@@ -1385,13 +1385,13 @@ void DictInterface::clientDone()
         global->databases = global->serverDatabases;
         for (int i = global->databaseSets.count()-1;i>=0;i--)
           global->databases.prepend(global->databaseSets.at(i)->first());
-        global->databases.prepend(i18n("all databases"));
+        global->databases.prepend(i18n("All Databases"));
         global->currentDatabase = 0;
 
         global->strategies.clear();
         for (it = job->strategies.begin(); it != job->strategies.end(); ++it)
           global->strategies.append(*it);
-        global->strategies.prepend(i18n("spell check"));
+        global->strategies.prepend(i18n("Spell Check"));
         global->currentStrategy = 0;
         message = i18n(" Received database/strategy list ");
         emit stopped(message);
