@@ -99,7 +99,8 @@ void GlobalData::read()
   f_onts[Ftext]=config->readFontEntry("textFont",&defFont);
   f_ontNames[Ftext]=i18n("Text");
 
-  defFont.setPointSize(defFont.pointSize()+5);
+  defFont.setBold( true );
+  defFont.setPointSize(defFont.pointSize()+2);
   f_onts[Fheadings]=config->readFontEntry("headingsFont",&defFont);
   f_ontNames[Fheadings]=i18n("Headings");
 
@@ -138,9 +139,9 @@ void GlobalData::read()
   currentDatabase = config->readNumEntry("Current_Database",0);
   strategies = config->readListEntry("Strategies");
   if (strategies.isEmpty()) {
-    strategies.append(i18n("spell check"));
-    strategies.append("exact");
-    strategies.append("prefix");
+    strategies.append(i18n("Spell Check"));
+    strategies.append(i18n("Exact"));
+    strategies.append(i18n("Prefix"));
   } else {
     strategies.remove(strategies.begin());
     strategies.prepend(i18n("spell check"));
@@ -487,7 +488,7 @@ OptionsDialog::OptionsDialog(QWidget *parent, const char *name)
 {
 
   //******** Server ************************************
-  serverTab = addPage(i18n("Server"),i18n("DICT server configuration"), BarIcon("network", KIcon::SizeMedium ));
+  serverTab = addPage(i18n("Server"),i18n("DICT Server Configuration"), BarIcon("network", KIcon::SizeMedium ));
   QGridLayout* grid = new QGridLayout(serverTab,10,3,5);
 
   w_server = new KLineEdit(serverTab);
@@ -520,7 +521,7 @@ OptionsDialog::OptionsDialog(QWidget *parent, const char *name)
   w_pipesize = new KIntSpinBox(100,5000,2,2,10,serverTab);
   w_pipesize->setSuffix(i18n(" bytes"));
   w_pipesize->setValue(global->pipeSize);
-  l = new QLabel(w_pipesize, i18n("Command &Buffer:"), serverTab);
+  l = new QLabel(w_pipesize, i18n("Command &buffer:"), serverTab);
   grid->addWidget(l,4,0);
   grid->addWidget(w_pipesize,4,1);
 
@@ -565,7 +566,7 @@ OptionsDialog::OptionsDialog(QWidget *parent, const char *name)
   grid->setColStretch(2,2);
 
   //************ Appearance ***************************
-  appTab = addPage(i18n("Appearance"),i18n("Customize visual appearance"), BarIcon("appearance", KIcon::SizeMedium ));
+  appTab = addPage(i18n("Appearance"),i18n("Customize Visual Appearance"), BarIcon("appearance", KIcon::SizeMedium ));
 
   QGridLayout *topL=new QGridLayout(appTab, 8,3, 5,5);
 
@@ -579,7 +580,7 @@ OptionsDialog::OptionsDialog(QWidget *parent, const char *name)
   topL->addWidget(c_olorCB,0,0);
   connect(c_olorCB, SIGNAL(toggled(bool)), this, SLOT(slotColCheckBoxToggled(bool)));
 
-  c_olChngBtn=new QPushButton(i18n("Cha&nge"), appTab);
+  c_olChngBtn=new QPushButton(i18n("Cha&nge..."), appTab);
   connect(c_olChngBtn, SIGNAL(clicked()), SLOT(slotColChangeBtnClicked()));
   topL->addWidget(c_olChngBtn,1,2);
 
@@ -597,7 +598,7 @@ OptionsDialog::OptionsDialog(QWidget *parent, const char *name)
   topL->addWidget(f_ontCB,4,0);
   connect(f_ontCB, SIGNAL(toggled(bool)), SLOT(slotFontCheckBoxToggled(bool)));
 
-  f_ntChngBtn=new QPushButton(i18n("Chang&e"), appTab);
+  f_ntChngBtn=new QPushButton(i18n("Chang&e..."), appTab);
   connect(f_ntChngBtn, SIGNAL(clicked()), SLOT(slotFontChangeBtnClicked()));
   topL->addWidget(f_ntChngBtn,5,2);
 
@@ -623,7 +624,7 @@ OptionsDialog::OptionsDialog(QWidget *parent, const char *name)
     f_List->insertItem(new FontListItem(global->fontName(i), global->font(i)));
 
   //************ Layout ***************************
-  layoutTab = addPage(i18n("Layout"),i18n("Customize output format"), BarIcon("text_left", KIcon::SizeMedium ));
+  layoutTab = addPage(i18n("Layout"),i18n("Customize Output Format"), BarIcon("text_left", KIcon::SizeMedium ));
 
   QVBoxLayout *vbox = new QVBoxLayout(layoutTab, 5, 0);
 
@@ -645,7 +646,7 @@ OptionsDialog::OptionsDialog(QWidget *parent, const char *name)
   vbox->addStretch(1);
 
   //************ Other ***************************
-  otherTab = addPage(i18n("Miscellaneous"),i18n("Various settings"), BarIcon("misc", KIcon::SizeMedium ));
+  otherTab = addPage(i18n("Miscellaneous"),i18n("Various Settings"), BarIcon("misc", KIcon::SizeMedium ));
 
   vbox = new QVBoxLayout(otherTab, 5, 0);
 
@@ -662,13 +663,13 @@ OptionsDialog::OptionsDialog(QWidget *parent, const char *name)
 
   w_Maxbrowse = new KIntSpinBox(1,100,1,1,10,group);
   w_Maxbrowse->setValue(global->maxBrowseListEntrys);
-  l = new QLabel(w_Maxbrowse, i18n("Cached &Results:"), group);
+  l = new QLabel(w_Maxbrowse, i18n("Cached &results:"), group);
   grid->addWidget(l,2,0);
   grid->addWidget(w_Maxbrowse,2,1);
 
   w_Maxhist = new KIntSpinBox(10,5000,10,10,10,group);
   w_Maxhist->setValue(global->maxHistEntrys);
-  l = new QLabel(w_Maxhist, i18n("Hi&story Entrys:"), group);
+  l = new QLabel(w_Maxhist, i18n("Hi&story entries:"), group);
   grid->addWidget(l,3,0);
   grid->addWidget(w_Maxhist,3,1);
 
