@@ -29,10 +29,10 @@ KitProfileInfo::KitProfileInfo(QWidget *parent, const char *name)
 	profileList->setMinimumHeight(fontMetrics().height() * 5);
 
 	KButtonBox *buttonBox = new KButtonBox(mainBox, Qt::Vertical);
-	
+
 	useButton = buttonBox->addButton(i18n("Use"));
 	setButton = buttonBox->addButton(i18n("Set Default"));
-	addButton = buttonBox->addButton(i18n("Add"));
+	addButton = buttonBox->addButton(i18n("Add..."));
 	deleteButton = buttonBox->addButton(i18n("Delete"));
 
 	buttonBox->layout();
@@ -77,7 +77,7 @@ void KitProfileInfo::addClicked(void)
 	if(KMessageBox::warningYesNo(0, i18n("Would you like to go to AOL and create a new account?")) == KMessageBox::Yes)
 		KApplication::kApplication()->invokeBrowser("http://aim.aol.com/aimnew/Aim/register.adp");
 	// now go
-	KLineEditDlg lineDialog(i18n("Enter a name for the profile."), QString::null, this);
+	KLineEditDlg lineDialog(i18n("Enter a name for the profile:"), QString::null, this);
 	if(lineDialog.exec() == KLineEditDlg::Accepted)
 	{
 		KSimpleConfig config("kitprofilerc");
