@@ -324,7 +324,7 @@ void TAim::tocConnect1(void)
 		tocConnectError(i18n("No data to read. This shouldn't happen, since this function is called only by the emit readData."));
 	
 	if(i != SFLAP_SIGNON)
-		tocConnectError(i18n("Missed signon frame from server"));
+		tocConnectError(i18n("Missed sign-on frame from server"));
 
 	socket->writeSignon( 1, 1, tocNormalize(username) );
 
@@ -373,7 +373,7 @@ void TAim::tocConnect3(void)
 	sendUserInfo();
 	sendConfig();
 
-	emit updateProgress(7, i18n("Completing signon..."));
+	emit updateProgress(7, i18n("Completing sign-on..."));
 
 	tocInitDone();
 	emit endProgress();
@@ -470,7 +470,7 @@ void TAim::tocServer(void)
 		}
 		else
 		{
- 			QString error = i18n("Error in changing the password.");
+ 			QString error = i18n("Error while changing the password.");
 			emit displayError(error);
 		}
 	end_case_tocServer
@@ -502,7 +502,7 @@ void TAim::updateIdleness(void)
 void TAim::tocSignon(const QString &auth_host, const QString &auth_port)
 {
 	QString data;
-	data.sprintf("toc_signon %s %s  %s %s %s \"TIK:\\$Revision: 1.22 $\"",
+	data.sprintf("toc_signon %s %s  %s %s %s \"TIK:\\$Revision: 1.23 $\"",
 	             auth_host.latin1(), auth_port.latin1(),
 	             (tocNormalize(username)).latin1(), r_password.latin1(), TOC_LANG);
 
@@ -797,7 +797,7 @@ void TAim::doError(const QString &data)
 		doError_errorCase(981, i18n("The service is temporarily unavailable."));
 		doError_errorCase(982, i18n("Your warning level is currently too high to sign on."));
 		doError_errorCase(983, i18n("You have been connecting and disconnecting too frequently. Wait 10 minutes and try again. If you continue to try, you will need to wait even longer."));
-		doError_errorCase(989, i18n("Unknown signon failure: %1"));
+		doError_errorCase(989, i18n("Unknown sign-on failure: %1"));
 	}
 #undef doError_errorCase
 
