@@ -7,11 +7,13 @@
 #include "config.h"
 
 #include <kaction.h>
+#include <kglobalsettings.h>
+#include <kglobal.h>
 #include <kcolorbutton.h>
 #include <kcolordialog.h>
 #include <kfile.h>
 #include <kiconloader.h>
-#include <klineedit.h>
+#include <ktextedit.h>
 #include <khtmlview.h>
 #include <kmenubar.h>
 #include <knotifyclient.h>
@@ -19,8 +21,10 @@
 #include <kpopupmenu.h>
 #include <kmessagebox.h>
 #include <klocale.h>
+#include <qlayout.h>
 #include <qtextcodec.h>
 #include <qvbox.h>
+#include <kdebug.h>
 
 #include "aim.h"
 #include "kitapp.h"
@@ -33,6 +37,8 @@ inline bool tagIsOpen(const QString &html, const QString &tag)
 	return html.contains("<" + tag, false) > html.contains("</" + tag + ">", false);
 }
 
+#define SKIP_TO_SEMI {while(newstr[++i] != ';' && i < newstr.length());}
+
 #define SKIP_TO_GT {while(newstr[++i] != '>' && i < newstr.length());}
 
 #define SKIP_TO_GT_CHECK_QUOTE {int quotecount = 0;\
@@ -57,12 +63,26 @@ void parseAndProcess(QString &newstr)
 			newstr.replace(i, 1, "&gt;");
 			i += 3;
 		}
+		else if(newstr[i] == '&')
+		{
+			QRegExp entity("^&(?=[^;<>&]*;)");
+			if(entity.search(newstr.mid(i)) == 0)
+			{
+				SKIP_TO_SEMI
+			}
+			else
+			{
+				newstr.replace(i, 1, "&amp;");
+				i += 4;
+			}
+		}
 		else if(newstr[i] == '<')
 		{
 			if(newstr.mid(i, 3).upper() == "<A ") SKIP_TO_GT_CHECK_QUOTE
 			else if(newstr.mid(i, 3).upper() == "<B>") i += 2;
 			else if(newstr.mid(i, 5).upper() == "<BIG>") i += 4;
 			else if(newstr.mid(i, 4).upper() == "<BR>") i += 3;
+			else if(newstr.mid(i, 5).upper() == "<BR/>") i += 4;
 			else if(newstr.mid(i, 6).upper() == "<CODE>") i += 5;
 			else if(newstr.mid(i, 3).upper() == "<EM") SKIP_TO_GT_CHECK_QUOTE
 			else if(newstr.mid(i, 5).upper() == "<FONT") SKIP_TO_GT_CHECK_QUOTE
@@ -152,7 +172,7 @@ TMessage::TMessage(const QString &display, const QString &internal,
 	name = internal;
 	myName = mine;
 	setCaption(displayName);
-	
+
 	// away messages
 	isAway = false;
 	awayMessage = QString::null;
@@ -170,14 +190,16 @@ TMessage::TMessage(const QString &display, const QString &internal,
 	bold = Bold;
 
 	isICQ = icq;
-	
+	lineBreak = false;
+
 	// menu & toolbar
 	initActions();
-	
+
 	// the stuff in the middle
-	centralWidget = new QVBox(this);
-	centralWidget->setMargin(KDialog::marginHint());
-	centralWidget->setSpacing(KDialog::spacingHint());
+	centralWidget = new QFrame(this);
+	QVBoxLayout *layout = new QVBoxLayout(centralWidget);
+	layout->setMargin(0);
+	layout->setSpacing(KDialog::spacingHint());
 	setCentralWidget(centralWidget);
 
 	htmlpart = new KitHTMLPart(centralWidget, 0, this);
@@ -193,21 +215,27 @@ TMessage::TMessage(const QString &display, const QString &internal,
 	htmlpart->setPluginsEnabled(false);
 	htmlpart->show();
 	initializeHTML();
-	
+
 	QHBox *bottomRow = new QHBox(centralWidget);
-	editFore = new KColorButton(bottomRow);
-	editBack = new KColorButton(bottomRow);
-	edit = new KLineEdit(bottomRow);
-	bottomRow->setMinimumHeight(edit->sizeHint().height());
-	bottomRow->setMaximumHeight(edit->sizeHint().height());
-	editFore->setMaximumWidth(edit->sizeHint().height());
-	editBack->setMaximumWidth(edit->sizeHint().height());
+	// No margin, that's the rule
+	bottomRow->setMargin(0);
+	bottomRow->setSpacing(KDialog::spacingHint());
+	// make color buttons in their own vertical arrangement - jason
+	QVBox *colors = new QVBox(bottomRow); 
+	colors->setMargin(0);
+	colors->setSpacing(KDialog::spacingHint());
+	editFore = new KColorButton(colors);
+	editBack = new KColorButton(colors);
+	edit = new KTextEdit(bottomRow);
 	connect(edit, SIGNAL(returnPressed()), this, SLOT(send()) );
 	connect( editFore, SIGNAL(changed(const QColor &)), this, SLOT(forePressed2(const QColor &)) );
 	connect( editBack, SIGNAL(changed(const QColor &)), this, SLOT(backPressed2(const QColor &)) );
 
+	layout->addWidget(htmlpart->view(), 20, 0);
+	layout->addWidget(bottomRow, 5, 0);
+
 	centralWidget->setMinimumHeight(KDialog::marginHint() * 2 + KDialog::spacingHint() + htmlpart->view()->minimumHeight() + bottomRow->minimumHeight());
-	resize(400,200 >= centralWidget->minimumHeight() ? 200 : centralWidget->minimumHeight());
+	resize(400, 300);
 	setColors(fore, back, bold);
 	setIcon(DesktopIcon("kit"));
 
@@ -253,11 +281,9 @@ void TMessage::setMyName(const QString &newname)
 }
 void TMessage::setColors(const QColor &fg, const QColor &bg, bool Bold)
 {
-	if(fore != fg)
-		editFore->setColor(fore);
+	editFore->setColor(fore);
 	fore = fg;
-	if(back != bg)
-		editBack->setColor(back);
+	editBack->setColor(back);
 	back = bg;
 	bold = Bold;
 }
@@ -276,16 +302,45 @@ void TMessage::resizeEvent(QResizeEvent *)
 	if(htmlpart && htmlpart->view())
 		htmlpart->view()->ensureVisible(0, htmlpart->view()->contentsHeight());
 }
+
+void TMessage::keyPressEvent(QKeyEvent *e)
+{
+	if(e->key() == Qt::Key_Shift)
+	{
+		edit->grabKeyboard();
+		lineBreak = true;
+	}
+}
+
+void TMessage::keyReleaseEvent(QKeyEvent *e)
+{
+	if(e->key() == Qt::Key_Shift)
+	{
+		edit->releaseKeyboard();
+		lineBreak = false;
+	}
+}
+
 // * class TMessage -- protected slots
 void TMessage::send(void)
 {
-	// avoid doubles
-	disconnect(edit, SIGNAL(returnPressed()), this, SLOT(send()) );
+	// Don't send on these
+	if(lineBreak) return;
+
+	edit->doKeyboardAction(QTextEdit::ActionBackspace);
 	QString message = edit->text();
-	edit->clear();
-	connect(edit, SIGNAL(returnPressed()), this, SLOT(send()) );
+
 	// don't send empty messages
-	if(message == "") return;
+	if(message.stripWhiteSpace().isEmpty())
+	{
+		edit->clear();
+		return;
+	}
+
+	edit->clear();
+
+	message.replace(QRegExp("\n$"), "");
+
 	// turn into AOL-HTML
 	if(isICQ)
 	{
@@ -298,6 +353,10 @@ void TMessage::send(void)
 		aolHTML.sprintf("<HTML><BODY BGCOLOR=\"#%02X%02X%02X\"><FONT COLOR=\"#%02X%02X%02X\">",
 			back.red(), back.green(), back.blue(), fore.red(), fore.green(), fore.blue());
 		if(bold) aolHTML += "<B>";
+		message.replace(QRegExp("&"), "&amp;");
+		message.replace(QRegExp("<"), "&lt;");
+		message.replace(QRegExp(">"), "&gt;");
+		message.replace(QRegExp("\n"), "<br/>");
 		aolHTML += message;
 		if(bold) aolHTML += "</B>";
 		aolHTML += "</FONT></BODY></HTML>";
@@ -395,9 +454,9 @@ void TMessage::initActions(void)
 	NEWTOGGLEACTION(aTimestamping, "kit_timestamping", i18n("&Timestamping"), timestampingPressed);
 	NEWTOGGLEACTION(aSound, "kit_sound", i18n("&Sound"), soundPressed);
 	NEWTOGGLEACTION(aICQ, "kit_icq", i18n("ICQ &Mode"), icqPressed);
-	NEWACTION(aFore, "pencil", i18n("&Text color..."), forePressed);
-	NEWACTION(aBack, "fill", i18n("Bac&kground color..."), backPressed);
-	NEWTOGGLEACTION(aBold, "kit_bold", i18n("&Bold text"), boldPressed);
+	NEWACTION(aFore, "pencil", i18n("&Text Color..."), forePressed);
+	NEWACTION(aBack, "fill", i18n("Bac&kground Color..."), backPressed);
+	NEWTOGGLEACTION(aBold, "kit_bold", i18n("&Bold Text"), boldPressed);
 #undef NEW_ACTION
 	aLogging->setChecked(isLogging);
 	aTimestamping->setChecked(isTimestamping);
@@ -430,7 +489,7 @@ void TMessage::initActions(void)
 	// place menus into menubar
 	menuBar()->insertItem(i18n("&User"), user);
 	menuBar()->insertItem(i18n("&Settings"), settings);
-	
+
 	// place actions into toolbar
 	aInfo->plug(toolBar(0));
 		toolBar(0)->insertSeparator();
@@ -507,10 +566,8 @@ void TMessage::updateHTML(QString message, QString sender, QString HTMLColor, bo
 	if(isTimestamping)
 	{
 		QDateTime time = QDateTime::currentDateTime();
-		timeStr = QString("<b>[%1 %2 - %3]</b>")
-		           .arg(time.date().day())
-		           .arg(QDate::shortMonthName(time.date().month()))
-		           .arg(time.time().toString());
+		timeStr = QString("<b>[%1]</b>")
+		           .arg(KGlobal::locale()->formatDateTime(time));
 	}
 	// HTMLize
 	QString HTMLMessage("<p>");
