@@ -8,20 +8,25 @@
 
 #include <kaboutapplication.h>
 #include <kaboutkde.h>
+#include <kaction.h>
 #include <kbugreport.h>
 #include <kconfig.h>
 #include <kiconloader.h>
 #include <klineeditdlg.h>
-#include <kmenubar.h>
-#include <knotifyclient.h>
-#include <kwin.h>
-#include <kmessagebox.h>
-#include <qheader.h>
 #include <klocale.h>
+#include <kmenubar.h>
+#include <kmessagebox.h>
+#include <knotifyclient.h>
+#include <knotifydialog.h>
+#include <kpopupmenu.h>
+#include <kstdaction.h>
+#include <kwin.h>
+#include <qheader.h>
 
 #include "kitapp.h"
 #include "kitdirectorysearch.h"
 #include "kitpasswordchanger.h"
+#include "kitsystemtray.h"
 #include "twindow.h"
 
 inline bool sanityCheck(TAim &aim)
@@ -50,7 +55,7 @@ void TWindow::updateTreeItem(QListViewItem *item, TBuddy &currentBuddy, int list
 		// signon time (in UNIX time.. needs conversion to something readable)
 		QDateTime signonTime;
 		signonTime.setTime_t(currentBuddy.signonTime);
-		sTime = signonTime.toString();
+		sTime = KGlobal::locale()->formatDateTime(signonTime);
 
 		// user class
 		switch(currentBuddy.userClass)
@@ -115,15 +120,13 @@ void TWindow::starsAndTheMoon(void)
 	// nowhere else to put this
 	awayMessage = QString::null;
 
-	// dock
-	// dock comes before menu because we need the dockWindow's context menu
-	dockWindow = new TDockWindow(this);
-	dockWindow->show();
-	dockMenu = dockWindow->contextMenu();
-	
 	// menu
-	initMenu();
-	
+	initActions();
+
+	// tray comes after actions because we need the createGUI
+	tray = new KitSystemTray("trayMenu", this);
+	tray->show();
+
 	// aim
 	connect( &aim, SIGNAL(statusChanged(int)), this, SLOT(setStatus(int)) );
 	connect( &aim, SIGNAL(userInfoReceived(QString)), this, SLOT(displayInfo(QString)) );
@@ -142,7 +145,7 @@ void TWindow::starsAndTheMoon(void)
 
 	// Visible Widgets
 	mainView = new QVBox(this);
-	mainView->setMargin(KDialog::marginHint());
+	mainView->setMargin(0);
 	mainView->setSpacing(KDialog::spacingHint());
 	mainView->setMinimumSize(100, 250);
 	setCentralWidget(mainView);
@@ -187,107 +190,98 @@ void TWindow::starsAndTheMoon(void)
 	profileUpdated();
 
 	if(profile->data()->autoConnect)
-		commandCallback(ID_TONLINE);
+		online();
 }
-void TWindow::commandCallback(int command)
+
+void TWindow::online(void)
 {
-	switch(command)
+	setStatus(TAIM_ONLINE);
+	awayMessage = QString::null;
+	emit globalAway(false, QString::null);
+	if(aim.getStatus() != TAIM_ONLINE)
 	{
-		case ID_TONLINE:
-			setStatus(TAIM_ONLINE);
-			awayMessage = QString::null;
-			emit globalAway(false, QString::null);
-			if(aim.getStatus() != TAIM_ONLINE)
-			{
-				// if no password or username specified, ask for one
-				// if the dialog returns successfully, try again
-				if( profile->data()->name.isNull() ||
-				    (!profile->data()->password.length() && !profile->data()->savePassword)
-				  )
-					profile->execEditor();
-				aim.setPassword(profile->data()->password);
-				aim.setStatus(TAIM_ONLINE);
-				initTree();
-			}
-			break;
-		case ID_TAWAY:
-			if(aim.getStatus() != TAIM_ONLINE) break;
-			awayMessage = doAway();
-			if(awayMessage != QString::null)
-			{
-				emit globalAway(true, awayMessage);
-				setStatus(TAIM_AWAY);
-			}
-			else setStatus(TAIM_ONLINE);
-			break;
-		case ID_TOFFLINE:
-			aim.setStatus(TAIM_OFFLINE);
-			break;
-		case ID_TOPEN:
-			if(!sanityCheck(aim)) return;
-			{
-			KLineEditDlg lineDialog(i18n("Enter the screen name to open."), QString::null, this);
-			if(lineDialog.exec() != KLineEditDlg::Accepted) return;
-			TMessage &targetWindow = messageWindow(lineDialog.text());
-			targetWindow.show();
-			}
-			break;
-		case ID_TEXIT:
-			if(QApplication::sendEvent(this, new QCloseEvent())) delete this;
-			break;
-		case ID_TPREFERENCES:
+		// if no password or username specified, ask for one
+		// if the dialog returns successfully, try again
+		if( profile->data()->name.isNull() ||
+		    (!profile->data()->password.length() && !profile->data()->savePassword)
+		  )
 			profile->execEditor();
-			break;
-		case ID_TPROFILES:
-			((KitApp *)KApplication::kApplication())->popProfileInfo();
-			break;
-		case ID_TABOUT:
-			{
-			KAboutApplication aboutBox;
-			aboutBox.exec();
-			}
-			break;
-		case ID_TABOUTKDE:
-			{
-			KAboutKDE aboutBox;
-			aboutBox.exec();
-			}
-			break;
-		case ID_TBUG:
-			{
-			KBugReport bugBox;
-			bugBox.exec();
-			}
-			break;
-		case ID_TDIRECTORY:
-			if(!sanityCheck(aim)) return;
-			{
-			KitDirectorySearch searchBox;
-			TAim::directory directory;
-			if(searchBox.exec())
-			{
-				directory = searchBox.result();
-				aim.searchDirectory(directory);
-			}
-			}
-			break;
-		case ID_THELP:
-			appHelpActivated();
-			break;
-		case ID_TPASSCHANGE:
-			if(!sanityCheck(aim)) return;
-			{
-			KitPasswordChanger dlg;
-			dlg.setOldPassword(profile->data()->password);
-			if(dlg.exec())
-			{
-				aim.changePassword(dlg.oldPassword(), dlg.newPassword());
-				newPassword = dlg.newPassword();
-			}
-			}
-			break;
-  	};
+		aim.setPassword(profile->data()->password);
+		aim.setStatus(TAIM_ONLINE);
+		initTree();
+	}
 }
+
+void TWindow::away(void)
+{
+	if(aim.getStatus() != TAIM_ONLINE) return;
+	awayMessage = doAway();
+	if(awayMessage != QString::null)
+	{
+		emit globalAway(true, awayMessage);
+		setStatus(TAIM_AWAY);
+	}
+	else setStatus(TAIM_ONLINE);
+}
+
+void TWindow::offline(void)
+{
+	aim.setStatus(TAIM_OFFLINE);
+}
+
+void TWindow::openChat(void)
+{
+	if(!sanityCheck(aim)) return;
+	KLineEditDlg lineDialog(i18n("Enter the screen name to open:"), QString::null, this);
+	if(lineDialog.exec() != KLineEditDlg::Accepted) return;
+	TMessage &targetWindow = messageWindow(lineDialog.text());
+	targetWindow.show();
+}
+
+void TWindow::quit(void)
+{
+	if(QApplication::sendEvent(this, new QCloseEvent())) delete this;
+}
+
+void TWindow::configure(void)
+{
+	profile->execEditor();
+}
+
+void TWindow::profiles(void)
+{
+	((KitApp *)KApplication::kApplication())->popProfileInfo();
+}
+
+void TWindow::searchDirectory(void)
+{
+	if(!sanityCheck(aim)) return;
+	KitDirectorySearch searchBox;
+	TAim::directory directory;
+	if(searchBox.exec())
+	{
+		directory = searchBox.result();
+		aim.searchDirectory(directory);
+	}
+}
+
+void TWindow::changePassword(void)
+{
+	if(!sanityCheck(aim)) return;
+	KitPasswordChanger dlg;
+	dlg.setOldPassword(profile->data()->password);
+	if(dlg.exec())
+	{
+		aim.changePassword(dlg.oldPassword(), dlg.newPassword());
+		newPassword = dlg.newPassword();
+	}
+}
+
+void TWindow::configureNotifications(void)
+{
+	KNotifyDialog::configure(this);
+}
+
 // *****************************
 // * class TWindow - protected *
 // *****************************
@@ -354,7 +348,7 @@ void TWindow::initTree(void)
 	if(profile->data()->listColumns & (1 << COL_NICK)) treelist->addColumn(i18n("Nickname"), -1);
 	if(profile->data()->listColumns & (1 << COL_IDLE)) treelist->addColumn(i18n("Idle"), -1);
 	if(profile->data()->listColumns & (1 << COL_WARN)) treelist->addColumn(i18n("Warn"), -1);
-	if(profile->data()->listColumns & (1 << COL_SIGNON)) treelist->addColumn(i18n("Signon"), -1);
+	if(profile->data()->listColumns & (1 << COL_SIGNON)) treelist->addColumn(i18n("Sign-on"), -1);
 	if(profile->data()->listColumns & (1 << COL_CLASS)) treelist->addColumn(i18n("Class"), -1);
 
 	// get list to use...
@@ -435,63 +429,63 @@ void TWindow::updateTree(int updated)
 	}
 	treelist->triggerUpdate();
 }
-void TWindow::selectedTree(QListViewItem *listItem)
+void TWindow::selectedTree(QListViewItem *item)
 {
 	// if the item has no parent, then it's a group, and I don't care if it is selected
-	if(listItem->parent() == 0) return;
+	if(item->parent() == 0) return;
 	// if the person's offline, back out
-	int num = aim.buddyList().getNum(listItem->text(COL_NAME));
+	int num = aim.buddyList().getNum(item->text(COL_NAME));
 	if(aim.buddyList().getStatus(num) != TAIM_ONLINE)
 	{
-		QString errorMessage = QString::fromLatin1("%1 is not available.").arg(listItem->text(COL_NAME));
+		QString errorMessage = QString::fromLatin1("%1 is not available.").arg(item->text(COL_NAME));
 		KMessageBox::error(0, errorMessage);
 		return;
 	}
 	// get the name of the requested person
-	QString name = listItem->text(COL_NAME);
+	QString name = item->text(COL_NAME);
 	TMessage &targetWindow = messageWindow(name);
 	targetWindow.show();
 }
-void TWindow::infoTree(QListViewItem *listItem)
+void TWindow::infoTree(void)
 {
 	// if the item has no parent, then it's a group, and I don't care if it is selected
-	if(!listItem || !listItem->parent()) return;
+	if(!treeCurrent || !treeCurrent->parent()) return;
 	// if the person's offline, back out
-	int num = aim.buddyList().getNum(listItem->text(COL_NAME));
+	int num = aim.buddyList().getNum(treeCurrent->text(COL_NAME));
 	if(aim.buddyList().getStatus(num) != TAIM_ONLINE)
 	{
-		QString errorMessage = QString::fromLatin1("%1 is not available.").arg(listItem->text(COL_NAME));
+		QString errorMessage = QString::fromLatin1("%1 is not available.").arg(treeCurrent->text(COL_NAME));
 		KMessageBox::error(0, errorMessage);
 		return;
 	}
 	// send the request
-	aim.requestUserInfo( listItem->text(COL_NAME) );
+	aim.requestUserInfo( treeCurrent->text(COL_NAME) );
 }
-void TWindow::directoryTree(QListViewItem *listItem)
+void TWindow::directoryTree(void)
 {
 	// if the item has no parent, then it's a group, and I don't care if it is selected
-	if(!listItem || !listItem->parent()) return;
+	if(!treeCurrent || !treeCurrent->parent()) return;
 	// if the person's offline, back out
-	int num = aim.buddyList().getNum(listItem->text(COL_NAME));
+	int num = aim.buddyList().getNum(treeCurrent->text(COL_NAME));
 	if(aim.buddyList().getStatus(num) != TAIM_ONLINE)
 	{
-		QString errorMessage = QString::fromLatin1("%1 is not available.").arg(listItem->text(COL_NAME));
+		QString errorMessage = QString::fromLatin1("%1 is not available.").arg(treeCurrent->text(COL_NAME));
 		KMessageBox::error(0, errorMessage);
 		return;
 	}
 	// send the request
-	aim.requestDirectory( listItem->text(COL_NAME) );
+	aim.requestDirectory( treeCurrent->text(COL_NAME) );
 }
-void TWindow::nickTree(QListViewItem *listItem)
+void TWindow::nickTree(void)
 {
 	// if the item has no parent, then it's a group, and I don't care if it is selected
-	if(!listItem || !listItem->parent()) return;
+	if(!treeCurrent || !treeCurrent->parent()) return;
 
-	QString name = listItem->text(COL_NAME);
+	QString name = treeCurrent->text(COL_NAME);
 
-	QString msg(i18n("Enter a nickname for %1"));
+	QString msg(i18n("Enter a nickname for %1:"));
 	msg = msg.arg(name);
-	
+
 	KLineEditDlg nameDialog(msg, profile->data()->nicknames[name], this);
 	if(nameDialog.exec() != KLineEditDlg::Accepted) return;
 
@@ -515,57 +509,30 @@ void TWindow::clickedTree(QListViewItem *item, const QPoint &point, int)
 	menu->insertTitle(item->text(COL_NAME), -1, 0);
 	menu->popup(point);
 }
-void TWindow::clickedTreeMenu(int item)
-{
-	switch(item)
-	{
-	case ID_TNICK:
-		nickTree(treeCurrent);
-		break;
-	case ID_TDIRECTORY:
-		directoryTree(treeCurrent);
-		break;
-	case ID_TMESSAGE:
-		selectedTree(treeCurrent);
-		break;
-	case ID_TINFO:
-		infoTree(treeCurrent);
-		break;
-	case ID_TADDBUDDY:
-		addBuddyToTree();
-		break;
-	case ID_TADDGROUP:
-		addGroupToTree();
-		break;
-	case ID_TDELBUDDY:
-		delBuddyFromTree();
-		break;
-	};
-}
 void TWindow::statusClicked(const QString &status)
 {
 	if(status == i18n("Online"))
-		commandCallback(ID_TONLINE);
+		online();
 	else if(status == i18n("Away"))
-		commandCallback(ID_TAWAY);
+		away();
 	else if(status == i18n("Offline"))
-		commandCallback(ID_TOFFLINE);
+		offline();
 }
 void TWindow::setStatus(int status)
 {
 	switch(status)
 	{
 	case TAIM_ONLINE:
-		dockWindow->setPixmap( BarIcon("kit_online") );
+		tray->setPixmap( BarIcon("kit_online") );
 		statusButton->setCurrentItem(0);
 		KNotifyClient::event("Online");
 		break;
 	case TAIM_AWAY:
-		dockWindow->setPixmap( BarIcon("kit_away") );
+		tray->setPixmap( BarIcon("kit_away") );
 		statusButton->setCurrentItem(1);
 		break;
 	case TAIM_OFFLINE:
-		dockWindow->setPixmap( BarIcon("kit_offline") );
+		tray->setPixmap( BarIcon("kit_offline") );
 		statusButton->setCurrentItem(2);
 		initTree();
 		KNotifyClient::event("Offline");
@@ -672,8 +639,9 @@ void TWindow::profileUpdated(void)
 	emit globalColors(profile->data()->foregroundColor, profile->data()->backgroundColor, profile->data()->bold);
 	emit globalICQ(profile->data()->icqMode);
 
-	dockMenu->removeItemAt(0);
-	dockMenu->insertTitle(profile->data()->name, -1, 0);
+	tray->changeTitle(BarIcon("kit"), profile->data()->name);
+
+	profile->save();
 }
 void TWindow::passwordChanged(void)
 {
@@ -682,78 +650,34 @@ void TWindow::passwordChanged(void)
 		profile->data()->password = newPassword;
 }
 
-
-// ****************************
-// * class TWindow -- private *
-// ****************************
-void TWindow::initMenu(void)
+void TWindow::initActions(void)
 {
-	file = new QPopupMenu(this);
-	file->insertItem( SmallIconSet("kit_online"), i18n("&Online"), ID_TONLINE );
-	file->insertItem( SmallIconSet("kit_away"), i18n("&Away..."), ID_TAWAY );
-	file->insertItem( SmallIconSet("kit_offline"), i18n("O&ffline"), ID_TOFFLINE );
-	file->insertSeparator();
-	file->insertItem( SmallIconSet("mail_send"), i18n("&Open Chat..."), ID_TOPEN );
-	file->insertSeparator();
-	file->insertItem( i18n("&Change Password..."), ID_TPASSCHANGE );
-	file->insertItem( SmallIconSet("find"), i18n("&Search Directory..."), ID_TDIRECTORY);
-	file->insertSeparator();
-	file->insertItem( SmallIconSet("exit"), i18n("E&xit"), ID_TEXIT );
-	connect( file, SIGNAL(activated(int)), SLOT(commandCallback(int)) );
+	(void)new KAction(i18n("&Online"), 0, this, SLOT(online(void)), actionCollection(), "online");
+	(void)new KAction(i18n("&Away..."), 0, this, SLOT(away(void)), actionCollection(), "away");
+	(void)new KAction(i18n("O&ffline"), 0, this, SLOT(offline(void)), actionCollection(), "offline");
+	(void)new KAction(i18n("&Open Chat..."), 0, this, SLOT(openChat(void)), actionCollection(), "openchat");
+	(void)new KAction(i18n("&Change Password..."), 0, this, SLOT(changePassword(void)), actionCollection(), "changepassword");
+	(void)new KAction(i18n("&Search Directory..."), 0, this, SLOT(searchDirectory(void)), actionCollection(), "searchdirectory");
+	(void)KStdAction::quit(this, SLOT(quit(void)), actionCollection());
+	(void)KStdAction::preferences(this, SLOT(configure(void)), actionCollection());
+	(void)KStdAction::configureNotifications(this, SLOT(configureNotifications(void)), actionCollection());
+	(void)new KAction(i18n("Configure &Profiles..."), 0, this, SLOT(profiles(void)), actionCollection(), "profiles");
 
-	settings = new QPopupMenu(this);
-	settings->insertItem( SmallIconSet("configure"), i18n("&Preferences..."), ID_TPREFERENCES  );
-	settings->insertSeparator();
-	settings->insertItem( SmallIconSet("fileopen"), i18n("&Profiles..."), ID_TPROFILES );
-	connect( settings, SIGNAL(activated(int)), SLOT(commandCallback(int)) );
+	(void)new KAction(i18n("&Set Nickname..."), 0, this, SLOT(nickTree(void)), actionCollection(), "treesetnickname");
+	(void)new KAction(i18n("&Open Chat..."), 0, this, SLOT(selectedTree(void)), actionCollection(), "treeopenchat");
+	(void)new KAction(i18n("&Information..."), 0, this, SLOT(infoTree(void)), actionCollection(), "treeinformation");
+	(void)new KAction(i18n("&Search Directory..."), 0, this, SLOT(directoryTree(void)), actionCollection(), "treesearchdirectory");
+	(void)new KAction(i18n("&Remove Buddy"), 0, this, SLOT(delBuddyFromTree(void)), actionCollection(), "treeremovebuddy");
+	(void)new KAction(i18n("&Add Buddy..."), 0, this, SLOT(addBuddyToTree(void)), actionCollection(), "treeaddbuddy");
+	(void)new KAction(i18n("&Add Group..."), 0, this, SLOT(addGroupToTree(void)), actionCollection(), "treeaddgroup");
 
-	help = new QPopupMenu(this);
-	help->insertItem( SmallIconSet("help"), i18n("&Help"), ID_THELP );
-	help->insertSeparator();
-	help->insertItem( i18n("&Report Bug..."), ID_TBUG );
-	help->insertSeparator();
-	help->insertItem( SmallIconSet("kit"), i18n("&About Kit..."), ID_TABOUT );
-	help->insertItem( i18n("About &KDE..."), ID_TABOUTKDE );
-	connect( help, SIGNAL(activated(int)), SLOT(commandCallback(int)) );
-	
-	treeMenu = new KPopupMenu(this);
-	treeMenu->insertTitle(("Howdy Doody"));
-	treeMenu->insertItem( i18n("Set nickname..."), ID_TNICK );
-	treeMenu->insertSeparator();
-	treeMenu->insertItem( SmallIconSet("mail_send"), i18n("Open chat..."), ID_TMESSAGE );
-	treeMenu->insertItem( SmallIconSet("kit_info"), i18n("Information"), ID_TINFO );
-	treeMenu->insertItem( SmallIconSet("find"), i18n("Directory"), ID_TDIRECTORY);
-	treeMenu->insertSeparator();
-	treeMenu->insertItem( i18n("Delete Buddy..."), ID_TDELBUDDY );
-	treeMenu->insertSeparator();
-	treeMenu->insertItem( i18n("Add Group..."), ID_TADDGROUP );
-	connect( treeMenu, SIGNAL(activated(int)), this, SLOT(clickedTreeMenu(int)) );
+	createGUI();
 
-	treeGroupMenu = new KPopupMenu(this);
-	treeGroupMenu->insertTitle(("Howdy Doody"));
-	treeGroupMenu->insertItem( i18n("Add Buddy..."), ID_TADDBUDDY );
-	treeGroupMenu->insertSeparator();
-	treeGroupMenu->insertItem( i18n("Add Group..."), ID_TADDGROUP );
-	connect( treeGroupMenu, SIGNAL(activated(int)), this, SLOT(clickedTreeMenu(int)) );
-
-	treeEmptyMenu = new KPopupMenu(this);
-	treeEmptyMenu->insertItem( i18n("Add Group..."), ID_TADDGROUP );
-	connect( treeEmptyMenu, SIGNAL(activated(int)), this, SLOT(clickedTreeMenu(int)) );
-
-	dockMenu->clear();
-	dockMenu->setTitle(profile->data()->name);
-	dockMenu->insertItem( SmallIconSet("kit_online"), i18n("Online"), ID_TONLINE );
-	dockMenu->insertItem( SmallIconSet("kit_away"), i18n("Away..."), ID_TAWAY );
-	dockMenu->insertItem( SmallIconSet("kit_offline"), i18n("Offline"), ID_TOFFLINE );
-	dockMenu->insertSeparator();
-	dockMenu->insertItem( SmallIconSet("exit"), i18n("Exit"), ID_TEXIT );
-	connect( dockMenu, SIGNAL(activated(int)), this, SLOT(commandCallback(int)) );
-	
-	menuBar()->insertItem(i18n("&File"), file );
-	menuBar()->insertItem(i18n("&Settings"), settings );
-	menuBar()->insertItem(i18n("&Help"), help );
-
-	menuBar()->show();
+	treeMenu = static_cast<KPopupMenu *>(guiFactory()->container("treeMenu", this));
+	treeMenu->insertTitle("", -1, 0);
+	treeGroupMenu = static_cast<KPopupMenu *>(guiFactory()->container("treeGroupMenu", this));
+	treeGroupMenu->insertTitle("", -1, 0);
+	treeEmptyMenu = static_cast<KPopupMenu *>(guiFactory()->container("treeEmptyMenu", this));
 }
 QString TWindow::doAway(void)
 {
@@ -772,7 +696,7 @@ void TWindow::addBuddyToTree(void)
 	}
 	newBuddy.group = profile->data()->names.getNumGroup(item->text(COL_NAME));
 	// now get the name from the user
-	KLineEditDlg nameDialog(i18n("Enter the screen name to add."), QString::null, this);
+	KLineEditDlg nameDialog(i18n("Enter the screen name to add:"), QString::null, this);
 	if(nameDialog.exec() != KLineEditDlg::Accepted) return;
 	newBuddy.name = nameDialog.text();
 	// warn if the add fails
@@ -800,7 +724,7 @@ void TWindow::addGroupToTree(void)
 	TBuddy newBuddy;
 
 	// get a name
-	KLineEditDlg groupDialog(i18n("Name the new group."), QString::null, this);
+	KLineEditDlg groupDialog(i18n("Name the new group:"), QString::null, this);
 	if(groupDialog.exec() != KLineEditDlg::Accepted) return;
 	// save the group number, and check if it's valid
 	profile->data()->names = aim.buddyList();
@@ -811,7 +735,7 @@ void TWindow::addGroupToTree(void)
 		return;
 	}
 	// get a name to start off the group
-	KLineEditDlg nameDialog(i18n("Enter the screen name to add."), QString::null, this);
+	KLineEditDlg nameDialog(i18n("Enter the screen name to add:"), QString::null, this);
 	if(nameDialog.exec() != KLineEditDlg::Accepted)
 	{
 		profile->data()->names.delGroup(newBuddy.group);
