@@ -1,5 +1,5 @@
 /*
- *   kmail: KDE mail client  
+ *   kmail: KDE mail client
  *   This file: Copyright (C) 2000 Espen Sand, espen@kde.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,7 @@
  *
  */
 
+#include <config.h>
 #include <qbuttongroup.h>
 #include <qcheckbox.h>
 #include <qlineedit.h>
@@ -27,6 +28,8 @@
 #include <qvalidator.h>
 #include <qlabel.h>
 #include <qpushbutton.h>
+#include <qwhatsthis.h>
+#include <qhbox.h>
 
 #include <kfiledialog.h>
 #include <klocale.h>
@@ -226,7 +229,7 @@ AccountDialog::AccountDialog( const QString & caption, KMAccount *account,
   : KDialogBase( parent, name, modal, caption, Ok|Cancel|Help, Ok, true ),
     mAccount(account)
 {
-  mServerTest = NULL;
+  mServerTest = 0;
   setHelp("receiving-mail");
 
   QString accountType = mAccount->type();
@@ -259,7 +262,7 @@ AccountDialog::AccountDialog( const QString & caption, KMAccount *account,
 
 AccountDialog::~AccountDialog()
 {
-  if (mServerTest) delete mServerTest;
+  delete mServerTest;
 }
 
 void AccountDialog::makeLocalAccountPage()
@@ -298,7 +301,7 @@ void AccountDialog::makeLocalAccountPage()
   connect( choose, SIGNAL(clicked()), this, SLOT(slotLocationChooser()) );
   topLayout->addWidget( choose, 3, 2 );
 
-  QButtonGroup *group = new QButtonGroup(i18n("Locking method"), page );
+  QButtonGroup *group = new QButtonGroup(i18n("Locking Method"), page );
   group->setColumnLayout(0, Qt::Horizontal);
   group->layout()->setSpacing( 0 );
   group->layout()->setMargin( 0 );
@@ -345,11 +348,12 @@ void AccountDialog::makeLocalAccountPage()
   topLayout->addMultiCellWidget( mLocal.intervalCheck, 6, 6, 0, 2 );
   connect( mLocal.intervalCheck, SIGNAL(toggled(bool)),
 	   this, SLOT(slotEnableLocalInterval(bool)) );
-  mLocal.intervalLabel = new QLabel( i18n("Check inter&val (minutes):"), page );
+  mLocal.intervalLabel = new QLabel( i18n("Check inter&val:"), page );
   topLayout->addWidget( mLocal.intervalLabel, 7, 0 );
   mLocal.intervalSpin = new KIntNumInput( page );
   mLocal.intervalLabel->setBuddy( mLocal.intervalSpin );
   mLocal.intervalSpin->setRange( 1, 10000, 1, FALSE );
+  mLocal.intervalSpin->setSuffix( i18n(" min") );
   mLocal.intervalSpin->setValue( 1 );
   topLayout->addWidget( mLocal.intervalSpin, 7, 1 );
 
@@ -371,7 +375,7 @@ void AccountDialog::makeLocalAccountPage()
 
   //mLocal.identityCombo->setEnabled(false);
 
-  label = new QLabel( i18n("&Precommand:"), page );
+  label = new QLabel( i18n("&Pre-Command:"), page );
   topLayout->addWidget( label, 9, 0 );
   mLocal.precommand = new QLineEdit( page );
   label->setBuddy( mLocal.precommand );
@@ -399,47 +403,50 @@ void AccountDialog::makeMaildirAccountPage()
   hline->setFrameStyle( QFrame::Sunken | QFrame::HLine );
   topLayout->addMultiCellWidget( hline, 1, 1, 0, 2 );
 
-  QLabel *label = new QLabel( i18n("Name:"), page );
-  topLayout->addWidget( label, 2, 0 );
   mMaildir.nameEdit = new QLineEdit( page );
   topLayout->addWidget( mMaildir.nameEdit, 2, 1 );
+  QLabel *label = new QLabel( mMaildir.nameEdit, i18n("&Name:"), page );
+  topLayout->addWidget( label, 2, 0 );
 
-  label = new QLabel( i18n("Location:"), page );
-  topLayout->addWidget( label, 3, 0 );
   mMaildir.locationEdit = new QComboBox( true, page );
   topLayout->addWidget( mMaildir.locationEdit, 3, 1 );
   mMaildir.locationEdit->insertStringList(procmailrcParser.getSpoolFilesList());
+  label = new QLabel( mMaildir.locationEdit, i18n("&Location:"), page );
+  topLayout->addWidget( label, 3, 0 );
 
-  QPushButton *choose = new QPushButton( i18n("Choose..."), page );
+  QPushButton *choose = new QPushButton( i18n("Choo&se..."), page );
   choose->setAutoDefault( false );
   connect( choose, SIGNAL(clicked()), this, SLOT(slotMaildirChooser()) );
   topLayout->addWidget( choose, 3, 2 );
 
   mMaildir.excludeCheck =
-    new QCheckBox( i18n("Exclude from \"Check Mail\""), page );
+    new QCheckBox( i18n("E&xclude from \"Check Mail\""), page );
   topLayout->addMultiCellWidget( mMaildir.excludeCheck, 5, 5, 0, 2 );
 
   mMaildir.intervalCheck =
-    new QCheckBox( i18n("Enable interval mail checking"), page );
+    new QCheckBox( i18n("Enable &interval mail checking"), page );
   topLayout->addMultiCellWidget( mMaildir.intervalCheck, 6, 6, 0, 2 );
   connect( mMaildir.intervalCheck, SIGNAL(toggled(bool)),
 	   this, SLOT(slotEnableMaildirInterval(bool)) );
-  mMaildir.intervalLabel = new QLabel( i18n("Check interval (minutes):"), page );
+  mMaildir.intervalLabel = new QLabel( i18n("Check inter&val:"), page );
   topLayout->addWidget( mMaildir.intervalLabel, 7, 0 );
   mMaildir.intervalSpin = new KIntNumInput( page );
   mMaildir.intervalSpin->setRange( 1, 10000, 1, FALSE );
+  mMaildir.intervalSpin->setSuffix( i18n(" min") );
   mMaildir.intervalSpin->setValue( 1 );
+  mMaildir.intervalLabel->setBuddy( mMaildir.intervalSpin );
   topLayout->addWidget( mMaildir.intervalSpin, 7, 1 );
 
-  label = new QLabel( i18n("Destination folder:"), page );
-  topLayout->addWidget( label, 8, 0 );
   mMaildir.folderCombo = new QComboBox( false, page );
   topLayout->addWidget( mMaildir.folderCombo, 8, 1 );
+  label = new QLabel( mMaildir.folderCombo,
+		      i18n("&Destination folder:"), page );
+  topLayout->addWidget( label, 8, 0 );
 
-  label = new QLabel( i18n("Precommand:"), page );
-  topLayout->addWidget( label, 9, 0 );
   mMaildir.precommand = new QLineEdit( page );
   topLayout->addWidget( mMaildir.precommand, 9, 1 );
+  label = new QLabel( mMaildir.precommand, i18n("&Pre-Command:"), page );
+  topLayout->addWidget( label, 9, 0 );
 
   connect(kapp,SIGNAL(kdisplayFontChanged()),SLOT(slotFontChanged()));
 }
@@ -451,7 +458,7 @@ void AccountDialog::makePopAccountPage()
   QVBoxLayout *topLayout = new QVBoxLayout( page, 0, spacingHint() );
 
   mPop.titleLabel = new QLabel( page );
-  mPop.titleLabel->setText( i18n("Account type: Pop Account") );
+  mPop.titleLabel->setText( i18n("Account type: POP Account") );
   QFont titleFont( mPop.titleLabel->font() );
   titleFont.setBold( true );
   mPop.titleLabel->setFont( titleFont );
@@ -465,9 +472,9 @@ void AccountDialog::makePopAccountPage()
   QWidget *page1 = new QWidget( tabWidget );
   tabWidget->addTab( page1, i18n("&General") );
 
-  QGridLayout *grid = new QGridLayout( page1, 16, 2, spacingHint() );
+  QGridLayout *grid = new QGridLayout( page1, 15, 2, marginHint(), spacingHint() );
   grid->addColSpacing( 1, fontMetrics().maxWidth()*15 );
-  grid->setRowStretch( 13, 10 );
+  grid->setRowStretch( 14, 10 );
   grid->setColStretch( 1, 10 );
 
   QLabel *label = new QLabel( i18n("&Name:"), page1 );
@@ -477,6 +484,7 @@ void AccountDialog::makePopAccountPage()
   grid->addWidget( mPop.nameEdit, 0, 1 );
 
   label = new QLabel( i18n("&Login:"), page1 );
+  QWhatsThis::add( label, i18n("Your Internet Service Provider gave you a <em>user name</em> which is used to authenticate you with their servers. It usually is the first part of your email address (the part before <em>@</em>).") );
   grid->addWidget( label, 1, 0 );
   mPop.loginEdit = new QLineEdit( page1 );
   label->setBuddy( mPop.loginEdit );
@@ -492,6 +500,10 @@ void AccountDialog::makePopAccountPage()
   label = new QLabel( i18n("Ho&st:"), page1 );
   grid->addWidget( label, 3, 0 );
   mPop.hostEdit = new QLineEdit( page1 );
+  // only letters, digits, '-', '.', ':' (IPv6) and '_' (for Windows
+  // compatibility) are allowed
+  mPop.hostEdit->setValidator(
+    new QRegExpValidator( QRegExp( "[A-Za-z0-9-_:.]*" ), 0 ) );
   label->setBuddy( mPop.hostEdit );
   grid->addWidget( mPop.hostEdit, 3, 1 );
 
@@ -507,55 +519,61 @@ void AccountDialog::makePopAccountPage()
   grid->addMultiCellWidget( mPop.storePasswordCheck, 6, 6, 0, 1 );
 
   mPop.deleteMailCheck =
-    new QCheckBox( i18n("&Delete mail from server"), page1 );
+    new QCheckBox( i18n("&Delete message from server after fetching"), page1 );
   grid->addMultiCellWidget( mPop.deleteMailCheck, 7, 7, 0, 1 );
 
   mPop.excludeCheck =
     new QCheckBox( i18n("E&xclude from \"Check Mail\""), page1 );
   grid->addMultiCellWidget( mPop.excludeCheck, 8, 8, 0, 1 );
 
+  QHBox * hbox = new QHBox( page1 );
+  hbox->setSpacing( KDialog::spacingHint() );
   mPop.filterOnServerCheck =
-    new QCheckBox( i18n("&Filter mail on server"), page1 );
-  grid->addMultiCellWidget( mPop.filterOnServerCheck, 9, 9, 0, 1 );
-  connect( mPop.filterOnServerCheck, SIGNAL(toggled(bool)),
-	   this, SLOT(slotEnableCheckSize(bool)) );
-  mPop.filterOnServerSizeLabel =
-    new QLabel( i18n("Si&ze of messages to filter:"), page1 );
-  grid->addWidget(mPop.filterOnServerSizeLabel, 10, 0 );
-  mPop.filterOnServerSizeSpin = new KIntNumInput ( page1 );
+    new QCheckBox( i18n("&Filter messages if they are greater than"), hbox );
+  mPop.filterOnServerSizeSpin = new KIntNumInput ( hbox );
+  mPop.filterOnServerSizeSpin->setEnabled( false );
+  hbox->setStretchFactor( mPop.filterOnServerSizeSpin, 1 );
   mPop.filterOnServerSizeSpin->setRange( 1, 10000000, 100, FALSE );
   mPop.filterOnServerSizeSpin->setValue( 50000 );
-  mPop.filterOnServerSizeLabel->setBuddy( mPop.filterOnServerSizeSpin );
-  grid->addWidget(mPop.filterOnServerSizeSpin, 10, 1 );
+  mPop.filterOnServerSizeSpin->setSuffix( i18n(" byte") );
+  grid->addMultiCellWidget( hbox, 9, 9, 0, 1 );
+  connect( mPop.filterOnServerCheck, SIGNAL(toggled(bool)),
+	   mPop.filterOnServerSizeSpin, SLOT(setEnabled(bool)) );
+  QString msg = i18n("If you select this option, POP Filters will be used to "
+		     "decide what to do with messages. You can then select "
+		     "to download, delete or keep them on the server." );
+  QWhatsThis::add( mPop.filterOnServerCheck, msg );
+  QWhatsThis::add( mPop.filterOnServerSizeSpin, msg );
 
   mPop.intervalCheck =
     new QCheckBox( i18n("Enable &interval mail checking"), page1 );
-  grid->addMultiCellWidget( mPop.intervalCheck, 11, 11, 0, 1 );
+  grid->addMultiCellWidget( mPop.intervalCheck, 10, 10, 0, 1 );
   connect( mPop.intervalCheck, SIGNAL(toggled(bool)),
 	   this, SLOT(slotEnablePopInterval(bool)) );
-  mPop.intervalLabel = new QLabel( i18n("Check inter&val (minutes):"), page1 );
-  grid->addWidget( mPop.intervalLabel, 12, 0 );
+  mPop.intervalLabel = new QLabel( i18n("Check inter&val:"), page1 );
+  grid->addWidget( mPop.intervalLabel, 11, 0 );
   mPop.intervalSpin = new KIntNumInput( page1 );
   mPop.intervalSpin->setRange( 1, 10000, 1, FALSE );
+  mPop.intervalSpin->setSuffix( i18n(" min") );
   mPop.intervalSpin->setValue( 1 );
   mPop.intervalLabel->setBuddy( mPop.intervalSpin );
-  grid->addWidget( mPop.intervalSpin, 12, 1 );
+  grid->addWidget( mPop.intervalSpin, 11, 1 );
 
   label = new QLabel( i18n("Des&tination folder:"), page1 );
-  grid->addWidget( label, 13, 0 );
+  grid->addWidget( label, 12, 0 );
   mPop.folderCombo = new QComboBox( false, page1 );
   label->setBuddy( mPop.folderCombo );
-  grid->addWidget( mPop.folderCombo, 13, 1 );
+  grid->addWidget( mPop.folderCombo, 12, 1 );
 
   label = new QLabel( i18n("Precom&mand:"), page1 );
-  grid->addWidget( label, 14, 0 );
+  grid->addWidget( label, 13, 0 );
   mPop.precommand = new QLineEdit( page1 );
   label->setBuddy(mPop.precommand);
-  grid->addWidget( mPop.precommand, 14, 1 );
+  grid->addWidget( mPop.precommand, 13, 1 );
 
   QWidget *page2 = new QWidget( tabWidget );
   tabWidget->addTab( page2, i18n("&Extras") );
-  QVBoxLayout *vlay = new QVBoxLayout( page2, spacingHint() );
+  QVBoxLayout *vlay = new QVBoxLayout( page2, marginHint(), spacingHint() );
 
   mPop.usePipeliningCheck =
     new QCheckBox( i18n("&Use pipelining for faster mail download"), page2 );
@@ -578,7 +596,7 @@ void AccountDialog::makePopAccountPage()
   vlay->addWidget( mPop.encryptionGroup );
 
   mPop.authGroup = new QButtonGroup( 1, Qt::Horizontal,
-    i18n("Authentication method"), page2 );
+    i18n("Authentication Method"), page2 );
   mPop.authUser = new QRadioButton( i18n("Clear te&xt") , mPop.authGroup );
   mPop.authPlain = new QRadioButton( i18n("Please translate this "
     "authentication method only if you have a good reason", "&PLAIN"),
@@ -595,7 +613,7 @@ void AccountDialog::makePopAccountPage()
 
   QHBoxLayout *buttonLay = new QHBoxLayout( vlay );
   mPop.checkCapabilities =
-    new QPushButton( i18n("Check &what the server supports"), page2 );
+    new QPushButton( i18n("Check &What the Server Supports"), page2 );
   connect(mPop.checkCapabilities, SIGNAL(clicked()),
     SLOT(slotCheckPopCapabilities()));
   buttonLay->addStretch();
@@ -625,7 +643,7 @@ void AccountDialog::makeImapAccountPage()
   QWidget *page1 = new QWidget( tabWidget );
   tabWidget->addTab( page1, i18n("&General") );
 
-  QGridLayout *grid = new QGridLayout( page1, 14, 2, spacingHint() );
+  QGridLayout *grid = new QGridLayout( page1, 14, 2, marginHint(), spacingHint() );
   grid->addColSpacing( 1, fontMetrics().maxWidth()*15 );
   grid->setRowStretch( 13, 10 );
   grid->setColStretch( 1, 10 );
@@ -637,6 +655,7 @@ void AccountDialog::makeImapAccountPage()
   grid->addWidget( mImap.nameEdit, 0, 1 );
 
   label = new QLabel( i18n("&Login:"), page1 );
+  QWhatsThis::add( label, i18n("Your Internet Service Provider gave you a <em>user name</em> which is used to authenticate you with their servers. It usually is the first part of your email address (the part before <em>@</em>).") );
   grid->addWidget( label, 1, 0 );
   mImap.loginEdit = new QLineEdit( page1 );
   label->setBuddy( mImap.loginEdit );
@@ -652,6 +671,10 @@ void AccountDialog::makeImapAccountPage()
   label = new QLabel( i18n("Ho&st:"), page1 );
   grid->addWidget( label, 3, 0 );
   mImap.hostEdit = new QLineEdit( page1 );
+  // only letters, digits, '-', '.', ':' (IPv6) and '_' (for Windows
+  // compatibility) are allowed
+  mImap.hostEdit->setValidator(
+    new QRegExpValidator( QRegExp( "[A-Za-z0-9-_:.]*" ), 0 ) );
   label->setBuddy( mImap.hostEdit );
   grid->addWidget( mImap.hostEdit, 3, 1 );
 
@@ -692,22 +715,23 @@ void AccountDialog::makeImapAccountPage()
   grid->addMultiCellWidget( mImap.intervalCheck, 11, 11, 0, 2 );
   connect( mImap.intervalCheck, SIGNAL(toggled(bool)),
 	   this, SLOT(slotEnableImapInterval(bool)) );
-  mImap.intervalLabel = new QLabel( i18n("Check inter&val (minutes):"), page1 );
+  mImap.intervalLabel = new QLabel( i18n("Check inter&val:"), page1 );
   grid->addWidget( mImap.intervalLabel, 12, 0 );
   mImap.intervalSpin = new KIntNumInput( page1 );
-  mImap.intervalSpin->setRange( 1, 10000, 1, FALSE );
+  mImap.intervalSpin->setRange( 1, 60, 1, FALSE );
   mImap.intervalSpin->setValue( 1 );
+  mImap.intervalSpin->setSuffix( i18n( " min" ) );
   mImap.intervalLabel->setBuddy( mImap.intervalSpin );
   grid->addWidget( mImap.intervalSpin, 12, 1 );
 
   mImap.trashCombo = new KMFolderComboBox( page1 );
   mImap.trashCombo->showOutboxFolder( FALSE );
-  grid->addMultiCellWidget( mImap.trashCombo, 13, 13, 1, 2 );
-  grid->addWidget( new QLabel( mImap.trashCombo, i18n("&Trash Folder:"), page1 ), 13, 0 );
+  grid->addWidget( mImap.trashCombo, 13, 1 );
+  grid->addWidget( new QLabel( mImap.trashCombo, i18n("&Trash folder:"), page1 ), 13, 0 );
 
   QWidget *page2 = new QWidget( tabWidget );
   tabWidget->addTab( page2, i18n("S&ecurity") );
-  QVBoxLayout *vlay = new QVBoxLayout( page2, spacingHint() );
+  QVBoxLayout *vlay = new QVBoxLayout( page2, marginHint(), spacingHint() );
 
   mImap.encryptionGroup = new QButtonGroup( 1, Qt::Horizontal,
     i18n("Encryption"), page2 );
@@ -724,7 +748,7 @@ void AccountDialog::makeImapAccountPage()
   vlay->addWidget( mImap.encryptionGroup );
 
   mImap.authGroup = new QButtonGroup( 1, Qt::Horizontal,
-    i18n("Authentication method"), page2 );
+    i18n("Authentication Method"), page2 );
   mImap.authUser = new QRadioButton( i18n("Clear te&xt"), mImap.authGroup );
   mImap.authPlain = new QRadioButton( i18n("Please translate this "
     "authentication method only if you have a good reason", "&PLAIN"),
@@ -741,7 +765,7 @@ void AccountDialog::makeImapAccountPage()
 
   QHBoxLayout *buttonLay = new QHBoxLayout( vlay );
   mImap.checkCapabilities =
-    new QPushButton( i18n("Check &what the server supports"), page2 );
+    new QPushButton( i18n("Check &What the Server Supports"), page2 );
   connect(mImap.checkCapabilities, SIGNAL(clicked()),
     SLOT(slotCheckImapCapabilities()));
   buttonLay->addStretch();
@@ -820,7 +844,6 @@ void AccountDialog::setupSettings()
     else mPop.authUser->setChecked( TRUE );
 
     slotEnablePopInterval( interval >= 1 );
-    slotEnableCheckSize( ap.filterOnServer() );
     folderCombo = mPop.folderCombo;
   }
   else if( accountType == "imap" )
@@ -849,7 +872,7 @@ void AccountDialog::setupSettings()
     QString trashfolder = ai.trash();
     if (trashfolder.isEmpty())
       trashfolder = kernel->trashFolder()->idString();
-    mImap.trashCombo->setFolder( trashfolder );	
+    mImap.trashCombo->setFolder( trashfolder );
     slotEnableImapInterval( interval >= 1 );
     if (ai.useSSL())
       mImap.encryptionSSL->setChecked( TRUE );
@@ -932,7 +955,7 @@ void AccountDialog::setupSettings()
 void AccountDialog::slotPipeliningClicked()
 {
   if (mPop.usePipeliningCheck->isChecked())
-    KMessageBox::information(0, 
+    KMessageBox::information(0,
       i18n("Please note that this feature can cause some POP3 servers "
       "that don't support pipelining to send corrupted mails.\n"
       "This is configurable, because some servers support pipelining "
@@ -962,7 +985,7 @@ void AccountDialog::slotImapEncryptionChanged(int id)
 
 void AccountDialog::slotCheckPopCapabilities()
 {
-  if (mServerTest) delete mServerTest;
+  delete mServerTest;
   mServerTest = new KMServerTest("pop3", mPop.hostEdit->text(),
     mPop.portEdit->text());
   connect(mServerTest, SIGNAL(capabilities(const QStringList &)),
@@ -973,7 +996,7 @@ void AccountDialog::slotCheckPopCapabilities()
 
 void AccountDialog::slotCheckImapCapabilities()
 {
-  if (mServerTest) delete mServerTest;
+  delete mServerTest;
   mServerTest = new KMServerTest("imap", mImap.hostEdit->text(),
     mImap.portEdit->text());
   connect(mServerTest, SIGNAL(capabilities(const QStringList &)),
@@ -997,8 +1020,8 @@ void AccountDialog::slotPopCapabilities(const QStringList &list)
   mPop.authAPOP->setEnabled(list.findIndex("APOP") != -1);
   checkHighest(mPop.encryptionGroup);
   checkHighest(mPop.authGroup);
-  if (mServerTest) delete mServerTest;
-  mServerTest = NULL;
+  delete mServerTest;
+  mServerTest = 0;
 }
 
 
@@ -1016,8 +1039,8 @@ void AccountDialog::slotImapCapabilities(const QStringList &list)
   mImap.authAnonymous->setEnabled(list.findIndex("AUTH=ANONYMOUS") != -1);
   checkHighest(mImap.encryptionGroup);
   checkHighest(mImap.authGroup);
-  if (mServerTest) delete mServerTest;
-  mServerTest = NULL;
+  delete mServerTest;
+  mServerTest = 0;
 }
 
 
@@ -1118,7 +1141,7 @@ void AccountDialog::saveSettings()
     mAccount->setCheckInterval( mImap.intervalCheck->isChecked() ?
                                 mImap.intervalSpin->value() : 0 );
     mAccount->setCheckExclude( mImap.excludeCheck->isChecked() );
-    mAccount->setFolder( NULL );
+    mAccount->setFolder( 0 );
 
     KMAcctImap &epa = *(KMAcctImap*)mAccount;
     epa.setHost( mImap.hostEdit->text().stripWhiteSpace() );
@@ -1167,7 +1190,7 @@ void AccountDialog::saveSettings()
   }
 
   kernel->acctMgr()->writeConfig(TRUE);
-  
+
   // get the new account and register the new destination folder
   KMAccount* newAcct = kernel->acctMgr()->find(mAccount->name());
   if (newAcct)
@@ -1203,7 +1226,7 @@ void AccountDialog::slotLocationChooser()
   }
   if( url.isLocalFile() == false )
   {
-    KMessageBox::sorry( 0L, i18n( "Only local files supported yet." ) );
+    KMessageBox::sorry( 0, i18n( "Only local files are currently supported." ) );
     return;
   }
 
@@ -1231,12 +1254,6 @@ void AccountDialog::slotEnablePopInterval( bool state )
   mPop.intervalLabel->setEnabled( state );
 }
 
-void AccountDialog::slotEnableCheckSize( bool state )
-{
-  mPop.filterOnServerSizeSpin->setEnabled( state );
-  mPop.filterOnServerSizeLabel->setEnabled( state );
-}
-
 void AccountDialog::slotEnableImapInterval( bool state )
 {
   mImap.intervalSpin->setEnabled( state );
