@@ -1,18 +1,13 @@
+// -*- c++ -*-
 // configuredialog_p.h: classes internal to ConfigureDialog
 // see configuredialog.h for details.
 
 #ifndef _CONFIGURE_DIALOG_PRIVATE_H_
 #define _CONFIGURE_DIALOG_PRIVATE_H_
 
-#define DEFAULT_EDITOR_STR "kate %f"
-
-
-#include "kmheaders.h"
-
 #include <qlineedit.h>
 #include <qcombobox.h>
 #include <qguardedptr.h>
-#include <qwidget.h>
 #include <qptrlist.h>
 #include <qstring.h>
 #include <qvaluelist.h>
@@ -20,13 +15,11 @@
 
 #include <kdialogbase.h>
 #include <klistview.h>
-#include <klocale.h> // for i18n
 
 class QPushButton;
 class QLabel;
 class QCheckBox;
 class KURLRequester;
-class QMultiLineEdit;
 class KFontChooser;
 class QRadioButton;
 class ColorListBox;
@@ -44,133 +37,16 @@ class ConfigureDialog;
 class KIntSpinBox;
 class SimpleStringListEditor;
 class KConfig;
+class QPoint;
+class CryptPlugWrapperList;
 namespace Kpgp {
   class Config;
 };
-
-//
-//
-// Identity handling
-//
-//
-
-class IdentityEntry
-{
-public:
-  IdentityEntry() : mSignatureFileIsAProgram( false ),
-    mUseSignatureFile( true ), mIsDefault( false ) {}
-  /** Convenience constructor for the most-often used settings */
-  IdentityEntry( const QString & fullName,
-		 const QString & email,
-		 const QString & organization=QString::null,
-		 const QString & replyTo=QString::null )
-    : mFullName( fullName ), mOrganization( organization),
-    mEmailAddress( email ), mReplyToAddress( replyTo ),
-    mSignatureFileIsAProgram( false ),
-    mUseSignatureFile( true ), mIsDefault( false ) {}
-
-  /** Constructs an IdentituEntry from the control center default
-      settings */
-  static IdentityEntry fromControlCenter();
-
-  /** used for sorting */
-  bool operator<( const IdentityEntry & other ) const {
-    // default is lower than any other, so it is always sorted to the
-    // top:
-    if ( isDefault() ) return true;
-    if ( other.isDefault() ) return false;
-    return identityName() < other.identityName();
-  }
-  // begin stupid stl headers workaround
-  bool operator>( const IdentityEntry & other ) const {
-    if ( isDefault() ) return false;
-    if ( other.isDefault() ) return true;
-    return identityName() > other.identityName();
-  }
-  bool operator<=( const IdentityEntry & other ) const {
-    return !operator>( other );
-  }
-  bool operator>=( const IdentityEntry & other ) const {
-    return !operator<( other );
-  }
-  // end stupid stl headers workaround
-  /** used for searching */
-  bool operator==( const IdentityEntry & other ) const {
-    return identityName() == other.identityName();
-  }
-
-  QString identityName() const { return mIdentityName; }
-  QString fullName() const { return mFullName; }
-  QString organization() const { return mOrganization; }
-  QString pgpIdentity() const { return mPgpIdentity ; }
-  QString emailAddress() const { return mEmailAddress; }
-  QString replyToAddress() const { return mReplyToAddress; }
-  QString signatureFileName() const { return mSignatureFileName; }
-  QString signatureInlineText() const { return mSignatureInlineText; }
-  bool    signatureFileIsAProgram() const { return mSignatureFileIsAProgram; }
-  bool    useSignatureFile() const { return mUseSignatureFile; }
-  bool    isDefault() const { return mIsDefault; }
-  QString transport() const { return mTransport; }
-  QString fcc() const { return mFcc; }
-  QString drafts() const { return mDrafts; }
-  
-  void setIdentityName( const QString & i ) { mIdentityName = i; }
-  void setFullName( const QString & f ) { mFullName = f; }
-  void setOrganization( const QString & o ) { mOrganization = o; }
-  void setPgpIdentity( const QString & p ) { mPgpIdentity = p; }
-  void setEmailAddress( const QString & e ) { mEmailAddress = e; }
-  void setReplyToAddress( const QString & r ) { mReplyToAddress = r; }
-  void setSignatureFileName( const QString & s ) { mSignatureFileName = s; }
-  void setSignatureInlineText( const QString & t ) { mSignatureInlineText = t; }
-  void setSignatureFileIsAProgram( bool b ) { mSignatureFileIsAProgram = b; }
-  void setUseSignatureFile( bool b ) { mUseSignatureFile = b; }
-  void setIsDefault( bool b ) { mIsDefault = b; }
-  void setTransport( const QString & t ) { mTransport = t; }
-  void setFcc( const QString & f ) { mFcc = f; }
-  void setDrafts( const QString & f ) { mDrafts = f; }
-
-private:
-  // only add members that have an operator= defined or where the
-  // compiler can generate one by itself!!
-  QString mIdentityName;
-  QString mFullName;
-  QString mOrganization;
-  QString mPgpIdentity;
-  QString mEmailAddress;
-  QString mReplyToAddress;
-  QString mSignatureFileName;
-  QString mSignatureInlineText;
-  QString mTransport;
-  QString mFcc;
-  QString mDrafts;
-  bool    mSignatureFileIsAProgram;
-  bool    mUseSignatureFile;
-  bool    mIsDefault;
+namespace KMail {
+  class IdentityDialog;
+  class IdentityListView;
 };
 
-
-class IdentityList : public QValueList<IdentityEntry>
-{
-public:
-  // Let the compiler generate c'tors, d'tor and operator=. They are
-  // sufficient.
-
-  /** generates a list of display names of identies for use in
-      widgets. */
-  QStringList names() const;
-  /** Gets and @ref IdentityEntry by name */
-  IdentityEntry & getByName( const QString & identity );
-
-  // remove when no longer commented out in qvaluelist.h:
-  void sort() { qHeapSort( *this ); }
-
-  /** Load system settings */
-  void importData();
-  /** Save state to system */
-  void exportData() const;
-};
-
-
 class NewIdentityDialog : public KDialogBase
 {
   Q_OBJECT
@@ -180,13 +56,13 @@ public:
 
   NewIdentityDialog( const QStringList & identities,
 		     QWidget *parent=0, const char *name=0, bool modal=true );
-  
+
   QString identityName() const { return mLineEdit->text(); }
-  int duplicateIdentity() const { return mComboBox->currentItem(); }
+  QString duplicateIdentity() const { return mComboBox->currentText(); }
   DuplicateMode duplicateMode() const;
-  
+
 protected slots:
-  virtual void slotEnableOK( const QString & ); 
+  virtual void slotEnableOK( const QString & );
 
 private:
   QLineEdit  *mLineEdit;
@@ -210,7 +86,7 @@ struct LanguageItem
 		const QString & indentPrefix=QString::null ) :
     mLanguage( language ), mReply( reply ), mReplyAll( replyAll ),
     mForward( forward ), mIndentPrefix( indentPrefix ) {}
-  
+
   QString mLanguage, mReply, mReplyAll, mForward, mIndentPrefix;
 };
 
@@ -254,6 +130,9 @@ public:
     : QWidget( parent, name ) {}
   ~ConfigurationPage() {};
 
+  /** Should return the help anchor for this page or tab */
+  virtual QString helpAnchor() const = 0;
+
   /** Should set the page up (ie. read the setting from the @ref
       KConfig object into the widgets) after creating it in the
       constructor. Called from @ref ConfigureDialog. */
@@ -292,10 +171,17 @@ class TabbedConfigurationPage : public ConfigurationPage {
 public:
   TabbedConfigurationPage( QWidget * parent=0, const char * name=0 );
 
+  void setup();
+  void dismiss();
+  void installProfile( KConfig * profile );
+  void apply();
+
 protected:
   void addTab( QWidget * tab, const QString & title );
 
 private:
+  ConfigurationPage * configTab( int index, const char * debugMsg ) const;
+
   QTabWidget *mTabWidget;
 
 };
@@ -315,56 +201,42 @@ public:
   static QString iconLabel();
   static QString title();
   static const char * iconName();
-  static QString helpAnchor();
+  QString helpAnchor() const;
 
   void setup();
   void apply();
+  void dismiss();
 
 public slots:
   void slotUpdateTransportCombo( const QStringList & );
 
 protected slots:
   void slotNewIdentity();
-  void slotRenameIdentity();
+  void slotModifyIdentity();
   void slotRemoveIdentity();
-  void slotIdentitySelectorChanged();
-  void slotChangeDefaultPGPKey();
-  void slotSignatureEdit();
-  void slotEnableSignatureEditButton( const QString &filename );
+  /** Connected to @p mRenameButton's clicked() signal. Just does a
+      KListView::rename on the selected item */
+  void slotRenameIdentity();
+  /** connected to @p mIdentityList's renamed() signal. Validates the
+      new name and sets it in the @ref IdentityManager */
+  void slotRenameIdentity( QListViewItem *, const QString &, int );
+  void slotContextMenu( KListView*, QListViewItem *, const QPoint & );
+  void slotSetAsDefault();
+  void slotIdentitySelectionChanged( QListViewItem * );
 
 protected: // methods
-  void saveActiveIdentity();
-  void setIdentityInformation( const QString & identityName );
-  QStringList identityStrings() const;
+  void refreshList();
 
 protected: // data members
-  QString        mActiveIdentity;
-  IdentityList   mIdentities;
+  KMail::IdentityDialog   * mIdentityDialog;
+  int            mOldNumberOfIdentities;
 
-  // Widgets:
-  // outside tabwidget:
-  QComboBox        *mIdentityCombo;
-  QPushButton      *mRenameButton;
-  QPushButton      *mRemoveButton;
+  KMail::IdentityListView * mIdentityList;
+  QPushButton             * mModifyButton;
+  QPushButton             * mRenameButton;
+  QPushButton             * mRemoveButton;
+  QPushButton             * mSetAsDefaultButton;
 
-  // "general" tab:
-  QLineEdit        *mNameEdit;
-  QLineEdit        *mOrganizationEdit;
-  QLineEdit        *mEmailEdit;
-  // "advanced" tab:
-  QLineEdit        *mReplyToEdit;
-  QLabel           *mPgpIdentityLabel;
-  KMFolderComboBox *mFccCombo;
-  KMFolderComboBox *mDraftsCombo;
-  QCheckBox        *mTransportCheck;
-  QComboBox        *mTransportCombo; // should be a KMTransportCombo...
-  // "signature" tab:
-  QCheckBox        *mSignatureEnabled;
-  QComboBox        *mSignatureSourceCombo;
-  KURLRequester    *mSignatureFileRequester;
-  QPushButton      *mSignatureEditButton;
-  KURLRequester    *mSignatureCommandRequester;
-  QMultiLineEdit   *mSignatureTextEdit;
 };
 
 
@@ -379,20 +251,20 @@ class NetworkPageSendingTab : public ConfigurationPage {
   Q_OBJECT
 public:
   NetworkPageSendingTab( QWidget * parent=0, const char * name=0 );
-    
+
   // no icon:
   static QString iconLabel() { return QString::null; }
   static const char * iconName() { return 0; }
-  
+
   static QString title();
-  static QString helpAnchor();
-  
+  QString helpAnchor() const;
+
   void setup();
   void apply();
-  
+
 signals:
   void transportListChanged( const QStringList & );
-  
+
 protected slots:
   void slotTransportSelected();
   void slotAddTransport();
@@ -400,7 +272,7 @@ protected slots:
   void slotRemoveSelectedTransport();
   void slotTransportUp();
   void slotTransportDown();
-  
+
 protected:
   ListView    *mTransportList;
   QPushButton *mModifyTransportButton;
@@ -411,9 +283,10 @@ protected:
   QCheckBox   *mSendOutboxCheck;
   QComboBox   *mSendMethodCombo;
   QComboBox   *mMessagePropertyCombo;
-  
+  QLineEdit   *mDefaultDomainEdit;
+
   QPtrList< KMTransportInfo > mTransportInfoList;
-  
+
 };
 
 
@@ -421,18 +294,18 @@ class NetworkPageReceivingTab : public ConfigurationPage {
   Q_OBJECT
 public:
   NetworkPageReceivingTab( QWidget * parent=0, const char * name=0 );
-  
+
   // no icon:
   static QString iconLabel() { return QString::null; }
   static const char * iconName() { return 0; }
-  
+
   static QString title();
-  static QString helpAnchor();
-  
+  QString helpAnchor() const;
+
   void setup();
   void apply();
   void dismiss(); // needed for account list cleanup
-  
+
 signals:
   void accountListChanged( const QStringList & );
 
@@ -441,6 +314,7 @@ protected slots:
   void slotAddAccount();
   void slotModifySelectedAccount();
   void slotRemoveSelectedAccount();
+  void slotEditNotifications();
 
 protected:
   QStringList occupiedNames();
@@ -450,10 +324,8 @@ protected:
   QPushButton   *mModifyAccountButton;
   QPushButton   *mRemoveAccountButton;
   QCheckBox     *mBeepNewMailCheck;
-  QCheckBox     *mShowMessageBoxCheck;
-  QCheckBox     *mMailCommandCheck;
-  KURLRequester *mMailCommandRequester;
-  
+  QPushButton   *mOtherNewMailActionsButton;
+
   QValueList< QGuardedPtr<KMAccount> > mAccountsToDelete;
   QValueList< QGuardedPtr<KMAccount> > mNewAccounts;
   struct ModifiedAccountsType {
@@ -472,12 +344,7 @@ public:
   static QString iconLabel();
   static QString title();
   static const char * iconName();
-  static QString helpAnchor();
-
-  void setup();
-  void apply();
-  void dismiss(); // needed for account list cleanup.
-  void installProfile( KConfig * profile );
+  QString helpAnchor() const;
 
   // hrmpf. moc doesn't like nested classes with slots/signals...:
   typedef NetworkPageSendingTab SendingTab;
@@ -507,14 +374,14 @@ public:
   // no icons:
   static QString iconLabel() { return QString::null; }
   static const char * iconName() { return 0; }
-  
+
   static QString title();
-  static QString helpAnchor();
-  
+  QString helpAnchor() const;
+
   void setup();
   void apply();
   void installProfile( KConfig * profile );
-  
+
 protected slots:
   void slotFontSelectorChanged( int );
 
@@ -525,7 +392,7 @@ protected:
   QCheckBox    *mCustomFontCheck;
   QComboBox    *mFontLocationCombo;
   KFontChooser *mFontChooser;
-  
+
   int          mActiveFontIndex;
   QFont        mFont[10];
 };
@@ -538,14 +405,14 @@ public:
   // no icons:
   static QString iconLabel() { return QString::null; }
   static const char * iconName() { return 0; }
-  
+
   static QString title();
-  static QString helpAnchor();
-  
+  QString helpAnchor() const;
+
   void setup();
   void apply();
   void installProfile( KConfig * profile );
-  
+
 protected:
   QCheckBox    *mCustomColorCheck;
   ColorListBox *mColorList;
@@ -556,53 +423,80 @@ class AppearancePageLayoutTab : public ConfigurationPage {
   Q_OBJECT
 public:
   AppearancePageLayoutTab( QWidget * parent=0, const char * name=0 );
-  
+
   // no icons:
   static QString iconLabel() { return QString::null; }
   static const char * iconName() { return 0; }
-  
+
   static QString title();
-  static QString helpAnchor();
+  QString helpAnchor() const;
 
   void setup();
   void apply();
   void installProfile( KConfig * profile );
-  
-protected:
-  QCheckBox    *mLongFolderCheck;
+
+
+protected slots:
+  /** Used to switch the Layout preview items if Mime Tree Viewer is enabled
+      or disabled */
+  void showMIMETreeClicked( int id );
+
+protected: // methods
+  QPixmap pixmapFor( int num, int type );
+
+protected: // data
   QCheckBox    *mShowColorbarCheck;
+  QButtonGroup *mShowMIMETreeMode;
+  QButtonGroup *mWindowLayoutBG;
+  int mShowMIMETreeModeLastValue;
+};
+
+class AppearancePageHeadersTab : public ConfigurationPage {
+  Q_OBJECT
+public:
+  AppearancePageHeadersTab( QWidget * parent=0, const char * name=0 );
+
+  // no icons:
+  static QString iconLabel() { return QString::null; }
+  static const char * iconName() { return 0; }
+
+  static QString title();
+  QString helpAnchor() const;
+
+  void setup();
+  void apply();
+  void installProfile( KConfig * profile );
+
+protected: // methods
+  void setDateDisplay( int id, const QString & format );
+
+protected: // data
   QCheckBox    *mMessageSizeCheck;
   QCheckBox    *mNestedMessagesCheck;
+  QCheckBox    *mCryptoIconsCheck;
   QButtonGroup *mNestingPolicy;
   QButtonGroup *mDateDisplay;
-  
-  enum { numDateDisplayConfig = 3 };
-  static const struct dateDisplayConfigType {
-    const char *  configName;
-    const char *  displayName;
-    KMDateDisplay dateDisplay;
-  } dateDisplayConfig[ numDateDisplayConfig ];
-
+  QLineEdit    *mCustomDateFormatEdit;
 };
 
 class AppearancePageProfileTab : public ConfigurationPage {
   Q_OBJECT
 public:
   AppearancePageProfileTab( QWidget * parent=0, const char * name=0 );
-  
+
   // no icons:
   static QString iconLabel() { return QString::null; }
   static const char * iconName() { return 0; }
-  
+
   static QString title();
-  static QString helpAnchor();
-  
+  QString helpAnchor() const;
+
   void setup();
   void apply();
-  
+
 signals:
   void profileSelected( KConfig * profile );
-  
+
 protected:
   KListView   *mListView;
   QStringList mProfileList;
@@ -612,20 +506,19 @@ class AppearancePage : public TabbedConfigurationPage {
   Q_OBJECT
 public:
   AppearancePage( QWidget * parent=0, const char * name=0 );
-  
+
   static QString iconLabel();
   static QString title();
   static const char * iconName();
-  static QString helpAnchor();
+  QString helpAnchor() const;
 
-  void setup();
-  void apply();
-  void installProfile( KConfig * profile );
+  void apply(); // is special
 
   // hrmpf. moc doesn't like nested classes with slots/signals...:
   typedef AppearancePageFontsTab FontsTab;
   typedef AppearancePageColorsTab ColorsTab;
   typedef AppearancePageLayoutTab LayoutTab;
+  typedef AppearancePageHeadersTab HeadersTab;
   typedef AppearancePageProfileTab ProfileTab;
 
 signals:
@@ -635,8 +528,9 @@ signals:
 
 protected:
   FontsTab   *mFontsTab;
-  ColorsTab  *mColorsTab; 
+  ColorsTab  *mColorsTab;
   LayoutTab  *mLayoutTab;
+  HeadersTab *mHeadersTab;
   ProfileTab *mProfileTab;
 };
 
@@ -656,7 +550,7 @@ public:
   static const char * iconName() { return 0; }
 
   static QString title();
-  static QString helpAnchor();
+  QString helpAnchor() const;
 
   void setup();
   void apply();
@@ -665,8 +559,6 @@ public:
 protected:
   QCheckBox     *mAutoAppSignFileCheck;
   QCheckBox     *mSmartQuoteCheck;
-  QCheckBox     *mPgpAutoSignatureCheck;
-  QCheckBox     *mPgpAutoEncryptCheck;
   QCheckBox     *mWordWrapCheck;
   KIntSpinBox   *mWrapColumnSpin;
   QCheckBox     *mExternalEditorCheck;
@@ -683,11 +575,11 @@ public:
   static const char * iconName() { return 0; }
 
   static QString title();
-  static QString helpAnchor();
-  
+  QString helpAnchor() const;
+
   void setup();
   void apply();
-  
+
 protected slots:
   void slotNewLanguage();
   void slotRemoveLanguage();
@@ -705,7 +597,7 @@ protected:
   QLineEdit        *mPhraseReplyAllEdit;
   QLineEdit        *mPhraseForwardEdit;
   QLineEdit        *mPhraseIndentPrefixEdit;
-  
+
   int              mActiveLanguageItem;
   LanguageItemList mLanguageList;
 };
@@ -714,17 +606,17 @@ class ComposerPageSubjectTab : public ConfigurationPage {
   Q_OBJECT
 public:
   ComposerPageSubjectTab( QWidget * parent=0, const char * name=0 );
-  
+
   // no icons:
   static QString iconLabel() { return QString::null; }
   static const char * iconName() { return 0; }
-  
+
   static QString title();
-  static QString helpAnchor();
-  
+  QString helpAnchor() const;
+
   void setup();
   void apply();
-  
+
 
 protected:
   SimpleStringListEditor *mReplyListEditor;
@@ -743,7 +635,7 @@ public:
   static const char * iconName() { return 0; }
 
   static QString title();
-  static QString helpAnchor();
+  QString helpAnchor() const;
 
   void setup();
   void apply();
@@ -755,7 +647,7 @@ protected:
   SimpleStringListEditor *mCharsetListEditor;
   QCheckBox              *mKeepReplyCharsetCheck;
 };
-  
+
 class ComposerPageHeadersTab : public ConfigurationPage {
   Q_OBJECT
 public:
@@ -764,20 +656,20 @@ public:
   // no icons:
   static QString iconLabel() { return QString::null; }
   static const char * iconName() { return 0; }
-  
+
   static QString title();
-  static QString helpAnchor();
-  
+  QString helpAnchor() const;
+
   void setup();
   void apply();
-  
+
 protected slots:
   void slotMimeHeaderSelectionChanged();
   void slotMimeHeaderNameChanged( const QString & );
   void slotMimeHeaderValueChanged( const QString & );
   void slotNewMimeHeader();
   void slotRemoveMimeHeader();
-  
+
 protected:
   QCheckBox   *mCreateOwnMessageIdCheck;
   QLineEdit   *mMessageIdSuffixEdit;
@@ -789,20 +681,16 @@ protected:
   QLabel      *mTagNameLabel;
   QLabel      *mTagValueLabel;
 };
-  
+
 class ComposerPage : public TabbedConfigurationPage {
   Q_OBJECT
 public:
   ComposerPage( QWidget * parent=0, const char * name=0 );
-  
+
   static QString iconLabel();
   static QString title();
   static const char * iconName();
-  static QString helpAnchor();
-
-  void setup();
-  void apply();
-  void installProfile( KConfig * profile );
+  QString helpAnchor() const;
 
   // hrmpf. moc doesn't like nested classes with slots/signals...:
   typedef ComposerPageGeneralTab GeneralTab;
@@ -835,7 +723,7 @@ public:
   static const char * iconName() { return 0; }
 
   static QString title();
-  static QString helpAnchor();
+  QString helpAnchor() const;
 
   void setup();
   void apply();
@@ -847,122 +735,130 @@ protected:
   QCheckBox *mSendReceiptCheck;
 };
 
+
+class SecurityPageOpenPgpTab : public ConfigurationPage {
+  Q_OBJECT
+public:
+  SecurityPageOpenPgpTab( QWidget * parent=0, const char * name=0 );
+
+  // no icons:
+  static QString iconLabel() { return QString::null; }
+  static const char * iconName() { return 0; }
+
+  static QString title();
+  QString helpAnchor() const;
+
+  void setup();
+  void apply();
+  void installProfile( KConfig * profile );
+
+protected:
+  Kpgp::Config *mPgpConfig;
+  QCheckBox    *mPgpAutoSignatureCheck;
+  QCheckBox    *mPgpAutoEncryptCheck;
+};
+
+
+class SecurityPageCryptPlugTab : public ConfigurationPage
+{
+  Q_OBJECT
+public:
+  SecurityPageCryptPlugTab( QWidget * parent = 0, const char* name = 0 );
+
+  // no icons:
+  static QString iconLabel() { return QString::null; }
+  static const char * iconName() { return 0; }
+
+  static QString title();
+  QString helpAnchor() const;
+
+  void setup();
+  void apply();
+  //void savePluginsConfig( bool silent );
+
+public slots:
+  void slotNewPlugIn();
+  void slotDeletePlugIn();
+  void slotActivatePlugIn();
+  void slotConfigurePlugIn();
+  void slotPlugNameChanged( const QString& );
+  void slotPlugLocationChanged( const QString& );
+  void slotPlugUpdateURLChanged( const QString& );
+
+protected slots:
+  void slotPlugSelectionChanged();
+
+private:
+  KListView     *mPlugList;
+
+  QPushButton   *mNewButton;
+  QPushButton   *mRemoveButton;
+  QPushButton   *mActivateButton;
+  QPushButton   *mConfigureButton;
+
+  QLineEdit     *mNameEdit;
+  KURLRequester *mLocationRequester;
+  QLineEdit     *mUpdateURLEdit;
+};
+
 class SecurityPage : public TabbedConfigurationPage {
   Q_OBJECT
 public:
-  SecurityPage( QWidget * parent=0, const char * name=0 );
+  SecurityPage(	QWidget * parent=0, const char * name=0 );
 
   static QString iconLabel();
   static const char * iconName();
   static QString title();
-  static QString helpAnchor();
+  QString helpAnchor() const;
 
+  // OpenPGP tab is special:
   void setup();
   void apply();
+  void dismiss() {}
   void installProfile( KConfig * profile );
 
   typedef SecurityPageGeneralTab GeneralTab;
+  typedef SecurityPageOpenPgpTab OpenPgpTab;
+  typedef SecurityPageCryptPlugTab CryptPlugTab;
 
 protected:
   GeneralTab   *mGeneralTab;
-  Kpgp::Config *mPgpTab;
+  OpenPgpTab   *mOpenPgpTab;
+  CryptPlugTab *mCryptPlugTab;
 };
 
 
 //
 //
-// MiscPage
+// FolderPage
 //
 //
 
-class MiscPageFoldersTab : public ConfigurationPage {
+class FolderPage : public ConfigurationPage {
   Q_OBJECT
 public:
-  MiscPageFoldersTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
-  static QString helpAnchor();
-
-  void setup();
-  void apply();
-
-protected:
-  QCheckBox  *mEmptyFolderConfirmCheck;
-  QCheckBox  *mWarnBeforeExpire;
-  QCheckBox  *mLoopOnGotoUnread;
-  QComboBox  *mMailboxPrefCombo;
-  QCheckBox  *mCompactOnExitCheck;
-  QCheckBox  *mEmptyTrashCheck;
-  QCheckBox  *mExpireAtExit;
-};
-
-class MiscPageAddressbookTab : public ConfigurationPage {
-  Q_OBJECT
-public:
-  MiscPageAddressbookTab( QWidget * parent=0, const char * name=0 );
-
-  // no icons:
-  static QString iconLabel() { return QString::null; }
-  static const char * iconName() { return 0; }
-
-  static QString title();
-  static QString helpAnchor();
-
-  void setup();
-  void apply();
-
-protected:
-  QComboBox *mAddressbookCombo;
-};
-
-
-class MiscPage : public TabbedConfigurationPage {
-  Q_OBJECT
-public:
-  MiscPage( QWidget * parent=0, const char * name=0 );
+  FolderPage( QWidget * parent=0, const char * name=0 );
 
   static QString iconLabel();
   static const char * iconName();
   static QString title();
-  static QString helpAnchor();
+  QString helpAnchor() const;
 
   void setup();
   void apply();
-  void installProfile( KConfig * profile );
-
-  typedef MiscPageFoldersTab FoldersTab;
-  typedef MiscPageAddressbookTab AddressbookTab;
-
-protected:
-  FoldersTab     *mFoldersTab;
-  AddressbookTab *mAddressbookTab;
-};
-
-//
-//
-// PluginPage
-//
-//
-
-class PluginPage : public ConfigurationPage {
-  Q_OBJECT
-public:
-  PluginPage( QWidget * parent=0, const char * name=0 );
-
-  static QString iconLabel();
-  static const char * iconName();
-  static QString title();
-  static QString helpAnchor();
-
-  void setup();
-  void apply();
-  void installProfile( KConfig * profile );
 
 protected:
+  QCheckBox    *mEmptyFolderConfirmCheck;
+  QCheckBox    *mWarnBeforeExpire;
+  QCheckBox    *mLoopOnGotoUnread;
+  QCheckBox    *mJumpToUnread;
+  QComboBox    *mMailboxPrefCombo;
+  QCheckBox    *mCompactOnExitCheck;
+  QCheckBox    *mEmptyTrashCheck;
+  QCheckBox    *mExpireAtExit;
+  QCheckBox    *mDelayedMarkAsRead;
+  KIntSpinBox  *mDelayedMarkTime;
+  QCheckBox    *mShowPopupAfterDnD;
 };
 
 //
@@ -971,31 +867,19 @@ protected:
 //
 //
 
-class ApplicationLaunch {
-public:
-  ApplicationLaunch( const QString &cmd ) : mCmdLine( cmd ) {}
-  void run();
-  
-private:
-  void doIt();
-  
-private:
-  QString mCmdLine;
-};
-
 class ListView : public KListView {
   Q_OBJECT
 public:
   ListView( QWidget *parent=0, const char *name=0, int visibleItem=10 );
   void resizeColums();
-  
+
   void setVisibleItem( int visibleItem, bool updateSize=true );
   virtual QSize sizeHint() const;
-  
+
 protected:
   virtual void resizeEvent( QResizeEvent *e );
   virtual void showEvent( QShowEvent *e );
-  
+
 private:
   int mVisibleItem;
 };
