@@ -1,5 +1,6 @@
 // KMail Account
 
+#include <config.h>
 #include <stdlib.h>
 #include <unistd.h>
 
@@ -16,6 +17,7 @@
 #include "kmfiltermgr.h"
 #include "kmsender.h"
 #include "kmbroadcaststatus.h"
+#include "kmmessage.h"
 
 //----------------------
 #include "kmaccount.moc"
@@ -27,6 +29,9 @@ KMPrecommand::KMPrecommand(const QString &precommand, QObject *parent)
   KMBroadcastStatus::instance()->setStatusMsg(
       i18n("Executing precommand %1").arg(precommand ));
 
+#if KDE_VERSION >= 306
+  mPrecommandProcess.setUseShell(true);
+#endif
   mPrecommandProcess << precommand;
 
   connect(&mPrecommandProcess, SIGNAL(processExited(KProcess *)),
@@ -72,7 +77,7 @@ KMAccount::KMAccount(KMAcctMgr* aOwner, const QString& aName)
     mCheckingMail(false),
     mPrecommandSuccess(true)
 {
-  assert(aOwner != NULL);
+  assert(aOwner != 0);
 
   connect(&mReceiptTimer,SIGNAL(timeout()),SLOT(sendReceipts()));
 }
@@ -104,8 +109,8 @@ void KMAccount::setFolder(KMFolder* aFolder, bool addAccount)
 {
   if(!aFolder)
     {
-    kdDebug(5006) << "KMAccount::setFolder() : aFolder == NULL" << endl;
-    mFolder = NULL;
+    kdDebug(5006) << "KMAccount::setFolder() : aFolder == 0" << endl;
+    mFolder = 0;
     return;
     }
   mFolder = (KMAcctFolder*)aFolder;
@@ -118,7 +123,7 @@ void KMAccount::readConfig(KConfig& config)
 {
   QString folderName;
 
-  mFolder = NULL;
+  mFolder = 0;
   folderName = config.readEntry("Folder", "");
   setCheckInterval(config.readNumEntry("check-interval", 0));
   setCheckExclude(config.readBoolEntry("check-exclude", false));
@@ -167,7 +172,7 @@ bool KMAccount::processNewMsg(KMMessage* aMsg)
 {
   int rc, processResult;
 
-  assert(aMsg != NULL);
+  assert(aMsg != 0);
 
   // checks whether we should send delivery receipts
   // and sends them.
@@ -179,7 +184,14 @@ bool KMAccount::processNewMsg(KMMessage* aMsg)
     aMsg->setStatus(KMMsgStatusUnread);  // -sanders
   //    aMsg->setStatus(KMMsgStatusRead);
   else aMsg->setStatus(KMMsgStatusNew);
-
+/*
+QFile fileD0( "testdat_xx-kmaccount-0" );
+if( fileD0.open( IO_WriteOnly ) ) {
+    QDataStream ds( &fileD0 );
+    ds.writeRawBytes( aMsg->asString(), aMsg->asString().length() );
+    fileD0.close();  // If data is 0 we just create a zero length file.
+}
+*/
   // 0==message moved; 1==processing ok, no move; 2==critical error, abort!
   processResult = kernel->filterMgr()->process(aMsg,KMFilterMgr::Inbound);
   if (processResult == 2) {
@@ -192,6 +204,14 @@ bool KMAccount::processNewMsg(KMMessage* aMsg)
   {
     kernel->filterMgr()->tempOpenFolder(mFolder);
     rc = mFolder->addMsg(aMsg);
+/*
+QFile fileD0( "testdat_xx-kmaccount-1" );
+if( fileD0.open( IO_WriteOnly ) ) {
+    QDataStream ds( &fileD0 );
+    ds.writeRawBytes( aMsg->asString(), aMsg->asString().length() );
+    fileD0.close();  // If data is 0 we just create a zero length file.
+}
+*/
     if (rc) {
       perror("failed to add message");
       KMessageBox::information(0, i18n("Failed to add message:\n") +
@@ -248,10 +268,8 @@ void KMAccount::installTimer()
 //-----------------------------------------------------------------------------
 void KMAccount::deinstallTimer()
 {
-  if(mTimer) {
-    delete mTimer;
-    mTimer = NULL;
-  }
+  delete mTimer;
+  mTimer = 0;
 }
 
 //-----------------------------------------------------------------------------
