@@ -2,8 +2,8 @@
 // Authors: Don Sanders, (based on kmacctpop by)
 //          Stefan Taferner and Markus Wuebben
 
-#include "kmacctexppop.moc"
-
+#include <config.h>
+#include "kmacctexppop.h"
 #include <netdb.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -26,8 +26,8 @@ KMAcctExpPop::KMAcctExpPop(KMAcctMgr* aOwner, const QString& aAccountName):
     headerIt(headersOnServer)
 {
   init();
-  job = 0L;
-  slave = 0L;
+  job = 0;
+  slave = 0;
   stage = Idle;
   indexOfCurrentMsg = -1;
   curMsgStrm = 0;
@@ -404,8 +404,8 @@ void KMAcctExpPop::slotAbortRequested()
           this, SLOT(slotAbortRequested()));
   stage = Quit;
   if (job) job->kill();
-  job = 0L;
-  slave = 0L;
+  job = 0;
+  slave = 0;
   slotCancel();
 }
 
@@ -463,14 +463,14 @@ void KMAcctExpPop::startJob() {
     mSlaveConfig.insert("sasl", mAuth);
   }
   else mSlaveConfig.insert("auth", mAuth);
-  slave = KIO::Scheduler::getConnectedSlave( url.url(), mSlaveConfig );
+  slave = KIO::Scheduler::getConnectedSlave( url, mSlaveConfig );
   if (!slave)
   {
     slotSlaveError(0, KIO::ERR_CANNOT_LAUNCH_PROCESS, url.protocol());
     return;
   }
   url.setPath(QString("/index"));
-  job = KIO::get( url.url(), false, false );
+  job = KIO::get( url, false, false );
   connectJob();
 }
 
@@ -514,7 +514,7 @@ void KMAcctExpPop::slotJobFinished() {
     kdDebug(5006) << "stage == List" << endl;
     KURL url = getUrl();
     url.setPath(QString("/uidl"));
-    job = KIO::get( url.url(), false, false );
+    job = KIO::get( url, false, false );
     connectJob();
     stage = Uidl;
   }
@@ -577,7 +577,7 @@ void KMAcctExpPop::slotJobFinished() {
       }
       headerIt.toFirst();
       url.setPath(QString("/headers/") + headerIds);
-      job = KIO::get( url.url(), false, false );
+      job = KIO::get( url, false, false );
       connectJob();
       slotGetNextHdr();
       stage = Head;
@@ -634,7 +634,7 @@ void KMAcctExpPop::slotJobFinished() {
           dlgPopup = true;
           break;
         case Later:
-          if (kernel->popFilterMgr()->showLaterMsgs()) 
+          if (kernel->popFilterMgr()->showLaterMsgs())
             dlgPopup = true;
         default:
           headersOnServer.current()->setAction(action);
@@ -650,7 +650,7 @@ void KMAcctExpPop::slotJobFinished() {
       KMPopFilterCnfrmDlg dlg(&headersOnServer, this->name(), kernel->popFilterMgr()->showLaterMsgs());
       dlg.exec();
     }
-	
+
     for (headersOnServer.first(); headersOnServer.current(); headersOnServer.next()) {
       if (headersOnServer.current()->action() == Delete ||
           headersOnServer.current()->action() == Later) {
@@ -706,7 +706,7 @@ void KMAcctExpPop::slotJobFinished() {
     KURL url = getUrl();
     if (mLeaveOnServer || idsOfMsgsToDelete.isEmpty()) {
       url.setPath(QString("/commit"));
-      job = KIO::get(url.url(), false, false );
+      job = KIO::get(url, false, false );
     }
     else {
       stage = Dele;
@@ -720,15 +720,15 @@ void KMAcctExpPop::slotJobFinished() {
     kdDebug(5006) << "stage == Dele" << endl;
     KURL url = getUrl();
     url.setPath(QString("/commit"));
-    job = KIO::get( url.url(), false, false );
+    job = KIO::get( url, false, false );
     stage = Quit;
     connectJob();
   }
   else if (stage == Quit) {
     kdDebug(5006) << "stage == Quit" << endl;
-    job = 0L;
+    job = 0;
     if (slave) KIO::Scheduler::disconnectSlave(slave);
-    slave = NULL;
+    slave = 0;
     stage = Idle;
     KMBroadcastStatus::instance()->setStatusProgressPercent( "P" + mName, 100 );
     int numMessages = (KMBroadcastStatus::instance()->abortRequested()) ?
@@ -736,19 +736,19 @@ void KMAcctExpPop::slotJobFinished() {
     QString statusMsg;
     if (numMessages > 0)
       if (numBytesToRead != numBytes && mLeaveOnServer)
-	statusMsg = i18n("Transmission completed, %n new message in %1 KB "
-			 "(%2 KB remain on the server).",
-			 "Transmission completed, %n new messages in %1 KB "
-			 "(%2 KB remain on the server).",
+	statusMsg = i18n("Transmission complete. %n new message in %1 KB "
+			 "(%2 KB remaining on the server).",
+			 "Transmission complete. %n new messages in %1 KB "
+			 "(%2 KB remaining on the server).",
 			 numMessages)
 	  .arg(numBytesRead/1024).arg(numBytes/1024);
       else
-	statusMsg = i18n("Transmission completed, %n message in %1 KB.",
-			 "Transmission completed, %n messages in %1 KB.",
+	statusMsg = i18n("Transmission complete. %n message in %1 KB.",
+			 "Transmission complete. %n messages in %1 KB.",
 			 numMessages)
 	  .arg(numBytesRead/1024);
     else
-      statusMsg = i18n("Transmission completed, no new messages." );
+      statusMsg = i18n("Transmission complete. No new messages." );
     KMBroadcastStatus::instance()->setStatusMsg( statusMsg );
     KMBroadcastStatus::instance()->setStatusProgressEnable( "P" + mName,
       false );
@@ -849,8 +849,9 @@ void KMAcctExpPop::slotData( KIO::Job* job, const QByteArray &data)
       }
       KMBroadcastStatus::instance()->setStatusMsg( msg );
       KMBroadcastStatus::instance()->setStatusProgressPercent("P" + mName,
-        (numBytesToRead == 0) ? 50  // We never know what the server tells us
-        : (numBytesRead * 100 / numBytesToRead) );
+        (numBytesToRead <= 100) ? 50  // We never know what the server tells us
+        // This way of dividing is reqired for > 21MB of mail
+        : (numBytesRead / (numBytesToRead / 100)) );
     }
     return;
   }
@@ -899,10 +900,10 @@ void KMAcctExpPop::slotData( KIO::Job* job, const QByteArray &data)
   else {
     stage = Idle;
     if (job) job->kill();
-    job = 0L;
-    slave = 0L;
+    job = 0;
+    slave = 0;
     KMessageBox::error(0, i18n( "Unable to complete LIST operation" ),
-                          i18n("Invalid response from server"));
+                          i18n("Invalid Response From Server"));
     return;
   }
 }
@@ -920,9 +921,9 @@ void KMAcctExpPop::slotResult( KIO::Job* )
       }
       if (stage == Head && job->error() == KIO::ERR_COULD_NOT_READ)
       {
-        KMessageBox::error(0, i18n("Sorry, your server doesn't support the "
+        KMessageBox::error(0, i18n("Your server does not support the "
           "TOP command. Therefore it is not possible to fetch the headers "
-	  "of large emails first, before downloading them.")); 
+	  "of large emails first, before downloading them."));
         slotCancel();
         return;
       }
@@ -942,7 +943,7 @@ void KMAcctExpPop::slotSlaveError(KIO::Slave *aSlave, int error,
   const QString &errorMsg)
 {
   if (aSlave != slave) return;
-  if (error == KIO::ERR_SLAVE_DIED) slave = NULL;
+  if (error == KIO::ERR_SLAVE_DIED) slave = 0;
   if (interactive) {
     KMessageBox::error(0, KIO::buildErrorString(error, errorMsg));
   }
@@ -968,3 +969,5 @@ void KMAcctExpPop::slotGetNextHdr(){
 
   curMsgStrm = new QDataStream( curMsgData, IO_WriteOnly );
 }
+
+#include "kmacctexppop.moc"
