@@ -19,8 +19,10 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <config.h>
 #include "kmacctimap.moc"
 
+#include "kmmainwin.h"
 #include "kmbroadcaststatus.h"
 #include "kmfoldertree.h"
 #include "kmfoldermgr.h"
@@ -47,9 +49,14 @@ KMAcctImap::KMAcctImap(KMAcctMgr* aOwner, const QString& aAccountName):
   KMAcctImapInherited(aOwner, aAccountName)
 {
   init();
-  mSlave = NULL;
+  mSlave = 0;
   mTotal = 0;
   mFolder = 0;
+  mCountUnread = 0;
+  mCountLastUnread = 0;
+  mCountRemainChecks = 0;  
+  errorDialogIsActive = false;
+  mOpenFolders.setAutoDelete(true);
   connect(KMBroadcastStatus::instance(), SIGNAL(signalAbortRequested()),
           this, SLOT(slotAbortRequested()));
   connect(&mIdleTimer, SIGNAL(timeout()), SLOT(slotIdleTimeout()));
@@ -100,7 +107,7 @@ void KMAcctImap::pseudoAssign(KMAccount* account)
   mIdleTimer.stop();
   killAllJobs();
   if (mSlave) KIO::Scheduler::disconnectSlave(mSlave);
-  mSlave = NULL;
+  mSlave = 0;
   if (mFolder)
   {
     mFolder->setContentState(KMFolderImap::imapNoInformation);
@@ -268,18 +275,29 @@ void KMAcctImap::initJobData(jobData &jd)
 {
   jd.total = 1;
   jd.done = 0;
-  jd.parent = NULL;
+  jd.parent = 0;
   jd.quiet = FALSE;
   jd.inboxOnly = FALSE;
 }
 
 
+//-----------------------------------------------------------------------------
+int KMAcctImap::tempOpenFolder(KMFolder *folder)
+{
+  int rc = folder->open();
+  if (rc) return rc;
+  mOpenFolders.append(new QGuardedPtr<KMFolder>(folder));
+  return 0;
+}
+
+
 //-----------------------------------------------------------------------------
 void KMAcctImap::setAutoExpunge(bool aAutoExpunge)
 {
   mAutoExpunge = aAutoExpunge;
 }
 
+
 //-----------------------------------------------------------------------------
 void KMAcctImap::setHiddenFolders(bool aHiddenFolders)
 {
@@ -322,8 +340,8 @@ KURL KMAcctImap::getUrl()
   url.setPort(mPort);
   return url;
 }
- 
- 
+
+
 //-----------------------------------------------------------------------------
 bool KMAcctImap::makeConnection()
 {
@@ -347,14 +365,14 @@ bool KMAcctImap::makeConnection()
   mSlave = KIO::Scheduler::getConnectedSlave(getUrl(), mSlaveConfig);
   if (!mSlave)
   {
-    KMessageBox::error(0, QString("Could not start process for %1.")
+    KMessageBox::error(0, i18n("Could not start process for %1.")
       .arg(getUrl().protocol()));
     return FALSE;
   }
   return TRUE;
 }
- 
- 
+
+
 //-----------------------------------------------------------------------------
 void KMAcctImap::slotSlaveError(KIO::Slave *aSlave, int errorCode,
   const QString &errorMsg)
@@ -362,8 +380,25 @@ void KMAcctImap::slotSlaveError(KIO::Slave *aSlave, int errorCode,
   if (aSlave != mSlave) return;
   if (errorCode == KIO::ERR_SLAVE_DIED) slaveDied();
   if (errorCode == KIO::ERR_COULD_NOT_LOGIN && !mStorePasswd) mAskAgain = TRUE;
-  KMessageBox::error(0, KIO::buildErrorString(errorCode, errorMsg));
+  if (errorCode == KIO::ERR_DOES_NOT_EXIST)
+  {
+    // folder is gone, so reload the folderlist
+    if (mFolder) mFolder->listDirectory();
+    return;
+  }
+  // check if we still display an error
   killAllJobs();
+  if ( !errorDialogIsActive )
+  {
+    errorDialogIsActive = true;
+    if ( KMessageBox::messageBox(kernel->mainWin(), KMessageBox::Error,
+          KIO::buildErrorString(errorCode, errorMsg),
+          i18n("Error")) == KMessageBox::Ok )
+    {
+      errorDialogIsActive = false;
+    }
+  } else
+    kdDebug(5006) << "suppressing error:" << errorMsg << endl;
 }
 
 
@@ -375,7 +410,13 @@ void KMAcctImap::displayProgress()
     mProgressEnabled = !mapJobData.isEmpty();
     KMBroadcastStatus::instance()->setStatusProgressEnable( "I" + mName,
       mProgressEnabled );
-    if (!mProgressEnabled) kernel->filterMgr()->cleanup();
+    if (!mProgressEnabled)
+    {
+      QPtrListIterator<QGuardedPtr<KMFolder> > it(mOpenFolders);
+      for ( it.toFirst() ; it.current() ; ++it )
+        if (*it) (*(*it))->close();
+      mOpenFolders.clear();
+    }
   }
   mIdle = FALSE;
   if (mapJobData.isEmpty())
@@ -407,12 +448,17 @@ void KMAcctImap::slotIdleTimeout()
   if (mIdle)
   {
     if (mSlave) KIO::Scheduler::disconnectSlave(mSlave);
-    mSlave = NULL;
+    mSlave = 0;
     mIdleTimer.stop();
   } else {
     if (mSlave)
     {
-      KIO::SimpleJob *job = KIO::special(getUrl(), QCString("NOOP"), FALSE);
+      QByteArray packedArgs;
+      QDataStream stream( packedArgs, IO_WriteOnly);
+
+      stream << (int) 'N';
+
+      KIO::SimpleJob *job = KIO::special(getUrl(), packedArgs, FALSE);
       KIO::Scheduler::assignJobToSlave(mSlave, job);
       connect(job, SIGNAL(result(KIO::Job *)),
         this, SLOT(slotSimpleResult(KIO::Job *)));
@@ -436,6 +482,7 @@ void KMAcctImap::killAllJobs()
   for (it = mapJobData.begin(); it != mapJobData.end(); it++)
     if ((*it).parent)
     {
+      // clear folder state
       KMFolderImap *fld = (*it).parent;
       fld->setContentState(KMFolderImap::imapNoInformation);
       fld->setSubfolderState(KMFolderImap::imapNoInformation);
@@ -445,12 +492,19 @@ void KMAcctImap::killAllJobs()
   if (mSlave && mapJobData.begin() != mapJobData.end())
   {
     mSlave->kill();
-    mSlave = NULL;
+    mSlave = 0;
   }
+  // remove the jobs
   mapJobData.clear();
   mJobList.setAutoDelete(true);
   mJobList.clear();
   mJobList.setAutoDelete(false);
+  // make sure that no new-mail-check is blocked
+  if (mCountRemainChecks > 0)
+  {
+    mCountRemainChecks = 0;
+    emit finishedCheck(false);
+  }
   displayProgress();
 }
 
@@ -461,7 +515,7 @@ void KMAcctImap::killJobsForItem(KMFolderTreeItem * fti)
   QMap<KIO::Job *, jobData>::Iterator it = mapJobData.begin();
   while (it != mapJobData.end())
   {
-    if (it.data().parent == fti->folder)
+    if (it.data().parent == fti->folder())
     {
       killAllJobs();
       break;
@@ -483,7 +537,8 @@ void KMAcctImap::slotSimpleResult(KIO::Job * job)
   }
   if (job->error())
   {
-    if (!quiet) job->showErrorDialog();
+    if (!quiet) slotSlaveError(mSlave, job->error(),
+        job->errorText() );
     if (job->error() == KIO::ERR_SLAVE_DIED) slaveDied();
   }
   displayProgress();
@@ -494,8 +549,10 @@ void KMAcctImap::slotSimpleResult(KIO::Job * job)
 void KMAcctImap::processNewMail(bool interactive)
 {
   emit newMailsProcessed(-1);
-  if (!mFolder || !mFolder->child())
+  if (!mFolder || !mFolder->child() ||
+      !makeConnection())
   {
+    mCountRemainChecks = 0;
     emit finishedCheck(false);
     return;
   }
@@ -504,6 +561,18 @@ void KMAcctImap::processNewMail(bool interactive)
   kernel->imapFolderMgr()->createFolderList(&strList, &folderList,
     mFolder->child(), QString::null, false);
   QValueList<QGuardedPtr<KMFolder> >::Iterator it;
+  // first get the current count of unread-messages
+  mCountRemainChecks = 0;
+  mCountLastUnread = 0;
+  for (it = folderList.begin(); it != folderList.end(); it++)
+  {
+    KMFolder *folder = *it;
+    if (folder && !folder->noContent())
+    {
+      mCountLastUnread += folder->countUnread();
+    }
+  }
+  // then check for new mails
   for (it = folderList.begin(); it != folderList.end(); it++)
   {
     KMFolder *folder = *it;
@@ -512,11 +581,52 @@ void KMAcctImap::processNewMail(bool interactive)
       KMFolderImap *imapFolder = static_cast<KMFolderImap*>(folder);
       if (imapFolder->getContentState() != KMFolderImap::imapInProgress)
       {
-        if (imapFolder->isSelected())
+        // connect the result-signals for new-mail-notification
+        mCountRemainChecks++;
+        if (imapFolder->isSelected()) {
+          connect(imapFolder, SIGNAL(folderComplete(KMFolderImap*, bool)),
+              this, SLOT(postProcessNewMail(KMFolderImap*, bool)));
           imapFolder->getFolder();
-        else imapFolder->processNewMail(interactive);
+        }
+        else {
+          connect(imapFolder, SIGNAL(numUnreadMsgsChanged(KMFolder*)),
+              this, SLOT(postProcessNewMail(KMFolder*)));
+          imapFolder->processNewMail(interactive);
+        }
       }
     }
   }
-  emit finishedCheck(false);
 }
+
+//-----------------------------------------------------------------------------
+void KMAcctImap::postProcessNewMail(KMFolderImap* folder, bool)
+{
+  disconnect(folder, SIGNAL(folderComplete(KMFolderImap*, bool)),
+      this, SLOT(postProcessNewMail(KMFolderImap*, bool)));
+  postProcessNewMail(static_cast<KMFolder*>(folder));
+}
+
+//-----------------------------------------------------------------------------
+void KMAcctImap::postProcessNewMail(KMFolder* folder)
+{
+  disconnect(folder, SIGNAL(numUnreadMsgsChanged(KMFolder*)),
+      this, SLOT(postProcessNewMail(KMFolder*)));
+
+  mCountRemainChecks--;
+
+  // count the unread messages
+  mCountUnread += folder->countUnread();
+  if (mCountRemainChecks == 0)
+  {
+    // all checks are done
+    if (mCountUnread > 0 && mCountUnread > mCountLastUnread)
+    {
+      emit finishedCheck(true);
+      mCountLastUnread = mCountUnread;
+    } else {
+      emit finishedCheck(false);
+    }
+    mCountUnread = 0;
+  }
+}
+
