@@ -180,7 +180,7 @@ public:
    * Get the Slave used for the account
    */
   KIO::Slave * slave() { return mSlave; }
-  void slaveDied() { mSlave = NULL; killAllJobs(); }
+  void slaveDied() { mSlave = 0; killAllJobs(); }
 
   /**
    * Set the top level pseudo folder
@@ -192,9 +192,19 @@ public:
    */
   static void initJobData(jobData &jd);
 
+  /**
+   * Open a folder and close it again when the network transfer is finished
+   */
+  int tempOpenFolder(KMFolder *folder);
+
 public slots:
   void processNewMail() { processNewMail(TRUE); }
 
+  /**
+   * Display an error message
+   */
+  void slotSlaveError(KIO::Slave *aSlave, int, const QString &errorMsg);
+
 signals:
   /**
    * Emitted, when the account is deleted
@@ -226,6 +236,9 @@ protected:
   KIO::MetaData mSlaveConfig;
   QPtrList<KMImapJob> mJobList;
   KMFolderImap *mFolder;
+  int mCountUnread, mCountLastUnread;
+  int mCountRemainChecks;
+  QPtrList<QGuardedPtr<KMFolder> > mOpenFolders;
 
 protected slots:
   /**
@@ -243,10 +256,14 @@ protected slots:
    */
   void slotSimpleResult(KIO::Job * job);
 
-  /**
-   * Display an error message, that connecting failed
-   */
-  void slotSlaveError(KIO::Slave *aSlave, int, const QString &errorMsg);
+  /** new-mail-notification for the current folder (is called via folderComplete) */
+  void postProcessNewMail(KMFolderImap*, bool);
+
+  /** new-mail-notification for not-selected folders (is called via numUnreadMsgsChanged) */
+  void postProcessNewMail(KMFolder*);
+
+private:
+  bool errorDialogIsActive;
 };
 
 #endif /*KMAcctImap_h*/
