@@ -7,22 +7,28 @@
 #include "kmtopwidget.h"
 
 #include <qlabel.h>
+#include <qlistview.h>
 
 #include <qcheckbox.h>
 #include <qpushbutton.h>
 #include <qclipboard.h>
 #include <qpalette.h>
 #include <qfont.h>
+#include <qptrlist.h>
 
 #include <keditcl.h>
 #include <klineedit.h>
 #include <kio/job.h>
 #include <kglobalsettings.h>
 
+#include <kpgp.h>
+
 #include "kmmsgpart.h"
 #include "kmmsgbase.h"
 #include "mailcomposerIface.h"
 
+#include "cryptplugwrapper.h"
+
 class _StringPair {
  public:
    QString name;
@@ -36,9 +42,9 @@ class QFrame;
 class QGridLayout;
 class QLineEdit;
 class QListView;
-class QListViewItem;
 class QPopupMenu;
 class QPushButton;
+class QCString;
 class KCompletion;
 class KEdit;
 class KMComposeWin;
@@ -53,17 +59,21 @@ class KTempFile;
 class KToolBar;
 class KToggleAction;
 class KURL;
+class IdentityCombo;
+class CryptPlugWrapperList;
+class SpellingFilter;
 
 typedef QPtrList<KMMessagePart> KMMsgPartList;
 
+
 //-----------------------------------------------------------------------------
 #define KMEditInherited KEdit
 class KMEdit: public KEdit
 {
   Q_OBJECT
 public:
-  KMEdit(QWidget *parent=0L,KMComposeWin* composer=0L,
-	 const char *name=0L);
+  KMEdit(QWidget *parent=0,KMComposeWin* composer=0,
+	 const char *name=0);
   virtual ~KMEdit();
 
   /**
@@ -86,7 +96,7 @@ signals:
   void spellcheck_done(int result);
 public slots:
   void slotSpellcheck2(KSpell*);
-  void slotSpellResult(const QString &newtext);
+  void slotSpellResult(const QString&);
   void slotSpellDone();
   void slotExternalEditorDone(KProcess*);
 
@@ -99,10 +109,12 @@ protected:
   KMComposeWin* mComposer;
 private:
   KSpell *mKSpell;
+  SpellingFilter* mSpellingFilter;
   KTempFile *mTempFile;
   KProcess  *mExtEditorProcess;
   bool      extEditor;
   QString   mExtEditor;
+  bool      mWasModifiedBeforeSpellCheck;
 };
 
 
@@ -113,8 +125,8 @@ class KMLineEdit : public KLineEdit
   Q_OBJECT
 
 public:
-  KMLineEdit(KMComposeWin* composer, bool useCompletion, QWidget *parent = 0L,
-             const char *name = 0L);
+  KMLineEdit(KMComposeWin* composer, bool useCompletion, QWidget *parent = 0,
+             const char *name = 0);
   virtual ~KMLineEdit();
 
   virtual void setFont( const QFont& );
@@ -141,6 +153,17 @@ private slots:
 
 private:
   void loadAddresses();
+  /**
+   * Smart insertion of email addresses. If @p pos is -1 then
+   * @p str is inserted at the end of the current contents of this
+   * lineedit. Else @p str is inserted at @p pos.
+   * Features:
+   * - Automatically adds ',' if necessary to separate email addresses
+   * - Correctly decodes mailto URLs
+   * - Recognizes email addresses which are protected against address
+   *   harvesters, i.e. "name at kde dot org" and "name(at)kde.org"
+   */
+  void smartInsert( const QString &str, int pos = -1 );
 
   QString m_previousAddresses;
   bool m_useCompletion;
@@ -152,6 +175,32 @@ private:
 };
 
 
+//-----------------------------------------------------------------------------
+class KMAtmListViewItem : public QObject, public QListViewItem
+{
+  Q_OBJECT
+  friend class KMComposeWin;
+
+public:
+  KMAtmListViewItem(QListView * parent);
+  virtual ~KMAtmListViewItem();
+  virtual void paintCell( QPainter * p, const QColorGroup & cg,
+                          int column, int width, int align );
+
+protected:
+  void enableCryptoCBs(bool on);
+  void setEncrypt(bool on);
+  bool isEncrypt();
+  void setSign(bool on);
+  bool isSign();
+
+private:
+  QListView* mListview;
+  QCheckBox* mCBEncrypt;
+  QCheckBox* mCBSign;
+};
+
+
 class KMHeaders;
 
 //-----------------------------------------------------------------------------
@@ -161,7 +210,7 @@ class KMComposeWin : public KMTopLevelWidget, virtual public MailComposerIface
   friend class KMHeaders;         // needed for the digest forward
 
 public:
-  KMComposeWin(KMMessage* msg=0L, QString id=QString::fromLatin1("unknown"));
+  KMComposeWin( KMMessage* msg=0, uint identity=0 );
   ~KMComposeWin();
 
   /**
@@ -219,6 +268,20 @@ public:
    */
    bool applyChanges(void);
 
+  /**
+   * Internal helper function called from applyChanges(void) to allow
+   * processing several messages (encrypted or unencrypted) based on
+   * the same composer content.
+   * That's usefull for storing decrypted versions of messages which
+   * were sent in encrypted form.                  (khz, 2002/06/24)
+   */
+   Kpgp::Result composeMessage( QCString pgpUserId,
+                                KMMessage& theMessage,
+                                bool doSign,
+                                bool doEncrypt,
+                                bool ignoreBcc,
+                                QCString& signCertFingerprint );
+
   /**
    * If this flag is set the message of the composer is deleted when
    * the composer is closed and the message was not sent. Default: FALSE
@@ -243,6 +306,13 @@ public:
    */
    void setReplyFocus( bool hasMessage = true );
 
+   /**
+    * Tries to find the given mimetype @p type in the KDE Mimetype registry.
+    * If found, returns its localized description, otherwise the @p type
+    * in lowercase.
+    */
+   static QString prettyMimeType( const QString& type );
+
 public slots:
   /**
    * Actions:
@@ -268,9 +338,17 @@ public slots:
   void slotCopy();
   void slotPaste();
   void slotMarkAll();
-  
+
   void slotFolderRemoved(KMFolder*);
 
+    /**
+       Tell the composer to always send the message, even if the user
+       hasn't changed the next. This is useful if a message is
+       autogenerated (e.g., via a DCOP call), and the user should
+       simply be able to confirm the message and send it.
+    */
+    void slotSetAlwaysSend( bool bAlwaysSend );
+    
   /**
    * toggle fixed width font.
    */
@@ -292,6 +370,12 @@ public slots:
   void slotSpellcheck();
   void slotSpellcheckConfig();
 
+  /**
+   * Change crypto plugin to be used for signing/encrypting messages,
+   * or switch to built-in OpenPGP code.
+   */
+  void slotSelectCryptoModule();
+  
   /**
    * XML-GUI stuff
    */
@@ -311,9 +395,15 @@ public slots:
 
   /**
    * Switch the icon to lock or unlock respectivly.
+   * Change states of all encrypt check boxes in the attachments listview
    */
   void slotEncryptToggled(bool);
 
+  /**
+   * Change states of all sign check boxes in the attachments listview
+   */
+  void slotSignToggled(bool);
+
   /**
    * Switch wordWrap on/off
    */
@@ -388,7 +478,7 @@ public slots:
   /**
    * Update composer field to reflect new identity
    */
-  void slotIdentityActivated(int idx);
+  void slotIdentityChanged(uint);
 
   /**
    * KIO slots for attachment insertion
@@ -417,12 +507,17 @@ protected:
    */
   void rethinkFields(bool fromslot=false);
 
+  /**
+   * Enable/disable some actions in the Attach menu
+   */
+  void enableAttachActions();
+
   /**
    * Show or hide header lines
    */
   void rethinkHeaderLine(int value, int mask, int& row,
 				 const QString& labelStr, QLabel* lbl,
-				 QLineEdit* edt, QPushButton* btn=NULL);
+				 QLineEdit* edt, QPushButton* btn=0);
   void rethinkHeaderLine(int value, int mask, int& row,
 				 const QString& labelStr, QLabel* lbl,
 				 QComboBox* cbx, QCheckBox *chk);
@@ -449,8 +544,8 @@ protected:
 
   /**
    * Use the given folder as sent-mail folder if the given folder exists.
-   * Else show an error message (not in KDE_3_0_BRANCH) and use the default
-   * sent-mail folder as sent-mail folder.
+   * Else show an error message and use the default sent-mail folder as
+   * sent-mail folder.
    */
   void setFcc( const QString &idString );
 
@@ -472,7 +567,7 @@ protected:
    /**
     * Updates an item in the QListView to represnet a given message part
     */
-   void msgPartToItem(const KMMessagePart* msgPart, QListViewItem *lvi);
+   void msgPartToItem(const KMMessagePart* msgPart, KMAtmListViewItem *lvi);
 
   /**
    * Open addressbook and append selected addresses to the given
@@ -481,6 +576,87 @@ protected:
    void addrBookSelInto(KMLineEdit* destEdit);
 
 private:
+  /**
+   * Get message ready for sending or saving.
+   * This must be done _before_ signing and/or encrypting it.
+   *
+   */
+  QCString breakLinesAndApplyCodec();
+
+  /**
+   * Get signature for a message.
+   * To build nice S/MIME objects signing and encoding must be separeted.
+   *
+   */
+  QByteArray pgpSignedMsg( QCString cText,
+                           StructuringInfoWrapper& structuring,
+                           QCString& signCertFingerprint );
+
+  /**
+   * Get encrypted message.
+   * To build nice S/MIME objects signing and encrypting must be separat.
+   *
+   */
+  QByteArray pgpEncryptedMsg( QCString cText, const QStringList& recipients,
+                              StructuringInfoWrapper& structuring,
+                              QCString& encryptCertFingerprints );
+
+  /**
+   * Get encryption certificate for a recipient (the Aegypten way).
+   */
+  QCString getEncryptionCertificate( const QString& recipient );
+
+  /**
+   * Check for expiry of various certificates.
+   */
+  bool checkForEncryptCertificateExpiry( const QString& recipient,
+                                         const QCString& certFingerprint );
+
+  /**
+   * Build a MIME object (or a flat text resp.) based upon
+   * structuring information returned by a crypto plugin that was
+   * called via pgpSignedMsg() (or pgpEncryptedMsg(), resp.).
+   *
+   * NOTE: The c string representation of the MIME object (or the
+   *       flat text, resp.) is returned in resultingData, so just
+   *       use this string as body text of the surrounding MIME object.
+   *       This string *is* encoded according to contentTEncClear
+   *       and thus should be ready for neing sended via SMTP.
+   */
+  bool processStructuringInfo( const QString   bugURL,
+                               uint            boundaryLevel,
+                               const QString   contentDescriptionClear,
+                               const QCString  contentTypeClear,
+                               const QCString  contentSubtypeClear,
+                               const QCString  contentDispClear,
+                               const QCString  contentTEncClear,
+                               const QCString& bodytext,
+                               const QString   contentDescriptionCiph,
+                               const QByteArray& ciphertext,
+                               const StructuringInfoWrapper& structuring,
+                               KMMessagePart&  resultingPart );
+
+  /**
+   * Retrieve encrypt flag of an attachment
+   * ( == state of it's check box in the attachments list view )
+   */
+  bool encryptFlagOfAttachment(int idx);
+
+  /**
+   * Retrieve sign flag of an attachment
+   * ( == state of it's check box in the attachments list view )
+   */
+  bool signFlagOfAttachment(int idx);
+
+
+  Kpgp::Result encryptMessage( KMMessage* msg,
+                       const QStringList& recipients, bool doSign, bool doEncrypt,
+                       const QCString& encodedBody,int previousBoundaryLevel,
+                       const KMMessagePart& oldBodyPart,
+                       bool earlyAddAttachments, bool allAttachmentsAreInBody,
+                       KMMessagePart newBodyPart,
+                       QCString& signCertFingerprint );
+
   /**
    * Decrypt an OpenPGP block or strip off the OpenPGP envelope of a text
    * block with a clear text signature. This is only done if the given
@@ -502,7 +678,8 @@ private:
 
 protected:
   QWidget   *mMainWidget;
-  QComboBox *mIdentity, *mTransport;
+  QComboBox *mTransport;
+  IdentityCombo    *mIdentity;
   KMFolderComboBox *mFcc;
   KMLineEdit *mEdtFrom, *mEdtReplyTo, *mEdtTo, *mEdtCc, *mEdtBcc, *mEdtSubject;
   QLabel    *mLblIdentity, *mLblTransport, *mLblFcc;
@@ -516,7 +693,12 @@ protected:
   KMEdit* mEditor;
   QGridLayout* mGrid;
   KMMessage *mMsg;
+  QPtrList<KMMessage> bccMsgList;
   QListView *mAtmListBox;
+  int mAtmColEncrypt;
+  int mAtmColSign;
+  int mAtmEncryptColWidth;
+  int mAtmSignColWidth;
   QPtrList<QListViewItem> mAtmItemList;
   KMMsgPartList mAtmList;
   QPopupMenu *mAttachMenu;
@@ -539,25 +721,30 @@ protected:
   QPtrList<QWidget> mEdtList;
   QPtrList<KTempFile> mAtmTempList;
   QPalette mPalette;
-  QString mId, mOldSigText;
+  uint mId;
+  QString mOldSigText;
   QStringList mTransportHistory;
 
-  KAction *attachPK, *attachMPK;
+  KAction *attachPK, *attachMPK,
+          *attachRemoveAction, *attachSaveAction, *attachPropertiesAction;
 
   KToggleAction *signAction, *encryptAction, *confirmDeliveryAction;
-  KToggleAction *confirmReadAction, *urgentAction, *allFieldsAction, *fromAction;
+  KToggleAction *requestMDNAction, *urgentAction, *allFieldsAction, *fromAction;
   KToggleAction *replyToAction, *toAction, *ccAction, *bccAction, *subjectAction;
   KToggleAction *identityAction, *transportAction, *fccAction;
   KToggleAction *toolbarAction, *statusbarAction;
   KToggleAction *wordWrapAction, *fixedFontAction;
 
   KSelectAction *encodingAction;
+  KSelectAction *cryptoModuleAction;
 
   QCString mCharset;
   QCString mDefCharset;
   QStringList mCharsets;
   bool bAutoCharset;
 
+    bool bAlwaysSend;
+
   QStringList mFolderNames;
   QValueList<QGuardedPtr<KMFolder> > mFolderList;
 
@@ -576,6 +763,21 @@ private:
   };
   QMap<KIO::Job *, atmLoadData> mapAtmLoadData;
   bool mForceReplyCharset;
+  
+  QString mErrorProcessingStructuringInfo;
+  QString mErrorNoCryptPlugAndNoBuildIn;
+
+  /**
+   * Store the cryptplug that was selected for signing and/or encrypting.
+   *
+   * This is either the 'active' cryptplug of the global wrapper list
+   * or another plugin (or 0 if none selected) if the user decided to
+   * override the global setting using KDComposeWin's options_select_crypto action.
+   */
+  CryptPlugWrapper* mSelectedCryptPlug;
+  
+public:
+  bool mDebugComposerCrypto;
 };
 #endif
 
