@@ -45,14 +45,19 @@ public:
 
   /** Constructor that initializes from given config file, if given.
     * Filters are stored one by one in config groups, i.e. one filter, one group.
-    * The config group has to be preset if config is not NULL. */
+    * The config group has to be preset if config is not 0. */
   KMFilter( KConfig* aConfig=0 , bool popFilter = false);
 
   /** Copy constructor. Constructs a deep copy of @p aFilter. */
-  KMFilter( KMFilter* aFilter );
+  KMFilter( const KMFilter & other );
 
   /** Cleanup. */
-  virtual ~KMFilter() {};
+  ~KMFilter() {};
+
+  /** Equivalent to @pattern()->name(). @return name of the filter */
+  QString name() const {
+    return mPattern.name();
+  }
 
   /** Execute the filter action(s) on the given message.
       Returns:
@@ -65,7 +70,7 @@ public:
       TRUE if the caller may apply other filters and FALSE if he shall
       stop the filtering of this message.
   */
-  virtual ReturnCode execActions( KMMessage* msg, bool& stopIt ) const ;
+  ReturnCode execActions( KMMessage* msg, bool& stopIt ) const ;
 
   /** No descriptions */
   KMPopFilterAction action();
@@ -77,25 +82,28 @@ public:
       constructor above) has to be preset.  The config object will be
       deleted by higher levels, so it is not allowed to store a
       pointer to it anywhere inside this function. */
-  virtual void writeConfig( KConfig* config ) const;
+  void writeConfig( KConfig* config ) const;
 
   /** Initialize from given config file. The config group (see
       constructor above) has to be preset. The config object will be
       deleted by higher levels, so it is not allowed to store a
       pointer to it anywhere inside this function. */
-  virtual void readConfig( KConfig* config );
+  void readConfig( KConfig* config );
 
   /** Remove empty rules (and actions one day). */
-  virtual void purify();
+  void purify();
 
   /** Check for empty pattern and action list. */
-  virtual bool isEmpty() const;
+  bool isEmpty() const;
 
   /** Provides a reference to the internal action list. If your used
       the @p setAction() and @p action() functions before, please
       convert to using myFilter->actions()->at() and friends now. */
   QPtrList<KMFilterAction>* actions() { return &mActions; }
 
+  /** Provides a reference to the internal action list. Const version. */
+  const QPtrList<KMFilterAction>* actions() const { return &mActions; }
+
   /** Provides a reference to the internal pattern. If you used the
       @p matches() function before, please convert to using
       myFilter->pattern()->matches() now. */
@@ -147,14 +155,16 @@ public:
    * @return TRUE if a change in some action occured,
    * FALSE if no action was affected.
    */
-  virtual bool folderRemoved( KMFolder* aFolder, KMFolder* aNewFolder );
+  bool folderRemoved( KMFolder* aFolder, KMFolder* aNewFolder );
 
   /** Returns the filter in a human-readable form. useful for
       debugging but not much else. Don't use, as it may well go away
       in the future... */
   const QString asString() const;
   /** No descriptions */
-  bool isPopFilter();
+  bool isPopFilter() const {
+    return bPopFilter;
+  }
 
 private:
   KMSearchPattern mPattern;
