@@ -4,6 +4,7 @@
  */
 // kmfldsearch.cpp
 
+#include <config.h>
 #include "kbusyptr.h"
 #include "kmcomposewin.h"
 #include "kmfldsearch.h"
@@ -30,16 +31,18 @@
 #include <qpopupmenu.h>
 #include <qpushbutton.h>
 #include <qradiobutton.h>
+#include <qbuttongroup.h>
 #include <qregexp.h>
 #include <qtextcodec.h>
 
 #include <kaction.h>
 #include <kapplication.h>
+#include <kcombobox.h>
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kglobal.h>
 #include <kiconloader.h>
-#include <kfiledialog.h>
+#include <klineedit.h>
 #include <kstatusbar.h>
 #include <kstdaction.h>
 #include <kwin.h>
@@ -48,67 +51,68 @@
 
 #include <stdlib.h>
 
-const int KMFldSearch::FOLDER_COLUMN = 3;
 const int KMFldSearch::MSGID_COLUMN = 4;
 
 //-----------------------------------------------------------------------------
 KMFldSearch::KMFldSearch(KMMainWin* w, const char* name,
                          KMFolder *curFolder, bool modal):
-  KDialogBase(NULL, name, modal, i18n("Search in Folders"),
-              KDialogBase::User1 | KDialogBase::User2 | KDialogBase::Close,
-              KDialogBase::User1, false, i18n("&Search"), i18n("S&top")),
-  mSearchAllFolders(false),
+  KDialogBase(0, name, modal, i18n("Search in Folders"),
+              User1 | User2 | Close, User1, false,
+              KGuiItem( i18n("&Search"), "find" ),
+              KGuiItem( i18n("S&top"), "cancel" )),
   mSearching(false),
   mStopped(false),
   mCloseRequested(false),
-  mNumRules(2),
   mNumMatches(0),
   mCount(0),
   mLastFocus(0),
   mMainWin(w)
 {
+  int numRules = 2;
   KWin::setIcons(winId(), kapp->icon(), kapp->miniIcon());
-  
+
   KConfig* config = KGlobal::config();
   config->setGroup("SearchDialog");
 
   QWidget* searchWidget = new QWidget(this);
-  mGrid = new QGridLayout(searchWidget, mNumRules+5, 5, 0, spacingHint());
+  mGrid = new QGridLayout(searchWidget, numRules+5, 5, 0, spacingHint());
+
+  QButtonGroup * radioGroup = new QButtonGroup( searchWidget );
+  radioGroup->hide();
 
   mChkbxAllFolders = new QRadioButton(i18n("Search in &all local folders"), searchWidget);
   mGrid->addMultiCellWidget(mChkbxAllFolders, 0, 0, 0, 3);
-  connect(mChkbxAllFolders, SIGNAL(toggled(bool)),
-          this, SLOT(slotSearchAllFolders(bool)));
+  radioGroup->insert( mChkbxAllFolders );
 
   mChkbxSpecificFolders = new QRadioButton(i18n("Search &only in:"), searchWidget);
   mGrid->addWidget(mChkbxSpecificFolders, 1, 0);
   mChkbxSpecificFolders->setChecked(true);
-  connect(mChkbxSpecificFolders, SIGNAL(toggled(bool)),
-          this, SLOT(slotSearchSpecificFolder(bool)));
+  radioGroup->insert( mChkbxSpecificFolders );
 
   mCbxFolders = new KMFolderComboBox(false, searchWidget);
-  mCbxFolders->setMinimumSize(mCbxFolders->sizeHint());
-  mCbxFolders->setMaximumSize(1024, mCbxFolders->sizeHint().height());
   mCbxFolders->setFolder(curFolder);
   mGrid->addMultiCellWidget(mCbxFolders, 1, 1, 1, 2);
 
   connect(mCbxFolders, SIGNAL(activated(int)),
           this, SLOT(slotFolderActivated(int)));
 
-  mRules = new KMFldSearchRule*[mNumRules];
-
-  KMFldSearchRule* rule = 0;
-  for (int i=0; i<mNumRules; ++i)
-  {
-    rule = new KMFldSearchRule(searchWidget, mGrid, i+2, 0);
-    rule->updateFunctions(mCbxFolders->getFolder());
-    mRules[i] = rule;
-  }
-
   mChkSubFolders = new QCheckBox(i18n("I&nclude sub-folders"), searchWidget);
   mChkSubFolders->setChecked(true);
   mGrid->addWidget(mChkSubFolders, 1, 3);
 
+  for (int i=0; i<numRules; ++i)
+  {
+    KMFldSearchRule* rule = new KMFldSearchRule(searchWidget, mGrid, i+2, 0);
+    rule->updateFunctions(mCbxFolders->getFolder());
+    mRules.append(rule);
+  }
+
+  // enable/disable widgets depending on radio buttons:
+  connect( mChkbxSpecificFolders, SIGNAL(toggled(bool)),
+	   mCbxFolders, SLOT(setEnabled(bool)) );
+  connect( mChkbxSpecificFolders, SIGNAL(toggled(bool)),
+	   mChkSubFolders, SLOT(setEnabled(bool)) );
+
   mLbxMatches = new QListView(searchWidget, "Search in Folders");
   /* Default is to sort by date. TODO: Unfortunately this sorts *while*
      inserting, which looks rather strange - the user cannot read
@@ -117,13 +121,13 @@ KMFldSearch::KMFldSearch(KMMainWin* w, const char* name,
   mLbxMatches->setSorting(2, false);
   mLbxMatches->setShowSortIndicator(true);
   mLbxMatches->setAllColumnsShowFocus(true);
-  mLbxMatches->addColumn(i18n("Subject"), 
+  mLbxMatches->addColumn(i18n("Subject"),
                          config->readNumEntry("SubjectWidth", 150));
-  mLbxMatches->addColumn(i18n("Sender/Receiver"), 
+  mLbxMatches->addColumn(i18n("Sender/Receiver"),
                          config->readNumEntry("SenderWidth", 120));
-  mLbxMatches->addColumn(i18n("Date"), 
+  mLbxMatches->addColumn(i18n("Date"),
                          config->readNumEntry("DateWidth", 120));
-  mLbxMatches->addColumn(i18n("Folder"), 
+  mLbxMatches->addColumn(i18n("Folder"),
                          config->readNumEntry("FolderWidth", 100));
 
   mLbxMatches->addColumn(""); // should be hidden
@@ -133,7 +137,7 @@ KMFldSearch::KMFldSearch(KMMainWin* w, const char* name,
 
   connect(mLbxMatches, SIGNAL(doubleClicked(QListViewItem *)),
           this, SLOT(slotShowMsg(QListViewItem *)));
-  mGrid->addMultiCellWidget(mLbxMatches, mNumRules+2, mNumRules+2, 0, 3);
+  mGrid->addMultiCellWidget(mLbxMatches, numRules+2, numRules+2, 0, 3);
 
   mStatusBar = new KStatusBar(searchWidget);
   mStatusBar->insertFixedItem(i18n("AMiddleLengthText..."), 0, true);
@@ -141,7 +145,7 @@ KMFldSearch::KMFldSearch(KMMainWin* w, const char* name,
   mStatusBar->setItemAlignment(0, AlignLeft | AlignVCenter);
   mStatusBar->insertItem(QString::null, 1, 1, true);
   mStatusBar->setItemAlignment(1, AlignLeft | AlignVCenter);
-  mGrid->addMultiCellWidget(mStatusBar, mNumRules+4, mNumRules+4, 0, 3);
+  mGrid->addMultiCellWidget(mStatusBar, numRules+4, numRules+4, 0, 3);
 
   mGrid->setColStretch(0, 0);
   mGrid->setColStretch(1, 2);
@@ -149,20 +153,20 @@ KMFldSearch::KMFldSearch(KMMainWin* w, const char* name,
   mGrid->setColStretch(3, 100);
   mGrid->setColStretch(4, 0);
 
-  mGrid->setRowStretch(mNumRules+2, 100);
-  mGrid->setRowStretch(mNumRules+3, 0);
+  mGrid->setRowStretch(numRules+2, 100);
+  mGrid->setRowStretch(numRules+3, 0);
 
   mGrid->activate();
-  mRules[0]->setFocus();
-  
+  mRules.at(0)->setFocus();
+
   int mainWidth = config->readNumEntry("SearchWidgetWidth", 0);
   int mainHeight = config->readNumEntry("SearchWidgetHeight", 0);
-  
+
   if (mainWidth || mainHeight)
   {
     resize(mainWidth, mainHeight);
   }
-  
+
   setMainWidget(searchWidget);
   setButtonBoxOrientation(QWidget::Vertical);
 
@@ -172,7 +176,7 @@ KMFldSearch::KMFldSearch(KMMainWin* w, const char* name,
 
   connect(this, SIGNAL(user1Clicked()), SLOT(slotSearch()));
   connect(this, SIGNAL(user2Clicked()), SLOT(slotStop()));
-  connect(this, SIGNAL(finished()), this, SLOT(slotDelayedDestruct()));
+  connect(this, SIGNAL(finished()), this, SLOT(deleteLater()));
 }
 
 //-----------------------------------------------------------------------------
@@ -187,7 +191,6 @@ KMFldSearch::~KMFldSearch()
   config->writeEntry("SearchWidgetWidth", width());
   config->writeEntry("SearchWidgetHeight", height());
   config->sync();
-  delete [] mRules;
 }
 
 //-----------------------------------------------------------------------------
@@ -195,18 +198,18 @@ void KMFldSearch::updStatus(void)
 {
   QString genMsg, detailMsg;
 
-  if (!mSearching) 
+  if (!mSearching)
   {
     if(!mStopped)
     {
-      genMsg = i18n("Done"); 
+      genMsg = i18n("Done");
       detailMsg = i18n("%n match (%1)", "%n matches (%1)", mNumMatches)
                  .arg(i18n("%n message processed",
                            "%n messages processed", mCount));
     }
     else
     {
-      genMsg = i18n("Search canceled"); 
+      genMsg = i18n("Search canceled");
       detailMsg = i18n("%n match so far (%1)",
                        "%n matches so far (%1)", mNumMatches)
                  .arg(i18n("%n message processed",
@@ -242,14 +245,15 @@ void KMFldSearch::keyPressEvent(QKeyEvent *evt)
 //-----------------------------------------------------------------------------
 bool KMFldSearch::searchInMessage(KMMessage* aMsg, const QCString& aMsgStr)
 {
-  int i;
   bool matches = true;
 
   mCount++;
   if (mCount % 100 == 0) updStatus();
-  for(i=0; matches && i<mNumRules; i++)
-    if (!mRules[i]->matches(aMsg, aMsgStr))
+  for(KMFldSearchRuleIt it(mRules); *it && matches; ++it)
+  {
+    if (!(*it)->matches(aMsg, aMsgStr))
       matches = false;
+  }
 
   return matches;
 }
@@ -260,13 +264,13 @@ void KMFldSearch::slotFolderComplete(KMFolderImap *folder, bool success)
   disconnect(folder, SIGNAL(folderComplete(KMFolderImap*, bool)),
              this, SLOT(slotFolderComplete(KMFolderImap*, bool)));
   mFetchingInProgress--;
-  
+
   if (success)
   {
     searchInFolder(folder, mChkSubFolders->isChecked(), FALSE);
     if (mFetchingInProgress == 0) searchDone();
   }
-  else 
+  else
   {
     searchDone();
   }
@@ -279,6 +283,7 @@ void KMFldSearch::slotFolderComplete(KMFolderImap *folder, bool success)
 void KMFldSearch::searchInFolder(QGuardedPtr<KMFolder> aFld, bool recursive,
   bool fetchHeaders)
 {
+  //kdDebug(5006) << "Searching folder '" << aFld->name() << "'" << endl;
   if (fetchHeaders && aFld->protocol() == "imap")
   {
     KMFolder *fld = aFld;
@@ -316,22 +321,28 @@ void KMFldSearch::searchInFolder(QGuardedPtr<KMFolder> aFld, bool recursive,
     return;
   }
 
-  for(i=0; i<mNumRules; i++)
-    if (!mRules[i]->isHeaderField()) fastMode = false;
+  for(KMFldSearchRuleIt it(mRules); *it; ++it)
+  {
+    if (!(*it)->isHeaderField())
+    {
+      fastMode = false;
+      break;
+    }
+  }
 
   num = aFld->count();
   for (i=0, upd=0; i<num && mSearching; i++, upd++)
   {
     if (fastMode)
     {
-      found = searchInMessage(NULL, aFld->getMsgString(i, cStr));
+      found = searchInMessage(0, aFld->getMsgString(i, cStr));
       if (found)
       {
         unget = !aFld->isMessage(i);
         msg = aFld->getMsg(i);
       } else unget = false;
-    } 
-    else 
+    }
+    else
     {
       unget = !aFld->isMessage(i);
       msg = aFld->getMsg(i);
@@ -340,11 +351,11 @@ void KMFldSearch::searchInFolder(QGuardedPtr<KMFolder> aFld, bool recursive,
     if (found)
     {
       QString from;
-      if(mSearchFolder == i18n("sent-mail"))
+      if(aFld->type() == KFolderTreeItem::SentMail)
         from = msg->to();
       else
         from = msg->from();
-      
+
       (void)new QListViewItem(mLbxMatches,
                               msg->subject(), from, msg->dateIsoStr(),
                               mSearchFolder,
@@ -397,12 +408,12 @@ void KMFldSearch::searchInAllFolders(void)
   QValueList<QGuardedPtr<KMFolder> > folders;
   QGuardedPtr<KMFolder> folder;
   QStringList str;
-  int i = 0;
 
   mMainWin->folderTree()->createFolderList( &str, &folders );
-  for (; folders.at(i) != folders.end(); ++i)
+  QValueListConstIterator<QGuardedPtr<KMFolder> > it;
+  for (it = folders.begin(); it != folders.end(); ++it)
   {
-    folder = *folders.at(i);
+    folder = *it;
     // Stop pressed?
     if(!mSearching)
     {
@@ -411,7 +422,7 @@ void KMFldSearch::searchInAllFolders(void)
 
     if (folder && (folder->protocol() != "imap"))
     {
-      searchInFolder(folder);
+      searchInFolder(folder, false);
     }
   }
 }
@@ -421,13 +432,13 @@ void KMFldSearch::searchInAllFolders(void)
 void KMFldSearch::slotFolderActivated(int /*nr*/)
 {
   KMFolder* folder = mCbxFolders->getFolder();
-  
+
   mChkbxSpecificFolders->setChecked(true);
   mBtnSearch->setEnabled(folder);
 
-  for (int i = 0; i < mNumRules; i++)
+  for(KMFldSearchRuleIt it(mRules); *it; ++it)
   {
-    mRules[i]->updateFunctions(folder);
+    (*it)->updateFunctions(folder);
   }
 }
 
@@ -437,7 +448,7 @@ void KMFldSearch::activateFolder(KMFolder *curFolder)
 {
   mChkbxSpecificFolders->setChecked(true);
   mCbxFolders->setFolder(curFolder);
-  mRules[0]->setFocus();
+  mRules.at(0)->setFocus();
 }
 
 //-----------------------------------------------------------------------------
@@ -459,10 +470,12 @@ void KMFldSearch::slotSearch()
   mLbxMatches->clear();
   kapp->processEvents();
 
-  for (int i = 0; i < mNumRules; ++i)
-    mRules[i]->prepare();
+  for(KMFldSearchRuleIt it(mRules); *it; ++it)
+  {
+    (*it)->prepare();
+  }
 
-  if (mSearchAllFolders)
+  if (mChkbxAllFolders->isChecked())
   {
     searchInAllFolders();
   }
@@ -520,46 +533,12 @@ void KMFldSearch::closeEvent(QCloseEvent *e)
     mCloseRequested = true;
     e->ignore();
   }
-  else 
+  else
   {
     KDialogBase::closeEvent(e);
   }
 }
 
-//-----------------------------------------------------------------------------
-void KMFldSearch::slotSearchAllFolders(bool on)
-{
-  if (on)
-  {
-    mChkbxSpecificFolders->setChecked(false);
-    mCbxFolders->setEnabled(false);
-    mChkSubFolders->setEnabled(false);
-    mSearchAllFolders = true;
-  }
-  else if (!mChkbxSpecificFolders->isChecked())
-  {
-    mChkbxAllFolders->setChecked(true);
-//    mChkbxSpecificFolders->setChecked(true);
-  }
-}
-
-//-----------------------------------------------------------------------------
-void KMFldSearch::slotSearchSpecificFolder(bool on)
-{
-  if (on)
-  {
-    mChkbxAllFolders->setChecked(false);
-    mCbxFolders->setEnabled(true);
-    mChkSubFolders->setEnabled(true);
-    mSearchAllFolders = false;
-  }
-  else if (!mChkbxAllFolders->isChecked())
-  {
-    mChkbxSpecificFolders->setChecked(true);
-//    mChkbxAllFolders->setChecked(true);
-  }
-}
-
 //-----------------------------------------------------------------------------
 bool KMFldSearch::slotShowMsg(QListViewItem *item)
 {
@@ -570,7 +549,7 @@ bool KMFldSearch::slotShowMsg(QListViewItem *item)
 
   KMFolder* folder;
   int msgIndex;
-  kernel->msgDict()->getLocation(item->text(MSGID_COLUMN).toUInt(), 
+  kernel->msgDict()->getLocation(item->text(MSGID_COLUMN).toUInt(),
                                  &folder, &msgIndex);
 
   if (!folder || msgIndex < 0)
@@ -591,7 +570,7 @@ bool KMFldSearch::slotShowMsg(QListViewItem *item)
 
 
 //-----------------------------------------------------------------------------
-void KMFldSearch::enableGUI() 
+void KMFldSearch::enableGUI()
 {
   actionButton(KDialogBase::Close)->setEnabled(!mSearching);
   mCbxFolders->setEnabled(!mSearching);
@@ -599,8 +578,10 @@ void KMFldSearch::enableGUI()
   mChkbxAllFolders->setEnabled(!mSearching);
   mChkbxSpecificFolders->setEnabled(!mSearching);
 
-  for(int i = 0; i < mNumRules; i++)
-    mRules[i]->setEnabled(!mSearching);
+  for(KMFldSearchRuleIt it(mRules); *it; ++it)
+  {
+    (*it)->setEnabled(!mSearching);
+  }
 }
 
 
@@ -608,29 +589,29 @@ void KMFldSearch::enableGUI()
 KMFldSearchRule::KMFldSearchRule(QWidget* aParent, QGridLayout* aGrid,
                                  int aRow, int aCol)
 {
-  assert(aParent!=NULL);
-  assert(aGrid!=NULL);
+  assert(aParent!=0);
+  assert(aGrid!=0);
 
   mRow = aRow;
-  mCbxField = new QComboBox(true, aParent);
+  mCbxField = new KComboBox(true, aParent);
   insertFieldItems(false);
   mCbxField->setMinimumSize(mCbxField->sizeHint());
   mCbxField->setMaximumSize(1024, mCbxField->sizeHint().height());
 
-  mCbxFunc = new QComboBox(false, aParent);
+  mCbxFunc = new KComboBox(false, aParent);
   mCbxFunc->insertItem(i18n("contains"));
   mCbxFunc->insertItem(i18n("doesn't contain"));
   mCbxFunc->insertItem(i18n("equals"));
-  mCbxFunc->insertItem(i18n("not equal"));
+  mCbxFunc->insertItem(i18n("doesn't equal"));
   mCbxFunc->insertItem(i18n("matches regular expr."));
   mCbxFunc->insertItem(i18n("doesn't match regular expr."));
   mCbxFunc->setMinimumSize(mCbxFunc->sizeHint());
   mCbxFunc->setMaximumSize(1024, mCbxFunc->sizeHint().height());
 
-  mEdtValue = new QLineEdit(aParent);
+  mEdtValue = new KLineEdit(aParent);
   mEdtValue->setMinimumSize(mCbxFunc->sizeHint());
   mEdtValue->setMaximumSize(1024, mCbxFunc->sizeHint().height());
-  
+
   QLabel* lbl;
 
   if (mRow > 2)
@@ -646,7 +627,7 @@ KMFldSearchRule::KMFldSearchRule(QWidget* aParent, QGridLayout* aGrid,
   }
 
   lbl->setMinimumSize(lbl->sizeHint());
-  
+
   aGrid->addWidget(lbl, aRow, aCol);
   aGrid->addWidget(mCbxField, aRow, aCol + 1);
   aGrid->addWidget(mCbxFunc,  aRow, aCol + 2);
@@ -705,7 +686,7 @@ void KMFldSearchRule::prepare(void)
 {
   mFieldIdx = mCbxField->currentItem();
   mField = mCbxField->currentText();
-  mHeaderField = QString("\n" + mField + ": ").latin1();
+  mHeaderField = QString('\n' + mField + ": ").latin1();
   mFieldLength = mField.length() + 3;
   mFunc = mCbxFunc->currentItem();
   mValue = mEdtValue->text();
@@ -755,9 +736,9 @@ bool KMFldSearchRule::matches(const KMMessage* aMsg, const QCString& aMsgStr)
     if (endOfHeader == -1) endOfHeader = aMsgStr.find("\n\r\n");
     if (endOfHeader < start) return false;
     start += mFieldLength;
-    stop = aMsgStr.find("\n", start);
+    stop = aMsgStr.find('\n', start);
     while (stop != -1 && (ch = aMsgStr.at(stop + 1)) == ' ' || ch == '\t')
-      stop = aMsgStr.find("\n", stop + 1);
+      stop = aMsgStr.find('\n', stop + 1);
     if (stop == -1) value = KMMsgBase::decodeRFC2047String(aMsgStr.mid(start));
     else value = KMMsgBase::decodeRFC2047String(aMsgStr.mid(start,
       stop - start));
