@@ -1,6 +1,7 @@
 // kmfolder.cpp
 // Author: Stefan Taferner <taferner@alpin.or.at>
 
+#include <config.h>
 #include <qfileinfo.h>
 #include <qsortedlist.h>
 
@@ -9,23 +10,23 @@
 #include "kmfolderimap.h"
 #include "kmundostack.h"
 #include "kmmsgdict.h"
+#include "kmkernel.h"
+#include "identitymanager.h"
 #include "kmidentity.h"
-#include "kmfiltermgr.h"
 
 #include <mimelib/mimepp.h>
 #include <qregexp.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
 #include <kcursor.h>
+#include <kiconloader.h>
 
 #include <stdio.h>
 #include <errno.h>
 #include <assert.h>
 #include <unistd.h>
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #if HAVE_FCNTL_H
 #include <fcntl.h>
@@ -80,7 +81,7 @@ static int _rename(const char* oldname, const char* newname)
 KMFolder :: KMFolder(KMFolderDir* aParent, const QString& aName) :
   KMFolderInherited(aParent, aName), mMsgList(INIT_MSGS)
 {
-  mIndexStream    = NULL;
+  mIndexStream    = 0;
   mOpenCount      = 0;
   mQuiet          = 0;
   mChanged        = FALSE;
@@ -88,16 +89,17 @@ KMFolder :: KMFolder(KMFolderDir* aParent, const QString& aName) :
   mAutoCreateIndex= TRUE;
   mIsSystemFolder = FALSE;
   mType           = "plain";
-  mAcctList       = NULL;
+  mAcctList       = 0;
   mDirty          = FALSE;
   mUnreadMsgs      = -1;
   mGuessedUnreadMsgs = -1;
+  mTotalMsgs      = -1;
   needsCompact    = FALSE;
   mChild          = 0;
   mConvertToUtf8  = FALSE;
   mMailingListEnabled = FALSE;
   mIndexId        = -1;
-  mIndexStreamPtr = NULL;
+  mIndexStreamPtr = 0;
   mIndexStreamPtrLength = 0;
   mIndexSwapByteOrder = false;
   mIndexSizeOfLong = sizeof(long);
@@ -109,12 +111,18 @@ KMFolder :: KMFolder(KMFolderDir* aParent, const QString& aName) :
   readExpireAge = 14;
   readExpireUnits = expireNever;
   mRDict = 0;
+  mNormalIcon = 0;
+  mUnreadIcon = 0;
+  mUseCustomIcons = 0;
+  mNeedsRepainting = false;
 }
 
 
 //-----------------------------------------------------------------------------
 KMFolder :: ~KMFolder()
 {
+  delete mNormalIcon;
+  delete mUnreadIcon;
   delete mAcctList;
   KMMsgDict::deleteRentry(mRDict);
 }
@@ -200,15 +208,17 @@ KMFolderDir* KMFolder::createChildFolder()
 }
 
 //-----------------------------------------------------------------------------
-bool KMFolder::isIndexOutdated()
+KMFolder::IndexStatus KMFolder::indexStatus()
 {
   QFileInfo contInfo(location());
   QFileInfo indInfo(indexLocation());
 
-  if (!contInfo.exists()) return FALSE;
-  if (!indInfo.exists()) return TRUE;
+  if (!contInfo.exists()) return KMFolder::IndexOk;
+  if (!indInfo.exists()) return KMFolder::IndexMissing;
 
-  return (contInfo.lastModified() > indInfo.lastModified());
+  return ( contInfo.lastModified() > indInfo.lastModified() )
+         ? KMFolder::IndexTooOld
+         : KMFolder::IndexOk;
 }
 
 
@@ -216,11 +226,9 @@ bool KMFolder::isIndexOutdated()
 int KMFolder::writeIndex()
 {
   QString tempName;
-  KMMsgBase* msgBase;
   int old_umask;
   int i=0, len;
-  long tmp;
-  const uchar *buffer = NULL;
+  const uchar *buffer = 0;
   old_umask = umask(077);
 
   //the sorted file must be removed, BIG kludge, don made me do it!
@@ -245,23 +253,24 @@ int KMFolder::writeIndex()
   Q_UINT32 byteOrder = 0x12345678;
   Q_UINT32 sizeOfLong = sizeof(long);
 
-  Q_UINT32 header_length = sizeof(byteOrder)+sizeof(sizeOfLong); 
+  Q_UINT32 header_length = sizeof(byteOrder)+sizeof(sizeOfLong);
   char pad_char = '\0';
   fwrite(&pad_char, sizeof(pad_char), 1, tmpIndexStream);
   fwrite(&header_length, sizeof(header_length), 1, tmpIndexStream);
 
-  // Write header 
+  // Write header
   fwrite(&byteOrder, sizeof(byteOrder), 1, tmpIndexStream);
   fwrite(&sizeOfLong, sizeof(sizeOfLong), 1, tmpIndexStream);
 
-  long nho = ftell(tmpIndexStream);
+  KMMsgBase* msgBase;
+  off_t nho = ftell(tmpIndexStream);
   for (i=0; i<mMsgList.high(); i++)
   {
     if (!(msgBase = mMsgList[i])) continue;
     buffer = msgBase->asIndexString(len);
     fwrite(&len,sizeof(len), 1, tmpIndexStream);
-	
-    tmp = ftell(tmpIndexStream);
+
+    off_t tmp = ftell(tmpIndexStream);
     msgBase->setIndexOffset(tmp);
     msgBase->setIndexLength(len);
     if(fwrite(buffer, len, 1, tmpIndexStream) != 1)
@@ -293,12 +302,16 @@ void KMFolder::setAutoCreateIndex(bool autoIndex)
   mAutoCreateIndex = autoIndex;
 }
 
+void KMFolder::setIdentity( uint identity ) {
+  mIdentity = identity;
+  kernel->slotRequestConfigSync();
+}
 
 //-----------------------------------------------------------------------------
 bool KMFolder::readIndexHeader(int *gv)
 {
   int indexVersion;
-  assert(mIndexStream != NULL);
+  assert(mIndexStream != 0);
   mIndexSwapByteOrder = false;
   mIndexSizeOfLong = sizeof(long);
 
@@ -317,14 +330,14 @@ bool KMFolder::readIndexHeader(int *gv)
       createIndexFromContents();
       return FALSE;
   } else if(indexVersion > INDEX_VERSION) {
-      kapp->setOverrideCursor(KCursor::arrowCursor());
+      QApplication::setOverrideCursor(KCursor::arrowCursor());
       int r = KMessageBox::questionYesNo(0,
 					 i18n(
 					    "The mail index for '%1' is from an unknown version of KMail (%2).\n"
 					    "This index can be regenerated from your mail folder, but some "
 					    "information, including status flags, may be lost. Do you wish "
 					    "to downgrade your index file?") .arg(name()) .arg(indexVersion) );
-      kapp->restoreOverrideCursor();
+      QApplication::restoreOverrideCursor();
       if (r == KMessageBox::Yes)
 	  createIndexFromContents();
       return FALSE;
@@ -339,8 +352,8 @@ bool KMFolder::readIndexHeader(int *gv)
       fread(&header_length, sizeof(header_length), 1, mIndexStream);
       if (header_length > 0xFFFF)
          header_length = kmail_swap_32(header_length);
-      
-      long endOfHeader = ftell(mIndexStream) + header_length;
+
+      off_t endOfHeader = ftell(mIndexStream) + header_length;
 
       bool needs_update = true;
       // Process available header parts
@@ -349,7 +362,7 @@ bool KMFolder::readIndexHeader(int *gv)
          fread(&byteOrder, sizeof(byteOrder), 1, mIndexStream);
          mIndexSwapByteOrder = (byteOrder == 0x78563412);
          header_length -= sizeof(byteOrder);
-      
+
          if (header_length >= sizeof(sizeOfLong))
          {
             fread(&sizeOfLong, sizeof(sizeOfLong), 1, mIndexStream);
@@ -379,26 +392,26 @@ bool KMFolder::readIndexHeader(int *gv)
 bool KMFolder::readIndex()
 {
   Q_INT32 len;
-  int offs;
   KMMsgInfo* mi;
 
-  assert(mIndexStream != NULL);
+  assert(mIndexStream != 0);
   rewind(mIndexStream);
 
   mMsgList.clear();
   int version;
 
   mDirty = false;
-  
+
   if (!readIndexHeader(&version)) return false;
 
   mUnreadMsgs = 0;
+  mTotalMsgs = 0;
   mHeaderOffset = ftell(mIndexStream);
 
   mMsgList.clear();
   while (!feof(mIndexStream))
   {
-    mi = NULL;
+    mi = 0;
     if(version >= 1505) {
       if(!fread(&len, sizeof(len), 1, mIndexStream))
         break;
@@ -406,25 +419,25 @@ bool KMFolder::readIndex()
       if (mIndexSwapByteOrder)
         len = kmail_swap_32(len);
 
-      offs = ftell(mIndexStream);
+      off_t offs = ftell(mIndexStream);
       if(fseek(mIndexStream, len, SEEK_CUR))
         break;
       mi = new KMMsgInfo(this, offs, len);
-    } 
-    else 
+    }
+    else
     {
       QCString line(MAX_LINE);
       fgets(line.data(), MAX_LINE, mIndexStream);
       if (feof(mIndexStream)) break;
       if (*line.data() == '\0') {
 	  fclose(mIndexStream);
-	  mIndexStream = NULL;
+	  mIndexStream = 0;
 	  mMsgList.clear();
 	  return false;
       }
       mi = new KMMsgInfo(this);
       mi->compat_fromOldIndexString(line, mConvertToUtf8);
-    }	
+    }
     if(!mi)
       break;
 
@@ -457,6 +470,7 @@ bool KMFolder::readIndex()
     mDirty = TRUE;
     writeIndex();
   }
+  mTotalMsgs = mMsgList.count();
   return true;
 }
 
@@ -480,7 +494,7 @@ void KMFolder::markNewAsUnread()
 
 void KMFolder::markUnreadAsRead()
 {
-  KMMsgBase* msgBase;
+  const KMMsgBase* msgBase;
   QValueList<int> items;
 
   for (int i=count()-1; i>=0; --i)
@@ -535,9 +549,9 @@ int operator==( KMMsgBase & m1, KMMsgBase & m2 )
 int KMFolder::reduceSize( int aSize )
 {
   kdDebug(5006) << "Reducing folder to size of " << aSize << " Mo" << endl;
-  QSortedList<KMMsgBase> * slice=0L;
+  QSortedList<KMMsgBase> * slice=0;
   QPtrList< QSortedList<KMMsgBase> > sliceArr;
-  KMMsgBase* mb;
+  const KMMsgBase* mb;
   ulong folderSize, msgSize, sliceSize, firstSliceSize, lastSliceSize, size;
   int sliceIndex;
   int delMsg = 0;
@@ -618,10 +632,10 @@ int KMFolder::expungeOldMsg(int days)
 {
   int i, msgnb=0;
   time_t msgTime, maxTime;
-  KMMsgBase* mb;
+  const KMMsgBase* mb;
   QValueList<int> rmvMsgList;
 
-  maxTime = time(0L) - days * 3600 * 24;
+  maxTime = time(0) - days * 3600 * 24;
 
   for (i=count()-1; i>=0; i--) {
     mb = getMsgBase(i);
@@ -670,7 +684,7 @@ void KMFolder::expireOldMessages() {
   int             days = 0;
   int             maxUnreadTime = 0;
   int             maxReadTime = 0;
-  KMMsgBase       *mb = NULL;
+  const KMMsgBase       *mb = 0;
   QValueList<int> rmvMsgList;
   int             i = 0;
   time_t          msgTime, maxTime = 0;
@@ -684,13 +698,13 @@ void KMFolder::expireOldMessages() {
   days = daysToExpire(getUnreadExpireAge(), getUnreadExpireUnits());
   if (days > 0) {
     kdDebug(5006) << "deleting unread older than "<< days << " days" << endl;
-    maxUnreadTime = time(0L) - days * 3600 * 24;
+    maxUnreadTime = time(0) - days * 3600 * 24;
   }
 
   days = daysToExpire(getReadExpireAge(), getReadExpireUnits());
   if (days > 0) {
     kdDebug(5006) << "deleting read older than "<< days << " days" << endl;
-    maxReadTime = time(0L) - days * 3600 * 24;
+    maxReadTime = time(0) - days * 3600 * 24;
   }
 
   if ((maxUnreadTime == 0) && (maxReadTime == 0)) {
@@ -699,7 +713,7 @@ void KMFolder::expireOldMessages() {
   open();
   for (i=mMsgList.count()-1; i>=0; i--) {
     mb = getMsgBase(i);
-    if (mb == NULL) {
+    if (mb == 0) {
 	  continue;
     }
     msgTime = mb->date();
@@ -715,7 +729,7 @@ void KMFolder::expireOldMessages() {
     }
   }
   close();
-  
+
   return;
 }
 
@@ -733,7 +747,7 @@ bool KMFolder::canAddMsgNow(KMMessage* aMsg, int* aIndex_ret)
     connect(imapJob, SIGNAL(messageRetrieved(KMMessage*)),
       SLOT(reallyAddMsg(KMMessage*)));
     aMsg->setTransferInProgress(TRUE);
-    kernel->filterMgr()->tempOpenFolder(this);
+    static_cast<KMFolderImap*>(msgParent)->account()->tempOpenFolder(this);
     return FALSE;
   }
   return TRUE;
@@ -743,27 +757,32 @@ bool KMFolder::canAddMsgNow(KMMessage* aMsg, int* aIndex_ret)
 //-----------------------------------------------------------------------------
 void KMFolder::reallyAddMsg(KMMessage* aMsg)
 {
+  aMsg->setTransferInProgress(FALSE);
   KMFolder *folder = aMsg->parent();
   int index;
   ulong serNum = aMsg->getMsgSerNum();
+  bool undo = aMsg->enableUndo();
   addMsg(aMsg, &index);
   if (index < 0) return;
   unGetMsg(index);
-  kernel->undoStack()->pushAction( serNum, folder, this );
+  if (undo)
+  {
+    kernel->undoStack()->pushAction( serNum, folder, this );
+  }
 }
 
 
 //-----------------------------------------------------------------------------
 void KMFolder::reallyAddCopyOfMsg(KMMessage* aMsg)
 {
-  aMsg->setParent( NULL );
+  aMsg->setParent( 0 );
   addMsg( aMsg );
   unGetMsg( count() - 1 );
 }
 
 
 //-----------------------------------------------------------------------------
-void KMFolder::removeMsg(KMMsgBasePtr aMsg)
+void KMFolder::removeMsg(const KMMsgBase* aMsg)
 {
   int idx = find(aMsg);
   assert( idx != -1);
@@ -797,15 +816,17 @@ void KMFolder::removeMsg(int idx, bool)
   needsCompact=true; // message is taken from here - needs to be compacted
 
   if (mb->status()==KMMsgStatusUnread ||
-      mb->status()==KMMsgStatusNew || 
+      mb->status()==KMMsgStatusNew ||
       (this == kernel->outboxFolder())) {
     --mUnreadMsgs;
     emit numUnreadMsgsChanged( this );
   }
+  --mTotalMsgs;
 
-  if (!mQuiet)
+  if (!mQuiet) {
     emit msgRemoved(idx, msgIdMD5);
-  else
+    emit msgRemoved(this);
+  } else
     mChanged = TRUE;
 }
 
@@ -819,23 +840,25 @@ KMMessage* KMFolder::take(int idx)
   assert(idx>=0 && idx<=mMsgList.high());
 
   mb = mMsgList[idx];
-  if (!mb) return NULL;
+  if (!mb) return 0;
   if (!mb->isMessage()) readMsg(idx);
 
   QString msgIdMD5 = mMsgList[idx]->msgIdMD5();
   msg = (KMMessage*)mMsgList.take(idx);
   if (msg->status()==KMMsgStatusUnread ||
-      msg->status()==KMMsgStatusNew || 
+      msg->status()==KMMsgStatusNew ||
       (this == kernel->outboxFolder())) {
     --mUnreadMsgs;
     emit numUnreadMsgsChanged( this );
   }
-  msg->setParent(NULL);
+  --mTotalMsgs;
+  msg->setParent(0);
   mDirty = TRUE;
   needsCompact=true; // message is taken from here - needs to be compacted
-  if (!mQuiet)
+  if (!mQuiet) {
     emit msgRemoved(idx,msgIdMD5);
-  else
+    emit msgRemoved(this);
+  } else
     mChanged = TRUE;
 
   return msg;
@@ -862,16 +885,17 @@ KMMessage* KMFolder::getMsg(int idx)
 
   // assert(idx>=0 && idx<=mMsgList.high());
   if(!(idx >= 0 && idx <= mMsgList.high()))
-    return 0L;
+    return 0;
 
   mb = mMsgList[idx];
-  if (!mb) return NULL;
+  if (!mb) return 0;
 
 #if 0
   if (mb->isMessage()) return ((KMMessage*)mb);
   return readMsg(idx);
 #else
   KMMessage *msg = 0;
+  bool undo = mb->enableUndo();
   if (mb->isMessage()) {
       msg = ((KMMessage*)mb);
   } else {
@@ -885,12 +909,13 @@ KMMessage* KMFolder::getMsg(int idx)
 	  writeConfig();
       }
   }
-  
+  msg->setEnableUndo(undo);
+
   if (msg->getMsgSerNum() == 0) {
     msg->setMsgSerNum(kernel->msgDict()->insert(0, msg, idx));
     kdDebug(5006) << "Serial number generated for message in folder " << label() << endl;
   }
-  
+
   return msg;
 #endif
 
@@ -904,10 +929,10 @@ KMMsgInfo* KMFolder::unGetMsg(int idx)
   KMMsgBase* mb;
 
   if(!(idx >= 0 && idx <= mMsgList.high()))
-    return 0L;
+    return 0;
 
   mb = mMsgList[idx];
-  if (!mb) return NULL;
+  if (!mb) return 0;
 
   if (mb->isMessage()) {
     KMMsgInfo *msgInfo = new KMMsgInfo( this );
@@ -916,7 +941,7 @@ KMMsgInfo* KMFolder::unGetMsg(int idx)
     return msgInfo;
   }
 
-  return 0L;
+  return 0;
 }
 
 
@@ -936,7 +961,7 @@ int KMFolder::moveMsg(KMMessage* aMsg, int* aIndex_ret)
   KMFolder* msgParent;
   int rc;
 
-  assert(aMsg != NULL);
+  assert(aMsg != 0);
   msgParent = aMsg->parent();
 
   if (msgParent)
@@ -959,7 +984,7 @@ int KMFolder::moveMsg(QPtrList<KMMessage> msglist, int* aIndex_ret)
   int rc;
 
   KMMessage* aMsg = msglist.first();
-  assert(aMsg != NULL);
+  assert(aMsg != 0);
   msgParent = aMsg->parent();
 
   if (msgParent)
@@ -1066,7 +1091,7 @@ int KMFolder::remove()
   if (kernel->msgDict()) kernel->msgDict()->removeFolderIds(this);
   unlink(indexLocation().local8Bit() + ".sorted");
   unlink(indexLocation().local8Bit());
-  
+
   int rc = removeContents();
   if (rc) return rc;
 
@@ -1088,10 +1113,10 @@ int KMFolder::expunge()
   kernel->msgDict()->removeFolderIds(this);
   if (mAutoCreateIndex) truncate(indexLocation().local8Bit(), mHeaderOffset);
   else unlink(indexLocation().local8Bit());
-  
+
   int rc = expungeContents();
   if (rc) return rc;
-  
+
   mDirty = FALSE;
   needsCompact = false; //we're cleared and truncated no need to compact
 
@@ -1102,6 +1127,7 @@ int KMFolder::expunge()
   }
 
   mUnreadMsgs = 0;
+  mTotalMsgs = 0;
   emit numUnreadMsgsChanged( this );
   if (mAutoCreateIndex)
     writeConfig();
@@ -1125,10 +1151,19 @@ const char* KMFolder::type() const
 QString KMFolder::label() const
 {
   if (mIsSystemFolder && !mLabel.isEmpty()) return mLabel;
-  if (mIsSystemFolder) return i18n(name().latin1());
+  if (mIsSystemFolder) return i18n(name().utf8());
   return name();
 }
 
+int KMFolder::count(bool cache) const
+{
+  if (cache && mTotalMsgs != -1)
+  {
+    return mTotalMsgs;
+  } else {
+    return mMsgList.count();
+  }
+}
 
 //-----------------------------------------------------------------------------
 int KMFolder::countUnread()
@@ -1198,7 +1233,7 @@ void KMFolder::headerOfMsgChanged(const KMMsgBase* aMsg, int idx)
     return;
   }
   if (idx < 0)
-    idx = mMsgList.find((KMMsgBasePtr)aMsg);
+    idx = mMsgList.find((KMMsgBase*)aMsg);
   if (idx >= 0)
     emit msgHeaderChanged(idx);
   else
@@ -1207,7 +1242,7 @@ void KMFolder::headerOfMsgChanged(const KMMsgBase* aMsg, int idx)
 
 
 //-----------------------------------------------------------------------------
-QString KMFolder::idString()
+QString KMFolder::idString() const
 {
   KMFolderNode* folderNode = parent();
   if (!folderNode)
@@ -1220,6 +1255,36 @@ QString KMFolder::idString()
     relativePath = relativePath.right( relativePath.length() - 1 ) + "/";
   return relativePath + QString(name());
 }
+
+//---------------------------------------------------------------------------
+void KMFolder::iconsFromPath()
+{
+  KInstance *instance=KGlobal::instance();
+  KIconLoader *loader = instance->iconLoader();
+  QPixmap tmp1(loader->loadIcon(mNormalIconPath, KIcon::Small, 0, KIcon::DefaultState, 0, true));
+  QPixmap tmp2(loader->loadIcon(mUnreadIconPath, KIcon::Small, 0, KIcon::DefaultState, 0, true));
+  bool tmp1Valid, tmp2Valid;
+
+  tmp1Valid = !tmp1.isNull();
+  tmp2Valid = !tmp2.isNull();
+
+  if ( tmp1Valid || tmp2Valid) { //we have a valid pixmap
+    // NOTE: can qpixmap constructors throw/fail?
+    if ( tmp1Valid ) {
+      delete mNormalIcon;
+      mNormalIcon = new QPixmap(tmp1);
+    }
+    if ( tmp2Valid ) {
+      delete mUnreadIcon;
+      mUnreadIcon = new QPixmap(tmp2);
+    } else { //make this one,same as the other one custom one
+      delete mUnreadIcon;
+      mUnreadIconPath = mNormalIconPath;
+      mUnreadIcon = new QPixmap(tmp1);
+    }
+  }
+}
+
 //-----------------------------------------------------------------------------
 void KMFolder::readConfig()
 {
@@ -1227,20 +1292,27 @@ void KMFolder::readConfig()
   KConfigGroupSaver saver(config, "Folder-" + idString());
   if (mUnreadMsgs == -1)
     mUnreadMsgs = config->readNumEntry("UnreadMsgs", -1);
+  if (mTotalMsgs == -1)
+    mTotalMsgs = config->readNumEntry("TotalMsgs", -1);
   mMailingListEnabled = config->readBoolEntry("MailingListEnabled");
   mMailingListPostingAddress = config->readEntry("MailingListPostingAddress");
   mMailingListAdminAddress = config->readEntry("MailingListAdminAddress");
-  mIdentity = config->readEntry("Identity");
-  if ( mIdentity.isEmpty() ) // backward compatiblity
-      mIdentity = config->readEntry("MailingListIdentity");
+  mIdentity = config->readUnsignedNumEntry("Identity",0);
   mCompactable = config->readBoolEntry("Compactable", TRUE);
 
+  mUseCustomIcons = config->readBoolEntry("UseCustomIcons", FALSE );
+  mNormalIconPath = config->readEntry("NormalIconPath");
+  mUnreadIconPath = config->readEntry("UnreadIconPath");
+  if ( mUseCustomIcons )
+      mNeedsRepainting = true;
+  iconsFromPath();
+
   expireMessages = config->readBoolEntry("ExpireMessages", FALSE);
   readExpireAge = config->readNumEntry("ReadExpireAge", 3);
   readExpireUnits = (ExpireUnits)config->readNumEntry("ReadExpireUnits", expireMonths);
   unreadExpireAge = config->readNumEntry("UnreadExpireAge", 12);
   unreadExpireUnits = (ExpireUnits)config->readNumEntry("UnreadExpireUnits", expireNever);
-	setUserWhoField( config->readEntry("WhoField") );
+  setUserWhoField( config->readEntry("WhoField") );
 }
 
 //-----------------------------------------------------------------------------
@@ -1249,6 +1321,7 @@ void KMFolder::writeConfig()
   KConfig* config = kapp->config();
   KConfigGroupSaver saver(config, "Folder-" + idString());
   config->writeEntry("UnreadMsgs", mUnreadMsgs);
+  config->writeEntry("TotalMsgs", mTotalMsgs);
   config->writeEntry("MailingListEnabled", mMailingListEnabled);
   config->writeEntry("MailingListPostingAddress", mMailingListPostingAddress);
   config->writeEntry("MailingListAdminAddress", mMailingListAdminAddress);
@@ -1259,6 +1332,9 @@ void KMFolder::writeConfig()
   config->writeEntry("ReadExpireUnits", readExpireUnits);
   config->writeEntry("UnreadExpireAge", unreadExpireAge);
   config->writeEntry("UnreadExpireUnits", unreadExpireUnits);
+  config->writeEntry("UseCustomIcons", mUseCustomIcons);
+  config->writeEntry("NormalIconPath", mNormalIconPath);
+  config->writeEntry("UnreadIconPath", mUnreadIconPath);
 	config->writeEntry("WhoField", mUserWhoField);
 }
 
@@ -1278,18 +1354,20 @@ bool KMFolder::updateIndexStreamPtr(bool)
 {
   // We touch the folder, otherwise the index is regenerated, if KMail is
   // running, while the clock switches from daylight savings time to normal time
-  KProcess p, q;
+  KProcess p, q, r;
   p << "touch" << location();
   p.start(KProcess::Block);
   q << "touch" << indexLocation();
   q.start(KProcess::Block);
+  r << "touch" << kernel->msgDict()->getFolderIdsLocation( this );
+  r.start(KProcess::Block);
 
   mIndexSwapByteOrder = false;
 #ifdef HAVE_MMAP
     if(just_close) {
 	if(mIndexStreamPtr)
 	    munmap(mIndexStreamPtr, mIndexStreamPtrLength);
-	mIndexStreamPtr = NULL;
+	mIndexStreamPtr = 0;
 	mIndexStreamPtrLength = 0;
 	return TRUE;
     }
@@ -1299,7 +1377,7 @@ bool KMFolder::updateIndexStreamPtr(bool)
     if(fstat(fileno(mIndexStream), &stat_buf) == -1) {
 	if(mIndexStreamPtr)
 	    munmap(mIndexStreamPtr, mIndexStreamPtrLength);
-	mIndexStreamPtr = NULL;
+	mIndexStreamPtr = 0;
 	mIndexStreamPtrLength = 0;
 	return FALSE;
     }
@@ -1309,7 +1387,7 @@ bool KMFolder::updateIndexStreamPtr(bool)
     mIndexStreamPtr = (uchar *)mmap(0, mIndexStreamPtrLength, PROT_READ, MAP_SHARED,
 				    fileno(mIndexStream), 0);
     if(mIndexStreamPtr == MAP_FAILED) {
-	mIndexStreamPtr = NULL;
+	mIndexStreamPtr = 0;
 	mIndexStreamPtrLength = 0;
 	return FALSE;
     }
@@ -1367,7 +1445,7 @@ void KMFolder::setRDict(KMMsgDictREntry *rentry) {
   if (rentry == mRDict)
 	return;
   KMMsgDict::deleteRentry(mRDict);
-  mRDict = rentry; 
+  mRDict = rentry;
 }
 
 //-----------------------------------------------------------------------------
@@ -1376,7 +1454,7 @@ void KMFolder::setStatus(QValueList<int>& ids, KMMsgStatus status)
   for ( QValueList<int>::Iterator it = ids.begin(); it != ids.end(); ++it )
   {
     KMFolder::setStatus(*it, status);
-  }  
+  }
 }
 
 //-----------------------------------------------------------------------------
@@ -1399,12 +1477,11 @@ void KMFolder::setUserWhoField(const QString &whoField)
   if ( whoField.isEmpty() )
   {
     // default setting
-    QString ident = mIdentity; if (ident.isEmpty()) ident = i18n("Default");
-    KMIdentity identity (ident);
-    identity.readConfig();
+    const KMIdentity & identity =
+      kernel->identityManager()->identityForUoidOrDefault( mIdentity );
 
-    if ( mIsSystemFolder && protocol() != "imap" ) 
-    {	
+    if ( mIsSystemFolder && protocol() != "imap" )
+    {
       // local system folders
       if ( this == kernel->inboxFolder() || this == kernel->trashFolder() ) mWhoField = "From";
       if ( this == kernel->outboxFolder() || this == kernel->sentFolder() || this == kernel->draftsFolder() ) mWhoField = "To";
@@ -1412,9 +1489,9 @@ void KMFolder::setUserWhoField(const QString &whoField)
     } else if ( identity.drafts() == idString() || identity.fcc() == idString() ) {
       // drafts or sent of the identity
       mWhoField = "To";
-    } else { 
+    } else {
       mWhoField = "From";
-    }	
+    }
 
   } else if ( whoField == "From" || whoField == "To" ) {
 
