@@ -1,5 +1,6 @@
 // kmfolderdia.cpp
 
+#include <config.h>
 #include <assert.h>
 
 #include <qcheckbox.h>
@@ -14,11 +15,14 @@
 #include <knuminput.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
+#include <kicondialog.h>
+#include <kiconloader.h>
 
 #include "kmmainwin.h"
 #include "kmacctfolder.h"
 #include "kmfoldermgr.h"
-#include "kmidentity.h"
+#include "identitycombo.h"
+#include "kmkernel.h"
 #include "kmfolderimap.h"
 #include "kmheaders.h"
 
@@ -53,10 +57,10 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
   hl->addWidget( label );
 
   nameEdit = new QLineEdit( fpGroup );
-  nameEdit->setFocus();
+  if( !folder )
+    nameEdit->setFocus();
   nameEdit->setText(folder ? folder->label() : i18n("unnamed"));
   nameEdit->setMinimumSize(nameEdit->sizeHint());
-  nameEdit->selectAll();
   label->setBuddy( nameEdit );
   hl->addWidget( nameEdit );
 
@@ -67,6 +71,75 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
   hl->addWidget( fileInFolder );
   label2->setBuddy( fileInFolder );
 
+  //start icons group
+  QString normalIcon = (folder ? folder->normalIconPath() : "");
+  QString unreadIcon = (folder ? folder->unreadIconPath() : "");
+  QGroupBox *iconGroup = new QGroupBox( i18n("Folder Icons"), page, "iconGroup" );
+  iconGroup->setColumnLayout( 0,  Qt::Vertical );
+
+  topLayout->addWidget( iconGroup );
+
+  QVBoxLayout *ivl = new QVBoxLayout( iconGroup->layout() );
+  ivl->setSpacing( 6 );
+
+  QHBoxLayout *ihl = new QHBoxLayout( ivl );
+  mIconsCheckBox = new QCheckBox( i18n("Use custom &icons"), iconGroup );
+  mIconsCheckBox->setChecked( folder ? folder->useCustomIcons() : false );
+  ihl->addWidget( mIconsCheckBox );
+
+  ihl = new QHBoxLayout( ivl );
+  QLabel *ilabel = new QLabel( i18n("&Normal:"), iconGroup );
+  ihl->addWidget( ilabel );
+
+  mNormalIconButton = new KIconButton( iconGroup );
+  ilabel->setBuddy( mNormalIconButton );
+  mNormalIconButton->setIconType( KIcon::NoGroup , KIcon::Any, true );
+  mNormalIconButton->setIconSize( 16 );
+  mNormalIconButton->setStrictIconSize( true );
+  mNormalIconButton->setFixedSize( 28, 28 );
+  if ( folder ) {
+    mNormalIconButton->setIcon( (!normalIcon.isEmpty())?normalIcon:"folder" );
+  } 
+  ihl->addWidget( mNormalIconButton );
+  ihl->addStretch( 1 );
+  
+  QLabel *ilabel2 = new QLabel( i18n("&Unread:"), iconGroup );
+  ihl->addWidget( ilabel2 );
+
+  mUnreadIconButton = new KIconButton( iconGroup );
+  ilabel2->setBuddy( mUnreadIconButton );
+  mUnreadIconButton->setIconType( KIcon::NoGroup, KIcon::Any, true );
+  mUnreadIconButton->setIconSize( 16 );
+  mUnreadIconButton->setStrictIconSize( true );
+  mUnreadIconButton->setFixedSize( 28, 28 );
+  if ( folder ) {
+    mUnreadIconButton->setIcon( (!unreadIcon.isEmpty())?unreadIcon:"folder_open" );
+  }
+  ihl->addWidget( mUnreadIconButton );
+  ihl->addStretch( 1 );
+  
+  if ( !mIconsCheckBox->isChecked() ) {
+    mNormalIconButton->setEnabled( false );
+    mUnreadIconButton->setEnabled( false );
+    ilabel->setEnabled( false );
+    ilabel2->setEnabled( false );
+    
+  }
+
+  connect( mIconsCheckBox, SIGNAL(toggled(bool)),
+	   mNormalIconButton, SLOT(setEnabled(bool)) );
+  connect( mIconsCheckBox, SIGNAL(toggled(bool)),
+	   mUnreadIconButton, SLOT(setEnabled(bool)) );
+  connect( mIconsCheckBox, SIGNAL(toggled(bool)),
+	   ilabel, SLOT(setEnabled(bool)) );
+  connect( mIconsCheckBox, SIGNAL(toggled(bool)),
+	   ilabel2, SLOT(setEnabled(bool)) );
+
+  connect( mNormalIconButton, SIGNAL(iconChanged(QString)),
+	   this, SLOT(slotChangeIcon(QString)) );
+
+  //end icons group
+  
   mtGroup = new QGroupBox( i18n("Folder Type"), page, "mtGroup" );
   mtGroup->setColumnLayout( 0,  Qt::Vertical );
 
@@ -125,8 +198,6 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
   QGridLayout *mlLayout = new QGridLayout(mlGroup->layout());
   mlLayout->setSpacing( 6 );
   holdsMailingList = new QCheckBox( i18n("&Folder holds a mailing list"), mlGroup);
-  QObject::connect( holdsMailingList, SIGNAL(toggled(bool)),
-                    SLOT(slotHoldsML(bool)) );
   topLayout->addWidget( mlGroup );
   mlLayout->addMultiCellWidget(holdsMailingList, 0, 0, 0, 1);
 
@@ -143,6 +214,9 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
   mlLayout->addWidget( mailingListPostAddress, 1, 1 );
   mailingListPostAddress->setEnabled(false);
 
+  connect( holdsMailingList, SIGNAL(toggled(bool)),
+	   mailingListPostAddress, SLOT(setEnabled(bool)) );
+
   //
   // Expiry data.
   //
@@ -158,7 +232,7 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
   expLayout->addMultiCellWidget(expireFolder, 0, 0, 0, 1);
 
   // Expiry time for read documents.
-  label = new QLabel(i18n("Expire &read email after"), expGroup);
+  label = new QLabel(i18n("Expire &read email after:"), expGroup);
   label->setEnabled(false);
   QObject::connect( expireFolder, SIGNAL(toggled(bool)),
 		    label, SLOT(setEnabled(bool)) );
@@ -170,16 +244,15 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
 
   readExpiryUnits = new QComboBox(expGroup);
   readExpiryUnits->insertItem(i18n("Never"));
-  readExpiryUnits->insertItem(i18n("day(s)"));
-  readExpiryUnits->insertItem(i18n("week(s)"));
-  readExpiryUnits->insertItem(i18n("month(s)"));
+  readExpiryUnits->insertItem(i18n("Day(s)"));
+  readExpiryUnits->insertItem(i18n("Week(s)"));
+  readExpiryUnits->insertItem(i18n("Month(s)"));
   expLayout->addWidget(readExpiryUnits, 1, 2);
   connect( readExpiryUnits, SIGNAL( activated( int ) ),
            this, SLOT( slotReadExpiryUnitChanged( int ) ) );
 
-
   // Expiry time for unread documents.
-  label = new QLabel(i18n("Expire &unread email after"), expGroup);
+  label = new QLabel(i18n("Expire unr&ead email after:"), expGroup);
   label->setEnabled(false);
   QObject::connect( expireFolder, SIGNAL(toggled(bool)),
 		    label, SLOT(setEnabled(bool)) );
@@ -191,14 +264,13 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
 
   unreadExpiryUnits = new QComboBox(expGroup);
   unreadExpiryUnits->insertItem(i18n("Never"));
-  unreadExpiryUnits->insertItem(i18n("day(s)"));
-  unreadExpiryUnits->insertItem(i18n("week(s)"));
-  unreadExpiryUnits->insertItem(i18n("month(s)"));
+  unreadExpiryUnits->insertItem(i18n("Day(s)"));
+  unreadExpiryUnits->insertItem(i18n("Week(s)"));
+  unreadExpiryUnits->insertItem(i18n("Month(s)"));
   expLayout->addWidget(unreadExpiryUnits, 2, 2);
   connect( unreadExpiryUnits, SIGNAL( activated( int ) ),
            this, SLOT( slotUnreadExpiryUnitChanged( int ) ) );
 
-
   expLayout->setColStretch(0, 3);
   expLayout->setColStretch(0, 100);
 
@@ -211,8 +283,7 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
 
   label = new QLabel( i18n("&Sender:"), idGroup );
   idLayout->addWidget( label );
-  identity = new QComboBox( idGroup );
-  identity->insertStringList( KMIdentity::identities() );
+  identity = new IdentityCombo( idGroup );
   label->setBuddy( identity );
   idLayout->addWidget( identity, 3 );
 
@@ -282,11 +353,7 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
     else
       mailboxType->setCurrentItem(0);
 
-    for (int i=0; i < identity->count(); ++i)
-      if (identity->text(i) == folder->identity()) {
-         identity->setCurrentItem(i);
-         break;
-      }
+    identity->setCurrentIdentity( folder->identity() );
 
     // Set the status of widgets to represent the folder
     // properties for auto expiry of old email.
@@ -346,6 +413,7 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
   }
   else if (folder && folder->isSystemFolder()) {
     fpGroup->hide();
+    iconGroup->hide();
     mtGroup->hide();
     mlGroup->hide();
     idGroup->hide();
@@ -360,7 +428,7 @@ KMFolderDialog::KMFolderDialog(KMFolder* aFolder, KMFolderDir *aFolderDir,
 //-----------------------------------------------------------------------------
 void KMFolderDialog::slotUpdateItems ( int current )
 {
-  KMFolder* selectedFolder = NULL;
+  KMFolder* selectedFolder = 0;
   // check if the index is valid (the top level has no entrance in the mFolders)
   if (current > 0) selectedFolder = *mFolders.at(current - 1);
   if (selectedFolder && selectedFolder->protocol() == "imap")
@@ -384,7 +452,7 @@ void KMFolderDialog::slotOk()
     QString acctName;
     QString fldName, oldFldName;
     KMFolderDir *selectedFolderDir = &(kernel->folderMgr()->dir());
-    KMFolder *selectedFolder = NULL;
+    KMFolder *selectedFolder = 0;
     int curFolder = fileInFolder->currentItem();
 
     if (folder) oldFldName = folder->name();
@@ -409,7 +477,7 @@ void KMFolderDialog::slotOk()
       return;
     }
 
-    message = i18n( "Cannot move a parent folder into a child folder." );
+    message = i18n( "Cannot move folder \"%1\" into a subfolder below itself." ).arg(fldName);
     KMFolderDir* folderDir = selectedFolderDir;
 
 
@@ -471,7 +539,7 @@ void KMFolderDialog::slotOk()
 //   folder->setMailingListAdminAddress( mailingListAdminAddress->text() );
     folder->setMailingListAdminAddress( QString::null );
 
-    folder->setIdentity( identity->currentText() );
+    folder->setIdentity( identity->currentIdentity() );
 // folder->setMarkAnyMessage( markAnyMessage->isChecked() );
 
     // Settings for auto expiry of old email messages.
@@ -480,6 +548,12 @@ void KMFolderDialog::slotOk()
     folder->setReadExpireAge(readExpiryTime->value());
     folder->setUnreadExpireUnits((ExpireUnits)unreadExpiryUnits->currentItem());
     folder->setReadExpireUnits((ExpireUnits)readExpiryUnits->currentItem());
+    //update the tree iff new icon paths are different and not empty
+    folder->setUseCustomIcons( mIconsCheckBox->isChecked() );
+    if ( (( mNormalIconButton->icon() != folder->normalIconPath() ) && ( !mNormalIconButton->icon().isEmpty())) || 
+	 (( mUnreadIconButton->icon() != folder->unreadIconPath() ) && ( !mUnreadIconButton->icon().isEmpty())) ) {
+      folder->setIconPaths( mNormalIconButton->icon(), mUnreadIconButton->icon() );
+    } 
 
     // set whoField
     if (senderType->currentItem() == 1)
@@ -496,22 +570,6 @@ void KMFolderDialog::slotOk()
   KDialogBase::slotOk();
 }
 
-//-----------------------------------------------------------------------------
-void KMFolderDialog::slotHoldsML( bool holdsML )
-{
-  if ( holdsML )
-  {
-    mailingListPostAddress->setEnabled(true);
-//     mailingListAdminAddress->setEnabled(true);
-  }
-  else
-  {
-    mailingListPostAddress->setEnabled(false);
-//     mailingListAdminAddress->setEnabled(false);
-  }
-
-//  mailingListIdentity->setEnabled(holdsML);
-}
 
 /**
  * Called when the 'auto expire' toggle is clicked.
@@ -555,3 +613,11 @@ KMFolderDialog::slotUnreadExpiryUnitChanged( int value )
   // disable the number field if "Never" is selected
   unreadExpiryTime->setEnabled( value != 0 );
 }
+
+
+void 
+KMFolderDialog::slotChangeIcon( QString icon )
+{
+  if ( mFolder && !mFolder->unreadIcon() ) 
+    mUnreadIconButton->setIcon( icon );
+}
