@@ -43,7 +43,7 @@ KMFolderImap::KMFolderImap(KMFolderDir* aParent, const QString& aName)
 {
   mContentState = imapNoInformation;
   mSubfolderState = imapNoInformation;
-  mAccount = NULL;
+  mAccount = 0;
   mIsSelected = FALSE;
   mLastUid = 0;
   mCheckFlags = TRUE;
@@ -122,12 +122,13 @@ void KMFolderImap::slotRemoveFolderResult(KIO::Job *job)
   if (it == mAccount->mapJobData.end()) return;
   if (job->error())
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) mAccount->slaveDied();
+    mAccount->slotSlaveError( mAccount->slave(), job->error(),
+        job->errorText() );
+  } else {
+    if (mAccount->slave()) mAccount->mapJobData.remove(it);
+    mAccount->displayProgress();
+    kernel->imapFolderMgr()->remove(this);
   }
-  if (mAccount->slave()) mAccount->mapJobData.remove(it);
-  mAccount->displayProgress();
-  if (!job->error()) kernel->imapFolderMgr()->remove(this);
 }
 
 //-----------------------------------------------------------------------------
@@ -155,7 +156,7 @@ void KMFolderImap::removeMsg(QPtrList<KMMessage> msgList, bool quiet)
   if (!quiet)
     deleteMessage(msgList);
 
-  mLastUid = 0; 
+  mLastUid = 0;
   KMFolderImapInherited::removeMsg(msgList);
 }
 
@@ -164,24 +165,41 @@ void KMFolderImap::removeMsg(QPtrList<KMMessage> msgList, bool quiet)
 void KMFolderImap::addMsgQuiet(KMMessage* aMsg)
 {
   KMFolder *folder = aMsg->parent();
-  if (folder) kernel->undoStack()->pushAction( aMsg->getMsgSerNum(), folder, this );
-  if (folder) folder->take(folder->find(aMsg));
+  if (folder) 
+  {
+    if (aMsg->enableUndo())
+    {
+      kernel->undoStack()->pushAction( aMsg->getMsgSerNum(), folder, this );
+    } else {
+      aMsg->setEnableUndo(true);
+    }
+    folder->take(folder->find(aMsg));
+  }
   delete aMsg;
-  if (mIsSelected) getFolder();
+  aMsg = 0;
+  getFolder();
 }
 
 //-----------------------------------------------------------------------------
 void KMFolderImap::addMsgQuiet(QPtrList<KMMessage> msgList)
 {
   KMFolder *folder = msgList.first()->parent();
-  for ( KMMessage* msg = msgList.first(); msg; msg = msgList.next() )
+  if (folder)
   {
-    kernel->undoStack()->pushAction( msg->getMsgSerNum(), folder, this );
+    for ( KMMessage* msg = msgList.first(); msg; msg = msgList.next() )
+    {
+      if (msg->enableUndo())
+      {
+        kernel->undoStack()->pushAction( msg->getMsgSerNum(), folder, this );
+      } else {
+        msg->setEnableUndo(true);
+      }
+    }
+    folder->take(msgList);
   }
-  if (folder) folder->take(msgList);
   msgList.setAutoDelete(true);
   msgList.clear();
-  if (mIsSelected) getFolder();
+  getFolder();
 }
 
 //-----------------------------------------------------------------------------
@@ -193,6 +211,7 @@ int KMFolderImap::addMsg(KMMessage* aMsg, int* aIndex_ret)
 
 int KMFolderImap::addMsg(QPtrList<KMMessage>& msgList, int* aIndex_ret)
 {
+  mAccount->tempOpenFolder(this);
   KMMessage *aMsg = msgList.getFirst();
   KMFolder *msgParent = aMsg->parent();
 
@@ -200,14 +219,15 @@ int KMFolderImap::addMsg(QPtrList<KMMessage>& msgList, int* aIndex_ret)
   for ( KMMessage* msg = msgList.first(); msg; msg = msgList.next() )
     msg->setTransferInProgress(true);
 
-  KMImapJob *imapJob = NULL;
+  KMImapJob *imapJob = 0;
   if (msgParent)
   {
+    mAccount->tempOpenFolder(msgParent);
     if (msgParent->protocol() == "imap")
     {
       if (static_cast<KMFolderImap*>(msgParent)->account() == account())
       {
-        if (this == msgParent) 
+        if (this == msgParent)
         {
           // transfer the whole message, e.g. a draft-message is canceled and re-added to the drafts-folder
           for ( KMMessage* msg = msgList.first(); msg; msg = msgList.next() )
@@ -219,7 +239,7 @@ int KMFolderImap::addMsg(QPtrList<KMMessage>& msgList, int* aIndex_ret)
             connect(imapJob, SIGNAL(messageCopied(KMMessage*)),
                 SLOT(addMsgQuiet(KMMessage*)));
           }
-          
+
         } else {
 
           /* get the messages and the uids
@@ -235,7 +255,7 @@ int KMFolderImap::addMsg(QPtrList<KMMessage>& msgList, int* aIndex_ret)
             // we need the messages that belong to the current set to pass them to the ImapJob
             QPtrList<KMMessage> temp_msgs = splitMessageList(*it, msgList);
 
-            imapJob = new KMImapJob(temp_msgs, *it, KMImapJob::tCopyMessage, this);
+            imapJob = new KMImapJob(temp_msgs, *it, KMImapJob::tMoveMessage, this);
             connect(imapJob, SIGNAL(messageCopied(QPtrList<KMMessage>)),
                 SLOT(addMsgQuiet(QPtrList<KMMessage>)));
           }
@@ -243,7 +263,7 @@ int KMFolderImap::addMsg(QPtrList<KMMessage>& msgList, int* aIndex_ret)
         if (aIndex_ret) *aIndex_ret = -1;
         return 0;
       }
-      else 
+      else
       {
         // different account, check if messages can be added
         QPtrListIterator<KMMessage> it( msgList );
@@ -271,7 +291,7 @@ int KMFolderImap::addMsg(QPtrList<KMMessage>& msgList, int* aIndex_ret)
     connect(imapJob, SIGNAL(messageStored(KMMessage*)),
         SLOT(addMsgQuiet(KMMessage*)));
   }
-  
+
   if (aIndex_ret) *aIndex_ret = -1;
   return 0;
 }
@@ -279,6 +299,13 @@ int KMFolderImap::addMsg(QPtrList<KMMessage>& msgList, int* aIndex_ret)
 //-----------------------------------------------------------------------------
 void KMFolderImap::copyMsg(QPtrList<KMMessage>& msgList)
 {
+  mAccount->tempOpenFolder(this);
+  KMMessage *aMsg = msgList.getFirst();
+  if (aMsg)
+  {
+    KMFolder* parent = aMsg->parent();
+    if (parent) mAccount->tempOpenFolder(parent);
+  }
   QValueList<int> uids;
   getUids(msgList, uids);
   QStringList sets = makeSets(uids, false);
@@ -294,8 +321,8 @@ void KMFolderImap::copyMsg(QPtrList<KMMessage>& msgList)
 //-----------------------------------------------------------------------------
 QPtrList<KMMessage> KMFolderImap::splitMessageList(QString set, QPtrList<KMMessage>& msgList)
 {
-  int lastcomma = set.findRev(","); 
-  int lastdub = set.findRev(":");
+  int lastcomma = set.findRev(',');
+  int lastdub = set.findRev(':');
   int last = 0;
   if (lastdub > lastcomma) last = lastdub;
   else last = lastcomma;
@@ -308,8 +335,8 @@ QPtrList<KMMessage> KMFolderImap::splitMessageList(QString set, QPtrList<KMMessa
   if (!last_uid.isEmpty())
   {
     QPtrListIterator<KMMessage> it( msgList );
-    KMMessage* msg = NULL;
-    while ( (msg = it.current()) != NULL )
+    KMMessage* msg = 0;
+    while ( (msg = it.current()) != 0 )
     {
       // append the msg to the new list and delete it from the old
       temp_msgs.append(msg);
@@ -327,12 +354,12 @@ QPtrList<KMMessage> KMFolderImap::splitMessageList(QString set, QPtrList<KMMessa
 
   return temp_msgs;
 }
-            
+
 //-----------------------------------------------------------------------------
 KMMessage* KMFolderImap::take(int idx)
 {
   KMMsgBase* mb(mMsgList[idx]);
-  if (!mb) return NULL;
+  if (!mb) return 0;
   if (!mb->isMessage()) readMsg(idx);
 
   KMMessage *msg = static_cast<KMMessage*>(mb);
@@ -349,14 +376,14 @@ void KMFolderImap::take(QPtrList<KMMessage> msgList)
   mLastUid = 0;
   KMFolderImapInherited::take(msgList);
 }
-  
+
 
 //-----------------------------------------------------------------------------
 // Originally from KMAcctImap
 //-----------------------------------------------------------------------------
 
 //-----------------------------------------------------------------------------
-void KMFolderImap::listDirectory(KMFolderTreeItem * fti, bool secondStep)
+bool KMFolderImap::listDirectory(bool secondStep)
 {
   mSubfolderState = imapInProgress;
   KMAcctImap::jobData jd;
@@ -368,10 +395,8 @@ void KMFolderImap::listDirectory(KMFolderTreeItem * fti, bool secondStep)
   url.setPath(((jd.inboxOnly) ? QString("/") : imapPath())
     + ";TYPE=" + ((mAccount->onlySubscribedFolders()) ? "LSUB" : "LIST"));
   if (!mAccount->makeConnection())
-  { 
-    if (fti) fti->setOpen( FALSE );
-    return;
-  }
+    return FALSE;
+
   if (!secondStep) mHasInbox = FALSE;
   mSubfolderNames.clear();
   mSubfolderPaths.clear();
@@ -384,6 +409,8 @@ void KMFolderImap::listDirectory(KMFolderTreeItem * fti, bool secondStep)
   connect(job, SIGNAL(entries(KIO::Job *, const KIO::UDSEntryList &)),
           this, SLOT(slotListEntries(KIO::Job *, const KIO::UDSEntryList &)));
   mAccount->displayProgress();
+  
+  return TRUE;
 }
 
 
@@ -396,8 +423,8 @@ void KMFolderImap::slotListResult(KIO::Job * job)
   assert(it != mAccount->mapJobData.end());
   if (job->error())
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) mAccount->slaveDied();
+    mAccount->slotSlaveError( mAccount->slave(), job->error(),
+        job->errorText() );
   }
   mSubfolderState = imapFinished;
   bool it_inboxOnly = (*it).inboxOnly;
@@ -405,7 +432,7 @@ void KMFolderImap::slotListResult(KIO::Job * job)
   if (!job->error())
   {
     kernel->imapFolderMgr()->quiet(TRUE);
-    if (it_inboxOnly) listDirectory(NULL, TRUE);
+    if (it_inboxOnly) listDirectory(TRUE);
     else {
       if (mIsSystemFolder && mImapPath == "/INBOX/"
         && mAccount->prefix() == "/INBOX/")
@@ -421,7 +448,7 @@ void KMFolderImap::slotListResult(KIO::Job * job)
         if (!node->isDir() && (node->name() != "INBOX" || !mHasInbox)
             && mSubfolderNames.findIndex(node->name()) == -1)
         {
-kdDebug(5006) << node->name() << " disappeared." << endl;
+          kdDebug(5006) << node->name() << " disappeared." << endl;
           kernel->imapFolderMgr()->remove(static_cast<KMFolder*>(node));
           node = mChild->first();
         }
@@ -438,7 +465,7 @@ kdDebug(5006) << node->name() << " disappeared." << endl;
         folder->setImapPath("/INBOX/");
         folder->setLabel(i18n("inbox"));
         if (!node) folder->close();
-        folder->listDirectory(NULL);
+        folder->listDirectory();
         kernel->imapFolderMgr()->contentsChanged();
       }
       for (uint i = 0; i < mSubfolderNames.count(); i++)
@@ -464,7 +491,7 @@ kdDebug(5006) << node->name() << " disappeared." << endl;
           folder->setImapPath(mSubfolderPaths[i]);
           if (mSubfolderMimeTypes[i] == "message/directory" ||
               mSubfolderMimeTypes[i] == "inode/directory")
-            folder->listDirectory(NULL);
+            folder->listDirectory();
         }
       }
     }
@@ -481,7 +508,9 @@ void KMFolderImap::slotListEntries(KIO::Job * job, const KIO::UDSEntryList & uds
     mAccount->mapJobData.find(job);
   if (it == mAccount->mapJobData.end()) return;
   assert(it != mAccount->mapJobData.end());
-  QString name, url, mimeType;
+  QString name;
+  KURL url;
+  QString mimeType;
   for (KIO::UDSEntryList::ConstIterator udsIt = uds.begin();
     udsIt != uds.end(); udsIt++)
   {
@@ -492,7 +521,7 @@ void KMFolderImap::slotListEntries(KIO::Job * job, const KIO::UDSEntryList & uds
       if ((*eIt).m_uds == KIO::UDS_NAME)
         name = (*eIt).m_str;
       else if ((*eIt).m_uds == KIO::UDS_URL)
-        url = (*eIt).m_str;
+        url = KURL((*eIt).m_str, 106); // utf-8
       else if ((*eIt).m_uds == KIO::UDS_MIME_TYPE)
         mimeType = (*eIt).m_str;
     }
@@ -501,19 +530,19 @@ void KMFolderImap::slotListEntries(KIO::Job * job, const KIO::UDSEntryList & uds
         && name != ".." && (mAccount->hiddenFolders() || name.at(0) != '.')
         && (!(*it).inboxOnly || name == "INBOX"))
     {
-      if (((*it).inboxOnly || KURL(url).path() == "/INBOX/") && name == "INBOX")
+      if (((*it).inboxOnly || url.path() == "/INBOX/") && name == "INBOX")
         mHasInbox = TRUE;
       else {
         // Some servers send _lots_ of duplicates
         if (mSubfolderNames.findIndex(name) == -1)
         {
           mSubfolderNames.append(name);
-          mSubfolderPaths.append(KURL(url).path());
+          mSubfolderPaths.append(url.path());
           mSubfolderMimeTypes.append(mimeType);
         }
       }
 /*      static_cast<KMFolderTree*>((*it).parent->listView())
-        ->addImapChildFolder((*it).parent, name, KURL(url).path(),
+        ->addImapChildFolder((*it).parent, name, url.path(),
         mimeType, (*it).inboxOnly); */
     }
   }
@@ -572,10 +601,9 @@ kdDebug(5006) << "KMFolderImap::slotCheckValidityResult" << endl;
   if (it == mAccount->mapJobData.end()) return;
   if (job->error())
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) mAccount->slaveDied();
+    mAccount->slotSlaveError( mAccount->slave(), job->error(),
+        job->errorText() );
     emit folderComplete(this, FALSE);
-    if (mAccount->slave()) mAccount->mapJobData.remove(it);
     mAccount->displayProgress();
   } else {
     QCString cstr((*it).data.data(), (*it).data.size() + 1);
@@ -591,8 +619,10 @@ kdDebug(5006) << "KMFolderImap::slotCheckValidityResult" << endl;
     QString startUid;
     if (uidValidity() != uidv)
     {
+      // uidValidity changed
       expunge();
       mLastUid = 0;
+      uidmap.clear();
     } else {
       if (!mCheckFlags)
         startUid = QString::number(lastUid() + 1);
@@ -604,7 +634,7 @@ kdDebug(5006) << "KMFolderImap::slotCheckValidityResult" << endl;
 
 
 //-----------------------------------------------------------------------------
-void KMFolderImap::getFolder()
+void KMFolderImap::getFolder(bool force)
 {
   mGuessedUnreadMsgs = -1;
   if (mNoContent)
@@ -614,6 +644,10 @@ void KMFolderImap::getFolder()
     return;
   }
   mContentState = imapInProgress;
+  if (force) {
+    // force an update
+    mCheckFlags = TRUE;
+  }
   checkValidity();
 }
 
@@ -670,10 +704,9 @@ void KMFolderImap::slotListFolderResult(KIO::Job * job)
   jd.done = 0;
   if (job->error())
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) mAccount->slaveDied();
+    mAccount->slotSlaveError( mAccount->slave(), job->error(),
+        job->errorText() );
     emit folderComplete(this, FALSE);
-    if (mAccount->slave()) mAccount->mapJobData.remove(it);
     return;
   }
   mCheckFlags = FALSE;
@@ -690,10 +723,10 @@ void KMFolderImap::slotListFolderResult(KIO::Job * job)
     {
       getMsgString(idx, cstr);
       a = cstr.find("X-UID: ");
-      b = cstr.find("\n", a);
+      b = cstr.find('\n', a);
       if (a == -1 || b == -1) mailUid = -1;
       else mailUid = cstr.mid(a + 7, b - a - 7).toLong();
-      c = (*uid).find(",");
+      c = (*uid).find(',');
       serverUid = (*uid).left(c).toLong();
       serverFlags = (*uid).mid(c+1).toInt();
       if (mailUid < serverUid) removeMsg(idx, TRUE);
@@ -709,7 +742,7 @@ void KMFolderImap::slotListFolderResult(KIO::Job * job)
     while (idx < count()) removeMsg(idx, TRUE);
   }
   for (uid = (*it).items.begin(); uid != (*it).items.end(); uid++)
-    (*uid).truncate((*uid).find(","));
+    (*uid).truncate((*uid).find(','));
 //jd.items = (*it).items;
   jd.total = (*it).items.count();
   uid = (*it).items.begin();
@@ -724,7 +757,7 @@ void KMFolderImap::slotListFolderResult(KIO::Job * job)
   }
 
   QStringList sets;
-  if (jd.total == 1) sets.append(*uid + ":" + *uid);
+  if (jd.total == 1) sets.append(*uid + ':' + *uid);
   else sets = makeSets( (*it).items );
 
   for (QStringList::Iterator i = sets.begin(); i != sets.end(); ++i)
@@ -746,7 +779,7 @@ void KMFolderImap::slotListFolderResult(KIO::Job * job)
         : SLOT(slotGetMessagesResult(KIO::Job *)));
     connect(newJob, SIGNAL(data(KIO::Job *, const QByteArray &)),
         this, SLOT(slotGetMessagesData(KIO::Job *, const QByteArray &)));
-  }	
+  }
   mAccount->mapJobData.remove(it);
 }
 
@@ -775,7 +808,7 @@ void KMFolderImap::slotListFolderEntries(KIO::Job * job,
         flags = (*eIt).m_long;
     }
     if (mimeType == "message/rfc822-imap" && !(flags & 8))
-      (*it).items.append(name + "," + QString::number(flags));
+      (*it).items.append(name + ',' + QString::number(flags));
   }
 }
 
@@ -839,10 +872,24 @@ void KMFolderImap::slotGetMessagesData(KIO::Job * job, const QByteArray & data)
     msg->fromString((*it).cdata.mid(16, pos - 16));
     flags = msg->headerField("X-Flags").toInt();
     ulong uid = msg->headerField("X-UID").toULong();
-    if (flags & 8 || uid <= lastUid()) delete msg;
+    if (flags & 8 || uid <= lastUid()) {
+      delete msg;
+      msg = 0;
+    }
     else {
       msg->setStatus(flagsToStatus(flags));
-      KMFolderImapInherited::addMsg(msg, NULL);
+      if (uidmap.find(uid))
+      {
+        // assign the sernum from the cache
+        const ulong sernum = (ulong) uidmap[uid];
+        kdDebug() << "set sernum:" << sernum << " for " << uid << endl;
+        msg->setMsgSerNum(sernum);
+        // delete the entry
+        uidmap.remove(uid);
+      }
+      open();
+      KMFolderImapInherited::addMsg(msg, 0);
+      close();
       if (count() > 1) unGetMsg(count() - 1);
       mLastUid = uid;
 /*      if ((*it).total > 20 &&
@@ -869,8 +916,8 @@ void KMFolderImap::getMessagesResult(KIO::Job * job, bool lastSet)
   assert(it != mAccount->mapJobData.end());
   if (job->error())
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) mAccount->slaveDied();
+    mAccount->slotSlaveError( mAccount->slave(), job->error(),
+        job->errorText() );
     mContentState = imapNoInformation;
     emit folderComplete(this, FALSE);
   } else if (lastSet) mContentState = imapFinished;
@@ -922,10 +969,10 @@ void KMFolderImap::slotCreateFolderResult(KIO::Job * job)
   if (it == mAccount->mapJobData.end()) return;
   if (job->error())
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) mAccount->slaveDied();
+    mAccount->slotSlaveError( mAccount->slave(), job->error(),
+        job->errorText() );
   } else {
-    listDirectory(NULL);
+    listDirectory();
   }
   if (mAccount->slave()) mAccount->mapJobData.remove(it);
   mAccount->displayProgress();
@@ -933,7 +980,7 @@ void KMFolderImap::slotCreateFolderResult(KIO::Job * job)
 
 
 //-----------------------------------------------------------------------------
-static QTextCodec *sUtf7Codec = NULL;
+static QTextCodec *sUtf7Codec = 0;
 
 QTextCodec * KMFolderImap::utf7Codec()
 {
@@ -957,6 +1004,67 @@ QString KMFolderImap::decodeFileName(const QString &name)
   return utf7Codec()->toUnicode(result.latin1());
 }
 
+//-----------------------------------------------------------------------------
+QValueList<int> KMFolderImap::splitSets(QString uids)
+{
+  QValueList<int> uidlist;
+
+  // ex: 1205,1204,1203,1202,1236:1238
+  QString buffer = QString::null;
+  int setstart = -1;
+  // iterate over the uids
+  for (uint i = 0; i < uids.length(); i++)
+  {
+    QChar chr = uids[i];
+    if (chr == ',')
+    {
+      if (setstart > -1)
+      {
+        // a range (uid:uid) was before
+        for (int j = setstart; j <= buffer.toInt(); j++)
+        {
+          uidlist.append(j);
+        }
+        setstart = -1;
+      } else {
+        // single uid
+        uidlist.append(buffer.toInt());
+      }
+      buffer = "";
+    } else if (chr == ':') {
+      // remember the start of the range
+      setstart = buffer.toInt();
+      buffer = "";
+    } else if (chr.category() == QChar::Number_DecimalDigit) {
+      // digit
+      buffer += chr;
+    } else {
+      // ignore
+    }
+  }
+  // process the last data
+  if (setstart > -1)
+  {
+    for (int j = setstart; j <= buffer.toInt(); j++)
+    {
+      uidlist.append(j);
+    }
+  } else {
+    uidlist.append(buffer.toInt());
+  }
+
+  return uidlist;
+}
+
+//-----------------------------------------------------------------------------
+bool KMFolderImap::autoExpunge()
+{
+  if (mAccount)
+    return mAccount->autoExpunge();
+
+  return false;
+}
+
 //-----------------------------------------------------------------------------
 KMImapJob::KMImapJob(KMMessage *msg, JobType jt, KMFolderImap* folder)
 {
@@ -966,13 +1074,16 @@ KMImapJob::KMImapJob(KMMessage *msg, JobType jt, KMFolderImap* folder)
 }
 
 //-----------------------------------------------------------------------------
-KMImapJob::KMImapJob(QPtrList<KMMessage>& msgList, QString sets, JobType jt, KMFolderImap* folder)
+KMImapJob::KMImapJob(QPtrList<KMMessage>& msgList, QString sets, 
+    JobType jt, KMFolderImap* folder)
 {
   mMsg = msgList.first();
   init(jt, sets, folder, msgList);
 }
 
-void KMImapJob::init(JobType jt, QString sets, KMFolderImap* folder, QPtrList<KMMessage>& msgList)
+//-----------------------------------------------------------------------------
+void KMImapJob::init(JobType jt, QString sets, KMFolderImap* folder, 
+    QPtrList<KMMessage>& msgList)
 {
   assert(jt == tGetMessage || folder);
   KMMessage* msg = msgList.first();
@@ -988,14 +1099,14 @@ void KMImapJob::init(JobType jt, QString sets, KMFolderImap* folder, QPtrList<KM
     url.setPath(folder->imapPath() + ";SECTION="
       + QString::fromLatin1(KMFolderImap::statusToFlags(msg->status())));
     KMAcctImap::jobData jd;
-    jd.parent = NULL; jd.offset = 0;
+    jd.parent = 0; jd.offset = 0;
     jd.total = 1; jd.done = 0;
     jd.msgList.append(msg);
     QCString cstr(msg->asString());
     int a = cstr.find("\nX-UID: ");
-    int b = cstr.find("\n", a);
+    int b = cstr.find('\n', a);
     if (a != -1 && b != -1 && cstr.find("\n\n") > a) cstr.remove(a, b-a);
-    mData.resize(cstr.length() + cstr.contains("\n"));
+    mData.resize(cstr.length() + cstr.contains('\n'));
     unsigned int i = 0;
     for (char *ch = cstr.data(); *ch; ch++)
     {
@@ -1017,38 +1128,43 @@ void KMImapJob::init(JobType jt, QString sets, KMFolderImap* folder, QPtrList<KM
         SLOT(slotPutMessageResult(KIO::Job *)));
     connect(mJob, SIGNAL(dataReq(KIO::Job *, QByteArray &)),
         SLOT(slotPutMessageDataReq(KIO::Job *, QByteArray &)));
+    connect(mJob, SIGNAL(infoMessage(KIO::Job *, const QString &)),
+        SLOT(slotPutMessageInfoData(KIO::Job *, const QString &)));    
     account->displayProgress();
   }
-  else if (jt == tCopyMessage)
+  else if (jt == tCopyMessage || jt == tMoveMessage)
   {
     KURL url = account->getUrl();
     KURL destUrl = account->getUrl();
     destUrl.setPath(folder->imapPath());
-    url.setPath(msg_parent->imapPath() + ";UID=" + sets); 
+    url.setPath(msg_parent->imapPath() + ";UID=" + sets);
     KMAcctImap::jobData jd;
-    jd.parent = NULL; mOffset = 0;
+    jd.parent = 0; mOffset = 0;
     jd.total = 1; jd.done = 0;
     jd.msgList = msgList;
-    QCString urlStr("C" + url.url().utf8());
-    QByteArray data;
-    QBuffer buff(data);
-    buff.open(IO_WriteOnly | IO_Append);
-    buff.writeBlock(urlStr.data(), urlStr.size());
-    urlStr = destUrl.url().utf8();
-    buff.writeBlock(urlStr.data(), urlStr.size());
-    buff.close();
+
+    QByteArray packedArgs;
+    QDataStream stream( packedArgs, IO_WriteOnly);
+
+    stream << (int) 'C' << url << destUrl;
+
     if (!account->makeConnection())
     {
       account->mJobList.remove(this);
       delete this;
       return;
     }
-    KIO::SimpleJob *simpleJob = KIO::special(url, data, FALSE);
+    KIO::SimpleJob *simpleJob = KIO::special(url, packedArgs, FALSE);
     KIO::Scheduler::assignJobToSlave(account->slave(), simpleJob);
     mJob = simpleJob;
     account->mapJobData.insert(mJob, jd);
     connect(mJob, SIGNAL(result(KIO::Job *)),
         SLOT(slotCopyMessageResult(KIO::Job *)));
+    if (jt == tMoveMessage)
+    {
+      connect(mJob, SIGNAL(infoMessage(KIO::Job *, const QString &)),
+          SLOT(slotCopyMessageInfoData(KIO::Job *, const QString &)));    
+    }
     account->displayProgress();
   } else {
     slotGetNextMessage();
@@ -1072,6 +1188,7 @@ KMImapJob::~KMImapJob()
         msg->setTransferInProgress(false);
     }
   }
+  emit finished();
 }
 
 
@@ -1091,7 +1208,7 @@ void KMImapJob::slotGetNextMessage()
   url.setPath(msgParent->imapPath() + ";UID="
     + mMsg->headerField("X-UID"));
   KMAcctImap::jobData jd;
-  jd.parent = NULL;
+  jd.parent = 0;
   jd.total = 1; jd.done = 0;
   if (!account->makeConnection())
   {
@@ -1114,23 +1231,32 @@ void KMImapJob::slotGetNextMessage()
 //-----------------------------------------------------------------------------
 void KMImapJob::slotGetMessageResult(KIO::Job * job)
 {
+  if( !mMsg || !mMsg->parent() || !job )
+    return;
   KMAcctImap *account = static_cast<KMFolderImap*>(mMsg->parent())->account();
+  if (!account) return;
   QMap<KIO::Job *, KMAcctImap::jobData>::Iterator it =
     account->mapJobData.find(job);
   if (it == account->mapJobData.end()) return;
   if (job->error())
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) account->slaveDied();
+    account->slotSlaveError( account->slave(), job->error(),
+        job->errorText() );
+    return;
   } else {
-    QString uid = mMsg->headerField("X-UID");
-    (*it).data.resize((*it).data.size() + 1);
-    (*it).data[(*it).data.size() - 1] = '\0';
-    mMsg->fromString(QCString((*it).data));
-    mMsg->setHeaderField("X-UID",uid);
-    mMsg->setComplete( TRUE );
-    emit messageRetrieved(mMsg);
-    mMsg = NULL;
+    if ((*it).data.size() > 0)
+    {
+      QString uid = mMsg->headerField("X-UID");
+      (*it).data.resize((*it).data.size() + 1);
+      (*it).data[(*it).data.size() - 1] = '\0';
+      mMsg->fromString(QCString((*it).data));
+      mMsg->setHeaderField("X-UID",uid);
+      mMsg->setComplete( TRUE );
+      emit messageRetrieved(mMsg);
+    } else {
+      emit messageRetrieved(0);
+    }
+    mMsg = 0;
   }
   if (account->slave()) account->mapJobData.remove(it);
   account->displayProgress();
@@ -1169,8 +1295,9 @@ void KMImapJob::slotPutMessageResult(KIO::Job *job)
   if (it == account->mapJobData.end()) return;
   if (job->error())
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) account->slaveDied();
+    account->slotSlaveError( account->slave(), job->error(),
+        job->errorText() );
+    return;
   } else {
     if ( !(*it).msgList.isEmpty() )
     {
@@ -1180,7 +1307,7 @@ void KMImapJob::slotPutMessageResult(KIO::Job *job)
     {
       emit messageStored(mMsg);
     }
-    mMsg = NULL;
+    mMsg = 0;
   }
   if (account->slave()) account->mapJobData.remove(it);
   account->displayProgress();
@@ -1188,6 +1315,30 @@ void KMImapJob::slotPutMessageResult(KIO::Job *job)
   delete this;
 }
 
+//-----------------------------------------------------------------------------
+void KMImapJob::slotPutMessageInfoData(KIO::Job *job, const QString &data)
+{
+  KMAcctImap *account = mDestFolder->account();
+  QMap<KIO::Job *, KMAcctImap::jobData>::Iterator it =
+    account->mapJobData.find(job);
+  if (it == account->mapJobData.end()) return;
+  if (data.find("UID") != -1)
+  {
+    int uid = (data.right(data.length()-4)).toInt();
+
+    if ( !(*it).msgList.isEmpty() )
+    {
+      const ulong * sernum = (ulong *)(*it).msgList.last()->getMsgSerNum();
+      kdDebug() << "insert sernum " << (*it).msgList.last()->getMsgSerNum() << " for " << uid << endl;
+      mDestFolder->insertUidSerNumEntry(uid, sernum);
+    } else if (mMsg)
+    {
+      const ulong * sernum = (ulong *)mMsg->getMsgSerNum();
+      kdDebug() << "insert sernum " << mMsg->getMsgSerNum() << " for " << uid << endl;
+      mDestFolder->insertUidSerNumEntry(uid, sernum);
+    }
+  }
+}
 
 //-----------------------------------------------------------------------------
 void KMImapJob::slotCopyMessageResult(KIO::Job *job)
@@ -1198,15 +1349,16 @@ void KMImapJob::slotCopyMessageResult(KIO::Job *job)
   if (it == account->mapJobData.end()) return;
   if (job->error())
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) account->slaveDied();
+    account->slotSlaveError( account->slave(), job->error(),
+        job->errorText() );
+    return;
   } else {
     if ( !(*it).msgList.isEmpty() )
     {
       emit messageCopied((*it).msgList);
     } else if (mMsg) {
       emit messageCopied(mMsg);
-      mMsg = NULL;
+      mMsg = 0;
     }
   }
   if (account->slave()) account->mapJobData.remove(it);
@@ -1215,6 +1367,51 @@ void KMImapJob::slotCopyMessageResult(KIO::Job *job)
   delete this;
 }
 
+//-----------------------------------------------------------------------------
+void KMImapJob::slotCopyMessageInfoData(KIO::Job * job, const QString & data)
+{
+  KMAcctImap *account = mDestFolder->account();
+  QMap<KIO::Job *, KMAcctImap::jobData>::Iterator it =
+    account->mapJobData.find(job);
+  if (it == account->mapJobData.end()) return;
+
+  if (data.find("UID") != -1)
+  {
+    // split
+    QString oldUid = data.section(' ', 1, 1);
+    QString newUid = data.section(' ', 2, 2);
+
+    // get lists of uids
+    QValueList<int> olduids = KMFolderImap::splitSets(oldUid);
+    QValueList<int> newuids = KMFolderImap::splitSets(newUid);
+
+    int index = -1;
+    if ( !(*it).msgList.isEmpty() )
+    {
+      KMMessage * msg;
+      for ( msg = (*it).msgList.first(); msg; msg = (*it).msgList.next() )
+      {
+        uint uid = msg->headerField("X-UID").toInt();
+        index = olduids.findIndex(uid);
+        if (index > -1)
+        {
+          // found, get the new uid
+          const ulong * sernum = (ulong *)msg->getMsgSerNum();
+          mDestFolder->insertUidSerNumEntry(newuids[index], sernum);
+        }
+      }
+    } else if (mMsg) {
+      uint uid = mMsg->headerField("X-UID").toInt();
+      index = olduids.findIndex(uid);
+      if (index > -1)
+      {
+        // found, get the new uid
+        const ulong * sernum = (ulong *)mMsg->getMsgSerNum();
+        mDestFolder->insertUidSerNumEntry(newuids[index], sernum);
+      }
+    }
+  }
+}
 
 //-----------------------------------------------------------------------------
 void KMFolderImap::slotSimpleData(KIO::Job * job, const QByteArray & data)
@@ -1261,7 +1458,7 @@ void KMFolderImap::deleteMessage(KMMessage * msg)
   KIO::SimpleJob *job = KIO::file_delete(url, FALSE);
   KIO::Scheduler::assignJobToSlave(mAccount->slave(), job);
   KMAcctImap::jobData jd;
-  jd.total = 1; jd.done = 0; jd.parent = NULL; jd.quiet = FALSE;
+  jd.total = 1; jd.done = 0; jd.parent = 0; jd.quiet = FALSE;
   mAccount->mapJobData.insert(job, jd);
   connect(job, SIGNAL(result(KIO::Job *)),
           mAccount, SLOT(slotSimpleResult(KIO::Job *)));
@@ -1283,7 +1480,7 @@ void KMFolderImap::deleteMessage(QPtrList<KMMessage> msgList)
     KIO::SimpleJob *job = KIO::file_delete(url, FALSE);
     KIO::Scheduler::assignJobToSlave(mAccount->slave(), job);
     KMAcctImap::jobData jd;
-    jd.total = 1; jd.done = 0; jd.parent = NULL; jd.quiet = FALSE;
+    jd.total = 1; jd.done = 0; jd.parent = 0; jd.quiet = FALSE;
     mAccount->mapJobData.insert(job, jd);
     connect(job, SIGNAL(result(KIO::Job *)),
         mAccount, SLOT(slotSimpleResult(KIO::Job *)));
@@ -1303,7 +1500,7 @@ void KMFolderImap::setStatus(QValueList<int>& ids, KMMsgStatus status)
   KMFolder::setStatus(ids, status);
   if (mReadOnly) return;
 
-  // get the uids 
+  // get the uids
   QValueList<int> uids;
   getUids(ids, uids);
 
@@ -1313,7 +1510,7 @@ void KMFolderImap::setStatus(QValueList<int>& ids, KMMsgStatus status)
   // get the sets of ranges..
   QStringList sets = makeSets(uids);
   // ..and pass them to the server
-  for ( QStringList::Iterator it = sets.begin(); it != sets.end(); ++it )	
+  for ( QStringList::Iterator it = sets.begin(); it != sets.end(); ++it )
   {
     setImapStatus(imapPath() + ";UID=" + *it, flags);
   }
@@ -1327,7 +1524,7 @@ QStringList KMFolderImap::makeSets(QStringList& uids, bool sort)
   QValueList<int> tmp;
   for ( QStringList::Iterator it = uids.begin(); it != uids.end(); ++it )
     tmp.append( (*it).toInt() );
-  return makeSets(tmp, sort);	
+  return makeSets(tmp, sort);
 }
 
 QStringList KMFolderImap::makeSets(QValueList<int>& uids, bool sort)
@@ -1339,7 +1536,7 @@ QStringList KMFolderImap::makeSets(QValueList<int>& uids, bool sort)
   {
     sets.append(QString::number(uids.first()));
     return sets;
-  }	
+  }
 
   if (sort) qHeapSort(uids);
 
@@ -1347,7 +1544,7 @@ QStringList KMFolderImap::makeSets(QValueList<int>& uids, bool sort)
   // needed to make a uid like 124 instead of 124:124
   bool inserted = false;
   /* iterate over uids and build sets like 120:122,124,126:150 */
-  for ( QValueList<int>::Iterator it = uids.begin(); it != uids.end(); ++it )	
+  for ( QValueList<int>::Iterator it = uids.begin(); it != uids.end(); ++it )
   {
     if (it == uids.begin() || set.isEmpty()) {
       set = QString::number(*it);
@@ -1358,25 +1555,25 @@ QStringList KMFolderImap::makeSets(QValueList<int>& uids, bool sort)
       {
         // end this range
         if (inserted)
-          set += "," + QString::number(*it);
-        else	
-          set += ":" + QString::number(last) + "," + QString::number(*it);
+          set += ',' + QString::number(*it);
+        else
+          set += ':' + QString::number(last) + ',' + QString::number(*it);
         inserted = true;
         if (set.length() > 100)
         {
           // just in case the server has a problem with longer lines..
           sets.append(set);
           set = "";
-        }	
+        }
       } else {
         inserted = false;
-      }	
+      }
     }
     last = *it;
   }
   // last element
   if (!inserted)
-    set += ":" + QString::number(uids.last());
+    set += ':' + QString::number(uids.last());
 
   if (!set.isEmpty()) sets.append(set);
 
@@ -1386,9 +1583,9 @@ QStringList KMFolderImap::makeSets(QValueList<int>& uids, bool sort)
 //-----------------------------------------------------------------------------
 void KMFolderImap::getUids(QValueList<int>& ids, QValueList<int>& uids)
 {
-  KMMessage *msg = NULL;
+  KMMessage *msg = 0;
   // get the uids
-  for ( QValueList<int>::Iterator it = ids.begin(); it != ids.end(); ++it )	
+  for ( QValueList<int>::Iterator it = ids.begin(); it != ids.end(); ++it )
   {
     bool unget = !isMessage(*it);
     msg = getMsg(*it);
@@ -1400,8 +1597,7 @@ void KMFolderImap::getUids(QValueList<int>& ids, QValueList<int>& uids)
 
 void KMFolderImap::getUids(QPtrList<KMMessage>& msgList, QValueList<int>& uids, KMFolder* msgParent)
 {
-  int idx = -1;
-  KMMessage *msg = NULL;
+  KMMessage *msg = 0;
 
   if (!msgParent) msgParent = msgList.first()->parent();
   if (!msgParent) return;
@@ -1420,18 +1616,17 @@ void KMFolderImap::setImapStatus(QString path, QCString flags)
   kdDebug(5006) << "setImapStatus path=" << path << endl;
   KURL url = mAccount->getUrl();
   url.setPath(path);
-  QCString urlStr("S" + url.url().utf8());
-  QByteArray data;
-  QBuffer buff(data);
-  buff.open(IO_WriteOnly | IO_Append);
-  buff.writeBlock(urlStr.data(), urlStr.size());
-  buff.writeBlock(flags.data(), flags.size());
-  buff.close();
+
+  QByteArray packedArgs;
+  QDataStream stream( packedArgs, IO_WriteOnly);
+
+  stream << (int) 'S' << url << flags;
+
   if (!mAccount->makeConnection()) return;
-  KIO::SimpleJob *job = KIO::special(url, data, FALSE);
+  KIO::SimpleJob *job = KIO::special(url, packedArgs, FALSE);
   KIO::Scheduler::assignJobToSlave(mAccount->slave(), job);
   KMAcctImap::jobData jd;
-  jd.total = 1; jd.done = 0; jd.parent = NULL;
+  jd.total = 1; jd.done = 0; jd.parent = 0;
   mAccount->mapJobData.insert(job, jd);
   connect(job, SIGNAL(result(KIO::Job *)),
           SLOT(slotSetStatusResult(KIO::Job *)));
@@ -1448,7 +1643,7 @@ void KMFolderImap::expungeFolder(KMFolderImap * aFolder, bool quiet)
   KIO::SimpleJob *job = KIO::file_delete(url, FALSE);
   KIO::Scheduler::assignJobToSlave(mAccount->slave(), job);
   KMAcctImap::jobData jd;
-  jd.parent = NULL;
+  jd.parent = 0;
   jd.quiet = quiet;
   jd.total = 1; jd.done = 0;
   mAccount->mapJobData.insert(job, jd);
@@ -1467,8 +1662,8 @@ void KMFolderImap::slotSetStatusResult(KIO::Job * job)
   mAccount->mapJobData.remove(it);
   if (job->error() && job->error() != KIO::ERR_CANNOT_OPEN_FOR_WRITING)
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) mAccount->slaveDied();
+    mAccount->slotSlaveError( mAccount->slave(), job->error(),
+        job->errorText() );
   }
   mAccount->displayProgress();
 }
@@ -1503,8 +1698,8 @@ void KMFolderImap::slotStatResult(KIO::Job * job)
   mAccount->mapJobData.remove(it);
   if (job->error())
   {
-    job->showErrorDialog();
-    if (job->error() == KIO::ERR_SLAVE_DIED) mAccount->slaveDied();
+    mAccount->slotSlaveError( mAccount->slave(), job->error(),
+        job->errorText() );
   } else {
     KIO::UDSEntry uds = static_cast<KIO::StatJob*>(job)->statResult();
     for (KIO::UDSEntry::ConstIterator it = uds.begin(); it != uds.end(); it++)
