@@ -31,6 +31,9 @@
 #include "kio/job.h"
 #include "kio/global.h"
 
+#include <qintdict.h>
+#include <qvaluelist.h>
+
 class KMFolderTreeItem;
 class KMFolderImap;
 
@@ -40,9 +43,9 @@ class KMImapJob : public QObject
 
 public:
   enum JobType { tListDirectory, tGetFolder, tCreateFolder, tDeleteMessage,
-    tGetMessage, tPutMessage, tCopyMessage };
-  KMImapJob(KMMessage *msg, JobType jt = tGetMessage, KMFolderImap *folder = NULL);
-  KMImapJob(QPtrList<KMMessage>& msgList, QString sets, JobType jt = tGetMessage, KMFolderImap *folder = NULL );
+    tGetMessage, tPutMessage, tCopyMessage, tMoveMessage };
+  KMImapJob(KMMessage *msg, JobType jt = tGetMessage, KMFolderImap *folder = 0);
+  KMImapJob(QPtrList<KMMessage>& msgList, QString sets, JobType jt = tGetMessage, KMFolderImap *folder = 0 );
   ~KMImapJob();
   static void ignoreJobsForMessage(KMMessage *msg);
 signals:
@@ -50,13 +53,18 @@ signals:
   void messageStored(KMMessage *);
   void messageCopied(KMMessage *);
   void messageCopied(QPtrList<KMMessage>);
+  void finished();
 private slots:
   void slotGetMessageResult(KIO::Job * job);
   void slotGetNextMessage();
   /** Feeds the message in pieces to the server */
   void slotPutMessageDataReq(KIO::Job *job, QByteArray &data);
   void slotPutMessageResult(KIO::Job *job);
+  void slotPutMessageInfoData(KIO::Job *, const QString &data);
+  /** result of a copy-operation */
   void slotCopyMessageResult(KIO::Job *job);
+  void slotCopyMessageInfoData(KIO::Job *, const QString &data);
+
 private:
   void init(JobType jt, QString sets, KMFolderImap *folder, QPtrList<KMMessage>& msgList);
   JobType mType;
@@ -116,7 +124,7 @@ public:
   virtual void removeOnServer();
 
   /** Automatically expunge deleted messages when leaving the folder */
-  bool autoExpunge() { return mAccount->autoExpunge(); }
+  bool autoExpunge();
 
   /** Write the config file */
   virtual void writeConfig();
@@ -125,14 +133,15 @@ public:
   virtual void readConfig();
 
   /**
-   * List a directory and add the contents to a KMFolderTreeItem
+   * List a directory and add the contents to kmfoldermgr
+   * returns false if the connection failed
    */
-  void listDirectory(KMFolderTreeItem * fti, bool secondStep = FALSE);
+  bool listDirectory(bool secondStep = FALSE);
 
   /**
    * Retrieve all mails in a folder
    */
-  void getFolder();
+  void getFolder(bool force = FALSE);
 
   /**
    * Get the whole message
@@ -172,7 +181,7 @@ public:
   void getUids(QValueList<int>& ids, QValueList<int>& uids);
  
   /** same as above but accepts a Message-List */
-  void getUids(QPtrList<KMMessage>& msgList, QValueList<int>& uids, KMFolder* msgParent = NULL);
+  void getUids(QPtrList<KMMessage>& msgList, QValueList<int>& uids, KMFolder* msgParent = 0);
 
   /**
    * Expunge deleted messages from the folder
@@ -214,6 +223,17 @@ public:
    */
   virtual QString fileName() const { return encodeFileName(name()); }
 
+  /**
+   * Insert a new entry into the uid <=> sernum cache
+   */ 
+  void insertUidSerNumEntry(ulong uid, const ulong * sernum) {
+    uidmap.insert(uid, sernum); }
+
+  /**
+   * Splits a uid-set into single uids
+   */
+  static QValueList<int> splitSets(QString); 
+
 signals:
   void folderComplete(KMFolderImap *folder, bool success);
 
@@ -233,8 +253,8 @@ public slots:
     is stored in index_return if given.
     Please note that the message is added as is to the folder and the folder
     takes ownership of the message (deleting it in the destructor).*/
-  virtual int addMsg(KMMessage* msg, int* index_return = NULL);
-  int addMsg(QPtrList<KMMessage>&, int* index_return = NULL);
+  virtual int addMsg(KMMessage* msg, int* index_return = 0);
+  int addMsg(QPtrList<KMMessage>&, int* index_return = 0);
 
   /** Copy the messages to this folder */
   void copyMsg(QPtrList<KMMessage>& msgList/*, KMFolder* parent*/);
@@ -330,6 +350,7 @@ protected:
   bool        mCheckFlags;
   bool        mReadOnly;
   QGuardedPtr<KMAcctImap> mAccount;
+  QIntDict<ulong> uidmap;
 };
 
 #endif // kmfolderimap_h
