@@ -5,6 +5,8 @@
 #include <config.h>
 #endif
 
+#include <limits.h>
+
 #include <qdir.h>
 #include <qregexp.h>
 
@@ -12,10 +14,12 @@
 #include "kmfoldermaildir.h"
 #include "kmmessage.h"
 #include "kmundostack.h"
-
+#include "kbusyptr.h"
+#include <kio/netaccess.h>
 #include <kapplication.h>
 #include <kdebug.h>
 #include <klocale.h>
+#include <kstaticdeleter.h>
 #include <kmessagebox.h>
 
 #include <dirent.h>
@@ -37,8 +41,6 @@
 #define INIT_MSGS 8
 #endif
 
-static QRegExp SUFFIX_REGEX(":2,?R?S?$");
-
 //-----------------------------------------------------------------------------
 KMFolderMaildir::KMFolderMaildir(KMFolderDir* aParent, const QString& aName)
   : KMFolderMaildirInherited(aParent, aName)
@@ -53,6 +55,27 @@ KMFolderMaildir::~KMFolderMaildir()
   if (kernel->undoStack()) kernel->undoStack()->folderDestroyed(this);
 }
 
+//-----------------------------------------------------------------------------
+int KMFolderMaildir::canAccess()
+{
+
+  assert(name() != "");
+
+  if (access(QFile::encodeName(location()), R_OK | W_OK | X_OK) != 0)
+    return 1;
+
+  if (access(QFile::encodeName(location() + "/new"), R_OK | W_OK | X_OK) != 0)
+    return 1;
+
+  if (access(QFile::encodeName(location() + "/cur"), R_OK | W_OK | X_OK) != 0)
+    return 1;
+
+  if (access(QFile::encodeName(location() + "/tmp"), R_OK | W_OK | X_OK) != 0)
+    return 1;
+
+  return 0;
+}
+
 //-----------------------------------------------------------------------------
 int KMFolderMaildir::open()
 {
@@ -63,19 +86,29 @@ int KMFolderMaildir::open()
 
   assert(name() != "");
 
+  if (canAccess() != 0) {
+    bool busy = kernel->kbp()->isBusy();
+    if (busy) kernel->kbp()->idle();
+    KMessageBox::sorry(0, i18n("Error opening %1. Either this is not a valid "
+      "maildir folder or you don't have sufficient access permissions.")
+      .arg(name()));
+    if (busy) kernel->kbp()->busy();
+    return EPERM;
+  }
+
   if (!path().isEmpty())
   {
-    if (isIndexOutdated()) // test if contents file has changed
+    if (KMFolder::IndexOk != indexStatus()) // test if contents file has changed
     {
       QString str;
-      mIndexStream = NULL;
+      mIndexStream = 0;
       str = i18n("Folder `%1' changed. Recreating index.")
 		  .arg(name());
       emit statusMsg(str);
     } else {
       mIndexStream = fopen(indexLocation().local8Bit(), "r+"); // index file
       updateIndexStreamPtr();
-    }	
+    }
 
     if (!mIndexStream)
       rc = createIndexFromContents();
@@ -183,7 +216,7 @@ void KMFolderMaildir::close(bool aForced)
     }
 
   mOpenCount   = 0;
-  mIndexStream = NULL;
+  mIndexStream = 0;
   mUnreadMsgs  = -1;
 
   mMsgList.reset(INIT_MSGS);
@@ -194,7 +227,7 @@ void KMFolderMaildir::sync()
 {
   if (mOpenCount > 0)
     if (!mIndexStream || fsync(fileno(mIndexStream))) {
-    KMKernel::self()->emergencyExit( i18n("Couldn't sync maildir folder.") );
+    kernel->emergencyExit( i18n("Couldn't sync maildir folder.") );
     }
 }
 
@@ -208,12 +241,12 @@ int KMFolderMaildir::expungeContents()
   QStringList::ConstIterator it(files.begin());
   for ( ; it != files.end(); ++it)
     QFile::remove(d.filePath(*it));
-  
+
   d.setPath(location() + "/cur");
   files = d.entryList();
   for (it = files.begin(); it != files.end(); ++it)
     QFile::remove(d.filePath(*it));
-  
+
   return 0;
 }
 
@@ -271,6 +304,14 @@ int KMFolderMaildir::compact()
 
 int KMFolderMaildir::addMsg(KMMessage* aMsg, int* index_return)
 {
+/*
+QFile fileD0( "testdat_xx-kmfoldermaildir-0" );
+if( fileD0.open( IO_WriteOnly ) ) {
+    QDataStream ds( &fileD0 );
+    ds.writeRawBytes( aMsg->asString(), aMsg->asString().length() );
+    fileD0.close();  // If data is 0 we just create a zero length file.
+}
+*/
   if (!canAddMsgNow(aMsg, index_return)) return 0;
 
   long len;
@@ -312,7 +353,7 @@ int KMFolderMaildir::addMsg(KMMessage* aMsg, int* index_return)
   tmp_file += filename;
 
   if (!kCStringToFile(msgText, tmp_file, false, false, false))
-    return 0;
+    kernel->emergencyExit( i18n("Not enough free disk space.") );
 
   QFile file(tmp_file);
   size = msgText.length();
@@ -332,7 +373,7 @@ int KMFolderMaildir::addMsg(KMMessage* aMsg, int* index_return)
   {
     file.remove();
     if (opened) close();
-    return 0;
+    return -1;
   }
 
   if (msgParent)
@@ -341,7 +382,7 @@ int KMFolderMaildir::addMsg(KMMessage* aMsg, int* index_return)
   if (filename != aMsg->fileName())
     aMsg->setFileName(filename);
 
-  if (aMsg->status() == KMMsgStatusUnread || 
+  if (aMsg->status() == KMMsgStatusUnread ||
       aMsg->status() == KMMsgStatusNew ||
       this == kernel->outboxFolder())
   {
@@ -351,21 +392,23 @@ int KMFolderMaildir::addMsg(KMMessage* aMsg, int* index_return)
       ++mUnreadMsgs;
     emit numUnreadMsgsChanged( this );
   }
+  ++mTotalMsgs;
 
   // store information about the position in the folder file in the message
   aMsg->setParent(this);
   aMsg->setMsgSize(size);
   idx = mMsgList.append(aMsg);
-  aMsg->setMsgSerNum();
+  if (aMsg->getMsgSerNum() <= 0)
+    aMsg->setMsgSerNum();
 
   // write index entry if desired
   if (mAutoCreateIndex)
   {
-    assert(mIndexStream != NULL);
+    assert(mIndexStream != 0);
     clearerr(mIndexStream);
     fseek(mIndexStream, 0, SEEK_END);
-    long revert = ftell(mIndexStream);
-	
+    off_t revert = ftell(mIndexStream);
+
     int len;
     const uchar *buffer = aMsg->asIndexString(len);
     fwrite(&len,sizeof(len), 1, mIndexStream);
@@ -376,17 +419,18 @@ int KMFolderMaildir::addMsg(KMMessage* aMsg, int* index_return)
 
     fflush(mIndexStream);
     int error = ferror(mIndexStream);
-    
+
     error |= appendtoMsgDict(idx);
-    
+
     if (error) {
       kdDebug(5006) << "Error: Could not add message to folder (No space left on device?)" << endl;
       if (ftell(mIndexStream) > revert) {
 	kdDebug(5006) << "Undoing changes" << endl;
 	truncate( QFile::encodeName(indexLocation()), revert );
       }
-      kdDebug(5006) << "Abnormally terminating to prevent data loss, now." << endl;
-      exit(1);
+      kernel->emergencyExit(i18n("KMFolderMaildir::addMsg: abnormally terminating to prevent data loss."));
+      // exit(1); // don't ever use exit(), use the above!
+
       /* This code may not be 100% reliable
       bool busy = kernel->kbp()->isBusy();
       if (busy) kernel->kbp()->idle();
@@ -405,14 +449,23 @@ int KMFolderMaildir::addMsg(KMMessage* aMsg, int* index_return)
   if (index_return)
     *index_return = idx;
 
-  if (!mQuiet)
+  if (!mQuiet) {
     emit msgAdded(idx);
-  else
+    emit msgAdded(this);
+  } else
     mChanged = TRUE;
 
   needsCompact = true;
 
   if (opened) close();
+/*
+QFile fileD1( "testdat_xx-kmfoldermaildir-1" );
+if( fileD1.open( IO_WriteOnly ) ) {
+    QDataStream ds( &fileD1 );
+    ds.writeRawBytes( aMsg->asString(), aMsg->asString().length() );
+    fileD1.close();  // If data is 0 we just create a zero length file.
+}
+*/
   return 0;
 }
 
@@ -435,7 +488,7 @@ QCString& KMFolderMaildir::getMsgString(int idx, QCString& mDest)
 {
   KMMsgInfo* mi = (KMMsgInfo*)mMsgList[idx];
 
-  assert(mi!=NULL);
+  assert(mi!=0);
 
   QString abs_file(location() + "/cur/");
   abs_file += mi->fileName();
@@ -475,7 +528,7 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
   char line[MAX_LINE];
   bool atEof    = false;
   bool inHeader = true;
-  QCString *lastStr = NULL;
+  QCString *lastStr = 0;
 
   QCString dateStr, fromStr, toStr, subjStr;
   QCString xmarkStr, replyToIdStr, msgIdStr, referencesStr;
@@ -484,9 +537,10 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
   // iterate through this file until done
   while (!atEof)
   {
-    if (!f.readLine(line, MAX_LINE))
+    // if the end of the file has been reached or if there was an error
+    if ( f.atEnd() || ( -1 == f.readLine(line, MAX_LINE) ) )
       atEof = true;
-  
+
     // are we done with this file?  if so, compile our info and store
     // it in a KMMsgInfo object
     if (atEof || !inHeader)
@@ -513,7 +567,9 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
       }
 
       KMMsgInfo *mi = new KMMsgInfo(this);
-      mi->init(subjStr, fromStr, toStr, 0, status, xmarkStr, replyToIdStr, msgIdStr, file.local8Bit(), f.size());
+      mi->init(subjStr, fromStr, toStr, 0, status, xmarkStr, replyToIdStr,
+	       msgIdStr, file.local8Bit(), KMMsgEncryptionStateUnknown,
+	       KMMsgSignatureStateUnknown, f.size());
       if (!dateStr.isEmpty())
         mi->setDate(dateStr);
       mi->setDirty(false);
@@ -543,7 +599,7 @@ void KMFolderMaildir::readFileHeaderIntern(const QString& dir, const QString& fi
           *lastStr += line + i;
     }
     else
-      lastStr = NULL;
+      lastStr = 0;
 
     if (inHeader && (line[0] == '\n' || line[0] == '\r'))
       inHeader = false;
@@ -692,17 +748,19 @@ int KMFolderMaildir::createIndexFromContents()
   return 0;
 }
 
-bool KMFolderMaildir::isIndexOutdated()
+KMFolder::IndexStatus KMFolderMaildir::indexStatus()
 {
   QFileInfo new_info(location() + "/new");
   QFileInfo cur_info(location() + "/cur");
   QFileInfo index_info(indexLocation());
 
   if (!index_info.exists())
-    return TRUE;
+    return KMFolder::IndexMissing;
 
-  return ((new_info.lastModified() > index_info.lastModified()) || 
-          (cur_info.lastModified() > index_info.lastModified()));
+  return ((new_info.lastModified() > index_info.lastModified()) ||
+          (cur_info.lastModified() > index_info.lastModified()))
+         ? KMFolder::IndexTooOld
+         : KMFolder::IndexOk;
 }
 
 //-----------------------------------------------------------------------------
@@ -722,12 +780,12 @@ KMMessage* KMFolderMaildir::take(int idx)
   // first, we do the high-level stuff.. then delete later
   KMMessage *msg = KMFolderMaildirInherited::take(idx);
 
-  if (!msg || !msg->fileName()) return NULL;
+  if (!msg || !msg->fileName()) return 0;
 
   if (removeFile(msg->fileName()))
     return msg;
   else
-    return NULL;
+    return 0;
 }
 
 bool KMFolderMaildir::removeFile(const QString& filename)
@@ -760,25 +818,27 @@ bool KMFolderMaildir::removeFile(const QString& filename)
 //-----------------------------------------------------------------------------
 int KMFolderMaildir::removeContents()
 {
-  // it would be nice if QDir could delete recursively.. but since it
-  // doesn't, we have to do this hack
-  QCString cmd;
-  cmd.sprintf("rm -rf '%s'", QFile::encodeName(location()).data());
-  system(cmd.data());
-  
-  return 0;
+    if (KIO::NetAccess::del(KURL::fromPathOrURL(location())))
+        return 0;
+    return 1;
 }
 
+static QRegExp *suffix_regex = 0;
+static KStaticDeleter<QRegExp> suffix_regex_sd;
+
 //-----------------------------------------------------------------------------
 QString KMFolderMaildir::constructValidFileName(QString& aFileName, KMMsgStatus status)
 {
   if (aFileName.isEmpty())
   {
-    aFileName.sprintf("%ld.%d.", (long)time(NULL), getpid());
+    aFileName.sprintf("%ld.%d.", (long)time(0), getpid());
     aFileName += KApplication::randomString(5);
   }
 
-  aFileName.truncate(aFileName.findRev(SUFFIX_REGEX));
+  if (!suffix_regex)
+      suffix_regex = suffix_regex_sd.setObject(new QRegExp(":2,?R?S?$"));
+
+  aFileName.truncate(aFileName.findRev(*suffix_regex));
 
   QString suffix;
   if ((status != KMMsgStatusNew) && (status != KMMsgStatusUnread))
