@@ -1,21 +1,21 @@
 // kmfoldermbox.cpp
 // Author: Stefan Taferner <taferner@alpin.or.at>
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
 #include <qfileinfo.h>
 #include <qregexp.h>
 
 #include "kmfoldermbox.h"
 #include "kmmessage.h"
 #include "kmundostack.h"
+#include "kbusyptr.h"
 
 #include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <knotifyclient.h>
+#include <kapplication.h>
+#include <kprocess.h>
 
 #include <stdio.h>
 #include <errno.h>
@@ -52,7 +52,7 @@ static short msgSepLen = strlen(MSG_SEPERATOR_START);
 KMFolderMbox::KMFolderMbox(KMFolderDir* aParent, const QString& aName)
   : KMFolderMboxInherited(aParent, aName)
 {
-  mStream         = NULL;
+  mStream         = 0;
   mFilesLocked    = FALSE;
   mLockType       = None;
 }
@@ -79,7 +79,7 @@ int KMFolderMbox::open()
   mStream = fopen(location().local8Bit(), "r+"); // messages file
   if (!mStream)
   {
-    KNotifyClient::event("warning", 
+    KNotifyClient::event("warning",
     i18n("Cannot open file \"%1\":\n%2").arg(location()).arg(strerror(errno)));
     kdDebug(5006) << "Cannot open folder `" << location() << "': " << strerror(errno) << endl;
     mOpenCount = 0;
@@ -90,17 +90,79 @@ int KMFolderMbox::open()
 
   if (!path().isEmpty())
   {
-    if (isIndexOutdated()) // test if contents file has changed
+    KMFolder::IndexStatus index_status = indexStatus();
+    // test if index file exists and is up-to-date
+    if (KMFolder::IndexOk != index_status)
     {
+      // only show a warning if the index file exists, otherwise it can be
+      // silently regenerated
+      if (KMFolder::IndexTooOld == index_status)
+      {
+        QString msg = i18n("<qt><p>The index of folder '%2' seems "
+                           "to be out of date. To prevent message "
+                           "corruption the index will be "
+                           "regenerated. As a result deleted "
+                           "messages might reappear and status "
+                           "flags might be lost.</p>"
+                           "<p>Please read the corresponding entry "
+                           "in the <a href=\"%1\">FAQ section of the manual "
+                           "of KMail</a> for "
+                           "information about how to prevent this "
+                           "problem from happening again.</p></qt>")
+                      .arg("help:/kmail/faq.html#faq-index-regeneration")
+                      .arg(name());
+        // When KMail is starting up we have to show a non-blocking message
+        // box so that the initialization can continue. We don't show a
+        // queued message box when KMail isn't starting up because queued
+        // message boxes don't have a "Don't ask again" checkbox.
+        if (kernel->startingUp())
+        {
+          KConfigGroup configGroup( kapp->config(), "Notification Messages" );
+          bool showMessage =
+            configGroup.readBoolEntry( "showIndexRegenerationMessage", true );
+          if (showMessage)
+            KMessageBox::queuedMessageBox( 0, KMessageBox::Information,
+                                           msg, i18n("Index Out of Date"),
+                                           KMessageBox::AllowLink );
+        }
+        else
+        {
+          bool busy = kernel->kbp()->isBusy();
+          if (busy) kernel->kbp()->idle();
+          KMessageBox::information( 0, msg, i18n("Index Out of Date"),
+                                    "showIndexRegenerationMessage",
+                                    KMessageBox::AllowLink );
+          if (busy) kernel->kbp()->busy();
+        }
+// ######### FIXME-AFTER-MSG-FREEZE: Delete this after the msg freeze
+        if( 0 ) {
+        KMessageBox::information( 0,
+                                  i18n("<qt><p>The index of folder '%1' seems "
+                                       "to be out of date. To prevent message "
+                                       "corruption the index will be "
+                                       "regenerated. As a result deleted "
+                                       "messages might reappear and status "
+                                       "flags might be lost.</p>"
+                                       "<p>Please read the corresponding entry "
+                                       "in the FAQ section of the manual of "
+                                       "KMail for "
+                                       "information about how to prevent this "
+                                       "problem from happening again.</p></qt>")
+                                  .arg(name()),
+                                         i18n("Index Out of Date"),
+                                         "dontshowIndexRegenerationWarning");
+        }
+// ######### end of FIXME-AFTER-MSG-FREEZE: Delete this after the msg freeze
+      }
       QString str;
-      mIndexStream = NULL;
+      mIndexStream = 0;
       str = i18n("Folder `%1' changed. Recreating index.")
 		  .arg(name());
       emit statusMsg(str);
     } else {
       mIndexStream = fopen(indexLocation().local8Bit(), "r+"); // index file
       updateIndexStreamPtr();
-    }	
+    }
 
     if (!mIndexStream)
       rc = createIndexFromContents();
@@ -122,6 +184,17 @@ int KMFolderMbox::open()
   return rc;
 }
 
+//----------------------------------------------------------------------------
+int KMFolderMbox::canAccess()
+{
+  assert(name() != "");
+
+  if (access(location().local8Bit(), R_OK | W_OK) != 0) {
+    kdDebug(5006) << "KMFolderMbox::access call to access function failed" << endl;
+      return 1;
+  }
+  return 0;
+}
 
 //-----------------------------------------------------------------------------
 int KMFolderMbox::create(bool imap)
@@ -183,7 +256,7 @@ void KMFolderMbox::close(bool aForced)
 
   if (mAutoCreateIndex)
   {
-      if (isIndexOutdated()) {
+      if (KMFolder::IndexOk != indexStatus()) {
 	  kdDebug(5006) << "Critical error: " << location() <<
 	      " has been modified by an external application while KMail was running." << endl;
 	  //	  exit(1); backed out due to broken nfs
@@ -210,8 +283,8 @@ void KMFolderMbox::close(bool aForced)
     }
 
   mOpenCount   = 0;
-  mStream      = NULL;
-  mIndexStream = NULL;
+  mStream      = 0;
+  mIndexStream = 0;
   mFilesLocked = FALSE;
   mUnreadMsgs  = -1;
 
@@ -224,7 +297,7 @@ void KMFolderMbox::sync()
   if (mOpenCount > 0)
     if (!mStream || fsync(fileno(mStream)) ||
 	!mIndexStream || fsync(fileno(mIndexStream))) {
-    KMKernel::self()->emergencyExit( i18n("Not enough free disk space." ));
+    kernel->emergencyExit( i18n("Not enough free disk space." ));
     }
 }
 
@@ -238,8 +311,8 @@ int KMFolderMbox::lock()
   fl.l_start=0;
   fl.l_len=0;
   fl.l_pid=-1;
-  QString cmd_str;
-  assert(mStream != NULL);
+  QCString cmd_str;
+  assert(mStream != 0);
   mFilesLocked = FALSE;
 
   switch( mLockType )
@@ -271,13 +344,13 @@ int KMFolderMbox::lock()
       break;
 
     case procmail_lockfile:
-      cmd_str = "lockfile ";
+      cmd_str = "lockfile -l20 -r5 ";
       if (!mProcmailLockFileName.isEmpty())
-	cmd_str += mProcmailLockFileName;
+	cmd_str += QFile::encodeName(KProcess::quote(mProcmailLockFileName));
       else
-        cmd_str += location() + ".lock";
+        cmd_str += QFile::encodeName(KProcess::quote(location() + ".lock"));
 
-      rc = system( cmd_str.local8Bit() );
+      rc = system( cmd_str.data() );
       if( rc != 0 )
       {
 	kdDebug(5006) << "Cannot lock folder `" << location() << "': "
@@ -286,8 +359,8 @@ int KMFolderMbox::lock()
       }
       if( mIndexStream )
       {
-        cmd_str = "lockfile " + indexLocation() + ".lock";
-        rc = system( cmd_str.local8Bit() );
+        cmd_str = "lockfile -l20 -r5 " + QFile::encodeName(KProcess::quote(indexLocation() + ".lock"));
+        rc = system( cmd_str.data() );
         if( rc != 0 )
         {
           kdDebug(5006) << "Cannot lock index of folder `" << location() << "': "
@@ -298,8 +371,8 @@ int KMFolderMbox::lock()
       break;
 
     case mutt_dotlock:
-      cmd_str = "mutt_dotlock " + location();
-      rc = system( cmd_str.local8Bit() );
+      cmd_str = "mutt_dotlock " + QFile::encodeName(KProcess::quote(location()));
+      rc = system( cmd_str.data() );
       if( rc != 0 )
       {
         kdDebug(5006) << "Cannot lock folder `" << location() << "': "
@@ -308,8 +381,8 @@ int KMFolderMbox::lock()
       }
       if( mIndexStream )
       {
-        cmd_str = "mutt_dotlock " + indexLocation();
-        rc = system( cmd_str.local8Bit() );
+        cmd_str = "mutt_dotlock " + QFile::encodeName(KProcess::quote(indexLocation()));
+        rc = system( cmd_str.data() );
         if( rc != 0 )
         {
           kdDebug(5006) << "Cannot lock index of folder `" << location() << "': "
@@ -320,8 +393,8 @@ int KMFolderMbox::lock()
       break;
 
     case mutt_dotlock_privileged:
-      cmd_str = "mutt_dotlock -p " + location();
-      rc = system( cmd_str.local8Bit() );
+      cmd_str = "mutt_dotlock -p " + QFile::encodeName(KProcess::quote(location()));
+      rc = system( cmd_str.data() );
       if( rc != 0 )
       {
         kdDebug(5006) << "Cannot lock folder `" << location() << "': "
@@ -330,8 +403,8 @@ int KMFolderMbox::lock()
       }
       if( mIndexStream )
       {
-        cmd_str = "mutt_dotlock -p " + indexLocation();
-        rc = system( cmd_str.local8Bit() );
+        cmd_str = "mutt_dotlock -p " + QFile::encodeName(KProcess::quote(indexLocation()));
+        rc = system( cmd_str.data() );
         if( rc != 0 )
         {
           kdDebug(5006) << "Cannot lock index of folder `" << location() << "': "
@@ -361,9 +434,9 @@ int KMFolderMbox::unlock()
   fl.l_whence=0;
   fl.l_start=0;
   fl.l_len=0;
-  QString cmd_str;
+  QCString cmd_str;
 
-  assert(mStream != NULL);
+  assert(mStream != 0);
   mFilesLocked = FALSE;
 
   switch( mLockType )
@@ -377,35 +450,35 @@ int KMFolderMbox::unlock()
     case procmail_lockfile:
       cmd_str = "rm -f ";
       if (!mProcmailLockFileName.isEmpty())
-        cmd_str += mProcmailLockFileName;
+        cmd_str += QFile::encodeName(KProcess::quote(mProcmailLockFileName));
       else
-        cmd_str += location() + ".lock";
+        cmd_str += QFile::encodeName(KProcess::quote(location() + ".lock"));
 
-      rc = system( cmd_str.latin1() );
+      rc = system( cmd_str.data() );
       if( mIndexStream )
       {
-        cmd_str = "rm -f " + indexLocation() + ".lock";
-        rc = system( cmd_str.latin1() );
+        cmd_str = "rm -f " + QFile::encodeName(KProcess::quote(indexLocation() + ".lock"));
+        rc = system( cmd_str.data() );
       }
       break;
 
     case mutt_dotlock:
-      cmd_str = "mutt_dotlock -u " + location();
-      rc = system( cmd_str.latin1() );
+      cmd_str = "mutt_dotlock -u " + QFile::encodeName(KProcess::quote(location()));
+      rc = system( cmd_str.data() );
       if( mIndexStream )
       {
-        cmd_str = "mutt_dotlock -u " + indexLocation();
-        rc = system( cmd_str.latin1() );
+        cmd_str = "mutt_dotlock -u " + QFile::encodeName(KProcess::quote(indexLocation()));
+        rc = system( cmd_str.data() );
       }
       break;
 
     case mutt_dotlock_privileged:
-      cmd_str = "mutt_dotlock -p -u " + location();
-      rc = system( cmd_str.latin1() );
+      cmd_str = "mutt_dotlock -p -u " + QFile::encodeName(KProcess::quote(location()));
+      rc = system( cmd_str.data() );
       if( mIndexStream )
       {
-        cmd_str = "mutt_dotlock -p -u " + indexLocation();
-        rc = system( cmd_str.latin1() );
+        cmd_str = "mutt_dotlock -p -u " + QFile::encodeName(KProcess::quote(indexLocation()));
+        rc = system( cmd_str.data() );
       }
       break;
 
@@ -420,15 +493,20 @@ int KMFolderMbox::unlock()
 
 
 //-----------------------------------------------------------------------------
-bool KMFolderMbox::isIndexOutdated()
+KMFolder::IndexStatus KMFolderMbox::indexStatus()
 {
   QFileInfo contInfo(location());
   QFileInfo indInfo(indexLocation());
 
-  if (!contInfo.exists()) return FALSE;
-  if (!indInfo.exists()) return TRUE;
+  if (!contInfo.exists()) return KMFolder::IndexOk;
+  if (!indInfo.exists()) return KMFolder::IndexMissing;
 
-  return (contInfo.lastModified() > indInfo.lastModified());
+  // Check whether the mbox file is more than 5 seconds newer than the index
+  // file. The 5 seconds are added to reduce the number of false alerts due
+  // to slightly out of sync clocks of the NFS server and the local machine.
+  return ( contInfo.lastModified() > indInfo.lastModified().addSecs(5) )
+         ? KMFolder::IndexTooOld
+         : KMFolder::IndexOk;
 }
 
 
@@ -437,28 +515,26 @@ int KMFolderMbox::createIndexFromContents()
 {
   char line[MAX_LINE];
   char status[8], xstatus[8];
-  QCString subjStr, dateStr, fromStr, toStr, xmarkStr, *lastStr=NULL;
+  QCString subjStr, dateStr, fromStr, toStr, xmarkStr, *lastStr=0;
   QCString replyToIdStr, referencesStr, msgIdStr;
-  QString whoFieldName;
-  unsigned long offs, size, pos;
   bool atEof = FALSE;
   bool inHeader = TRUE;
   KMMsgInfo* mi;
   QString msgStr;
   QRegExp regexp(MSG_SEPERATOR_REGEX);
-  int i, num, numStatus, whoFieldLen;
+  int i, num, numStatus;
   short needStatus;
 
   quiet(TRUE);
-  assert(mStream != NULL);
+  assert(mStream != 0);
   rewind(mStream);
 
   mMsgList.clear();
 
   num     = -1;
   numStatus= 11;
-  offs    = 0;
-  size    = 0;
+  off_t offs = 0;
+  size_t size = 0;
   dateStr = "";
   fromStr = "";
   toStr = "";
@@ -469,14 +545,10 @@ int KMFolderMbox::createIndexFromContents()
   replyToIdStr = "";
   msgIdStr = "";
   needStatus = 3;
-  whoFieldName = QString(whoField()) + ':'; //unused (sven)
-  whoFieldLen = whoFieldName.length();      //unused (sven)
-
-  //kdDebug(5006) << "***whoField: " << //      (const char*)whoFieldName << " (" << whoFieldLen << ")" << endl;
 
   while (!atEof)
   {
-    pos = ftell(mStream);
+    off_t pos = ftell(mStream);
     if (!fgets(line, MAX_LINE, mStream)) atEof = TRUE;
 
     if (atEof ||
@@ -502,7 +574,8 @@ int KMFolderMbox::createIndexFromContents()
 	    replyToIdStr = referencesStr;
 	  }
 	  mi = new KMMsgInfo(this);
-	  mi->init(subjStr, fromStr, toStr, 0, KMMsgStatusNew, xmarkStr, replyToIdStr, msgIdStr, offs, size);
+	  mi->init(subjStr, fromStr, toStr, 0, KMMsgStatusNew, xmarkStr, replyToIdStr, msgIdStr, 
+		   KMMsgEncryptionStateUnknown, KMMsgSignatureStateUnknown, offs, size);
 	  mi->setStatus("RO","O");
 	  mi->setDate(dateStr);
 	  mi->setDirty(FALSE);
@@ -536,7 +609,7 @@ int KMFolderMbox::createIndexFromContents()
       if (line [i] < ' ' && line [i]>0) inHeader = FALSE;
       else if (lastStr) *lastStr += line + i;
     }
-    else lastStr = NULL;
+    else lastStr = 0;
 
     if (inHeader && (line [0]=='\n' || line [0]=='\r'))
       inHeader = FALSE;
@@ -620,7 +693,8 @@ int KMFolderMbox::createIndexFromContents()
   correctUnreadMsgsCount();
 
   if (kernel->outboxFolder() == this && count() > 0)
-    KMessageBox::information(0, i18n("Your outbox contains messages which were "
+    KMessageBox::queuedMessageBox(0, KMessageBox::Information,
+                                  i18n("Your outbox contains messages which were "
     "most likely not created by KMail.\nPlease remove them from there, if you "
     "don't want KMail to send them."));
 
@@ -636,8 +710,8 @@ KMMessage* KMFolderMbox::readMsg(int idx)
   QCString msgText;
   KMMsgInfo* mi = (KMMsgInfo*)mMsgList[idx];
 
-  assert(mi!=NULL && !mi->isMessage());
-  assert(mStream != NULL);
+  assert(mi!=0 && !mi->isMessage());
+  assert(mStream != 0);
 
   msgSize = mi->msgSize();
   msgText.resize(msgSize+2);
@@ -661,8 +735,8 @@ QCString& KMFolderMbox::getMsgString(int idx, QCString &mDest)
   unsigned long msgSize;
   KMMsgInfo* mi = (KMMsgInfo*)mMsgList[idx];
 
-  assert(mi!=NULL);
-  assert(mStream != NULL);
+  assert(mi!=0);
+  assert(mStream != 0);
 
   msgSize = mi->msgSize();
   mDest.resize(msgSize+2);
@@ -679,8 +753,6 @@ QCString& KMFolderMbox::getMsgString(int idx, QCString &mDest)
 int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
 {
   if (!canAddMsgNow(aMsg, aIndex_ret)) return 0;
-
-  long offs, size, len, revert;
   bool opened = FALSE;
   QCString msgText;
   char endStr[3];
@@ -690,7 +762,7 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
   int growth = 0;
 
 /* Then we can also disable it completely, this wastes time, at least for IMAP
-  if (isIndexOutdated()) {
+  if (KMFolder::IndexOk != indexStatus()) {
       kdDebug(5006) << "Critical error: " << location() <<
 	  " has been modified by an external application while KMail was running." << endl;
       //      exit(1); backed out due to broken nfs
@@ -726,15 +798,31 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
 
   if (protocol() != "imap")
   {
+/*
+QFile fileD0( "testdat_xx-kmfoldermbox-0" );
+if( fileD0.open( IO_WriteOnly ) ) {
+    QDataStream ds( &fileD0 );
+    ds.writeRawBytes( aMsg->asString(), aMsg->asString().length() );
+    fileD0.close();  // If data is 0 we just create a zero length file.
+}
+*/  
     aMsg->setStatusFields();
+/*  
+QFile fileD1( "testdat_xx-kmfoldermbox-1" );
+if( fileD1.open( IO_WriteOnly ) ) {
+    QDataStream ds( &fileD1 );
+    ds.writeRawBytes( aMsg->asString(), aMsg->asString().length() );
+    fileD1.close();  // If data is 0 we just create a zero length file.
+}
+*/  
     if (aMsg->headerField("Content-Type").isEmpty())  // This might be added by
       aMsg->removeHeaderField("Content-Type");        // the line above
   }
   msgText = aMsg->asString();
   msgText.replace(QRegExp("\nFrom "),"\n>From ");
-  len = msgText.length();
+  size_t len = msgText.length();
 
-  assert(mStream != NULL);
+  assert(mStream != 0);
   clearerr(mStream);
   if (len <= 0)
   {
@@ -746,7 +834,7 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
   // Make sure the file is large enough to check for an end
   // character
   fseek(mStream, 0, SEEK_END);
-  revert = ftell(mStream);
+  off_t revert = ftell(mStream);
   if (ftell(mStream) >= 2) {
       // write message to folder file
       fseek(mStream, -2, SEEK_END);
@@ -771,11 +859,11 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
 
   fprintf(mStream, "From %s %s\n", (const char *)aMsg->fromEmail(),
           (const char *)aMsg->dateShortStr());
-  offs = ftell(mStream);
+  off_t offs = ftell(mStream);
   fwrite(msgText, len, 1, mStream);
   if (msgText[(int)len-1]!='\n') fwrite("\n\n", 1, 2, mStream);
   fflush(mStream);
-  size = ftell(mStream) - offs;
+  size_t size = ftell(mStream) - offs;
 
   error = ferror(mStream);
   if (error) {
@@ -784,7 +872,7 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
       kdDebug(5006) << "Undoing changes" << endl;
       truncate( location().local8Bit(), revert );
     }
-    KMKernel::self()->emergencyExit( i18n("Not enough free disk space.") );
+    kernel->emergencyExit( i18n("Not enough free disk space.") );
 
     /* This code is not 100% reliable
     bool busy = kernel->kbp()->isBusy();
@@ -812,13 +900,15 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
     else ++mUnreadMsgs;
     emit numUnreadMsgsChanged( this );
   }
+  ++mTotalMsgs;
 
   // store information about the position in the folder file in the message
   aMsg->setParent(this);
   aMsg->setFolderOffset(offs);
   aMsg->setMsgSize(size);
   idx = mMsgList.append(aMsg);
-  aMsg->setMsgSerNum();
+  if (aMsg->getMsgSerNum() <= 0)
+    aMsg->setMsgSerNum();
 
   // change the length of the previous message to encompass white space added
   if ((idx > 0) && (growth > 0)) {
@@ -830,11 +920,11 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
   // write index entry if desired
   if (mAutoCreateIndex)
   {
-    assert(mIndexStream != NULL);
+    assert(mIndexStream != 0);
     clearerr(mIndexStream);
     fseek(mIndexStream, 0, SEEK_END);
     revert = ftell(mIndexStream);
-	
+
 	int len;
 	const uchar *buffer = aMsg->asIndexString(len);
 	fwrite(&len,sizeof(len), 1, mIndexStream);
@@ -845,16 +935,16 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
 
     fflush(mIndexStream);
     error = ferror(mIndexStream);
-    
+
     error |= appendtoMsgDict(idx);
-    
+
     if (error) {
       kdWarning(5006) << "Error: Could not add message to folder (No space left on device?)" << endl;
       if (ftell(mIndexStream) > revert) {
 	kdWarning(5006) << "Undoing changes" << endl;
 	truncate( indexLocation().local8Bit(), revert );
       }
-      KMKernel::self()->emergencyExit( i18n("Not enough free disk space.") );
+      kernel->emergencyExit( i18n("Not enough free disk space.") );
 
       /* This code may not be 100% reliable
       bool busy = kernel->kbp()->isBusy();
@@ -869,12 +959,13 @@ int KMFolderMbox::addMsg(KMMessage* aMsg, int* aIndex_ret)
       return error;
     }
   }
-  
+
   // some "paper work"
   if (aIndex_ret) *aIndex_ret = idx;
-  if (!mQuiet)
+  if (!mQuiet) {
     emit msgAdded(idx);
-  else
+    emit msgAdded(this);
+  } else
     mChanged = TRUE;
 
   if (opened) close();
@@ -901,9 +992,9 @@ int KMFolderMbox::compact()
     kdDebug(5006) << location() << " compaction skipped." << endl;
     return 0;
   }
-  kdDebug(5006) << "Compacting " << endl;
+  kdDebug(5006) << "Compacting " << idString() << endl;
 
-  if (isIndexOutdated()) {
+  if (KMFolder::IndexOk != indexStatus()) {
       kdDebug(5006) << "Critical error: " << location() <<
 	  " has been modified by an external application while KMail was running." << endl;
       //      exit(1); backed out due to broken nfs
@@ -918,8 +1009,10 @@ int KMFolderMbox::compact()
   open();
 
   KMMsgInfo* mi;
-  uint msize, folder_offset;
-  int offs=0, msgs=0;
+  size_t msize;
+  off_t folder_offset;
+  off_t offs=0;
+  size_t msgs=0;
   QCString mtext;
   for(int idx = 0; idx < mMsgList.count(); idx++) {
     if(!(msgs++ % 10)) {
@@ -933,8 +1026,8 @@ int KMFolderMbox::compact()
     folder_offset = mi->folderOffset();
 
     //now we need to find the separator! grr...
-    for(int i = folder_offset-25; TRUE; i -= 20) {
-      int chunk_offset = i <= 0 ? 0 : i;
+    for(off_t i = folder_offset-25; TRUE; i -= 20) {
+      off_t chunk_offset = i <= 0 ? 0 : i;
       if(fseek(mStream, chunk_offset, SEEK_SET) == -1) {
         rc = errno;
 	break;
@@ -980,7 +1073,7 @@ int KMFolderMbox::compact()
     }
     if (rc)
       break;
-	
+
     //now actually write the message
     if(fseek(mStream, folder_offset, SEEK_SET) == -1 ||
        !fread(mtext.data(), msize, 1, mStream) || !fwrite(mtext.data(), msize, 1, tmpfile)) {
