@@ -1,3 +1,4 @@
+#include <config.h>
 #include <sys/types.h>
 #include <dirent.h>
 #include <sys/stat.h>
@@ -6,13 +7,23 @@
 #include <stdlib.h>
 #include <assert.h>
 
+#include <dcopclient.h>
+
+#include <qvbox.h>
+
+#include <kaboutdata.h>
 #include <kdebug.h>
 #include <kmessagebox.h>
 #include <knotifyclient.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
 #include <qutf7codec.h>
+#include <kio/job.h>
+#include <kprocess.h>
+#include <kprogress.h>
+#include <kpassivepopup.h>
 
+#include "kmreaderwin.h"
 #include "kmmainwin.h"
 #include "kmcomposewin.h"
 #include "kmfoldermgr.h"
@@ -29,8 +40,12 @@
 #include "kmversion.h"
 #include "kmrecentaddr.h"
 #include "kmmsgdict.h"
+#include "kmmessage.h"
 #include "kmidentity.h"
+#include "identitymanager.h"
+#include "configuredialog.h"
 #include <kwin.h>
+#include <ktip.h>
 
 #include <X11/Xlib.h>
 #include <kapplication.h>
@@ -42,10 +57,12 @@ KMKernel *KMKernel::mySelf = 0;
 /*                     Constructor and destructor                   */
 /********************************************************************/
 KMKernel::KMKernel (QObject *parent, const char *name) :
-  QObject(parent, name),  DCOPObject("KMailIface")
+  QObject(parent, name),  DCOPObject("KMailIface"),
+  mIdentityManager(0), mProgress(0), mConfigureDialog(0)
 {
   //kdDebug(5006) << "KMKernel::KMKernel" << endl;
   mySelf = this;
+  the_startingUp = true;
   closed_by_user = true;
   the_firstInstance = true;
   the_msgDict = 0;
@@ -66,6 +83,7 @@ KMKernel::KMKernel (QObject *parent, const char *name) :
   the_filterActionDict = 0;
   the_msgSender = 0;
   the_msgDict = 0;
+  mWin = 0;
 
   new KMpgpWrap();
   // register our own (libkdenetwork) utf-7 codec as long as Qt
@@ -99,6 +117,10 @@ KMKernel::~KMKernel ()
     job->kill();
     it = mPutJobs.begin();
   }
+  delete mConfigureDialog;
+  mConfigureDialog = 0;
+  delete mWin;
+  mWin = 0;
   mySelf = 0;
   kdDebug(5006) << "KMKernel::~KMKernel" << endl;
 }
@@ -110,19 +132,27 @@ KMKernel::~KMKernel ()
 void KMKernel::checkMail () //might create a new reader but wont show!!
 {
   kdDebug(5006) << "KMKernel::checkMail called" << endl;
-  KMMainWin *mWin = 0;
-  KMainWindow *kmWin = 0;
+  mainWin()->slotCheckMail();
+}
 
-  for (kmWin = KMainWindow::memberList->first(); kmWin;
-    kmWin = KMainWindow::memberList->next())
-      if (kmWin->isA("KMMainWin")) break;
-  if (kmWin && kmWin->isA("KMMainWin")) {
-    mWin = (KMMainWin *) kmWin;
-    mWin->slotCheckMail();
-  } else {
-    mWin = new KMMainWin;
-    mWin->slotCheckMail();
-    delete mWin;
+QStringList KMKernel::accounts()
+{
+  return kernel->acctMgr()->getAccounts();
+}
+
+void KMKernel::checkAccount (const QString &account) //might create a new reader but wont show!!
+{
+  kdDebug(5006) << "KMKernel::checkMail called" << endl;
+
+  if (!checkingMail())
+  {
+    setCheckingMail(true);
+
+    KMAccount* acct = kernel->acctMgr()->find(account);
+    if (acct)
+       kernel->acctMgr()->singleCheckMail(acct, false);
+
+    setCheckingMail(false);
   }
 }
 
@@ -165,8 +195,11 @@ int KMKernel::openComposer (const QString &to, const QString &cc,
   if (!subject.isEmpty()) msg->setSubject(subject);
   if (!to.isEmpty()) msg->setTo(to);
 
-  if (!messageFile.isEmpty() && messageFile.isLocalFile())
-    msg->setBody( kFileToString( messageFile.path(), true, false ) );
+  if (!messageFile.isEmpty() && messageFile.isLocalFile()) {
+    QCString str = kFileToString( messageFile.path(), true, false );
+    if( !str.isEmpty() )
+      msg->setBody( str );
+  }
 
   if (!body.isEmpty()) msg->setBody(body.utf8());
 
@@ -179,6 +212,7 @@ int KMKernel::openComposer (const QString &to, const QString &cc,
   return 1;
 }
 
+
 int KMKernel::openComposer (const QString &to, const QString &cc,
                             const QString &bcc, const QString &subject,
                             const QString &body, int hidden,
@@ -241,6 +275,35 @@ DCOPRef KMKernel::openComposer(const QString &to, const QString &cc,
   return DCOPRef(cWin);
 }
 
+
+int KMKernel::sendCertificate( const QString& to, const QByteArray& certData )
+{
+  KMMessage *msg = new KMMessage;
+  msg->initHeader();
+  msg->setCharset("utf-8");
+  msg->setSubject( i18n( "Certificate Signature Request" ) );
+  if (!to.isEmpty()) msg->setTo(to);
+  msg->setBody( i18n( "Please sign this certificate and return to sender." ).utf8() );
+
+  KMComposeWin *cWin = new KMComposeWin(msg);
+  cWin->setCharset("", TRUE);
+  cWin->slotSetAlwaysSend( true );
+  if (!certData.isEmpty()) {
+    KMMessagePart *msgPart = new KMMessagePart;
+    msgPart->setName("smime.p10");
+    msgPart->setCteStr("base64");
+    msgPart->setBodyEncodedBinary(certData);
+    msgPart->setTypeStr("application");
+    msgPart->setSubtypeStr("pkcs10");
+    msgPart->setContentDisposition("attachment; filename=smime.p10");
+    cWin->addAttach(msgPart);
+  }
+
+  cWin->show();
+  return 1;
+}
+
+
 void KMKernel::compactAllFolders ()
 {
   kdDebug(5006) << "KMKernel::compactAllFolders called" << endl;
@@ -257,14 +320,17 @@ int KMKernel::dcopAddMessage(const QString & foldername,const KURL & msgUrl)
 {
 int retval;
 QCString bericht;
-static QStringList *msgIds=NULL;
+static QStringList *msgIds=0;
 static QString      lastFolder="";
 bool readFolderMsgIds=false;
 
   //kdDebug(5006) << "KMKernel::dcopAddMessage called" << endl;
 
   if (foldername!=lastFolder) {
-    if (msgIds!=NULL) { delete msgIds; }
+    if (msgIds!=0) {
+      delete msgIds;
+      msgIds = 0;
+    }
     msgIds=new QStringList;
     readFolderMsgIds=true;
     lastFolder=foldername;
@@ -287,7 +353,7 @@ bool readFolderMsgIds=false;
 
     KMFolder  *F=the_folderMgr->findOrCreate(foldername, FALSE);
 
-    if (F==NULL) { retval=-1; }
+    if (F==0) { retval=-1; }
     else {
 
       if (readFolderMsgIds) {int i;
@@ -333,7 +399,7 @@ bool readFolderMsgIds=false;
       if (k==-1) {
         if (msgId!="") { msgIds->append(msgId); }
         if (F->addMsg(M)==0) { retval=1; }
-        else { retval=-2;delete M; }
+        else { retval=-2;delete M; M = 0; }
       }
       else { retval=-4; }
     }
@@ -418,7 +484,7 @@ void KMKernel::testDir(const char *_name)
 
   c += _name;
   dp = opendir(c);
-  if (dp == NULL) ::mkdir(c, S_IRWXU);
+  if (dp == 0) ::mkdir(c, S_IRWXU);
   else closedir(dp);
 }
 
@@ -455,7 +521,7 @@ void KMKernel::recoverDeadLetters(void)
     msg = folder.take(0);
     if (msg)
     {
-      win = new KMComposeWin;
+      win = new KMComposeWin();
       win->setMsg(msg, FALSE);
       win->show();
     }
@@ -474,28 +540,51 @@ void KMKernel::initFolders(KConfig* cfg)
   // in the base folder directory.
   //if (name.isEmpty()) name = getenv("MAIL");
 
-  if (name.isEmpty()) name = "inbox";
+  if (name.isEmpty()) name = I18N_NOOP("inbox");
 
   the_inboxFolder  = (KMFolder*)the_folderMgr->findOrCreate(name);
+
+  if (the_inboxFolder->canAccess() != 0) {
+    KMessageBox::sorry(0, i18n("You do not have read/write permission to your inbox folder.") );
+    exit(1);
+  }
+
   the_inboxFolder->setSystemFolder(TRUE);
   // inboxFolder->open();
 
-  the_outboxFolder = the_folderMgr->findOrCreate(cfg->readEntry("outboxFolder", "outbox"));
+  the_outboxFolder = the_folderMgr->findOrCreate(cfg->readEntry("outboxFolder", I18N_NOOP("outbox")));
+  if (the_outboxFolder->canAccess() != 0) {
+    KMessageBox::sorry(0, i18n("You do not have read/write permission to your outbox folder.") );
+    exit(1);
+  }
+
   the_outboxFolder->setType("Out");
   the_outboxFolder->setSystemFolder(TRUE);
   the_outboxFolder->open();
 
-  the_sentFolder = the_folderMgr->findOrCreate(cfg->readEntry("sentFolder", "sent-mail"));
+  the_sentFolder = the_folderMgr->findOrCreate(cfg->readEntry("sentFolder", I18N_NOOP("sent-mail")));
+  if (the_sentFolder->canAccess() != 0) {
+    KMessageBox::sorry(0, i18n("You do not have read/write permission to your sent-mail folder.") );
+    exit(1);
+  }
   the_sentFolder->setType("St");
   the_sentFolder->setSystemFolder(TRUE);
   the_sentFolder->open();
 
-  the_trashFolder  = the_folderMgr->findOrCreate(cfg->readEntry("trashFolder", "trash"));
+  the_trashFolder  = the_folderMgr->findOrCreate(cfg->readEntry("trashFolder", I18N_NOOP("trash")));
+  if (the_trashFolder->canAccess() != 0) {
+    KMessageBox::sorry(0, i18n("You do not have read/write permission to your trash folder.") );
+    exit(1);
+  }
   the_trashFolder->setType("Tr");
   the_trashFolder->setSystemFolder(TRUE);
   the_trashFolder->open();
 
-  the_draftsFolder = the_folderMgr->findOrCreate(cfg->readEntry("draftsFolder", "drafts"));
+  the_draftsFolder = the_folderMgr->findOrCreate(cfg->readEntry("draftsFolder", I18N_NOOP("drafts")));
+  if (the_draftsFolder->canAccess() != 0) {
+    KMessageBox::sorry(0, i18n("You do not have read/write permission to your drafts folder.") );
+    exit(1);
+  }
   the_draftsFolder->setType("Df");
   the_draftsFolder->setSystemFolder(TRUE);
   the_draftsFolder->open();
@@ -517,6 +606,8 @@ void KMKernel::init()
   cfg = kapp->config();
   //kdDebug(5006) << "1" << endl;
 
+  mCryptPlugList.loadFromConfig( cfg );
+
   QDir dir;
   QString d = locateLocal("data", "kmail/");
 
@@ -558,7 +649,7 @@ void KMKernel::init()
     KConfigGroupSaver saver(cfg, "Composer");
     if (cfg->readListEntry("pref-charsets").isEmpty())
     {
-      cfg->writeEntry("pref-charsets", "us-ascii,locale,utf-8");
+      cfg->writeEntry("pref-charsets", "us-ascii,iso-8859-1,locale,utf-8");
     }
   }
   // filterMgr->dump();
@@ -617,8 +708,6 @@ bool KMKernel::doSessionManagement()
 void KMKernel::cleanup(void)
 {
   the_shuttingDown = TRUE;
-  KConfig* config =  kapp->config();
-  KConfigGroupSaver saver(config, "General");
 
   delete the_acctMgr;
   the_acctMgr = 0;
@@ -633,6 +722,69 @@ void KMKernel::cleanup(void)
   delete the_popFilterMgr;
   the_popFilterMgr = 0;
 
+  KMReaderWin::deleteAllStandaloneWindows();
+
+  // Since the application has already quit we can't use
+  // kapp->processEvents() because it will return immediately:
+  // We first have to fire up a new event loop.
+  // We use the timer to transfer control to the cleanupLoop function
+  // once the event loop is running.
+
+  // Don't handle DCOP requests from the event loop
+  kapp->dcopClient()->suspend();
+
+  // Schedule execution of cleanupLoop
+  QTimer::singleShot(0, this, SLOT(cleanupLoop()));
+
+  // Start new event loop
+  kapp->enter_loop();
+}
+
+void KMKernel::cleanupProgress()
+{
+  mProgress->advance( 1 );
+}
+
+void KMKernel::cleanupLoop()
+{
+  mProgress = 0;
+  mCleanupLabel = 0;
+  mCleanupPopup = 0;
+  int nrFolders = the_folderMgr->folderCount();
+  if (closed_by_user)
+  {
+    mCleanupPopup = new KPassivePopup();
+    QVBox *box = mCleanupPopup->standardView( kapp->aboutData()->programName(),
+                                              QString::null, kapp->miniIcon());
+    mCleanupLabel = new QLabel( i18n("Cleaning up"), box );
+    mProgress = new KProgress( box, "kmail-cleanupProgress" );
+    mCleanupPopup->setView( box );
+    
+    mProgress->setTotalSteps(nrFolders*2+2);
+    mProgress->setProgress(1);
+    QApplication::syncX();
+    mCleanupPopup->adjustSize();
+    mCleanupPopup->show();
+    kapp->processEvents();
+    connect(the_folderMgr, SIGNAL(progress()), this, SLOT(cleanupProgress()));
+  }
+
+
+  KConfig* config =  kapp->config();
+  KConfigGroupSaver saver(config, "General");
+
+  bool expire = false;
+  // Expire old messages in all folders.
+  if (closed_by_user) {
+    if (config->readNumEntry("when-to-expire")==expireAtExit) {
+      expire = true;
+
+      if (config->readBoolEntry("warn-before-expire")) {
+	expire = canExpire();
+      }
+    }
+  }
+
   if (!closed_by_user) {
       if (the_trashFolder)
 	  the_trashFolder->close();
@@ -642,36 +794,55 @@ void KMKernel::cleanup(void)
     the_trashFolder->close(TRUE);
 
     if (config->readBoolEntry("empty-trash-on-exit", true))
+    {
+      if (mCleanupLabel)
+      {
+        mCleanupLabel->setText(i18n("Emptying trash..."));
+        QApplication::syncX();
+        kapp->processEvents();
+      }
       the_trashFolder->expunge();
-
-  } 
-
-  // Expire old messages in all folders.
-  if (closed_by_user) {
-    if (config->readNumEntry("when-to-expire")==expireAtExit) {
-      bool  expire = true;
-
-      if (config->readBoolEntry("warn-before-expire")) {
-	expire = canExpire();
-      }
-      if (expire) {
-	the_folderMgr->expireAllFolders(NULL);
-      }
     }
   }
 
+  if (mProgress)
+    mProgress->setProgress(2);
+
+  if (expire) {
+    if (mCleanupLabel)
+    {
+       mCleanupLabel->setText(i18n("Expiring old messages..."));
+       QApplication::syncX();
+       kapp->processEvents();
+    }
+    the_folderMgr->expireAllFolders(0);
+  }
+
+  if (mProgress)
+     mProgress->setProgress(2+nrFolders);
+
   if (closed_by_user && the_folderMgr) {
     if (config->readBoolEntry("compact-all-on-exit", true))
+    {
+      if (mCleanupLabel)
+      {
+        mCleanupLabel->setText(i18n("Compacting folders..."));
+        QApplication::syncX();
+        kapp->processEvents();
+      }
       the_folderMgr->compactAll(); // I can compact for ages in peace now!
+    }
   }
-  
+  if (mProgress)
+     mProgress->setProgress(2+2*nrFolders);
+
   if (the_inboxFolder) the_inboxFolder->close(TRUE);
   if (the_outboxFolder) the_outboxFolder->close(TRUE);
   if (the_sentFolder) the_sentFolder->close(TRUE);
   if (the_draftsFolder) the_draftsFolder->close(TRUE);
-  
+
   delete the_msgDict;
-  
+  the_msgDict = 0;
   delete the_folderMgr;
   the_folderMgr = 0;
   delete the_imapFolderMgr;
@@ -682,6 +853,15 @@ void KMKernel::cleanup(void)
   //qInstallMsgHandler(oldMsgHandler);
   KMRecentAddresses::self()->save( KGlobal::config() );
   kapp->config()->sync();
+  if (mCleanupPopup)
+  {
+    sleep(1); // Give the user some time to realize what's going on
+    delete mCleanupPopup;
+    mCleanupPopup = 0;
+    mCleanupLabel = 0; // auto-deleted child of mCleanupPopup
+    mProgress = 0;     // ditto
+  }
+  kapp->exit_loop();
 }
 
 //Isnt this obsolete? (sven)
@@ -839,6 +1019,54 @@ void KMKernel::slotResult(KIO::Job *job)
   mPutJobs.remove(it);
 }
 
+void KMKernel::slotCollectStdOut( KProcess * proc, char * buffer, int len )
+{
+  QByteArray & ba = mStdOutCollection[proc];
+  // append data to ba:
+  int oldsize = ba.size();
+  ba.resize( oldsize + len );
+  qmemmove( ba.begin() + oldsize, buffer, len );
+}
+
+void KMKernel::slotCollectStdErr( KProcess * proc, char * buffer, int len )
+{
+  QByteArray & ba = mStdErrCollection[proc];
+  // append data to ba:
+  int oldsize = ba.size();
+  ba.resize( oldsize + len );
+  qmemmove( ba.begin() + oldsize, buffer, len );
+}
+
+QByteArray KMKernel::getCollectedStdOut( KProcess * proc )
+{
+  QByteArray result = mStdOutCollection[proc];
+  mStdOutCollection.remove(proc);
+  return result;
+}
+
+QByteArray KMKernel::getCollectedStdErr( KProcess * proc )
+{
+  QByteArray result = mStdErrCollection[proc];
+  mStdErrCollection.remove(proc);
+  return result;
+}
+
+void KMKernel::slotRequestConfigSync() {
+  // ### FIXME: delay as promised in the kdoc of this function ;-)
+  kapp->config()->sync();
+}
+
+void KMKernel::slotShowConfigurationDialog()
+{
+  if( !mConfigureDialog )
+    mConfigureDialog = new ConfigureDialog( 0, "configure", false );
+
+  if( mConfigureDialog->isHidden() )
+    mConfigureDialog->show();
+  else
+    mConfigureDialog->raise();
+}
+
 void KMKernel::notClosedByUser()
 {
   closed_by_user = false;
@@ -890,28 +1118,53 @@ KMKernel::canExpire() {
 /**
  * Returns true if the folder is either the outbox or one of the drafts-folders
  */
-bool KMKernel::folderIsDraftOrOutbox(KMFolder * folder)
+bool KMKernel::folderIsDraftOrOutbox(const KMFolder * folder)
 {
-	bool test = false;
-	
-	if (folder == the_outboxFolder || folder == the_draftsFolder)
-	{
-		test = true;
-		return test;
-	}	
+  assert( folder );
+  if ( folder == the_outboxFolder || folder == the_draftsFolder )
+    return true;
 
-	// search the identities if the folder matches the drafts-folder
-	QStringList identities = KMIdentity::identities();
-	QStringList::Iterator it = identities.begin();
-	for( ; it != identities.end(); ++it )
-	{
-		KMIdentity ident( *it );
-		ident.readConfig();
-		if (!ident.drafts().isEmpty() &&
-			ident.drafts() == folder->idString()) test = true;
-	}
+  QString idString = folder->idString();
+  if ( idString.isEmpty() ) return false;
 
-	return test;
+  // search the identities if the folder matches the drafts-folder
+  const IdentityManager * im = identityManager();
+  for( IdentityManager::ConstIterator it = im->begin(); it != im->end(); ++it )
+    if ( (*it).drafts() == idString ) return true;
+  return false;
+}
+
+bool KMKernel::folderIsTrash(KMFolder * folder)
+{
+  assert(folder);
+  if (folder == the_trashFolder) return true;
+  if (folder->protocol() != "imap") return false;
+  KMFolderImap *fi = static_cast<KMFolderImap*>(folder);
+  if (fi->account()->trash() == fi->idString()) return true;
+  return false;
+}
+
+IdentityManager * KMKernel::identityManager() {
+  if ( !mIdentityManager ) {
+    kdDebug(5006) << "instantating IdentityManager" << endl;
+    mIdentityManager = new IdentityManager( this, "mIdentityManager" );
+  }
+  return mIdentityManager;
+}
+
+KMMainWin* KMKernel::mainWin()
+{
+  KMainWindow *kmWin = 0;
+
+  for (kmWin = KMainWindow::memberList->first(); kmWin;
+    kmWin = KMainWindow::memberList->next())
+      if (kmWin->isA("KMMainWin")) break;
+  if (kmWin && kmWin->isA("KMMainWin")) {
+    return (KMMainWin *) kmWin;
+  } else {
+    mWin = new KMMainWin;
+    return mWin;
+  }
 }
 
 #include "kmkernel.moc"
