@@ -1,10 +1,12 @@
 // kmmsgbase.cpp
 
+#include <config.h>
 #include <kdebug.h>
 #include <kglobal.h>
 #include <klocale.h>
 #include <kcharsets.h>
 
+#include <kmime_util.h>
 #include <mimelib/mimepp.h>
 #include <qregexp.h>
 #include <qtextcodec.h>
@@ -12,6 +14,7 @@
 #include <kmfolder.h>
 #include <kmheaders.h>
 #include <kmmsgdict.h>
+#include <krfcdate.h>
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -28,7 +31,7 @@
 /* Swap bytes in 16 bit value.  */
 #ifdef bswap_16
 #define kmail_swap_16(x) bswap_16(x)
-#else 
+#else
 #define kmail_swap_16(x) \
      ((((x) >> 8) & 0xff) | (((x) & 0xff) << 8))
 #endif
@@ -74,7 +77,8 @@ KMMsgBase::KMMsgBase(KMFolder* aParent)
 {
   mParent  = aParent;
   mDirty   = FALSE;
-  mIndexOffset = mIndexLength = 0;
+  mIndexOffset = 0;
+  mIndexLength = 0;
 }
 
 
@@ -102,6 +106,13 @@ KMMsgBase& KMMsgBase::operator=(const KMMsgBase& other)
 }
 
 
+//----------------------------------------------------------------------------
+KMMsgBase::KMMsgBase( const KMMsgBase& other )
+{
+    assign( &other );
+}
+
+
 //-----------------------------------------------------------------------------
 bool KMMsgBase::isMessage(void) const
 {
@@ -149,6 +160,56 @@ void KMMsgBase::setStatus(const char* aStatusStr, const char* aXStatusStr)
   }
 }
 
+void KMMsgBase::setEncryptionState( const KMMsgEncryptionState status, int idx )
+{
+    kdDebug() << "***setEncryptionState1( " << status << " )" << endl;
+    mDirty = TRUE;
+    if (mParent)
+        mParent->headerOfMsgChanged(this, idx);
+}
+
+void KMMsgBase::setEncryptionStateChar( QChar status, int idx )
+{
+    kdDebug() << "***setEncryptionState2( " << (status.isNull() ? '?' : status.latin1()) << " )" << endl;
+    
+    if( status.latin1() == (char)KMMsgEncryptionStateUnknown )
+            setEncryptionState( KMMsgEncryptionStateUnknown, idx );
+    else if( status.latin1() == (char)KMMsgNotEncrypted )
+            setEncryptionState( KMMsgNotEncrypted, idx );
+    else if( status.latin1() == (char)KMMsgPartiallyEncrypted )
+            setEncryptionState( KMMsgPartiallyEncrypted, idx );
+    else if( status.latin1() == (char)KMMsgFullyEncrypted )
+            setEncryptionState( KMMsgFullyEncrypted, idx );
+        else
+            setEncryptionState( KMMsgEncryptionStateUnknown, idx );
+}
+
+
+void KMMsgBase::setSignatureState( const KMMsgSignatureState status, int idx )
+{
+    kdDebug() << "***setSignatureState1( " << status << " )" << endl;
+    mDirty = TRUE;
+    if (mParent)
+         mParent->headerOfMsgChanged(this, idx);
+}
+
+
+
+void KMMsgBase::setSignatureStateChar( QChar status, int idx )
+{
+    kdDebug() << "***setSignatureState2( " << (status.isNull() ? '?' : status.latin1()) << " )" << endl;
+    
+    if( status.latin1() == (char)KMMsgSignatureStateUnknown )
+            setSignatureState( KMMsgSignatureStateUnknown, idx );
+    else if( status.latin1() == (char)KMMsgNotSigned )
+            setSignatureState( KMMsgNotSigned, idx );
+    else if( status.latin1() == (char)KMMsgPartiallySigned )
+            setSignatureState( KMMsgPartiallySigned,idx );
+    else if( status.latin1() == (char)KMMsgFullySigned )
+            setSignatureState( KMMsgFullySigned, idx );
+    else
+        setSignatureState( KMMsgSignatureStateUnknown, idx );
+}
 
 //-----------------------------------------------------------------------------
 bool KMMsgBase::isUnread(void) const
@@ -179,10 +240,7 @@ const char* KMMsgBase::statusToStr(KMMsgStatus aStatus)
 //-----------------------------------------------------------------------------
 void KMMsgBase::setDate(const QCString& aDateStr)
 {
-  DwDateTime dwDate;
-  dwDate.FromString(aDateStr);
-  dwDate.Parse();
-  setDate(dwDate.AsUnixTime());
+  setDate( KRFCDate::parseDate( aDateStr ) );
 }
 
 
@@ -190,7 +248,7 @@ void KMMsgBase::setDate(const QCString& aDateStr)
 QString KMMsgBase::dateStr(void) const
 {
   time_t d = date();
-  return KMHeaders::fancyDate(d);
+  return KMime::DateFormatter::formatDate(KMime::DateFormatter::Fancy, d);
 }
 
 
@@ -224,7 +282,7 @@ QString KMMsgBase::skipKeyword(const QString& aStr, QChar sepChar,
 //-----------------------------------------------------------------------------
 QTextCodec* KMMsgBase::codecForName(const QCString& _str)
 {
-  if (_str.isEmpty()) return NULL;
+  if (_str.isEmpty()) return 0;
   return KGlobal::charsets()->codecForName(_str.lower());
 }
 
@@ -236,7 +294,7 @@ const QCString KMMsgBase::toUsAscii(const QString& _str, bool *ok)
   QString result = _str;
   int len = result.length();
   for (int i = 0; i < len; i++)
-    if (result.at(i).unicode() >= 128) { 
+    if (result.at(i).unicode() >= 128) {
       result.at(i) = '?';
       all_ok = false;
     }
@@ -287,13 +345,13 @@ QString KMMsgBase::decodeRFC2047String(const QCString& aStr)
   {
     QString str = kernel->networkCodec()->toUnicode(aStr);
     if (str.find('\n') == -1) return str;
-    QString str2((QChar*)NULL, str.length());
+    QString str2((QChar*)0, str.length());
     uint i = 0;
     while (i < str.length())
     {
       if (str[i] == '\n')
       {
-        str2 += " ";
+        str2 += ' ';
         i += 2;
       } else {
         str2 += str[i];
@@ -418,7 +476,7 @@ QCString KMMsgBase::encodeRFC2047Quoted(const QCString& aStr, bool base64)
     ch = aStr.at(i);
     if (ch >= 128 || ch == '_' || especials.find(aStr.at(i)) != -1)
     {
-      result += "=";
+      result += '=';
       hex = ((ch & 0xF0) >> 4) + 48;
       if (hex >= 58) hex += 7;
       result += hex;
@@ -548,7 +606,7 @@ QCString KMMsgBase::encodeRFC2231String(const QString& _str,
     for (i = 0; i < 17; i++) if (*l == especials[i]) quote = true;
     if (quote)
     {
-      result += "%";
+      result += '%';
       hexcode = ((*l & 0xF0) >> 4) + 48;
       if (hexcode >= 58) hexcode += 7;
       result += hexcode;
@@ -567,12 +625,12 @@ QCString KMMsgBase::encodeRFC2231String(const QString& _str,
 //-----------------------------------------------------------------------------
 QString KMMsgBase::decodeRFC2231String(const QCString& _str)
 {
-  int p = _str.find("'");
+  int p = _str.find('\'');
   if (p < 0) return kernel->networkCodec()->toUnicode(_str);
 
   QCString charset = _str.left(p);
 
-  QCString st = _str.mid(_str.findRev("'") + 1);
+  QCString st = _str.mid(_str.findRev('\'') + 1);
   char ch, ch2;
   p = 0;
   while (p < (int)st.length())
@@ -609,7 +667,7 @@ QCString KMMsgBase::autoDetectCharset(const QCString &_encoding, const QStringLi
     }
 
     QStringList::ConstIterator it = charsets.begin();
-    for (; it != charsets.end(); ++it) 
+    for (; it != charsets.end(); ++it)
     {
        QCString encoding = (*it).latin1();
        if (encoding == "locale")
@@ -622,7 +680,7 @@ QCString KMMsgBase::autoDetectCharset(const QCString &_encoding, const QStringLi
          if (ok)
             return encoding;
        }
-       else 
+       else
        {
          QTextCodec *codec = KMMsgBase::codecForName(encoding);
          if (!codec) {
@@ -676,7 +734,7 @@ QCString KMMsgBase::decodeBase64(const QCString& aStr)
   QCString bStr = aStr;
   if (aStr.isNull())
     bStr = "";
-  while (bStr.length() < 16) bStr += "=";
+  while (bStr.length() < 16) bStr += '=';
 
   DwString dwsrc(bStr.data(), bStr.length());
   DwString dwdest;
@@ -709,7 +767,7 @@ unsigned long KMMsgBase::getMsgSerNum() const
 {
   unsigned long msn = 0;
   if (mParent) {
-    int index = mParent->find((KMMsgBasePtr)this);
+    int index = mParent->find((KMMsgBase*)this);
     msn = kernel->msgDict()->getMsgSerNum(mParent, index);
   }
   return msn;
@@ -729,7 +787,7 @@ void swapEndian(QString &str)
 
 //-----------------------------------------------------------------------------
 static int g_chunk_length = 0, g_chunk_offset=0;
-static uchar *g_chunk = NULL;
+static uchar *g_chunk = 0;
 
 #define COPY_DATA(x, length) do { \
      if(g_chunk_offset + ((int)length) > g_chunk_length) {\
@@ -761,7 +819,7 @@ QString KMMsgBase::getStringPart(MsgPartType t) const
       return ret;
     if (g_chunk_length < mIndexLength)
 	g_chunk = (uchar *)realloc(g_chunk, g_chunk_length = mIndexLength);
-    int first_off=ftell(mParent->mIndexStream);
+    off_t first_off=ftell(mParent->mIndexStream);
     fseek(mParent->mIndexStream, mIndexOffset, SEEK_SET);
     fread( g_chunk, mIndexLength, 1, mParent->mIndexStream);
     fseek(mParent->mIndexStream, first_off, SEEK_SET);
@@ -794,10 +852,10 @@ QString KMMsgBase::getStringPart(MsgPartType t) const
   }
   if(using_mmap) {
       g_chunk_length = 0;
-      g_chunk = NULL;
+      g_chunk = 0;
   }
   // Normally we need to swap the byte order because the QStrings are written
-  // in the style of Qt2 (MSB -> network ordered). 
+  // in the style of Qt2 (MSB -> network ordered).
   // QStrings in Qt3 expect host ordering.
   // On e.g. Intel host ordering is LSB, on e.g. Sparc it is MSB.
 
@@ -806,15 +864,15 @@ QString KMMsgBase::getStringPart(MsgPartType t) const
   swapEndian(ret);
 #else
   // #warning Byte order is big endian (swap is false)
-#endif  
+#endif
 
   return ret;
 }
 
 //-----------------------------------------------------------------------------
-unsigned long KMMsgBase::getLongPart(MsgPartType t) const
+off_t KMMsgBase::getLongPart(MsgPartType t) const
 {
-  unsigned long ret = 0;
+  off_t ret = 0;
 
   g_chunk_offset = 0;
   bool using_mmap = FALSE;
@@ -832,7 +890,7 @@ unsigned long KMMsgBase::getLongPart(MsgPartType t) const
     assert(mIndexLength >= 0);
     if (g_chunk_length < mIndexLength)
       g_chunk = (uchar *)realloc(g_chunk, g_chunk_length = mIndexLength);
-    int first_off=ftell(mParent->mIndexStream);
+    off_t first_off=ftell(mParent->mIndexStream);
     fseek(mParent->mIndexStream, mIndexOffset, SEEK_SET);
     fread( g_chunk, mIndexLength, 1, mParent->mIndexStream);
     fseek(mParent->mIndexStream, first_off, SEEK_SET);
@@ -908,7 +966,7 @@ unsigned long KMMsgBase::getLongPart(MsgPartType t) const
             // We swap the result to host order.
             ret = kmail_swap_32(ret);
          }
-         
+
       }
       break;
     }
@@ -916,7 +974,7 @@ unsigned long KMMsgBase::getLongPart(MsgPartType t) const
   }
   if(using_mmap) {
     g_chunk_length = 0;
-    g_chunk = NULL;
+    g_chunk = 0;
   }
   return ret;
 }
@@ -926,7 +984,7 @@ unsigned long KMMsgBase::getLongPart(MsgPartType t) const
 const uchar *KMMsgBase::asIndexString(int &length) const
 {
   unsigned int csize = 256;
-  static uchar *ret = NULL; //different static buffer here for we may use the other buffer in the functions below
+  static uchar *ret = 0; //different static buffer here for we may use the other buffer in the functions below
   if(!ret)
     ret = (uchar *)malloc(csize);
   length = 0;
@@ -944,7 +1002,7 @@ const uchar *KMMsgBase::asIndexString(int &length) const
 #ifndef WORDS_BIGENDIAN
   // #warning Byte order is little endian (call swapEndian)
 #define SWAP_TO_NETWORK_ORDER(x) swapEndian(x)
-#else 
+#else
   // #warning Byte order is big endian
 #define SWAP_TO_NETWORK_ORDER(x)
 #endif
@@ -983,6 +1041,8 @@ const uchar *KMMsgBase::asIndexString(int &length) const
   STORE_DATA(MsgOffsetPart, tmp);
   tmp = date();
   STORE_DATA(MsgDatePart, tmp);
+  tmp = (signatureState() << 16) | encryptionState();
+  STORE_DATA(MsgCryptoStatePart, tmp);
 #undef STORE_DATA_LEN
   return ret;
 }
@@ -1001,4 +1061,4 @@ bool KMMsgBase::syncIndexString() const
   }
   return FALSE;
 }
-	
+
