@@ -2,6 +2,7 @@
 // Author: Marc Mutz <Marc@Mutz.com>
 // This code is under GPL
 
+#include <config.h>
 #include "kmsearchpatternedit.h"
 
 #include <klocale.h>
@@ -28,7 +29,7 @@ static QStringList sFilterFieldList, sFilterFuncList;
 //=============================================================================
 
 KMSearchRuleWidget::KMSearchRuleWidget(QWidget *parent, KMSearchRule *aRule, const char *name, bool headersOnly)
-  : QHBox(parent,name), 
+  : QHBox(parent,name),
     mRuleEditBut(0),
     mRegExpEditDialog(0)
 {
@@ -50,7 +51,7 @@ void KMSearchRuleWidget::initWidget()
   mRuleValue = new QLineEdit( this, "mRuleValue" );
 
   if( !KTrader::self()->query("KRegExpEditor/KRegExpEditor").isEmpty() ) {
-    mRuleEditBut = new QPushButton( i18n("Edit"), this, "mRuleEditBut" );
+    mRuleEditBut = new QPushButton( i18n("Edit..."), this, "mRuleEditBut" );
     connect( mRuleEditBut, SIGNAL( clicked() ), this, SLOT( editRegExp()));
     connect( mRuleFunc, SIGNAL( activated(int) ), this, SLOT( functionChanged(int) ) );
     functionChanged( mRuleFunc->currentItem() );
@@ -201,7 +202,7 @@ void KMSearchRuleWidget::initLists(bool headersOnly) const
     sFilterFieldList.append("Subject");
     sFilterFieldList.append("From");
     sFilterFieldList.append("To");
-    sFilterFieldList.append("Cc");
+    sFilterFieldList.append("CC");
     sFilterFieldList.append("Reply-To");
     sFilterFieldList.append("List-Id");
     sFilterFieldList.append("Organization");
