@@ -1,6 +1,6 @@
 // kmsender.cpp
 
-
+#include <config.h>
 #include <kio/passdlg.h>
 #include <kio/scheduler.h>
 #include <kapplication.h>
@@ -18,32 +18,29 @@
 
 #include "kmsender.h"
 #include "kmidentity.h"
+#include "kmkernel.h"
+#include "identitymanager.h"
 #include "kmbroadcaststatus.h"
 #include "kmaccount.h"
 #include "kmtransport.h"
 #include "kmfoldermgr.h"
 #include "kmmsgdict.h"
 #include "kbusyptr.h"
-
-#ifdef HAVE_PATHS_H
-#include <paths.h>
-#endif
-
-#ifndef _PATH_SENDMAIL
-#define _PATH_SENDMAIL  "/usr/sbin/sendmail"
-#endif
+#include "kmmessage.h"
+#include "kmmsgpart.h"
+#include <mimelib/mediatyp.h>
 
 #define SENDER_GROUP "sending mail"
 
 //-----------------------------------------------------------------------------
 KMSender::KMSender()
 {
-  mPrecommand = NULL;
-  mSendDlg = NULL;
-  mSendProc = NULL;
+  mPrecommand = 0;
+  mSendDlg = 0;
+  mSendProc = 0;
   mSendProcStarted = FALSE;
   mSendInProgress = FALSE;
-  mCurrentMsg = NULL;
+  mCurrentMsg = 0;
   mTransportInfo = new KMTransportInfo();
   readConfig();
   mSendAborted = false;
@@ -56,8 +53,8 @@ KMSender::KMSender()
 KMSender::~KMSender()
 {
   writeConfig(FALSE);
-  if (mSendProc) delete mSendProc;
-  if (mPrecommand) delete mPrecommand;
+  delete mSendProc;
+  delete mPrecommand;
   delete mTransportInfo;
 }
 
@@ -91,25 +88,14 @@ void KMSender::writeConfig(bool aWithSync)
 
 
 //-----------------------------------------------------------------------------
-bool KMSender::settingsOk(void) const
+bool KMSender::settingsOk() const
 {
-  KMIdentity ident( i18n( "Default" ));
-  ident.readConfig();
   if (KMTransportInfo::availableTransports().isEmpty())
   {
-    KMessageBox::information(0,i18n("Please specify a send "
-				    "method in the settings "
-				    "and try again."));
-    return FALSE;
+    KMessageBox::information(0,i18n("Please create an account for sending and try again."));
+    return false;
   }
-  if (!ident.mailingAllowed())
-  {
-    KMessageBox::information(0,i18n("Please set the required fields in the "
-				    "identity settings:\n"
-				    "user-name and email-address"));
-    return FALSE;
-  }
-  return TRUE;
+  return true;
 }
 
 
@@ -118,7 +104,7 @@ bool KMSender::send(KMMessage* aMsg, short sendNow)
 {
   int rc;
 
-  //assert(aMsg != NULL);
+  //assert(aMsg != 0);
   if(!aMsg)
     {
       return false;
@@ -150,13 +136,12 @@ bool KMSender::send(KMMessage* aMsg, short sendNow)
 
   // Handle redirections
   QString f = aMsg->headerField("X-KMail-Redirect-From");
-  if(f.length() > 0) {
-    QString idStr = aMsg->headerField("X-KMail-Identity");
-    KMIdentity ident( idStr.isEmpty() ? i18n( "Default" ) : idStr );
-    ident.readConfig();
-
-    aMsg->setFrom(f + QString(" (by way of %1 <%2>)").
-		  arg(ident.fullName()).arg(ident.emailAddr()));
+  if(!f.isEmpty()) {
+    uint id = aMsg->headerField("X-KMail-Identity").stripWhiteSpace().toUInt();
+    const KMIdentity & ident =
+      kernel->identityManager()->identityForUoidOrDefault( id );
+    aMsg->setFrom(f + QString(" (by way of %1 <%2>)")
+      .arg(ident.fullName()).arg(ident.emailAddr()));
   }
 
   rc = kernel->outboxFolder()->addMsg(aMsg);
@@ -181,6 +166,12 @@ bool KMSender::sendSingleMail( KMMessage*)
 
 }
 
+//-----------------------------------------------------------------------------
+void KMSender::outboxMsgAdded()
+{
+    ++mTotalMessages;
+}
+
 
 //-----------------------------------------------------------------------------
 bool KMSender::sendQueued(void)
@@ -196,7 +187,9 @@ bool KMSender::sendQueued(void)
   // open necessary folders
   kernel->outboxFolder()->open();
   mTotalMessages = kernel->outboxFolder()->count();
-  mCurrentMsg = NULL;
+  connect(kernel->outboxFolder(), SIGNAL(msgAdded(int)),
+          this, SLOT(outboxMsgAdded()));
+  mCurrentMsg = 0;
 
   kernel->sentFolder()->open();
 
@@ -211,6 +204,7 @@ void KMSender::doSendMsg()
 {
   KMFolder *sentFolder = 0, *imapSentFolder = 0;
   bool someSent = mCurrentMsg;
+  int rc;
   if (someSent) mSentMessages++;
   int percent = (mTotalMessages) ? (100 * mSentMessages / mTotalMessages) : 0;
   if (percent > 100) percent = 100;
@@ -220,13 +214,32 @@ void KMSender::doSendMsg()
   if (mCurrentMsg  && kernel->filterMgr())
   {
     mCurrentMsg->setTransferInProgress( FALSE );
+    if( mCurrentMsg->hasUnencryptedMsg() ) {
+kdDebug(5006) << "KMSender::doSendMsg() post-processing: replace mCurrentMsg body by unencryptedMsg data" << endl;
+      // delete all current body parts
+      mCurrentMsg->deleteBodyParts();
+      // copy Content-[..] headers from unencrypted message to current one
+      KMMessage & newMsg( *mCurrentMsg->unencryptedMsg() );
+      mCurrentMsg->dwContentType() = newMsg.dwContentType();
+      mCurrentMsg->setContentTransferEncodingStr( newMsg.contentTransferEncodingStr() );
+      QCString newDispo = newMsg.headerField("Content-Disposition").latin1();
+      if( newDispo.isEmpty() )
+        mCurrentMsg->removeHeaderField( "Content-Disposition" );
+      else
+        mCurrentMsg->setHeaderField( "Content-Disposition", newDispo );
+      // copy the body
+      mCurrentMsg->setBody( newMsg.body() );
+      // copy all the body parts
+      KMMessagePart msgPart;
+      for( int i = 0; i < newMsg.numBodyParts(); ++i ) {
+        newMsg.bodyPart( i, &msgPart );
+        mCurrentMsg->addBodyPart( &msgPart );
+      }
+    }
     mCurrentMsg->setStatus(KMMsgStatusSent);
 
-    QString msgIdentity = mCurrentMsg->headerField( "X-KMail-Identity" );
-		if (msgIdentity.isEmpty()) msgIdentity = i18n("Default");
-    kdDebug(5006) << "KMSender::doSendMsg: msgIdentity = " << msgIdentity << endl;
-    KMIdentity id( msgIdentity );
-    id.readConfig();
+    const KMIdentity & id = kernel->identityManager()
+      ->identityForUoidOrDefault( mCurrentMsg->headerField( "X-KMail-Identity" ).stripWhiteSpace().toUInt() );
     if ( !mCurrentMsg->fcc().isEmpty() )
     {
       sentFolder = kernel->folderMgr()->findIdString( mCurrentMsg->fcc() );
@@ -240,13 +253,17 @@ void KMSender::doSendMsg()
       if ( sentFolder == 0 )
         imapSentFolder = kernel->imapFolderMgr()->findIdString( id.fcc() );
     }
-    if (imapSentFolder && imapSentFolder->noContent()) imapSentFolder = NULL;
+    if (imapSentFolder && imapSentFolder->noContent()) imapSentFolder = 0;
 
     if ( sentFolder == 0 )
       sentFolder = kernel->sentFolder();
-    else
-      sentFolder->open();
-
+    else {
+      rc = sentFolder->open();
+      if (rc != 0) {
+	cleanup();
+        return;
+      }
+    }
 
     // 0==processed ok, 1==no filter matched, 2==critical error, abort!
     int processResult = kernel->filterMgr()->process(mCurrentMsg,KMFilterMgr::Outbound);
@@ -265,7 +282,17 @@ void KMSender::doSendMsg()
       return;
     case 1:
       sentFolder->quiet(TRUE);
-      sentFolder->moveMsg(mCurrentMsg);
+      if (sentFolder->moveMsg(mCurrentMsg) != 0)
+      {
+        KMessageBox::error(0, i18n("Moving the sent message \"%1\" from the "
+          "\"outbox\" to the \"sent-mail\" folder failed.\n"
+          "Possible reasons are lack of disk space or write permission. "
+          "Please try to fix the problem and move the message manually.")
+          .arg(mCurrentMsg->subject()));
+        cleanup();
+        sentFolder->quiet(FALSE);
+        return;
+      }
       if (imapSentFolder) imapSentFolder->moveMsg(mCurrentMsg);
       sentFolder->quiet(FALSE);
     default:
@@ -345,7 +372,7 @@ void KMSender::doSendMsg()
         if (!mPrecommand->start())
         {
           delete mPrecommand;
-          mPrecommand = NULL;
+          mPrecommand = 0;
         }
         return;
       }
@@ -367,11 +394,11 @@ void KMSender::sendProcStarted(bool success)
     if (mSendProc)
        mSendProc->finish(true);
     else
-      setStatusMsg(i18n("Unrecognised transport protocol, could not send message."));
+      setStatusMsg(i18n("Unrecognized transport protocol. Unable to send message."));
     mSendProc = 0;
     mSendProcStarted = false;
     cleanup();
-    return;
+    return;    
   }
   doSendMsgAux();
 }
@@ -403,15 +430,17 @@ void KMSender::doSendMsgAux()
 void KMSender::cleanup(void)
 {
   if (mSendProc && mSendProcStarted) mSendProc->finish(true);
-  mSendProc = NULL;
+  mSendProc = 0;
   mSendProcStarted = FALSE;
   if (mSendInProgress) kapp->deref();
   mSendInProgress = FALSE;
   if (mCurrentMsg)
   {
     mCurrentMsg->setTransferInProgress( FALSE );
-    mCurrentMsg = NULL;
+    mCurrentMsg = 0;
   }
+  disconnect(kernel->outboxFolder(), SIGNAL(msgAdded(int)),
+             this, SLOT(outboxMsgAdded()));
   kernel->sentFolder()->close();
   kernel->outboxFolder()->close();
   if (kernel->outboxFolder()->count()<0)
@@ -432,15 +461,15 @@ void KMSender::cleanup(void)
 void KMSender::slotAbortSend()
 {
   mSendAborted = true;
-  if (mPrecommand) delete mPrecommand;
-  mPrecommand = NULL;
+  delete mPrecommand;
+  mPrecommand = 0;
   if (mSendProc) mSendProc->abort();
 }
 
 //-----------------------------------------------------------------------------
 void KMSender::slotIdle()
 {
-  assert(mSendProc != NULL);
+  assert(mSendProc != 0);
 
   if (!mSendAborted) {
       if (mSendProc->sendOk()) {
@@ -479,7 +508,7 @@ void KMSender::slotIdle()
 void KMSender::slotPrecommandFinished(bool normalExit)
 {
   delete mPrecommand;
-  mPrecommand = NULL;
+  mPrecommand = 0;
   if (normalExit) mSendProc->start();
   else slotIdle();
 }
@@ -508,20 +537,20 @@ KMSendProc* KMSender::createSendProcFromString(QString transport)
   {
     mTransportInfo->readConfig(nr);
   } else {
-    if (transport.left(7) == "smtp://")
+    if (transport.startsWith("smtp://"))
     {
       mTransportInfo->type = "smtp";
       mTransportInfo->auth = FALSE;
       QString serverport = transport.mid(7);
       mTransportInfo->host = serverport;
       mTransportInfo->port = "25";
-      int colon = serverport.find(":");
+      int colon = serverport.find(':');
       if (colon != -1) {
         mTransportInfo->host = serverport.left(colon);
         mTransportInfo->port = serverport.mid(colon + 1);
       }
     }
-    else if (transport.left(7) == "file://")
+    else if (transport.startsWith("file://"))
     {
       mTransportInfo->type = "sendmail";
       mTransportInfo->host = transport.mid(7);
@@ -533,7 +562,7 @@ KMSendProc* KMSender::createSendProcFromString(QString transport)
   if (mTransportInfo->type == "smtp")
     return new KMSendSMTP(this);
 
-  return NULL;
+  return 0;
 }
 
 //-----------------------------------------------------------------------------
@@ -593,7 +622,7 @@ void KMSendProc::start(void)
 //-----------------------------------------------------------------------------
 bool KMSendProc::finish(bool destructive)
 {
-  if (destructive) delete this;
+  if (destructive) deleteLater();
   return TRUE;
 }
 
@@ -718,13 +747,13 @@ bool KMSendProc::addRecipients(const QStrList& aRecipientList)
 KMSendSendmail::KMSendSendmail(KMSender* aSender):
   KMSendSendmailInherited(aSender)
 {
-  mMailerProc = NULL;
+  mMailerProc = 0;
 }
 
 //-----------------------------------------------------------------------------
 KMSendSendmail::~KMSendSendmail()
 {
-  if (mMailerProc) delete mMailerProc;
+  delete mMailerProc;
 }
 
 //-----------------------------------------------------------------------------
@@ -743,12 +772,13 @@ void KMSendSendmail::start(void)
     .arg("sendmail://");
     KMessageBox::information(0,msg);
     emit started(false);
+    return;
   }
 
   if (!mMailerProc)
   {
     mMailerProc = new KProcess;
-    assert(mMailerProc != NULL);
+    assert(mMailerProc != 0);
     connect(mMailerProc,SIGNAL(processExited(KProcess*)),
 	    this, SLOT(sendmailExited(KProcess*)));
     connect(mMailerProc,SIGNAL(wroteStdin(KProcess*)),
@@ -762,18 +792,18 @@ void KMSendSendmail::start(void)
 //-----------------------------------------------------------------------------
 bool KMSendSendmail::finish(bool destructive)
 {
-  if (mMailerProc) delete mMailerProc;
-  mMailerProc = NULL;
+  delete mMailerProc;
+  mMailerProc = 0;
   if (destructive)
-      delete this;
+    	deleteLater(); 
   return TRUE;
 }
 
 //-----------------------------------------------------------------------------
 void KMSendSendmail::abort()
 {
-  if (mMailerProc) delete mMailerProc;
-  mMailerProc = NULL;
+  delete mMailerProc;
+  mMailerProc = 0;
   mSendOk = false;
   mMsgStr = 0;
   idle();
@@ -788,9 +818,17 @@ bool KMSendSendmail::send(KMMessage* aMsg)
   mMailerProc->clearArguments();
   *mMailerProc << mSender->transportInfo()->host;
   *mMailerProc << "-i";
-  addRecipients(aMsg->headerAddrField("To"));
-  if (!aMsg->cc().isEmpty()) addRecipients(aMsg->headerAddrField("Cc"));
-  if (!aMsg->bcc().isEmpty()) addRecipients(aMsg->headerAddrField("Bcc"));
+  
+  if( !aMsg->headerField("X-KMail-Recipients").isEmpty() ) {
+    // extended BCC handling to prevent TOs and CCs from seeing
+    // BBC information by looking at source of an OpenPGP encrypted mail
+    addRecipients(aMsg->headerAddrField("X-KMail-Recipients"));
+    aMsg->removeHeaderField( "X-KMail-Recipients" );
+  } else {
+    addRecipients(aMsg->headerAddrField("To"));
+    if (!aMsg->cc().isEmpty()) addRecipients(aMsg->headerAddrField("Cc"));
+    if (!aMsg->bcc().isEmpty()) addRecipients(aMsg->headerAddrField("Bcc"));
+  }
 
   mMsgStr = aMsg->asSendableString();
 
@@ -814,7 +852,7 @@ void KMSendSendmail::wroteStdin(KProcess *proc)
   char* str;
   int len;
 
-  assert(proc!=NULL);
+  assert(proc!=0);
 
   str = mMsgPos;
   len = (mMsgRest>1024 ? 1024 : mMsgRest);
@@ -837,7 +875,7 @@ void KMSendSendmail::wroteStdin(KProcess *proc)
 //-----------------------------------------------------------------------------
 void KMSendSendmail::receivedStderr(KProcess *proc, char *buffer, int buflen)
 {
-  assert(proc!=NULL);
+  assert(proc!=0);
   mMsg.replace(mMsg.length(), buflen, buffer);
 }
 
@@ -845,7 +883,7 @@ void KMSendSendmail::receivedStderr(KProcess *proc, char *buffer, int buflen)
 //-----------------------------------------------------------------------------
 void KMSendSendmail::sendmailExited(KProcess *proc)
 {
-  assert(proc!=NULL);
+  assert(proc!=0);
   mSendOk = (proc->normalExit() && proc->exitStatus()==0);
   if (!mSendOk) failed(i18n("Sendmail exited abnormally."));
   mMsgStr = 0;
@@ -856,7 +894,7 @@ void KMSendSendmail::sendmailExited(KProcess *proc)
 //-----------------------------------------------------------------------------
 bool KMSendSendmail::addOneRecipient(const QString& aRcpt)
 {
-  assert(mMailerProc!=NULL);
+  assert(mMailerProc!=0);
   if (!aRcpt.isEmpty()) *mMailerProc << aRcpt;
   return TRUE;
 }
@@ -885,35 +923,48 @@ KMSendSMTP::~KMSendSMTP()
 bool KMSendSMTP::send(KMMessage *aMsg)
 {
   KMTransportInfo *ti = mSender->transportInfo();
-  assert(aMsg != NULL);
+  assert(aMsg != 0);
 
   // email this is from
   mQuery = QString("headers=0&from=%1")
                   .arg(KURL::encode_string(KMMessage::getEmailAddr(aMsg->from())));
 
   // recipients
-  mQueryField = "&to=";
-  if(!addRecipients(aMsg->headerAddrField("To")))
-  {
-    return FALSE;
-  }
+  if( !aMsg->headerField("X-KMail-Recipients").isEmpty() ) {
+    // extended BCC handling to prevent TOs and CCs from seeing
+    // BBC information by looking at source of an OpenPGP encrypted mail
+    mQueryField = "&to=";
+    if( !addRecipients( aMsg->headerAddrField("X-KMail-Recipients")) ) {
+      return FALSE;
+    }
+    aMsg->removeHeaderField( "X-KMail-Recipients" );
+  } else {
+    mQueryField = "&to=";
+    if(!addRecipients(aMsg->headerAddrField("To")))
+    {
+      return FALSE;
+    }
 
-  if(!aMsg->cc().isEmpty())
-  {
-    mQueryField = "&cc=";
-    if(!addRecipients(aMsg->headerAddrField("Cc"))) return FALSE;
-  }
+    if(!aMsg->cc().isEmpty())
+    {
+      mQueryField = "&cc=";
+      if(!addRecipients(aMsg->headerAddrField("Cc"))) return FALSE;
+    }
 
-  QString bccStr = aMsg->bcc();
-  if(!bccStr.isEmpty())
-  {
-    mQueryField = "&bcc=";
-    if (!addRecipients(aMsg->headerAddrField("Bcc"))) return FALSE;
+    QString bccStr = aMsg->bcc();
+    if(!bccStr.isEmpty())
+    {
+      mQueryField = "&bcc=";
+      if (!addRecipients(aMsg->headerAddrField("Bcc"))) return FALSE;
+    }
   }
 
   if(!aMsg->subject().isEmpty())
     mQuery += QString("&subject=") + KURL::encode_string(aMsg->subject());
 
+  if (ti->specifyHostname)
+    mQuery += "&hostname=" + KURL::encode_string(ti->localHostname);
+
   KURL destination;
 
   destination.setProtocol((ti->encryption == "SSL") ? "smtps" : "smtp");
@@ -1034,7 +1085,7 @@ void KMSendSMTP::result(KIO::Job *_job)
   if(_job->error())
   {
     mSendOk = false;
-    if (_job->error() == KIO::ERR_SLAVE_DIED) mSlave = NULL;
+    if (_job->error() == KIO::ERR_SLAVE_DIED) mSlave = 0;
     failed(_job->errorString());
     abort();
   } else {
@@ -1046,9 +1097,9 @@ void KMSendSMTP::slaveError(KIO::Slave *aSlave, int error, const QString &errorM
 {
   if (aSlave == mSlave)
   {
-    if (error == KIO::ERR_SLAVE_DIED) mSlave = NULL;
+    if (error == KIO::ERR_SLAVE_DIED) mSlave = 0;
     mSendOk = false;
-    mJob = NULL;
+    mJob = 0;
     failed(KIO::buildErrorString(error, errorMsg));
     abort();
   }
