@@ -11,6 +11,8 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <qtimer.h>
+
 #include <kuniqueapplication.h>
 #include <klocale.h>
 #include <kglobal.h>
@@ -46,19 +48,19 @@
 static KCmdLineOptions kmoptions[] =
 {
   { "s", 0 , 0 },
-  { "subject <subject>",	I18N_NOOP("Set subject of msg."), 0 },
+  { "subject <subject>",	I18N_NOOP("Set subject of message."), 0 },
   { "c", 0 , 0 },
   { "cc <address>",		I18N_NOOP("Send CC: to 'address'."), 0 },
   { "b", 0 , 0 },
-  { "bcc <address>",		I18N_NOOP("Send BCC: to 'addres'."), 0 },
+  { "bcc <address>",		I18N_NOOP("Send BCC: to 'address'."), 0 },
   { "h", 0 , 0 },
-  { "header <header>",		I18N_NOOP("Add 'header' to msg."), 0 },
-  { "msg <file>",		I18N_NOOP("Read msg-body from 'file'."), 0 },
-  { "body <text>",              I18N_NOOP("Set body of msg."), 0 },
-  { "attach <url>",             I18N_NOOP("Add an attachment to the mail, this can be repeated"), 0 },
-  { "check",			I18N_NOOP("Check for new mail only."), 0 },
-  { "composer",			I18N_NOOP("Open only composer window."), 0 },
-  { "+[address]",		I18N_NOOP("Send msg to 'address'."), 0 },
+  { "header <header>",		I18N_NOOP("Add 'header' to message."), 0 },
+  { "msg <file>",		I18N_NOOP("Read message body from 'file'."), 0 },
+  { "body <text>",              I18N_NOOP("Set body of message."), 0 },
+  { "attach <url>",             I18N_NOOP("Add an attachment to the mail. This can be repeated."), 0 },
+  { "check",			I18N_NOOP("Only check for new mail."), 0 },
+  { "composer",			I18N_NOOP("Only open composer window."), 0 },
+  { "+[address]",		I18N_NOOP("Send message to 'address'."), 0 },
 //  { "+[file]",                  I18N_NOOP("Show message from file 'file'."), 0 },
   { 0, 0, 0}
 };
@@ -147,7 +149,7 @@ int KMailApplication::newInstance()
   if (args->getOption("msg"))
   {
      mailto = true;
-     messageFile = QString::fromLocal8Bit(args->getOption("msg"));
+     messageFile.setPath( QString::fromLocal8Bit(args->getOption("msg")) );
   }
 
   if (args->getOption("body"))
@@ -204,6 +206,20 @@ QString getMyHostName(void)
 }
 }
 
+static void checkConfigUpdates() {
+#if KDE_VERSION >= 306
+  KConfig * config = kapp->config();
+  const QString updateFile = QString::fromLatin1("kmail.upd");
+  QStringList updates;
+  updates << "9"
+	  << "3.1-update-identities"
+	  << "3.1-use-identity-uoids"
+          << "3.1-new-mail-notification";
+  for ( QStringList::const_iterator it = updates.begin() ; it != updates.end() ; ++it )
+    config->checkUpdate( *it, updateFile );
+#endif
+}
+
 int main(int argc, char *argv[])
 {
   // WABA: KMail is a KUniqueApplication. Unfortunately this makes debugging
@@ -218,10 +234,10 @@ int main(int argc, char *argv[])
                    I18N_NOOP("(c) 1997-2002, The KMail developers"),
 		   0,
 		   "http://kmail.kde.org");
-  about.addAuthor( "Michael H\303\244ckel", I18N_NOOP("Current maintainer"), "haeckel@kde.org" );
+  about.addAuthor( "Michael H\303\244ckel", I18N_NOOP("Current release coordinator"), "haeckel@kde.org" );
   about.addAuthor( "Don Sanders", I18N_NOOP("Core developer and former maintainer"), "sanders@kde.org" );
   about.addAuthor( "Stefan Taferner ", I18N_NOOP("Original author"), "taferner@kde.org" );
-  about.addAuthor( "Ingo Kl\303\266cker", I18N_NOOP("Encryption"), "ingo.kloecker@epost.de" );
+  about.addAuthor( "Ingo Kl\303\266cker", I18N_NOOP("Encryption"), "kloecker@kde.de" );
   about.addAuthor( "Marc Mutz", I18N_NOOP("Core developer"), "mutz@kde.org" );
   about.addAuthor( "Daniel Naber", I18N_NOOP("Documentation"), "daniel.naber@t-online.de" );
   about.addAuthor( "Andreas Gungl", I18N_NOOP("Encryption"), "a.gungl@gmx.de" );
@@ -230,6 +246,7 @@ int main(int argc, char *argv[])
   about.addAuthor( "Waldo Bastian", 0, "bastian@kde.org" );
   about.addAuthor( "Carsten Burghardt", 0, "carsten.burghardt@web.de" );
   about.addAuthor( "Steven Brown", 0, "swbrown@ucsd.edu" );
+  about.addAuthor( "Matthias Kalle Dalheimer", 0, "kalle@kde.org" );
   about.addAuthor( "Cristi Dumitrescu", 0, "cristid@chip.ro" );
   about.addAuthor( "Philippe Fremy", 0, "pfremy@chez.com" );
   about.addAuthor( "Kurt Granroth", 0, "granroth@kde.org" );
@@ -250,7 +267,9 @@ int main(int argc, char *argv[])
   about.addAuthor( "Sven Radej", 0, "radej@kde.org" );
   about.addAuthor( "Mark Roberts", 0, "mark@taurine.demon.co.uk" );
   about.addAuthor( "Wolfgang Rohdewald", 0, "wrohdewald@dplanet.ch" );
+  about.addAuthor( "Zack Rusin", 0, "zack@kde.org" );
   about.addAuthor( "Espen Sand", 0, "espen@kde.org" );
+  about.addAuthor( "Aaron J. Seigo", 0, "aseigo@olympusproject.org" );
   about.addAuthor( "Jan Simonson", 0, "jan@simonson.pp.se" );
   about.addAuthor( "George Staikos", 0, "staikos@kde.org" );
   about.addAuthor( "Jason Stephenson", 0, "panda@mis.net" );
@@ -264,6 +283,10 @@ int main(int argc, char *argv[])
   about.addAuthor( "Thorsten Zachmann", 0, "t.zachmann@zagge.de" );
   about.addAuthor( "Karl-Heinz Zimmer", 0, "khz@kde.org" );
 
+  about.addCredit( "Bernhard Reiter", 0, "bernhard@intevation.de" );
+  about.addCredit( "Jan-Oliver Wagner", 0, "jan@intevation.de" );
+  about.addCredit( "Wolfgang Westphal", 0, "wolfgang.westphal@gmx.de" );
+
   KCmdLineArgs::init(argc, argv, &about);
   KCmdLineArgs::addCmdLineOptions( kmoptions ); // Add kmail options
 
@@ -273,6 +296,9 @@ int main(int argc, char *argv[])
   KMailApplication app;
   KGlobal::locale()->insertCatalogue("libkdenetwork");
 
+  // Check that all updates have been run on the config file:
+  checkConfigUpdates();
+
   // Check and create a lock file to prevent concurrent access to kmail files
   const QString lockLocation = locateLocal("appdata", "lock");
   KSimpleConfig config(lockLocation);
@@ -286,10 +312,10 @@ int main(int argc, char *argv[])
   // proceed if the pid doesn't exist
       (kill(oldPid, 0) != -1 || errno != ESRCH))
   {
-  // Information left out here due to the message freeze.
     QString msg = i18n("Only one instance of KMail can be run at "
       "any one time. It is already running on a different display "
-      "with PID %1.").arg(oldPid);
+      "with PID %1 on host %2 according to the lock file located "
+      "at %3.").arg(oldPid).arg(oldHostName).arg(lockLocation);
 
     KNotifyClient::userEvent( msg,  KNotifyClient::Messagebox,
       KNotifyClient::Error );
@@ -318,6 +344,7 @@ int main(int argc, char *argv[])
   setSignalHandler(signalHandler);
 
   kapp->dcopClient()->resume(); // Ok. We are ready for DCOP requests.
+  kernel->setStartingUp( false ); // Starting up is finished
   // Go!
   int ret = kapp->exec();
 
