@@ -108,7 +108,7 @@ void NewsSourceItem::slotGotIcon(const KURL &url, const QPixmap &pixmap)
 
 KCMNewsTicker::KCMNewsTicker(QWidget *parent,  const char *name)
 	: KCModule(parent, name),
-	m_config(new KConfig(QString::fromLatin1("knewsticker_appletrc"), false, false)),
+	m_config(new KConfig(QString::fromLatin1("knewsticker_panelappletrc"), false, false)),
 	m_cfg(new ConfigAccess(m_config)),
 	m_child(new KCMNewsTickerWidget(this)),
 	m_newsIconMgr(NewsIconMgr::self())
@@ -119,7 +119,7 @@ KCMNewsTicker::KCMNewsTicker(QWidget *parent,  const char *name)
 	// Change various properties of the view which the Qt Designer cannot
 	// set at design time yet.
 	m_child->niInterval->setLabel(i18n("News query interval:"));
-	m_child->niInterval->setRange(0, 180, 1);
+	m_child->niInterval->setRange(4, 180);
 
 	m_child->lvNewsSources->setShowSortIndicator(true);
 	m_child->lvNewsSources->setSelectionModeExt(KListView::Extended);
@@ -414,9 +414,13 @@ void KCMNewsTicker::addFilter(const ArticleFilter &fd)
 
 void KCMNewsTicker::removeNewsSource()
 {
-	if (KMessageBox::warningYesNo(this, i18n("<p>Do you really want to remove these %1 news"
-							" source(s)?</p><p>Press 'Yes' to remove the news source from the list,"
-							" press 'No' to keep it and close this dialog.</p>").arg(m_child->lvNewsSources->selectedItems().count())) == KMessageBox::Yes) {
+	if (KMessageBox::warningYesNo(this, i18n("<p>Do you really want to remove %n news"
+							" source?</p><p>Press 'Yes' to remove the news source from the list,"
+							" press 'No' to keep it and close this dialog.</p>",
+						 "<p>Do you really want to remove these %n news"
+							" sources?</p><p>Press 'Yes' to remove the news sources from the list,"
+							" press 'No' to keep them and close this dialog.</p>",
+						 m_child->lvNewsSources->selectedItems().count())) == KMessageBox::Yes) {
 		int itemCount = m_child->lvNewsSources->selectedItems().count();
 		for (int j = 0; j < itemCount; j++) {
 			if (m_child->lvNewsSources->selectedItems().isEmpty()) { break; }
@@ -646,7 +650,7 @@ void KCMNewsTicker::slotFilterExpressionChanged(const QString &expression)
 
 extern "C"
 {
-	KCModule *create_newsticker(QWidget *parent, const char *)
+	KCModule *create_knewsticker(QWidget *parent, const char *)
 	{
 		return new KCMNewsTicker(parent, "kcmnewsticker");
 	};
