@@ -28,7 +28,7 @@ class NewsSourceDlgImpl : public NewsSourceDlg
 		NewsSourceDlgImpl(QWidget * = 0, const char * = 0, bool = FALSE, WFlags = 0);
 		~NewsSourceDlgImpl();
 
-		void setup(const NewsSourceBase::Data &);
+		void setup(const NewsSourceBase::Data &, bool);
 
 	signals:
 		void newsSource(const NewsSourceBase::Data &);
