@@ -17,11 +17,11 @@
 #ifndef KNARTICLEFACTORY_H
 #define KNARTICLEFACTORY_H
 
-#include <qsemimodal.h>
-#include <qlist.h>
+#include <qdialog.h>
+#include <qptrlist.h>
 
 #include "knjobdata.h"
-#include "knmime.h"
+#include "knarticle.h"
 #include "knwidgets.h"
 
 class QLabel;
@@ -55,7 +55,7 @@ class KNArticleFactory : public QObject , public KNJobConsumer {
     void createForward(KNArticle *a);
     void createCancel(KNArticle *a);
     void createSupersede(KNArticle *a);
-    void createMail(KNHeaders::AddressField *address);
+    void createMail(KMime::Headers::AddressField *address);
 
     // send a mail via an external program...
     void sendMailExternal(const QString &address=QString::null, const QString &subject=QString::null, const QString &body=QString::null);
@@ -86,7 +86,7 @@ class KNArticleFactory : public QObject , public KNJobConsumer {
     //send-errors
     void showSendErrorDialog();
 
-    QList<KNComposer> c_ompList;
+    QPtrList<KNComposer> c_ompList;
     KNSendErrorDialog *s_endErrDlg;
 
   protected slots:
@@ -96,7 +96,7 @@ class KNArticleFactory : public QObject , public KNJobConsumer {
 };
 
 
-class KNSendErrorDialog : public QSemiModal  {
+class KNSendErrorDialog : public QDialog  {
 
   Q_OBJECT
 
