@@ -46,7 +46,7 @@ class KNCleanUp {
 
   protected:
 
-    class ProgressDialog : public QSemiModal  {
+    class ProgressDialog : public QDialog  {
 
       public:
         ProgressDialog(int steps);
@@ -65,7 +65,7 @@ class KNCleanUp {
     };
 
     ProgressDialog *d_lg;
-    QList<KNArticleCollection> c_olList;
+    QPtrList<KNArticleCollection> c_olList;
     KNConfig::Cleanup *c_onfig;
 
 };
