@@ -34,6 +34,7 @@
 #include <kcombobox.h>
 #include <kspell.h>
 #include <ktempfile.h>
+#include <kpgp.h>
 #include <kpgpblock.h>
 #include <kprocess.h>
 #include <kqcstringsplitter.h>
@@ -95,7 +96,7 @@ KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &si
   new KAction(i18n("Send &Later"), "queue", 0, this,
     SLOT(slotSendLater()), actionCollection(), "send_later");
 
-  new KAction(i18n("Save As &Draft"),"filesave", 0 , this,
+  new KAction(i18n("Save as &Draft"),"filesave", 0 , this,
     SLOT(slotSaveAsDraft()), actionCollection(), "save_as_draft");
 
   new KAction(i18n("D&elete"),"editdelete", 0 , this,
@@ -187,7 +188,7 @@ KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &si
   new KAction(i18n("Re&move Box"), 0, v_iew->e_dit,
               SLOT(slotRemoveBox()), actionCollection(), "tools_unbox");
 
-  KAction *undoRewrap = new KAction(i18n("Get &Original Text (not rewraped)"), 0, this,
+  KAction *undoRewrap = new KAction(i18n("Get &Original Text (not re-wrapped)"), 0, this,
                                     SLOT(slotUndoRewrap()), actionCollection(), "tools_undoRewrap");
   undoRewrap->setEnabled(u_nwraped!=QString::null);
 
@@ -243,6 +244,18 @@ KNComposer::KNComposer(KNLocalArticle *a, const QString &text, const QString &si
     v_iew->e_dit->setCursorPosition(0,0);
   v_iew->e_dit->setFocus();
 
+  if (v_iew->s_ubject->text().length() == 0) {    
+    v_iew->s_ubject->setFocus();
+  }      
+
+  if (v_iew->g_roups->text().length() == 0 && m_ode == news) {    
+    v_iew->g_roups->setFocus();    
+  }
+
+  if (v_iew->t_o->text().length() == 0 && m_ode == mail) {    
+    v_iew->t_o->setFocus();    
+  }
+
   if(firstEdit && knGlobals.cfgManager->postNewsComposer()->appendOwnSignature())
     slotAppendSig();
 
@@ -387,47 +400,47 @@ bool KNComposer::hasValidData()
   }
 
   //GNKSA body checks
+  bool firstLine = true;
   bool empty = true;
   bool longLine = false;
   bool hasAttributionLine = false;
   int sigLength = 0;
   int notQuoted = 0;
-  int textLines = 0;
-  QString line;
-
-  for(int i=0; i<v_iew->e_dit->numLines(); i++) {
-    line=v_iew->e_dit->textLine(i);
-
-    if (!n_eeds8Bit && !KMime::isUsAscii(line))
+  int textLines = 0;  
+  QStringList text = v_iew->e_dit->processedText();
+  
+  for (QStringList::Iterator it = text.begin(); it != text.end(); ++it) {
+  
+    if (!n_eeds8Bit && !KMime::isUsAscii(*it))
       n_eeds8Bit=true;
 
-    if (line == "-- ") {   // signature text
-      for (int j=i+1; j<v_iew->e_dit->numLines(); j++) {
-        line=v_iew->e_dit->textLine(j);
+    if (*it == "-- ") {   // signature text
+      for (++it; it != text.end(); ++it) {        
 
-        if (!n_eeds8Bit && !KMime::isUsAscii(line))
+        if (!n_eeds8Bit && !KMime::isUsAscii(*it))
           n_eeds8Bit=true;
 
         sigLength++;
-        if(line.length()>80) {
-          longLine = true;
-          break;
+        if((*it).length()>80) {
+          longLine = true;          
         }
       }
       break;
     }
-    if(!line.isEmpty()) {
+    
+    if(!(*it).isEmpty()) {
       empty = false;
       textLines++;
-      if (line[0]!='>') {
+      if ((*it)[0]!='>') {
         notQuoted++;
-        if (i==0) hasAttributionLine = true;
+        if (firstLine) hasAttributionLine = true;
       }
     }
-    if(line.length()>80) {
-      longLine = true;
-      break;
+    if((*it).length()>80) {
+      longLine = true;      
     }
+    
+    firstLine = false;
   }
 
   if (n_eeds8Bit && (c_harset.lower()=="us-ascii")) {
@@ -453,7 +466,7 @@ bool KNComposer::hasValidData()
     }
   }
 
-  if ( longLine && ( (v_iew->e_dit->wordWrap() == QMultiLineEdit::NoWrap) ) )
+  if (longLine)
     if (!(KMessageBox::warningYesNo( this, 
           i18n("Your article contains lines longer than 80 characters.\n"
 	       "Do you want to re-edit the article or send it anyway?"),
@@ -470,6 +483,37 @@ bool KNComposer::hasValidData()
        KMessageBox::information(this, i18n("Your signature exceeds the widely accepted limit of 4 lines.\nPlease consider shortening your signature.\nOtherwise, you will probably annoy your readers"),
                                 QString::null,"longSignatureWarning");
 
+  // check if article can be signed
+  if ( a_ctPGPsign->isChecked() ) {
+    // try to get the signing key
+    QCString signingKey = knGlobals.cfgManager->identity()->signingKey();
+    KNNntpAccount *acc = knGlobals.accManager->account( a_rticle->serverId() );
+    if ( acc ) {
+      KMime::Headers::Newsgroups *grps = a_rticle->newsgroups();
+      KNGroup *grp = knGlobals.grpManager->group( grps->firstGroup(), acc );
+      if ( grp && grp->identity() && grp->identity()->hasSigningKey() )
+        signingKey = grp->identity()->signingKey();
+      else if ( acc->identity() && acc->identity()->hasSigningKey() )
+        signingKey = acc->identity()->signingKey();
+    }
+    
+    // the article can only be signed if we have a key  
+    if (signingKey.isEmpty()) {    
+          if ( KMessageBox::warningContinueCancel( this, 
+                   i18n("You haven't configured your preferred "
+                        "signing key yet.\n"
+                        "Please specify it in the global "
+                        "identity configuration,\n"
+                        "in the account properties or in the "
+                        "group properties!\n"
+                        "The article will be sent unsigned." ),
+                   QString::null, i18n( "Send Unsigned" ),
+                   "sendUnsignedDialog" ) 
+               == KMessageBox::Cancel )
+             return false;
+    }
+  }      
+
   v_alidated=true;
   return true;
 }
@@ -488,7 +532,7 @@ bool KNComposer::applyChanges()
 
   //Newsgroups
   if (m_ode != mail) {
-    a_rticle->newsgroups()->fromUnicodeString(v_iew->g_roups->text(), KMime::Headers::Latin1);
+    a_rticle->newsgroups()->fromUnicodeString(v_iew->g_roups->text().remove(QRegExp("\\s")), KMime::Headers::Latin1);
     a_rticle->setDoPost(true);
   } else
     a_rticle->setDoPost(false);
@@ -558,29 +602,13 @@ bool KNComposer::applyChanges()
     else
       text->contentTransferEncoding()->setCte(pnt->allow8BitBody()? KMime::Headers::CE8Bit : KMime::Headers::CEquPr);
   }
-
-  //auto-wrapped lines in v_iew->e_dit don't get an "\n", so we have to
+  
   //assemble the text line by line
-  QString tmp = QString::null;
-  if ( v_iew->e_dit->wordWrap() != QMultiLineEdit::NoWrap ) {
-      QString line;
-      int num_lines = v_iew->e_dit->numLines();
-      for (int i = 0; i < num_lines; ++i) {
-          int lastLine = 0;
-          line = v_iew->e_dit->textLine(i);
-          for (int j = 0; j < (int)line.length(); ++j) {
-              if (v_iew->e_dit->lineOfChar(i, j) > lastLine) {
-                  lastLine = v_iew->e_dit->lineOfChar(i, j);
-                  tmp += '\n';
-              }
-              tmp += line[j];
-          }
-	  tmp += '\n';
-      }
-  }
-  if ( tmp == QString::null)
-    tmp = v_iew->e_dit->text();
-
+  QString tmp;
+  QStringList textLines = v_iew->e_dit->processedText();
+  for (QStringList::Iterator it = textLines.begin(); it != textLines.end(); ++it)    
+    tmp += *it + "\n";
+  
   // Sign article if needed
   if ( a_ctPGPsign->isChecked() ) {
       // first get the signing key
@@ -597,33 +625,19 @@ bool KNComposer::applyChanges()
       // now try to sign the article
       if (!signingKey.isEmpty()) {
           QString tmpText = tmp;
-	  Kpgp::Block block;
+          Kpgp::Block block;
           bool ok=true;
           QTextCodec *codec=KGlobal::charsets()->codecForName(c_harset, ok);
           if(!ok) // no suitable codec found => try local settings and hope the best ;-)
               codec=KGlobal::locale()->codecForEncoding();
 
-	  block.setText( codec->fromUnicode(tmpText) );
+          block.setText( codec->fromUnicode(tmpText) );
           kdDebug(5003) << "signing article from " << article()->from()->email() << endl;
-	  if( block.clearsign( signingKey, codec->name() ) ) {
-	      QCString result = block.text();
+          if( block.clearsign( signingKey, codec->name() ) == Kpgp::Ok ) {
+              QCString result = block.text();
               tmp = codec->toUnicode(result.data(), result.length() );
           }
-      }
-      else {
-          if ( KMessageBox::warningContinueCancel( this, 
-                   i18n("You haven't configured your preferred "
-                        "signing key yet.\n"
-                        "Please specify it in the global "
-                        "identity configuration,\n"
-                        "in the account properties or in the "
-                        "group properties!\n"
-                        "The article will be sent unsigned." ),
-                   QString::null, i18n( "Send unsigned" ),
-                   "sendUnsignedDialog" ) 
-               == KMessageBox::Cancel )
-	      return false;
-      }
+      }     
   }
 
   text->fromUnicodeString(tmp);
@@ -661,12 +675,9 @@ void KNComposer::closeEvent(QCloseEvent *e)
     }
   }
 
-  d_oneSuccess = true;
-  emit composerDone(this);
-  if(d_oneSuccess)
     e->accept();
-  else
-    e->ignore();
+  emit composerDone(this);
+  // we're dead at this point, don't access members!
 }
 
 
@@ -941,10 +952,11 @@ void KNComposer::slotToggleDoMail()
       if (!s.contains(i18n("<posted & mailed>")))
         v_iew->e_dit->insertAt(i18n("<posted & mailed>\n\n"),0,0);
       QString tmp;
-      for(int i=0; i < v_iew->e_dit->numLines(); i++) {
-        if (v_iew->e_dit->textLine(i) == "-- ")   // try to be smart, don't include the signature,
-          break;                                  // kmail will append one, too.
-        tmp+=v_iew->e_dit->textLine(i)+"\n";
+      QStringList textLines = v_iew->e_dit->processedText();
+      for (QStringList::Iterator it = textLines.begin(); it != textLines.end(); ++it) {
+        if (*it == "-- ")   // try to be smart, don't include the signature,
+          break;            // kmail will append one, too.
+        tmp+=*it+"\n";
       }
       knGlobals.artFactory->sendMailExternal(v_iew->t_o->text(), v_iew->s_ubject->text(), tmp);
       a_ctDoMail->setChecked(false); //revert
@@ -1032,10 +1044,12 @@ void KNComposer::slotExternalEditor()
   QTextCodec *codec=KGlobal::charsets()->codecForName(c_harset, ok);
 
   QString tmp;
-  for(int i=0; i < v_iew->e_dit->numLines(); i++) {
-     tmp+=v_iew->e_dit->textLine(i);
-     if (i+1 < v_iew->e_dit->numLines())
-       tmp+="\n";
+  QStringList textLines = v_iew->e_dit->processedText();
+  for (QStringList::Iterator it = textLines.begin(); it != textLines.end();) {
+    tmp += *it;
+    ++it;
+    if (it != textLines.end())
+      tmp+="\n";
   }
 
   QCString local = codec->fromUnicode(tmp);
@@ -1138,15 +1152,15 @@ void KNComposer::slotUpdateStatusBar()
   statusBar()->changeItem(i18n(" Type: %1 ").arg(typeDesc), 1);
   statusBar()->changeItem(i18n(" Charset: %1 ").arg(c_harset), 2);
   statusBar()->changeItem(overwriteDesc, 3);
-  statusBar()->changeItem(i18n(" Column: %1 ").arg(v_iew->e_dit->currentColumn()), 4);
-  statusBar()->changeItem(i18n(" Line: %1 ").arg(v_iew->e_dit->currentLine()), 5);
+  statusBar()->changeItem(i18n(" Column: %1 ").arg(v_iew->e_dit->currentColumn() + 1), 4);
+  statusBar()->changeItem(i18n(" Line: %1 ").arg(v_iew->e_dit->currentLine() + 1), 5);
 }
 
 
 void KNComposer::slotUpdateCursorPos()
 {
-  statusBar()->changeItem(i18n(" Column: %1 ").arg(v_iew->e_dit->currentColumn()), 4);
-  statusBar()->changeItem(i18n(" Line: %1 ").arg(v_iew->e_dit->currentLine()), 5);
+  statusBar()->changeItem(i18n(" Column: %1 ").arg(v_iew->e_dit->currentColumn() + 1), 4);
+  statusBar()->changeItem(i18n(" Line: %1 ").arg(v_iew->e_dit->currentLine() + 1), 5);
 }
 
 
@@ -1235,7 +1249,7 @@ void KNComposer::slotGroupsBtnClicked()
   if(id==-1)
     a_rticle->setServerId(nntp->id());
 
-  KNGroupSelectDialog *dlg=new KNGroupSelectDialog(this, nntp, v_iew->g_roups->text());
+  KNGroupSelectDialog *dlg=new KNGroupSelectDialog(this, nntp, v_iew->g_roups->text().remove(QRegExp("\\s")));
 
   connect(dlg, SIGNAL(loadList(KNNntpAccount*)),
     knGlobals.grpManager, SLOT(slotLoadGroupList(KNNntpAccount*)));
@@ -1408,30 +1422,22 @@ KNComposer::ComposerView::ComposerView(QWidget *p, const char *n)
   QGridLayout *hdrL=new QGridLayout(hdrFrame, 4,3, 7,5);
   hdrL->setColStretch(1,1);
 
-  //subject
-  s_ubject=new KLineEdit(hdrFrame);
-  QLabel *l=new QLabel(s_ubject, i18n("S&ubject:"), hdrFrame);
-  hdrL->addWidget(l, 0,0);
-  hdrL->addMultiCellWidget(s_ubject, 0,0, 1,2);
-  connect(s_ubject, SIGNAL(textChanged(const QString&)),
-          parent(), SLOT(slotSubjectChanged(const QString&)));
-
   //To
   t_o=new KLineEdit(hdrFrame);
   l_to=new QLabel(t_o, i18n("T&o:"), hdrFrame);
   t_oBtn=new QPushButton(i18n("&Browse..."), hdrFrame);
-  hdrL->addWidget(l_to, 1,0);
-  hdrL->addWidget(t_o, 1,1);
-  hdrL->addWidget(t_oBtn, 1,2);
+  hdrL->addWidget(l_to, 0,0);
+  hdrL->addWidget(t_o, 0,1);
+  hdrL->addWidget(t_oBtn, 0,2);
   connect(t_oBtn, SIGNAL(clicked()), parent(), SLOT(slotToBtnClicked()));
 
   //Newsgroups
   g_roups=new KLineEdit(hdrFrame);
   l_groups=new QLabel(g_roups, i18n("&Groups:"), hdrFrame);
   g_roupsBtn=new QPushButton(i18n("B&rowse..."), hdrFrame);
-  hdrL->addWidget(l_groups, 2,0);
-  hdrL->addWidget(g_roups, 2,1);
-  hdrL->addWidget(g_roupsBtn, 2,2);
+  hdrL->addWidget(l_groups, 1,0);
+  hdrL->addWidget(g_roups, 1,1);
+  hdrL->addWidget(g_roupsBtn, 1,2);
   connect(g_roups, SIGNAL(textChanged(const QString&)),
           parent(), SLOT(slotGroupsChanged(const QString&)));
   connect(g_roupsBtn, SIGNAL(clicked()), parent(), SLOT(slotGroupsBtnClicked()));
@@ -1439,9 +1445,17 @@ KNComposer::ComposerView::ComposerView(QWidget *p, const char *n)
   //Followup-To
   f_up2=new KComboBox(true, hdrFrame);
   l_fup2=new QLabel(f_up2, i18n("Follo&wup-To:"), hdrFrame);
-  hdrL->addWidget(l_fup2, 3,0);
-  hdrL->addMultiCellWidget(f_up2, 3,3, 1,2);
+  hdrL->addWidget(l_fup2, 2,0);
+  hdrL->addMultiCellWidget(f_up2, 2,2, 1,2);
 
+  //subject
+  s_ubject=new KLineEdit(hdrFrame);
+  QLabel *l=new QLabel(s_ubject, i18n("S&ubject:"), hdrFrame);
+  hdrL->addWidget(l, 3,0);
+  hdrL->addMultiCellWidget(s_ubject, 3,3, 1,2);
+  connect(s_ubject, SIGNAL(textChanged(const QString&)),
+          parent(), SLOT(slotSubjectChanged(const QString&)));
+  
   //Editor
   e_dit=new Editor(main);
   e_dit->setMinimumHeight(50);
@@ -1603,29 +1617,59 @@ void KNComposer::ComposerView::hideExternalNotification()
 KNComposer::Editor::Editor(QWidget *parent, char *name)
   : KEdit(parent, name)
 {
-  setOverwriteEnabled(true);
-  installEventFilter(this);
+  setOverwriteEnabled(true);  
 }
 
 
 KNComposer::Editor::~Editor()
-{
-  removeEventFilter(this);
-}
+{}
 
 
-// expand tabs to avoid the "tab-damage"
-QString KNComposer::Editor::textLine(int line) const
+// expand tabs to avoid the "tab-damage",
+// auto-wraped paragraphs have to split (code taken from KEdit::saveText)
+QStringList KNComposer::Editor::processedText()
 {
-  QString temp=KEdit::textLine(line);
+  QStringList ret;
+  int lines = numLines()-1;
+  if (lines < 0)
+    return ret;
+
+  if (wordWrap() == NoWrap) {
+    for (int i = 0; i <= lines; i++)
+      ret.append(textLine(i));    
+  } else {
+    for (int i = 0; i <= lines; i++) {
+      int lines_in_parag = linesOfParagraph(i);
+
+      if (lines_in_parag == 1) {
+        ret.append(textLine(i));
+      } else {
+        QString parag_text = textLine(i);
+        int pos = 0;
+        int last_pos = 0;
+        int current_line = 0;       
+        while (current_line+1 < lines_in_parag) {
+          while (lineOfChar(i, pos) == current_line) pos++;            
+          ret.append(parag_text.mid(last_pos, pos - last_pos - 1));
+          current_line++;
+          last_pos = pos;         
+        }
+        // add last line
+        ret.append(parag_text.mid(pos));
+      }
+    }
+  }
+  
   QString replacement;
   int tabPos;
-
-  while((tabPos=temp.find('\t'))!=-1) {
-    replacement.fill(QChar(' '), 8-(tabPos%8));
-    temp.replace(tabPos, 1, replacement);
+  for (QStringList::Iterator it = ret.begin(); it != ret.end(); ++it ) {
+    while ((tabPos=(*it).find('\t'))!=-1) {
+      replacement.fill(QChar(' '), 8-(tabPos%8));
+      (*it).replace(tabPos, 1, replacement);
+    }
   }
-  return temp;
+
+  return ret;
 }
 
 
@@ -1659,14 +1703,14 @@ void KNComposer::Editor::slotReplace()
 void KNComposer::Editor::slotAddQuotes()
 {
   if (hasMarkedText()) {
-    QString s = markedText();
-    s.prepend("> ");
+    QString s = markedText();    
+    s.prepend("> ");    
     s.replace(QRegExp("\n"),"\n> ");
     insert(s);
   } else {
     int l = currentLine();
     int c = currentColumn();
-    QString s = KEdit::textLine(l);
+    QString s = textLine(l);
     s.prepend("> ");
     insertLine(s,l);
     removeLine(l+1);
@@ -1686,7 +1730,7 @@ void KNComposer::Editor::slotRemoveQuotes()
   } else {
     int l = currentLine();
     int c = currentColumn();
-    QString s = KEdit::textLine(l);
+    QString s = textLine(l);
     if (s.left(2) == "> ") {
       s.remove(0,2);
       insertLine(s,l);
@@ -1708,7 +1752,7 @@ void KNComposer::Editor::slotAddBox()
   } else {
     int l = currentLine();
     int c = currentColumn();
-    QString s = QString::fromLatin1(",----[  ]\n| %1\n`----").arg(KEdit::textLine(l));
+    QString s = QString::fromLatin1(",----[  ]\n| %1\n`----").arg(textLine(l));
     insertLine(s,l);
     removeLine(l+3);
     setCursorPosition(l+1,c+2);
@@ -1730,7 +1774,7 @@ void KNComposer::Editor::slotRemoveBox()
     int l = currentLine();
     int c = currentColumn();
 
-    QString s = KEdit::textLine(l);   // test if we are in a box
+    QString s = textLine(l);   // test if we are in a box
     if (!((s.left(2) == "| ")||(s.left(5)==",----")||(s.left(5)=="`----")))
       return;
 
@@ -1738,13 +1782,13 @@ void KNComposer::Editor::slotRemoveBox()
 
     // find & remove box begin
     int x = l;
-    while ((x>=0)&&(KEdit::textLine(x).left(5)!=",----"))
+    while ((x>=0)&&(textLine(x).left(5)!=",----"))
       x--;
-    if ((x>=0)&&(KEdit::textLine(x).left(5)==",----")) {
+    if ((x>=0)&&(textLine(x).left(5)==",----")) {
       removeLine(x);
       l--;
       for (int i=x;i<=l;i++) {     // remove quotation
-        s = KEdit::textLine(i);
+        s = textLine(i);
         if (s.left(2) == "| ") {
           s.remove(0,2);
           insertLine(s,i);
@@ -1755,12 +1799,12 @@ void KNComposer::Editor::slotRemoveBox()
 
     // find & remove box end
     x = l;
-    while ((x<numLines())&&(KEdit::textLine(x).left(5)!="`----"))
+    while ((x<numLines())&&(textLine(x).left(5)!="`----"))
       x++;
-    if ((x<numLines())&&(KEdit::textLine(x).left(5)=="`----")) {
+    if ((x<numLines())&&(textLine(x).left(5)=="`----")) {
       removeLine(x);
       for (int i=l+1;i<x;i++) {     // remove quotation
-        s = KEdit::textLine(i);
+        s = textLine(i);
         if (s.left(2) == "| ") {
           s.remove(0,2);
           insertLine(s,i);
@@ -1784,20 +1828,6 @@ void KNComposer::Editor::slotRot13()
 }
 
 
-// don't use Tab for focus handling
-bool KNComposer::Editor::eventFilter(QObject* o, QEvent* e)
-{
-  if (e->type() == QEvent::KeyPress) {
-    QKeyEvent *k = static_cast<QKeyEvent*>(e);
-    if (k->key()==Key_Tab) {
-      insert("\t");
-      return true;
-    }
-  }
-  return KEdit::eventFilter( o, e );
-}
-
-
 void KNComposer::Editor::dragEnterEvent(QDragEnterEvent *ev)
 {
   if (QUriDrag::canDecode(ev))
