@@ -36,8 +36,10 @@ class KLineEdit;
 class KComboBox;
 class KIntSpinBox;
 class KSpellConfig;
+class KURLCompletion;
 namespace Kpgp {
   class Config;
+  class SecretKeyRequester;
 };
 
 class KNNntpAccount;
@@ -165,7 +167,6 @@ class IdentityWidget : public BaseWidget {
 
     void apply();
 
-
   protected:
     QLabel          *f_ileName;
     KLineEdit       *n_ame,
@@ -174,24 +175,24 @@ class IdentityWidget : public BaseWidget {
                     *r_eplyTo,
                     *m_ailCopiesTo,
                     *s_ig;
-    QLabel          *s_igningKey;
     QRadioButton    *s_igFile,
                     *s_igEdit;
     QCheckBox       *s_igGenerator;
-    QPushButton     *c_hangeBtn,
-                    *c_hooseBtn,
+    QPushButton     *c_hooseBtn,
                     *e_ditBtn;
     QMultiLineEdit  *s_igEditor;
     QButtonGroup    *b_uttonGroup;
+    Kpgp::SecretKeyRequester
+                    *s_igningKey;
+    KURLCompletion  *c_ompletion;
 
     Identity        *d_ata;
 
   protected slots:
-    void slotSigningKeyChange();
     void slotSignatureType(int type);
     void slotSignatureChoose();
     void slotSignatureEdit();
-    void textFileNameChanged(const QString &);
+    void textFileNameChanged(const QString &);    
 
 };
 
