@@ -39,9 +39,9 @@ KNGroupDialog::KNGroupDialog(QWidget *parent, KNNntpAccount *a) :
 {
   rightLabel->setText(i18n("Current changes:"));
   subView=new QListView(page);
-  subView->addColumn(i18n("subscribe to"));
+  subView->addColumn(i18n("Subscribe To"));
   unsubView=new QListView(page);
-  unsubView->addColumn(i18n("unsubscribe from"));
+  unsubView->addColumn(i18n("Unsubscribe From"));
 
   QVBoxLayout *protL=new QVBoxLayout(3);
   listL->addLayout(protL, 1,2);
@@ -284,11 +284,11 @@ void KNGroupDialog::slotUser2()
   QDate lastDate = a_ccount->lastNewFetch();
   KDialogBase *dlg = new KDialogBase( this, 0L, true, i18n("New Groups"), Ok | Cancel, Ok);
 
-  QButtonGroup *btnGrp = new QButtonGroup(i18n("Check for new groups:"),dlg);
+  QButtonGroup *btnGrp = new QButtonGroup(i18n("Check for New Groups"),dlg);
   dlg->setMainWidget(btnGrp);
   QGridLayout *topL = new QGridLayout(btnGrp,4,2,25,10);
 
-  QRadioButton *takeLast = new QRadioButton( i18n("created since last check:"), btnGrp );
+  QRadioButton *takeLast = new QRadioButton( i18n("Created since last check:"), btnGrp );
   topL->addMultiCellWidget(takeLast, 0, 0, 0, 1);
 
   QLabel *l = new QLabel(KGlobal::locale()->formatDate(lastDate, false),btnGrp);
@@ -296,7 +296,7 @@ void KNGroupDialog::slotUser2()
 
   connect(takeLast, SIGNAL(toggled(bool)), l, SLOT(setEnabled(bool)));
 
-  QRadioButton *takeCustom = new QRadioButton( i18n("created since this date:"), btnGrp );
+  QRadioButton *takeCustom = new QRadioButton( i18n("Created since this date:"), btnGrp );
   topL->addMultiCellWidget(takeCustom, 2, 2, 0, 1);
 
   KDatePicker *dateSel = new KDatePicker(btnGrp, lastDate);
