@@ -17,17 +17,16 @@
 #include <stdlib.h>
 #include <klocale.h>
 #include <qtextcodec.h>
+#include <qmutex.h>
 
 #include "kngroupmanager.h"
 #include "knnntpclient.h"
 #include "utilities.h"
 
 
-KNNntpClient::KNNntpClient(int NfdPipeIn, int NfdPipeOut, pthread_mutex_t *nntpMutex, QObject *parent, const char *name)
-: KNProtocolClient(NfdPipeIn,NfdPipeOut,parent,name)
-{
-  mutex = nntpMutex;
-}
+KNNntpClient::KNNntpClient(int NfdPipeIn, int NfdPipeOut, QMutex& nntpMutex)
+: KNProtocolClient(NfdPipeIn,NfdPipeOut), mutex(nntpMutex)
+{}
 
 
 KNNntpClient::~KNNntpClient()
@@ -335,7 +334,7 @@ void KNNntpClient::doFetchNewHeaders()
   target->setLastFetchCount(0);
 
   sendSignal(TSdownloadNew);
-  errorPrefix=i18n("No new articles could be retrieved for\n%1/%2!\nThe following error ocurred:\n")
+  errorPrefix=i18n("No new articles could be retrieved for\n%1/%2!\nThe following error occurred:\n")
               .arg(account.server()).arg(target->groupname());
 
   cmd="GROUP ";
@@ -418,19 +417,10 @@ void KNNntpClient::doFetchNewHeaders()
     
   sendSignal(TSsortNew);
 
-  if (0==pthread_mutex_lock(mutex)) {
-    target->insortNewHeaders(&headers, this);
-    target->setLastNr(last);
-    if (0!=pthread_mutex_unlock(mutex)) {
-#ifndef NDEBUG
-      qDebug("knode: failed to unlock nntp mutex");
-#endif
-    }
-  } else {
-#ifndef NDEBUG
-    qDebug("knode: failed to lock nntp mutex");
-#endif
-  }
+  mutex.lock();
+  target->insortNewHeaders(&headers, this);
+  target->setLastNr(last);
+  mutex.unlock();
 }
 
 
@@ -468,7 +458,7 @@ void KNNntpClient::doFetchArticle()
     if ((code == 430) || (code == 423))  // 430 no such article found || 423 no such article number in this group
       job->setErrorString(
              errorPrefix + getCurrentLine() +
-             i18n("<br><br>The article you requested isn't available on your newsserver.<br>You can try to get it from <a href=\"http://groups.google.com/groups?q=msgid:%1&ic=1\">groups.google.com</a>.")
+             i18n("<br><br>The article you requested isn't available on your news server.<br>You can try to get it from <a href=\"http://groups.google.com/groups?q=msgid:%1&ic=1\">groups.google.com</a>.")
                   .arg(target->messageID()->as7BitString(false)));
     return;
   }
@@ -558,7 +548,7 @@ bool KNNntpClient::openConnection()
   currentGroup = QString::null;
 
   QString oldPrefix = errorPrefix;
-  errorPrefix=i18n("Unable to connect.\nThe following error ocurred:\n");
+  errorPrefix=i18n("Unable to connect.\nThe following error occurred:\n");
 
   if (!KNProtocolClient::openConnection())
     return false;
@@ -732,4 +722,3 @@ void KNNntpClient::handleErrors()
 
 //--------------------------------
 
-#include "knnntpclient.moc"
