@@ -37,14 +37,14 @@ int main(int argc, char* argv[])
                         KNODE_VERSION,
                         I18N_NOOP("A newsreader for KDE"),
                         KAboutData::License_GPL,
-                        I18N_NOOP("Copyright (c) 1999-2001 the KNode authors"),
+                        I18N_NOOP("Copyright (c) 1999-2002 the KNode authors"),
                         0,
                         "http://knode.sourceforge.net/");
-                        
-  aboutData.addAuthor("Christian Thurner",I18N_NOOP("Maintainer"),"cthurner@web.de");
-  aboutData.addAuthor("Christian Gebauer",0,"gebauer@kde.org");
+  
+  aboutData.addAuthor("Christian Gebauer",I18N_NOOP("Maintainer"),"gebauer@kde.org");                      
+  aboutData.addAuthor("Christian Thurner",0,"cthurner@web.de");  
   aboutData.addAuthor("Dirk Mueller",0,"mueller@kde.org");
-  aboutData.addAuthor("Roberto Teixeira",0,"maragato@kde.org");
+  aboutData.addAuthor("Roberto Teixeira",0,"roberto@kde.org");
   aboutData.addAuthor("Mathias Waack",0,"mathias@atoll-net.de");
   aboutData.addCredit("Stephan Johach",0,"lucardus@onlinehome.de");
   aboutData.addCredit("Matthias Kalle Dalheimer",0,"kalle@kde.org");
