@@ -1,7 +1,7 @@
 /*
 * ecfg.cpp -- Implementation of class KExternCfg.
 * Author:	Sirtaj Singh Kang
-* Version:	$Id: ecfg.cpp,v 1.7 2001/12/13 20:11:57 mueller Exp $
+* Version:	$Id: ecfg.cpp,v 1.8 2002/07/01 18:26:04 binner Exp $
 * Generated:	Sat Nov 21 18:50:13 EST 1998
 */
 
@@ -45,7 +45,7 @@ QWidget *KExternCfg::makeWidget( QWidget *parent )
 	QGridLayout *slay = new QGridLayout( aGroup, 2, 2, 10 );
 	slay->addRowSpacing( 0, 10 );
 
-	slay->addWidget( new QLabel( i18n( "Shell Command" ), aGroup ), 1, 0 );
+	slay->addWidget( new QLabel( i18n( "Shell command:" ), aGroup ), 1, 0 );
 
 	_cmdEdit = new QLineEdit(aGroup);
 	slay->addWidget( _cmdEdit, 1, 1 );
