@@ -1,7 +1,7 @@
 /*
 * qmailcfg.cpp -- Implementation of class KQMailCfg.
 * Author:	Sirtaj Singh Kang
-* Version:	$Id: qmailcfg.cpp,v 1.7 2002/02/13 13:37:54 mlaurent Exp $
+* Version:	$Id: qmailcfg.cpp,v 1.9 2002/07/04 01:00:49 waba Exp $
 * Generated:	Mon Aug  3 13:21:18 EST 1998
 */
 
@@ -39,10 +39,10 @@ QWidget *KQMailCfg::makeWidget( QWidget *parent )
 	QGridLayout *l = new QGridLayout( dlg, 2, 3, 10 );
 	l->addRowSpacing(0, 10);
 
-	l->addWidget (new QLabel( i18n( "Maildir Path" ), dlg ), 1, 0);
+	l->addWidget (new QLabel( i18n( "Maildir path:" ), dlg ), 1, 0);
 
 	_pathEdit = new KURLRequester( d->maildir(), dlg );
-	_pathEdit->fileDialog()->setMode(2); //directory
+	_pathEdit->setMode(2); //directory
 	l->addWidget( _pathEdit, 1, 1);
 	return dlg;
 }
