@@ -308,11 +308,11 @@ namespace KPF
     d->cb_share =
       new QCheckBox(i18n("Share this directory on the &web"), w);
 
-    d->l_listenPort       = new QLabel(i18n("&Listen port"),      w);
-    d->l_bandwidthLimit   = new QLabel(i18n("&Bandwidth limit"),  w);
+    d->l_listenPort       = new QLabel(i18n("&Listen port:"),      w);
+    d->l_bandwidthLimit   = new QLabel(i18n("&Bandwidth limit:"),  w);
 //    d->l_connectionLimit  = new QLabel(i18n("Connection &limit"), w);
 
-    d->sb_listenPort      = new QSpinBox(1000,  65535, 1, w);
+    d->sb_listenPort      = new QSpinBox(1000,  999999, 1, w);
     d->sb_bandwidthLimit  = new QSpinBox(1,     999999, 1, w);
 //    d->sb_connectionLimit = new QSpinBox(1,     9999,   1, w);
 
