@@ -348,6 +348,9 @@ namespace KPF
       case 501:
         s = i18n("Not implemented");
         break;
+      case 505:
+        s = i18n("HTTP version not supported");
+        break;
       default:
         s = i18n("Unknown");
         break;
@@ -393,6 +396,9 @@ namespace KPF
       case 501:
         s = "Not implemented";
         break;
+      case 505:
+        s = "HTTP version not supported";
+        break;
       default:
         s = "Unknown";
         break;
