@@ -33,6 +33,7 @@
 #include <kdebug.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <kpushbutton.h>
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -109,12 +110,12 @@ ConnectWidget::ConnectWidget(QWidget *parent, const char *name, PPPStats *st)
   messg = new QLabel(this, "messg");
   messg->setFrameStyle(QFrame::Panel|QFrame::Sunken);
   messg->setAlignment(AlignCenter);
-  messg->setText(i18n("Sorry, can't create modem lock file."));
+  messg->setText(i18n("Unable to create modem lock file."));
   messg->setMinimumHeight(messg->sizeHint().height() + 5);
   int messw = (messg->sizeHint().width() * 12) / 10;
   messw = QMAX(messw,280);
   messg->setMinimumWidth(messw);
-  messg->setText(i18n("Looking for Modem ..."));
+  messg->setText(i18n("Looking for modem..."));
   l0->addSpacing(10);
   l0->addWidget(messg);
   l0->addSpacing(10);
@@ -170,7 +171,7 @@ ConnectWidget::~ConnectWidget() {
 void ConnectWidget::preinit() {
   // this is all just to keep the GUI nice and snappy ....
   // you have to see to believe ...
-  messg->setText(i18n("Looking for Modem ..."));
+  messg->setText(i18n("Looking for modem..."));
   inittimer->start(100);
 }
 
@@ -223,13 +224,13 @@ void ConnectWidget::init() {
   int lock = Modem::modem->lockdevice();
 
   if (lock == 1) {
-    messg->setText(i18n("Sorry, modem device is locked."));
+    messg->setText(i18n("Modem device is locked."));
     vmain = 20; // wait until cancel is pressed
     return;
   }
 
   if (lock == -1) {
-    messg->setText(i18n("Sorry, can't create modem lock file."));
+    messg->setText(i18n("Unable to create modem lock file."));
     vmain = 20; // wait until cancel is pressed
     return;
   }
@@ -276,8 +277,8 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
     assert(PPPData::NumInitStrings > 0);
     // first init string ?
     if(substate == -1) {
-      messg->setText(i18n("Initializing Modem..."));
-      emit debugMessage(i18n("Initializing Modem..."));
+      messg->setText(i18n("Initializing modem..."));
+      emit debugMessage(i18n("Initializing modem..."));
       substate = 0;
     }
 
@@ -299,8 +300,8 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 
     /*
      * FIXME after 3.0: Make it possible to disable ATS11 since it
-     * seems to be incompatible with some ISDN adapters (e.g. DataBox 
-     * Speed Dragon). Even better would be to detect this when doing 
+     * seems to be incompatible with some ISDN adapters (e.g. DataBox
+     * Speed Dragon). Even better would be to detect this when doing
      * a "Modem Query"
      */
     if (MODEM_TONEDURATION != gpppdata.modemToneDuration())
@@ -405,12 +406,12 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       timeout_timer->stop();
       timeout_timer->start(gpppdata.modemTimeout()*1000);
 
-      messg->setText(i18n("Line Busy. Hanging up ..."));
+      messg->setText(i18n("Line busy. Hanging up..."));
       emit debugPutChar('\n');
       Modem::modem->hangup();
 
       if(gpppdata.busyWait() > 0) {
-	QString bm = i18n("Line Busy. Waiting: %1 seconds").arg(gpppdata.busyWait());
+	QString bm = i18n("Line busy. Waiting: %1 seconds").arg(gpppdata.busyWait());
 	messg->setText(bm);
 	emit debugMessage(bm);
 
@@ -533,8 +534,11 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	messg->setText(bm);
 	emit debugMessage(bm);
 
-        setExpect(scriptArgument);
+	// The incrementing of the scriptindex MUST be before the
+	// call to setExpect otherwise the expect will miss a string that is
+	// already in the buffer.
 	scriptindex++;
+	setExpect(scriptArgument);
         return;
       }
 
@@ -632,7 +636,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	messg->setText(bm);
 	emit debugMessage(bm);
 
-	QString pwstring = gpppdata.Password();
+	QString pwstring = gpppdata.password();
 
 	if(!pwstring.isEmpty() && firstrunPW) {
 	  // the user entered a password on the main kppp dialog
@@ -729,15 +733,19 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 
         QString bm = i18n("Loop Start %1").arg(scriptArgument);
 
+	// The incrementing of the scriptindex MUST be before the
+	// call to setExpect otherwise the expect will miss a string that is
+	// already in the buffer.
+	scriptindex++;
+
 	if ( loopnest > (MAXLOOPNEST-2) ) {
 		bm += i18n("ERROR: Nested too deep, ignored.");
 		vmain=20;
-		scriptindex++;
 		cancelbutton();
 	        KMessageBox::error(0, i18n("Loops nested too deeply!"));
 	} else {
         	setExpect(scriptArgument);
-		loopstartindex[loopnest] = scriptindex + 1;
+		loopstartindex[loopnest] = scriptindex;
 		loopstr[loopnest] = scriptArgument;
 		loopend = false;
 		loopnest++;
@@ -745,7 +753,6 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	messg->setText(bm);
 	emit debugMessage(bm);
 
-	scriptindex++;
       }
 
       if (scriptCommand == "LoopEnd") {
@@ -753,11 +760,14 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	if ( loopnest <= 0 ) {
 		bm = i18n("LoopEnd without matching Start! Line: %1").arg(bm);
 		vmain=20;
-		scriptindex++;
 		cancelbutton();
 	        KMessageBox::error(0, bm);
 		return;
 	} else {
+	        // NB! The incrementing of the scriptindex MUST be before the
+	        // call to setExpect otherwise the expect will miss a string
+	        // that is already in the buffer.
+        	scriptindex++;
         	setExpect(scriptArgument);
 		loopnest--;
 		loopend = true;
@@ -765,7 +775,6 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	messg->setText(bm);
 	emit debugMessage(bm);
 
-	scriptindex++;
       }
     }
   }
@@ -834,7 +843,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       semaphore = true;
       result = execppp();
 
-      emit debugMessage(i18n("Starting pppd ..."));
+      emit debugMessage(i18n("Starting pppd..."));
       kdDebug(5002) << "execppp() returned with return-code " << result << endl;
 
       if(result) {
@@ -970,7 +979,7 @@ void ConnectWidget::cancelbutton() {
     this->show();
   }
 
-  messg->setText(i18n("One Moment Please ..."));
+  messg->setText(i18n("One moment please..."));
 
   // just to be sure
   Requester::rq->removeSecret(AUTH_PAP);
@@ -1073,7 +1082,7 @@ void ConnectWidget::pppdDied()
 }
 
 void ConnectWidget::if_waiting_slot() {
-  messg->setText(i18n("Logging on to Network ..."));
+  messg->setText(i18n("Logging on to network..."));
 
   if(!stats->ifIsUp()) {
 
@@ -1104,7 +1113,7 @@ void ConnectWidget::if_waiting_slot() {
   auto_hostname();
 
   if(!gpppdata.command_on_connect().isEmpty()) {
-    messg->setText(i18n("Running Startup Command ..."));
+    messg->setText(i18n("Running startup command..."));
 
     // make sure that we don't get any async errors
     kapp->flushX();
@@ -1228,7 +1237,7 @@ bool ConnectWidget::execppp() {
   if (command.length() > MAX_CMDLEN) {
     KMessageBox::error(this, i18n(
 			      "pppd command + command-line arguments exceed "
-			      "2024 characters in length. What are you doing?"
+			      "2024 characters in length."
 			      ));
 
     return false; // nonsensically long command which would bust my buffer buf.
