@@ -1,7 +1,7 @@
 /*
  *            kPPP: A pppd front end for the KDE project
  *
- * $Id: pppdata.cpp,v 1.97 2002/02/01 14:55:32 winkvist Exp $
+ * $Id: pppdata.cpp,v 1.100 2002/08/31 23:54:33 porten Exp $
  *
  *            Copyright (C) 1997 Bernd Johannes Wuebben
  *                   wuebben@math.cornell.edu
@@ -56,8 +56,8 @@ bool PPPData::open() {
 
   if (config->getConfigState() == KConfig::NoAccess) {
     KMessageBox::error(0L,
-                       i18n("The application-specific config file could "
-                       "not be opened neither read-write nor read-only.\n"
+                       i18n("The application-specific config file could not "
+                       "be opened in either read-write or read-only mode.\n"
                        "The superuser might have to change its ownership "
                        "by issuing the following command in your home directory:\n"
                        "chown {YourUsername} .kde/share/config/kppprc"),
@@ -186,13 +186,13 @@ void PPPData::writeListConfig(const QString &group, const QString &key,
 //
 // functions to set/return general information
 //
-const QString PPPData::Password() {
-  return password;
+QString PPPData::password() const {
+  return passwd;
 }
 
 
 void PPPData::setPassword(const QString &pw) {
-  password = pw;
+  passwd = pw;
 }
 
 
@@ -788,9 +788,9 @@ int PPPData::copyaccount(int i) {
   setAccountbyIndex(i);
 
   QMap <QString, QString> map = config->entryMap(cgroup);
-  QMap <QString, QString>::Iterator it = map.begin();
+  QMap <QString, QString>::ConstIterator it = map.begin();
 
-  QString newname = QString("%1_copy").arg(accname());
+  QString newname = i18n("%1_copy").arg(accname());
 
   newaccount();
 
