@@ -11,21 +11,17 @@
 // derived from Jay Painters "ezppp"
 //
 //---------------------------------------------------------------------------
-//  
-//  $Id: providerdb.cpp,v 1.22 2002/01/04 21:27:43 binner Exp $
-//
-//---------------------------------------------------------------------------
 //
 //  This program is free software; you can redistribute it and-or
 //  modify it under the terms of the GNU Library General Public
 //  License as published by the Free Software Foundation; either
 //  version 2 of the License, or (at your option) any later version.
-// 
+//
 //  This program is distributed in the hope that it will be useful,
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 //  Library General Public License for more details.
-// 
+//
 //  You should have received a copy of the GNU Library General Public
 //  License along with this program; if not, write to the Free
 //  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
@@ -49,11 +45,11 @@
 
 QWizard* ProviderDB::wiz = 0L;
 
-ProviderDB::ProviderDB(QWidget *parent) : 
+ProviderDB::ProviderDB(QWidget *parent) :
   KWizard(parent, "", true),
   cfg(0)
 {
-  setCaption(i18n("Create New Account..."));
+  setCaption(i18n("Create New Account"));
 
   wiz = this;
 
@@ -95,8 +91,7 @@ ProviderDB::ProviderDB(QWidget *parent) :
 
 
 ProviderDB::~ProviderDB() {
-  if(cfg)
-    delete cfg;
+  delete cfg;
 }
 
 
@@ -122,8 +117,7 @@ void ProviderDB::pageSelected() {
 
 
 void ProviderDB::loadProviderInfo() {
-  if(cfg)
-    delete cfg;
+  delete cfg;
 
   QString loc = *page2->list->at(page2->lb->currentItem());
   QString provider = page3->lb->text(page3->lb->currentItem());
@@ -168,8 +162,8 @@ void ProviderDB::accept() {
 //
 /////////////////////////////////////////////////////////////////////////////
 PDB_Intro::PDB_Intro(QWidget *parent) : QWidget(parent) {
-  QLabel *l = new QLabel(i18n("You will be asked a few questions on informations\n"
-			    "which are needed to establish an Internet connection\n"
+  QLabel *l = new QLabel(i18n("You will be asked a few questions on information\n"
+			    "which is needed to establish an Internet connection\n"
 			    "with your Internet Service Provider (ISP).\n\n"
 			    "Make sure you have the registration form from your\n"
 			    "ISP handy. If you have any problems, try the online\n"
@@ -190,7 +184,7 @@ PDB_Country::PDB_Country(QWidget *parent) : QWidget(parent) {
 			    "account from the list below. If your country or\n"
 			    "location is not listed, you have to create the\n"
 			    "account with the normal, dialog based setup.\n\n"
-			    "If you hit \"Cancel\", the dialog based setup\n"
+			    "If you click \"Cancel\", the dialog based setup\n"
 			    "will start."),
 		       this);
   QVBoxLayout *tl = new QVBoxLayout(this, 10, 10);
@@ -220,7 +214,8 @@ PDB_Country::PDB_Country(QWidget *parent) : QWidget(parent) {
   if(flist) {
     QFileInfoListIterator it( *flist );
     QFileInfo *fi;
-    // traverse the flist and insert into the widget
+    // traverse the flist and insert into a map for sorting
+    QMap<QString, QString> countries;
     for(; (fi = it.current()) != 0; ++it) {
       if(fi->fileName() != "." && fi->fileName() != "..") {
         QString dirFile(fi->absFilePath()+"/.directory");
@@ -231,11 +226,17 @@ PDB_Country::PDB_Country(QWidget *parent) : QWidget(parent) {
           entryName = config.readEntry("Name");
         }
         if (entryName.isNull()) entryName = fi->fileName();
-
-        lb->insertItem(entryName);
-        list->append(fi->fileName());
+	countries.insert(entryName, fi->fileName());
       }
     }
+    // insert sorted entries into list box and string list
+    QMap<QString, QString>::const_iterator mit = countries.begin();
+    QMap<QString, QString>::const_iterator mend = countries.end();
+    while(mit != mend) {
+        lb->insertItem(mit.key());
+        list->append(*mit);
+	++mit;
+    }
   }
 
   tl->activate();
@@ -262,7 +263,7 @@ PDB_Provider::PDB_Provider(QWidget *parent) : QWidget(parent) {
 			    "you have to click on \"Cancel\" and create this\n"
 			    "account using the normal, dialog-based setup.\n\n"
 			    "Click on \"Next\" when you have finished your\n"
-			    "selection."), this); 
+			    "selection."), this);
   tl->addWidget(l);
 
   QHBoxLayout *l1 = new QHBoxLayout;
@@ -274,7 +275,7 @@ PDB_Provider::PDB_Provider(QWidget *parent) : QWidget(parent) {
 	  this, SLOT(selectionChanged(int)));
   lb->setMinimumSize(220, 100);
   l1->addWidget(lb, 2);
-  l1->addStretch(1);  
+  l1->addStretch(1);
 }
 
 void PDB_Provider::selectionChanged(int idx) {
@@ -285,25 +286,25 @@ void PDB_Provider::selectionChanged(int idx) {
 void PDB_Provider::setDir(const QString &_dir) {
   if(dir != _dir) {
     lb->clear();
-    
+
     // fill the listbox
     // set up filter
     dir = _dir;
-    
+
     QString dir1 = KGlobal::dirs()->findDirs("appdata", "Provider").first();
     QRegExp re1(" ");
     dir = dir.replace(re1, "_");
     dir1 += dir;
-    
+
     QDir d(dir1);
     d.setFilter(QDir::Files);
     d.setSorting(QDir::Name);
-    
+
     // read the list of files
     const QFileInfoList *list = d.entryInfoList();
     QFileInfoListIterator it( *list );
     QFileInfo *fi;
-    
+
     // traverse the list and insert into the widget
     QRegExp re("_");
     while((fi = it.current()) != NULL) {
@@ -314,7 +315,7 @@ void PDB_Provider::setDir(const QString &_dir) {
       }
       ++it;
     }
-    
+
     // TODO: Qt 1.x needs this if list is empty
     lb->update();
   }
@@ -390,7 +391,7 @@ PDB_DialPrefix::PDB_DialPrefix(QWidget *parent) : QWidget(parent) {
 			    "are using a telephone switch) you can specify\n"
 			    "this here. This prefix is dialed just before the\n"
 			    "phone number.\n\n"
-			    "If you have a telephone switch, you probly need\n"
+			    "If you have a telephone switch, you probably need\n"
 			    "to write \"0\" or \"0,\" here."),
 		       this);
   tl->addWidget(l);
@@ -423,7 +424,7 @@ void PDB_DialPrefix::activate() {
 PDB_Finished::PDB_Finished(QWidget *parent) : QWidget(parent) {
   QVBoxLayout *tl = new QVBoxLayout(this, 10, 10);
   QLabel *l = new QLabel(i18n("Finished!\n\n"
-			    "A new account has been created. Hit \"Finish\" to\n"
+			    "A new account has been created. Click \"Finish\" to\n"
 			    "go back to the setup dialog. If you want to\n"
 			    "check the settings of the newly created account,\n"
 			    "you can use \"Edit\" in the setup dialog."),
@@ -445,7 +446,7 @@ void urlDecode(QString &s) {
       s1 += s[i];
     }
   }
-  
+
   s = s1;
 }
 
