@@ -1,7 +1,7 @@
 /*
  *            kPPP: A pppd front end for the KDE project
  *
- * $Id: runtests.h,v 1.12 2000/05/29 09:03:38 porten Exp $
+ * $Id: runtests.h,v 1.13 2000/08/06 12:08:36 porten Exp $
  *
  *            Copyright (C) 1997 Bernd Johannes Wuebben
  *                   wuebben@math.cornell.edu
@@ -36,7 +36,7 @@ const int TEST_CRITICAL = 3;
 
 // Various tests to be run at starttime
 int runTests();
-QString getHomeDir();
+const char* getHomeDir();
 void pppdVersion(int *version, int *modification, int *patch);
 
 #ifdef __linux__
