@@ -191,7 +191,7 @@ void open_ksirc::setServer( const QString &serveraddress )
 void open_ksirc::setServerDesc( QString description )
 {
   if (description.isNull() || description.isEmpty()) {
-    Label_ServerDesc->setText( i18n("Not Available"));
+    Label_ServerDesc->setText( i18n("Not available"));
   } else {
     Label_ServerDesc->setText( description );
   }
