@@ -32,11 +32,14 @@ protected slots:
   void clickConnect();
   void clickCancel();
   void clickEdit();
+  void passwordChanged( const QString& );
 
 private:
   void insertGroupList();
   void insertServerList( const char * );
   void setServerDesc( QString );
+  QString encryptPassword( const QString & );
+  QString decryptPassword( const QString & );
 
 signals:
   void open_ksircprocess( const char *, int, const char * );
