@@ -20,21 +20,29 @@
 #include "page_startup.h"
 #include "page_rmbmenu.h"
 #include "page_servchan.h"
+#include "page_font.h"
+
 
 KSPrefs::KSPrefs(QWidget * parent, const char * name):
     KDialogBase(KDialogBase::IconList, i18n("Configure KSirc"),
                 KDialogBase::Help | KDialogBase::Ok | KDialogBase::Apply | KDialogBase::Cancel | KDialogBase::Default,
                 KDialogBase::Ok, parent, name)
 {
-
-    QFrame *itemGeneral  = addPage( i18n( "General" ), i18n( "General KSirc settings" ), BarIcon( "ksirc", KIcon::SizeMedium ) );
-    QFrame *itemStartup  = addPage( i18n( "Startup" ), i18n( "KSirc startup settings" ), BarIcon( "gear", KIcon::SizeMedium ) );
-    QFrame *itemColors   = addPage( i18n( "Colors" ), i18n( "KSirc color settings" ), BarIcon( "colors", KIcon::SizeMedium ) );
+    setWFlags( getWFlags() | WDestructiveClose );
+    QFrame *itemGeneral  = addPage( i18n( "General" ), i18n( "General KSirc Settings" ), BarIcon( "ksirc", KIcon::SizeMedium ) );
+    QFrame *itemStartup  = addPage( i18n( "Startup" ), i18n( "KSirc Startup Settings" ), BarIcon( "gear", KIcon::SizeMedium ) );
+    QFrame *itemColors   = addPage( i18n( "Colors" ), i18n( "KSirc Color Settings" ), BarIcon( "colors", KIcon::SizeMedium ) );
     /*
     QFrame *itemRMBMenu  = addPage( i18n( "User Menu" ), i18n( "User menu configuration" ), BarIcon( "", KIcon::SizeMedium ) );
     QFrame *itemServChan = addPage( i18n( "Server/Channel" ), i18n( "Server/channel configuration" ), BarIcon( "", KIcon::SizeMedium ) );
     */
 
+
+    QFrame *itemFont = addPage( i18n( "Fonts" ), i18n( "Font Settings" ), BarIcon( "fonts", KIcon::SizeMedium ) );
+    QVBoxLayout *fontTopLayout  = new QVBoxLayout( itemFont, 0, 6 );
+
+
+
     QVBoxLayout *generalTopLayout  = new QVBoxLayout( itemGeneral, 0, 6 );
     QVBoxLayout *startupTopLayout  = new QVBoxLayout( itemStartup, 0, 6 );
     QVBoxLayout *colorsTopLayout   = new QVBoxLayout( itemColors, 0, 6 );
@@ -46,6 +54,8 @@ KSPrefs::KSPrefs(QWidget * parent, const char * name):
     pageGeneral  = new PageGeneral( itemGeneral );
     pageStartup  = new PageStartup( itemStartup );
     pageColors   = new PageColors( itemColors );
+    pageFont     = new PageFont(itemFont);
+
     /*
     pageRMBMenu  = new PageRMBMenu( itemRMBMenu );
     pageServChan = new PageServChan( itemServChan );
@@ -54,18 +64,23 @@ KSPrefs::KSPrefs(QWidget * parent, const char * name):
     connect(pageGeneral, SIGNAL( modified() ), SLOT( modified() ) );
     connect(pageStartup, SIGNAL( modified() ), SLOT( modified() ) );
     connect(pageColors, SIGNAL( modified() ), SLOT( modified() ) );
+    connect(pageFont, SIGNAL( modified() ), SLOT( modified() ) );
+
+
     /*
     connect(pageRMBMenu, SIGNAL( modified() ), SLOT( modified() ) );
     connect(pageServChan, SIGNAL( modified() ), SLOT( modified() ) );
     */
 
     connect(this, SIGNAL( applyClicked() ), this, SLOT( saveConfig() ) );
-    connect(this, SIGNAL( okClicked() ), this, SLOT(closeDialog() ) );
+    connect(this, SIGNAL( okClicked() ), this, SLOT( saveConfig() ) );
     connect(this, SIGNAL( defaultClicked() ), this, SLOT(defaultConfig() ) );
 
     generalTopLayout->addWidget( pageGeneral );
     startupTopLayout->addWidget( pageStartup );
     colorsTopLayout->addWidget( pageColors );
+    fontTopLayout->addWidget( pageFont );
+
     /*
     rmbMenuTopLayout->addWidget( pageRMBMenu );
     servChanTopLayout->addWidget( pageServChan );
@@ -81,12 +96,6 @@ KSPrefs::~KSPrefs()
 {
 }
 
-void KSPrefs::closeDialog()
-{
-    saveConfig();
-    this->close();
-}
-
 void KSPrefs::readConfig()
 {
 	// apply by calling readConfig in each page
@@ -94,10 +103,12 @@ void KSPrefs::readConfig()
 	pageGeneral->readConfig();
 	pageColors->readConfig();
 	pageStartup->readConfig();
-        /*
+	pageFont->readConfig();
+
+ /*
 	pageRMBMenu->readConfig();
 	pageServChan->readConfig();
-        */
+ */
 
 }
 
@@ -113,15 +124,17 @@ void KSPrefs::saveConfig()
         pageStartup->saveConfig();
     if ( dirty & KSOptions::Colors )
         pageColors->saveConfig();
+    if ( dirty & KSOptions::Colors )
+        pageFont->saveConfig();
     /*
     if ( dirty & KSOptions::RMBMenu )
         pageRMBMenu->saveConfig();
     if ( dirty & KSOptions::ServChan )
         pageServChan->saveConfig();
         */
-    
+
     ksopts->save( dirty );
-	emit update( dirty );
+    emit update( dirty );
     enableButtonApply( false );
 }
 
@@ -141,6 +154,9 @@ void KSPrefs::defaultConfig()
         case 2:
             pageColors->defaultConfig();
             break;
+        case 3:
+            pageFont->defaultConfig();
+            break;
             /*
         case 3:
             pageRMBMenu->defaultConfig();
@@ -160,6 +176,8 @@ void KSPrefs::modified()
         dirty |= static_cast<int>( KSOptions::Startup );
     if ( sender() == pageColors )
         dirty |= static_cast<int>( KSOptions::Colors );
+    if ( sender() == pageFont )
+        dirty |= static_cast<int>( KSOptions::Colors );
     /*
     if ( sender() == pageRMBMenu )
         dirty |= static_cast<int>( KSOptions::RMBMenu );
@@ -169,4 +187,3 @@ void KSPrefs::modified()
 
     enableButtonApply( dirty );
 }
-
