@@ -2,7 +2,7 @@
 
  IO Notify Messanger
 
- $$Id: ioNotify.cpp,v 1.11 2001/09/01 14:45:52 mueller Exp $$
+ $$Id: ioNotify.cpp,v 1.12 2002/04/01 08:25:22 hausmann Exp $$
 
 **********************************************************************/
 
@@ -24,7 +24,7 @@ KSircIONotify::~KSircIONotify()
 {
 }
 
-void KSircIONotify::sirc_receive(QString str, bool)
+void KSircIONotify::sirc_receive(QCString str, bool)
 {
   if(str.contains("*)*")){
     int s1, s2;
