@@ -2,7 +2,7 @@
 
  IO Controller Object
 
- $$Id: iocontroller.cpp,v 1.42 2002/02/19 21:12:46 malte Exp $$
+ $$Id: iocontroller.cpp,v 1.44.2.2 2002/12/17 20:58:54 fawcett Exp $$
 
  Main io controller.  Reads and writes strings to sirc.  Input
  received in the following 2 formats:
@@ -65,6 +65,8 @@
 
 ***********************************************************************/
 
+#include <config.h>
+
 #include "ksopts.h"
 #include "control_message.h"
 #include "iocontroller.h"
@@ -149,9 +151,10 @@ void KSircIOController::stdout_read(KProcess *, char *_buffer, int buflen)
    */
 
   int pos,pos2,pos3;
-  QString name, line;
+  QCString name, line;
 
-  QString buffer(QString::fromLocal8Bit(_buffer, buflen));
+  //QCString buffer(QString::fromLocal8Bit(_buffer, buflen));
+  QCString buffer(_buffer, buflen+1);
   //  kdDebug() << "<-- read: " << buffer;
   name = "!default";
 
@@ -229,7 +232,7 @@ void KSircIOController::stderr_read(KProcess *p, char *b, int l)
   stdout_read(p, b, l);
 }
 
-void KSircIOController::stdin_write(QString s)
+void KSircIOController::stdin_write(QCString s)
 {
   if (!proc->isRunning())
   {
@@ -245,7 +248,7 @@ void KSircIOController::stdin_write(QString s)
       delete[] send_buf;
     }
     send_buf = new char[len];
-    strncpy(send_buf, buffer.local8Bit(), len);
+    memcpy(send_buf, buffer.data(), len);
     if(proc->writeStdin(send_buf, len) == FALSE){
       kdDebug() << "Failed to write but CTS HIGH! Setting low!: " << s << endl;
     }
@@ -262,12 +265,14 @@ void KSircIOController::stdin_write(QString s)
 
 }
 
-void KSircIOController::sircDied(KProcess *)
+void KSircIOController::sircDied(KProcess *process)
 {
+  if ( process->exitStatus() == 0 )
+      return;
   kdDebug() << "IOController: KProcess died!\n";
   ksircproc->TopList["!all"]->sirc_receive("*E* DSIRC IS DEAD");
   ksircproc->TopList["!all"]->sirc_receive("*E* KSIRC WINDOW HALTED");
-  ksircproc->TopList["!all"]->sirc_receive("*E* Tried to run: " + KGlobal::dirs()->findExe("dsirc") + "\n");
+  ksircproc->TopList["!all"]->sirc_receive( QCString( "*E* Tried to run: " ) + KGlobal::dirs()->findExe("dsirc").ascii() + QCString( "\n" ) );
   ksircproc->TopList["!all"]->sirc_receive("*E* DID YOU READ THE INSTALL INTRUCTIONS?");
 }
 
@@ -277,7 +282,7 @@ void KSircIOController::procCTS ( KProcess *)
   delete[] send_buf;
   send_buf = 0x0;
   if(!buffer.isEmpty()){
-    QString str = "";
+    QCString str = "";
     stdin_write(str);
   }
 }
