@@ -20,7 +20,7 @@ public slots:
   virtual void stdout_read(KProcess *proc, char *_buffer, int buflen);
   virtual void stderr_read(KProcess *proc, char *_buffer, int buflen);
 
-  virtual void stdin_write(QString);
+  virtual void stdin_write(QCString);
 
   virtual void sircDied(KProcess *);
 
@@ -29,13 +29,13 @@ protected slots:
   virtual void procCTS(KProcess *);
 
 private:
-  QString holder;
+  QCString holder;
   bool proc_CTS;
   KProcess *proc;
   KSircProcess *ksircproc;
   char *send_buf;
 
-  QString buffer;
+  QCString buffer;
 
   static int counter;
 };
