@@ -22,6 +22,14 @@ KSOGeneral::KSOGeneral()
     windowLength = 200;
     logging = false;
     beepOnMsg = false;
+    publicAway = true;
+}
+
+QPixmap KSOGeneral::backgroundPixmap()
+{
+    if ( m_backgroundPixmap.isNull() && !backgroundFile.isEmpty() )
+	m_backgroundPixmap = QPixmap( backgroundFile );
+    return m_backgroundPixmap;
 }
 
 KSOStartup::KSOStartup()
@@ -31,7 +39,7 @@ KSOStartup::KSOStartup()
     if ( pw )
     {
         nick = QString::fromLocal8Bit( pw->pw_name );
-        realName = QString::fromLocal8Bit( pw->pw_gecos );
+        realName = QString::fromLocal8Bit( pw->pw_gecos ).section( ',', 0, 0 );
     }
 }
 
@@ -102,6 +110,7 @@ void KSOptions::load( int sections )
         backgroundFile = conf->readEntry( "BackgroundFile", backgroundFile );
         logging = conf->readBoolEntry( "Logging", logging );
         beepOnMsg = conf->readBoolEntry( "BeepOnMessage", false );
+        publicAway = conf->readBoolEntry( "PublicAway", publicAway );
     }
 
     if ( sections & Startup )
@@ -158,7 +167,7 @@ void KSOptions::load( int sections )
         }
         conf->setGroup( "GlobalOptions" );
         defaultFont = conf->readFontEntry( "MainFont" );
-        QApplication::setFont( defaultFont, true, "KSTextView" );
+        QApplication::setFont( defaultFont, true, "KSirc::TextView" );
     }
 }
 
@@ -181,6 +190,7 @@ void KSOptions::save( int sections )
         conf->writeEntry( "BackgroundFile", backgroundFile );
         conf->writeEntry( "Logging", logging );
         conf->writeEntry( "BeepOnMessage", beepOnMsg );
+        conf->writeEntry( "PublicAway", publicAway );
     }
 
     if ( sections & Startup )
@@ -205,7 +215,14 @@ void KSOptions::save( int sections )
         conf->writeEntry( "Background", backgroundColor );
         for ( int i = 0; i < 16; ++i )
             conf->writeEntry( QString::fromLatin1( "IRC-%1" ).arg( i ), &ircColors[i] );
-    }
-
+		/* Store the font setting */
+		conf->setGroup( "GlobalOptions" );
+		conf->writeEntry("MainFont", defaultFont);
+		/* end of storing the font settings */
+	}
+	
     conf->sync();
 }
+
+/* vim: et sw=4 ts=4
+ */
