@@ -32,6 +32,8 @@
 #include <kcursor.h>
 #include <assert.h>
 
+using namespace KSirc;
+
 typedef StringParserState<QChar> ParsingState;
 
 static const int PaintBufferExtend = 128;
@@ -124,6 +126,11 @@ void Item::selectionOffsets( int &startOffset, int &endOffset )
     m_parag->textView()->selectionOffsets( startOffset, endOffset );
 }
 
+int Item::maxSelectionOffset() const
+{
+    return text().len - 1;
+}
+
 StringPtr Item::text() const
 {
     return StringPtr();
@@ -190,7 +197,7 @@ Item *TextChunk::breakLine( int width )
     if ( state.atEnd() ) // eh?
         return 0;
 
-    StringPtr firstWord; firstWord.ptr = state.current(); 
+    StringPtr firstWord; firstWord.ptr = state.current();
     firstWord.len = state.advanceTo( ' ' );
 
     int firstWordWidth = m_metrics.width( CONSTSTRING( firstWord ) );
@@ -304,7 +311,7 @@ Item::LayoutResetStatus TextChunk::resetLayout( Item *originalItem )
 int TextChunk::calcSelectionOffset( int x )
 {
     // ### how to optimize?
-    
+
     QConstString tmp( m_text.ptr, m_text.len );
     const QString &s = tmp.string();
 
@@ -312,14 +319,14 @@ int TextChunk::calcSelectionOffset( int x )
     int px = 0;
     for (; i < m_text.len; ++i )
     {
-        int cw = m_metrics.charWidth( s, i );
-        if ( px <= x && x <= ( px + cw ) )
+        int partialWidth = m_metrics.width( s, i + 1 );
+        if ( px <= x && x <= partialWidth )
             return i;
-        px += cw;
+        px = partialWidth;
     }
 
-    assert( false );
-    return -1;
+    //assert( false );
+    return m_text.len-1;
 }
 
 StringPtr TextChunk::text() const
@@ -347,7 +354,7 @@ void TextChunk::calcExtends() const
     if ( state.atEnd() ) // eh?
         return;
 
-    StringPtr firstWord; firstWord.ptr = state.current(); 
+    StringPtr firstWord; firstWord.ptr = state.current();
     firstWord.len = state.advanceTo( ' ' );
 
     m_minWidth = m_metrics.width( CONSTSTRING( firstWord ) );
@@ -451,7 +458,7 @@ void ImageItem::calcExtends() const
 }
 
 Tokenizer::Tokenizer( PString &text )
-    : m_text( text.data ), m_tags( text.tags ), 
+    : m_text( text.data ), m_tags( text.tags ),
       m_textBeforeFirstTagProcessed( false ), m_done( false )
 {
 //    qDebug( "Tokenizer::Tokenizer( %s )", m_text.ascii() );
@@ -666,13 +673,15 @@ void Tokenizer::resolveEntities( QString &text, TagIndexList &tags )
               i > (*tagInfoIt).index )
             ++tagInfoIt;
 
-        if ( *p == '&' )
-        {
+        QChar ch = *p;
+
+        if ( ch == '&' ) {
             ampersand = p;
             scanForSemicolon = true;
             continue;
         }
-        if ( *p != ';' || !scanForSemicolon )
+
+        if ( ch != ';' || !scanForSemicolon )
             continue;
 
         assert( ampersand );
@@ -680,25 +689,25 @@ void Tokenizer::resolveEntities( QString &text, TagIndexList &tags )
         scanForSemicolon = false;
 
         const QChar *entityBegin = ampersand + 1;
-        if ( entityBegin == p )
+
+        uint entityLength = p - entityBegin;
+        if ( entityLength == 0 )
+            continue;
+
+        QChar entityValue = KCharsets::fromEntity( QConstString( entityBegin, entityLength ).string() );
+        if ( entityValue.isNull() )
             continue;
 
         uint ampersandPos = ampersand - text.unicode();
-        uint entityLength = i - ampersandPos + 1;
 
-        QChar entityVal = KGlobal::charsets()->fromEntity( QConstString( entityBegin,
-                                                                         entityLength - 2 ).string() );
-        if ( entityVal == QChar::null )
-            continue;
-
-        text.remove( ampersandPos + 1, entityLength - 1 );
-        text[ ampersandPos ] = entityVal;
+        text[ ampersandPos ] = entityValue;
+        text.remove( ampersandPos + 1, entityLength + 1 );
         i = ampersandPos;
         p = text.unicode() + i;
         endP = text.unicode() + text.length();
         ampersand = 0;
 
-        uint adjustment = entityLength - 1;
+        uint adjustment = entityLength + 1;
         TagIndexList::Iterator it = tagInfoIt;
         for (; it != tags.end(); ++it )
             (*it).index -= adjustment;
@@ -731,7 +740,7 @@ void Tokenizer::parseTag( const StringPtr &text,
             start = ++p;
             continue;
         }
-        
+
        if ( state == ScanForEqual )
        {
            if ( ch == '=' )
@@ -777,7 +786,7 @@ void Tokenizer::parseTag( const StringPtr &text,
            const QChar *valueEnd = p - 1;
 
            // skip leading and trailing quotes
-           if ( *start == '\"' && 
+           if ( *start == '\"' &&
                 start != valueEnd &&
                 *valueEnd == '\"' )
                ++start;
@@ -790,7 +799,7 @@ void Tokenizer::parseTag( const StringPtr &text,
            state = ScanForName;
            continue;
        }
- 
+
        if ( state == ScanForName )
        {
            while ( p < endP && *p != ' ' && *p != '=' )
@@ -815,7 +824,7 @@ void Tokenizer::parseTag( const StringPtr &text,
     qDebug( "attribute:" );
     AttributeMap::ConstIterator it = attributes.begin();
     for (; it != attributes.end(); ++it )
-        qDebug( "%s -> %s", it.key().constString().string().ascii(), 
+        qDebug( "%s -> %s", it.key().constString().string().ascii(),
                             it.data().constString().string().ascii() );
                             */
 }
@@ -912,8 +921,10 @@ TextLine::TextLine( const QPtrList<Item> &items )
 QString TextLine::updateSelection( const SelectionPoint &start, const SelectionPoint &end )
 {
     QString selectedText;
-    if( m_items.isEmpty() )
-      return QString::null;
+
+    // fixes a crash where because of an empty list i becomes null
+    if ( m_items.isEmpty() )
+        return QString::null;
 
     if ( start.line == this )
     {
@@ -938,7 +949,7 @@ QString TextLine::updateSelection( const SelectionPoint &start, const SelectionP
         m_items.at( oldCurrent );
     }
 
-    for (; i != lastItem && i !=0L; i = m_items.next() )
+    for (; i != lastItem && i!=0L; i = m_items.next() )
     {
         if ( i == start.item )
         {
@@ -1010,7 +1021,7 @@ void TextLine::paint( QPainter *p, int y )
     }
 }
 
-Item *TextLine::itemAt( int px, SelectionPoint *selectionInfo, 
+Item *TextLine::itemAt( int px, SelectionPoint *selectionInfo,
                         Item::SelectionAccuracy accuracy )
 {
     QPtrListIterator<Item> it( m_items );
@@ -1034,12 +1045,12 @@ Item *TextLine::itemAt( int px, SelectionPoint *selectionInfo,
         x += width;
     }
 
-    if ( accuracy == Item::SelectFuzzy && selectionInfo && 
+    if ( accuracy == Item::SelectFuzzy && selectionInfo &&
          !m_items.isEmpty() && width > 0 )
     {
         Item *i = m_items.getLast();
         selectionInfo->pos.setX( x - width );
-        selectionInfo->offset = i->calcSelectionOffset( width - 1 );
+        selectionInfo->offset = i->maxSelectionOffset();
         selectionInfo->item = i;
         selectionInfo->line = this;
     }
@@ -1067,7 +1078,7 @@ void TextLine::fontChange( const QFont &newFont )
     }
 }
 
-TextParag::TextParag( KSTextView *textView, const QString &richText )
+TextParag::TextParag( TextView *textView, const QString &richText )
     : m_layouted( false ), m_height( 0 ), m_minWidth( 0 ),
       m_textView( textView )
 {
@@ -1153,7 +1164,7 @@ void TextParag::layout( int width )
         it.toFirst();
         remainingWidth = width;
     }
-    
+
     // append what's left
     if ( items.count() > 0 )
     {
@@ -1208,7 +1219,7 @@ Item *TextParag::itemAt( int px, int py, SelectionPoint *selectionInfo,
             }
             return i;
         }
-        y += height; 
+        y += height;
     }
 
     if ( accuracy == Item::SelectFuzzy && selectionInfo && !m_lines.isEmpty() )
@@ -1324,7 +1335,7 @@ void TextParag::setRichText( const QString &richText )
 
             assert( tok.value == tag.name );
 
-            // ...foo</bleh>... -> finish off 'foo' 
+            // ...foo</bleh>... -> finish off 'foo'
             if ( !lastTextToken.value.isNull() )
             {
                 Item *item = Item::create( this, lastTextToken, tag.props );
@@ -1345,7 +1356,7 @@ void TextParag::setRichText( const QString &richText )
         if ( item )
             line->appendItem( item );
     }
- 
+
     m_lines.append( line );
 }
 
@@ -1365,7 +1376,7 @@ void TextParag::fontChange( const QFont &newFont )
         it.current()->fontChange( newFont );
 }
 
-KSTextView::KSTextView( QWidget *parent, const char *name )
+TextView::TextView( QWidget *parent, const char *name )
     : QScrollView( parent, name, WRepaintNoErase ),
       m_paintBuffer( PaintBufferExtend, PaintBufferExtend ),
       m_selectionEndBeforeStart( false )
@@ -1376,11 +1387,11 @@ KSTextView::KSTextView( QWidget *parent, const char *name )
     m_autoScrollTimer = new QTimer( this );
 }
 
-KSTextView::~KSTextView()
+TextView::~TextView()
 {
 }
 
-void KSTextView::drawContents( QPainter *p, int cx, int cy, int , int ch )
+void TextView::drawContents( QPainter *p, int cx, int cy, int , int ch )
 {
     if ( m_parags.isEmpty() )
         return;
@@ -1408,7 +1419,7 @@ void KSTextView::drawContents( QPainter *p, int cx, int cy, int , int ch )
     int yEnd = cy + ch;
     while ( y < yEnd )
     {
-        m_paintBuffer.fill( colorGroup().base() );
+        m_paintBuffer.fill( viewport(), 0, y );
         QPainter painter( &m_paintBuffer );
         painter.translate( -cx, -y );
 
@@ -1442,22 +1453,22 @@ void KSTextView::drawContents( QPainter *p, int cx, int cy, int , int ch )
     }
 }
 
-void KSTextView::viewportResizeEvent( QResizeEvent *ev )
+void TextView::viewportResizeEvent( QResizeEvent *ev )
 {
     if ( ev->size().width() != ev->oldSize().width() )
         layout();
 }
 
-void KSTextView::clear()
+void TextView::clear()
 {
     stopAutoScroll();
-    clearSelection(); 
+    clearSelection();
     m_parags.clear();
     layout();
     viewport()->erase();
 }
 
-TextParagIterator KSTextView::appendParag( const QString &richText )
+TextParagIterator TextView::appendParag( const QString &richText )
 {
     TextParag *parag = new TextParag( this, richText );
     m_parags.append( parag );
@@ -1467,7 +1478,7 @@ TextParagIterator KSTextView::appendParag( const QString &richText )
     return TextParagIterator( it );
 }
 
-bool KSTextView::removeParag( const TextParagIterator &parag ) 
+bool TextView::removeParag( const TextParagIterator &parag )
 {
     if ( parag.atEnd() )
         return false;
@@ -1491,7 +1502,7 @@ bool KSTextView::removeParag( const TextParagIterator &parag )
     return true;
 }
 
-void KSTextView::clearSelection( bool repaint )
+void TextView::clearSelection( bool repaint )
 {
     m_selectionStart = SelectionPoint();
     m_selectionEnd = SelectionPoint();
@@ -1502,12 +1513,12 @@ void KSTextView::clearSelection( bool repaint )
         updateContents();
 }
 
-TextParagIterator KSTextView::firstParag() const
+TextParagIterator TextView::firstParag() const
 {
     return TextParagIterator( QPtrListIterator<TextParag>( m_parags ) );
 }
 
-QString KSTextView::plainText() const
+QString TextView::plainText() const
 {
     if ( m_parags.isEmpty() )
         return QString::null;
@@ -1525,12 +1536,12 @@ QString KSTextView::plainText() const
     return result;
 }
 
-void KSTextView::copy()
+void TextView::copy()
 {
     QApplication::clipboard()->setText( m_selectedText );
 }
 
-void KSTextView::clearSelectionInternal()
+void TextView::clearSelectionInternal()
 {
     m_selectionEndBeforeStart = false;
     TextParag *p = m_parags.first();
@@ -1538,7 +1549,7 @@ void KSTextView::clearSelectionInternal()
         p->clearSelection();
 }
 
-void KSTextView::viewportMousePressEvent( QMouseEvent *ev )
+void TextView::viewportMousePressEvent( QMouseEvent *ev )
 {
     if ( ev->button() & LeftButton )
     {
@@ -1554,13 +1565,13 @@ void KSTextView::viewportMousePressEvent( QMouseEvent *ev )
     }
 }
 
-void KSTextView::viewportMouseMoveEvent( QMouseEvent *ev )
+void TextView::viewportMouseMoveEvent( QMouseEvent *ev )
 {
     SelectionPoint p;
     Item *i = itemAt( viewportToContents( ev->pos() ), &p, Item::SelectFuzzy );
     if ( !i && !p.item )
         return;
-   
+
     if ( ev->state() & LeftButton && m_selectionStart.item && p.item )
     {
 
@@ -1613,11 +1624,11 @@ void KSTextView::viewportMouseMoveEvent( QMouseEvent *ev )
         viewport()->setCursor( c );
 }
 
-void KSTextView::viewportMouseReleaseEvent( QMouseEvent *ev )
+void TextView::viewportMouseReleaseEvent( QMouseEvent *ev )
 {
     stopAutoScroll();
 
-    if ( !m_selectedText.isEmpty() )
+    if ((ev->button() & Qt::LeftButton) && !m_selectedText.isEmpty() )
     {
         QClipboard *cb = QApplication::clipboard();
         if ( cb->supportsSelection() )
@@ -1643,9 +1654,18 @@ void KSTextView::viewportMouseReleaseEvent( QMouseEvent *ev )
             }
         }
     }
+    if (ev->button() & Qt::MidButton)
+    {
+        QClipboard *cp = KApplication::clipboard();
+        bool restoreMode = cp->selectionModeEnabled();
+        cp->setSelectionMode(true);
+        emit pasteReq( cp->text() );
+        cp->setSelectionMode(restoreMode);
+        return;
+    }
 }
 
-void KSTextView::fontChange( const QFont & )
+void TextView::fontChange( const QFont & )
 {
     QPtrListIterator<TextParag> it( m_parags );
     for (; it.current(); ++it )
@@ -1654,10 +1674,10 @@ void KSTextView::fontChange( const QFont & )
     layout( true );
 }
 
-void KSTextView::autoScroll()
+void TextView::autoScroll()
 {
     QPoint cursor = viewport()->mapFromGlobal( QCursor::pos() );
-    
+
     QPoint contentsPos = viewportToContents( cursor );
 
     cursor.rx() -= viewport()->x();
@@ -1669,21 +1689,21 @@ void KSTextView::autoScroll()
                        0, 5 );
 }
 
-void KSTextView::startAutoScroll()
+void TextView::startAutoScroll()
 {
     connect( m_autoScrollTimer, SIGNAL( timeout() ),
              this, SLOT( autoScroll() ) );
     m_autoScrollTimer->start( 100, false );
 }
 
-void KSTextView::stopAutoScroll()
+void TextView::stopAutoScroll()
 {
     disconnect( m_autoScrollTimer, SIGNAL( timeout() ),
                 this, SLOT( autoScroll() ) );
     m_autoScrollTimer->stop();
 }
 
-void KSTextView::selectionOffsets( int &startOffset, int &endOffset )
+void TextView::selectionOffsets( int &startOffset, int &endOffset )
 {
     assert( m_selectionStart.item );
 
@@ -1702,11 +1722,11 @@ void KSTextView::selectionOffsets( int &startOffset, int &endOffset )
         qSwap( startOffset, endOffset );
 }
 
-void KSTextView::updateSelectionOrder()
+void TextView::updateSelectionOrder()
 {
     int start = m_selectionStart.pos.y();
     int end = m_selectionEnd.pos.y();
-  
+
     if ( start == end )
     {
         start = m_selectionStart.pos.x();
@@ -1722,17 +1742,17 @@ void KSTextView::updateSelectionOrder()
     m_selectionEndBeforeStart = end < start;
 }
 
-SelectionPoint *KSTextView::selectionStart()
+SelectionPoint *TextView::selectionStart()
 {
     return m_selectionEndBeforeStart ? &m_selectionEnd : &m_selectionStart;
 }
 
-SelectionPoint *KSTextView::selectionEnd()
+SelectionPoint *TextView::selectionEnd()
 {
     return m_selectionEndBeforeStart ? &m_selectionStart : &m_selectionEnd;
 }
 
-QString KSTextView::updateSelection( const SelectionPoint &start, const SelectionPoint &end )
+QString TextView::updateSelection( const SelectionPoint &start, const SelectionPoint &end )
 {
     QString selectedText;
 
@@ -1774,7 +1794,7 @@ QString KSTextView::updateSelection( const SelectionPoint &start, const Selectio
                 else
                     selectedText = QString( text.ptr + end.offset,
                                             start.offset - end.offset + 1 );
-                
+
             }
         }
     }
@@ -1798,7 +1818,7 @@ QString KSTextView::updateSelection( const SelectionPoint &start, const Selectio
     return selectedText;
 }
 
-void KSTextView::layout( bool force )
+void TextView::layout( bool force )
 {
     int height = 0;
     int contentsWidth = visibleWidth();
@@ -1820,7 +1840,7 @@ void KSTextView::layout( bool force )
     resizeContents( contentsWidth, height );
 }
 
-Item *KSTextView::itemAt( const QPoint &pos, SelectionPoint *selectionInfo,
+Item *TextView::itemAt( const QPoint &pos, SelectionPoint *selectionInfo,
                           Item::SelectionAccuracy accuracy )
 {
     int px = pos.x();
@@ -1872,7 +1892,7 @@ void TextParagIterator::setRichText( const QString &richText )
 
     m_paragIt.current()->setRichText( richText );
 
-    KSTextView *textView = m_paragIt.current()->textView();
+    TextView *textView = m_paragIt.current()->textView();
     textView->layout( false );
 
     if ( textView->isUpdatesEnabled() )
