@@ -22,9 +22,6 @@ public:
   virtual void setWidget(QObject *_f);
   virtual KSProgress *widget();
 
-signals:
-  void outputMessage(int fd, PukeMessage *pm);
-
 protected slots:
   void cancelPressed();
 
