@@ -23,10 +23,6 @@ class PWidget : public PObject
   virtual void setWidget(QObject *w = 0x0);
   virtual QWidget *widget();
 
- signals:
-  void outputMessage(int fd, PukeMessage *pm);
-  void widgetDestroyed(widgetId wI);
-
  protected:
   bool eventFilter(QObject *o, QEvent *e);
 
