@@ -2,45 +2,43 @@
 #define USERCONTROLLERMENU_H
 
 #include <qstring.h>
-#include <qlist.h>
-
-#include <kconfig.h>
+#include <qptrlist.h>
 
 class UserControlMenu {
  public:
-  /** 
+  /**
       Basic constructor, everything defaults to a Seperator.  All data
       is public so there is no problems changing it in the future.
       Data is copied internally and deleted when finished.
   */
-  UserControlMenu(char *_title = 0, 
-		  char *_action = 0, 
-		  int _accel = 0, 
-		  int _type = 0, 
+  UserControlMenu(const QString& _title = 0,
+		  const QString& _action = 0,
+		  int _accel = 0,
+		  int _type = 0,
 		  bool _op_only = FALSE);
 
   ~UserControlMenu();
 
-  static QList<UserControlMenu> UserMenu;
+  static QPtrList<UserControlMenu> UserMenu;
 
-  static QList<UserControlMenu> *parseKConfig();
+  static QPtrList<UserControlMenu> *parseKConfig();
   static void writeKConfig();
 
   /**
     * title in the popup menu
   */
-  char *title;
-  /** 
+  QString title;
+  /**
     * Action to preform, must be of the format "blah lbah %s blah"
     * where %s will be expanded into the selected nick.
     */
-  char *action;
+  QString action;
 
   /**
     * Accelerator key, currently does nothing.
     */
   int accel;
-  /** 
+  /**
     * is this function only available to ops?
     */
   bool op_only;
