@@ -2,33 +2,33 @@
  * Copyright (c) 1983 Regents of the University of California, (c) 1998 David Faure
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
+ *  Redistribution and use in source and binary forms, with or
+ *    without modification, are permitted provided that the following
+ *    conditions are met:
  *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ *    - Redistributions of source code must retain the above
+ *      copyright notice, this list of conditions and the following
+ *      disclaimer.
+ *
+ *    - Redistributions in binary form must reproduce the above
+ *      copyright notice, this list of conditions and the following
+ *      disclaimer in the documentation and/or other materials
+ *      provided with the distribution.
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
+ *    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+ *    THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+ *    PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR
+ *    BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ *    EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+ *    TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ *    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ *    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ *    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
+ *    IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ *    THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * (BSD License, from kdelibs/doc/common/bsd-license.html)
  */
 
 /*
@@ -58,7 +58,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include "../print.h"
+#include "../proto.h"
 #include "../defs.h" // for hostname
 
 #ifndef SOMAXCONN
@@ -73,10 +73,10 @@ short int TalkConnection::talkDaemonPort;          // Port number of talk demon
 short int TalkConnection::ntalkDaemonPort;         // Port number of ntalk demon (518)
 
 
-TalkConnection::TalkConnection(struct in_addr caller_machine_addr, 
+TalkConnection::TalkConnection(struct in_addr caller_machine_addr,
                                char * r_name,
                                char * local_user,
-                               ProtocolType _protocol) : 
+                               ProtocolType _protocol) :
    protocol(_protocol), his_machine_addr(caller_machine_addr), ctl_sockt(-1), sockt(-1)
 {
     my_machine_addr = getReplyAddr(his_machine_addr);
@@ -88,9 +88,13 @@ TalkConnection::TalkConnection(struct in_addr caller_machine_addr,
     *old_msg.r_tty = '\0';
 
     strncpy(new_msg.l_name,local_user,NEW_NAME_SIZE);
+    new_msg.l_name[NEW_NAME_SIZE-1]='\0';
     strncpy(new_msg.r_name,r_name,NEW_NAME_SIZE);
+    new_msg.r_name[NEW_NAME_SIZE-1]='\0';
     strncpy(old_msg.l_name,local_user,OLD_NAME_SIZE);
+    old_msg.l_name[OLD_NAME_SIZE-1]='\0';
     strncpy(old_msg.r_name,r_name,OLD_NAME_SIZE);
+    old_msg.r_name[OLD_NAME_SIZE-1]='\0';
 
 }
 
@@ -150,12 +154,12 @@ void TalkConnection::open_sockets()
     /* open the ctl socket */
     ctl_sockt = open_socket(&ctl_addr, SOCK_DGRAM);
     /* store its address */
-    set_ctl_addr((const struct sockaddr *)&ctl_addr);
+    set_ctl_addr((const struct talk_addr *)&ctl_addr);
 
     /* open the text socket */
     sockt = open_socket(&my_addr, SOCK_STREAM);
     /* store its address */
-    set_addr((const struct sockaddr *)&my_addr);
+    set_addr((const struct talk_addr *)&my_addr);
 }
 
 /* Tries to find out the correct IP address that the daemon at host
@@ -166,7 +170,7 @@ struct in_addr TalkConnection::getReplyAddr (struct in_addr destination) {
   unsigned char *help1;
   unsigned char *help2;
 
- /* disabled caching - I don't have QIntDict ... 
+ /* disabled caching - I don't have QIntDict ...
   result = replyAddrList [(long) destination.s_addr];
   if (result) {
     return *result;
@@ -181,7 +185,7 @@ struct in_addr TalkConnection::getReplyAddr (struct in_addr destination) {
     client.sin_port = htons (0);
     daemon.sin_addr = destination;
     daemon.sin_port = i ? ntalkDaemonPort : talkDaemonPort;
- 
+
     // Connect to the daemon socket address
     // On some UNIXes (such as Linux) this works and sets the IP address queried
     // by getsockname to the local machine address used to reach the daemon.
@@ -196,7 +200,7 @@ struct in_addr TalkConnection::getReplyAddr (struct in_addr destination) {
         client.sin_addr.s_addr != htonl (INADDR_ANY))
     {
       *result = client.sin_addr;
-      message ("Found reply address");
+      debug("Found reply address");
       ::close (testsock);
       break;
     }
@@ -204,7 +208,7 @@ struct in_addr TalkConnection::getReplyAddr (struct in_addr destination) {
   }
   if (i == 2) {
     *result = defaultReplyAddr;
-    message ("Couldn't find reply address, using default");
+    debug("Couldn't find reply address, using default");
   }
   if (Options.debug_mode) {
       help1 = (unsigned char *) &destination;
@@ -223,7 +227,7 @@ struct in_addr TalkConnection::getReplyAddr (struct in_addr destination) {
  * @return 1 if succeeded to find at least 1 protocol */
 void TalkConnection::findProtocol() {
 
-    message("Remote protocol unknown. Trying to find it. findProtocol()");
+    debug("Remote protocol unknown. Trying to find it. findProtocol()");
     /* The existing ctl_sockt will be used for ntalk */
     int new_socket = ctl_sockt;
     /* We need a new SOCK_DGRAM socket for otalk */
@@ -231,8 +235,8 @@ void TalkConnection::findProtocol() {
     int old_socket = open_socket(&old_ctl_addr, SOCK_DGRAM);
 
     /* Fill the old_msg return-address to match the address of old_socket */
-    old_msg.ctl_addr = *(struct sockaddr *)&old_ctl_addr;
-    old_msg.ctl_addr.sa_family = htons(AF_INET);
+    old_msg.ctl_addr = *(struct talk_addr *)&old_ctl_addr;
+    old_msg.ctl_addr.ta_family = htons(AF_INET);
 
     /* Prepare two LOOK_UP ctl messages */
     old_msg.type = LOOK_UP;
@@ -265,14 +269,14 @@ void TalkConnection::findProtocol() {
 
     for (int retry = 0; (retry < 3) && (protocol==noProtocol); retry ++)
     {
-        message("Send packets. Retry = %d",retry);
+        debug("Send packets. Retry = %d",retry);
         /* Send the messages */
         daemon.sin_port = ntalkDaemonPort;
         int len = sendto (new_socket, (char *) &new_msg, sizeof new_msg, 0,
                           (struct sockaddr *) &daemon, sizeof daemon);
-        if (len != sizeof new_msg) 
+        if (len != sizeof new_msg)
             syslog(LOG_ERR, "findProtocol: sendto() for ntalk failed!");
-    
+
         daemon.sin_port = talkDaemonPort;
         len = sendto (old_socket, (char *) &old_msg, sizeof old_msg, 0,
                       (struct sockaddr *) &daemon, sizeof daemon);
@@ -291,11 +295,11 @@ void TalkConnection::findProtocol() {
                 // Timeout. Let's exit this loop and retry sending.
                 break;
             }
-            if (nready == 0) message("select returned 0 ! ");
+            if (nready == 0) debug("select returned 0 ! ");
 
             /* Analyze response */
             if (FD_ISSET(old_socket, &read_mask)) {
-                message("Reading on old_socket");
+                debug("Reading on old_socket");
                 cc = ::recv(old_socket, (char *)&old_resp, sizeof (old_resp), 0);
                 if (cc < 0) {
                     if (errno == EINTR)
@@ -305,7 +309,7 @@ void TalkConnection::findProtocol() {
                     if (old_resp.type == LOOK_UP) protocol=talkProtocol; // FOUND
             }
             if (FD_ISSET(new_socket, &read_mask)) {
-                message("Reading on new_socket");
+                debug("Reading on new_socket");
                 cc = ::recv(new_socket, (char *)&new_resp, sizeof (new_resp), 0);
                 if (cc < 0) {
                     if (errno == EINTR)
@@ -325,26 +329,26 @@ void TalkConnection::findProtocol() {
     /* restore old.ctl_addr */
     old_msg.ctl_addr = new_msg.ctl_addr;
     ::close(old_socket);
-    message("Exiting findProtocol");
-    if (protocol==ntalkProtocol) message("Exiting findProtocol with protocol = NTALK");
-    else if (protocol==talkProtocol) message("Exiting findProtocol with protocol = NTALK");
+    debug("Exiting findProtocol");
+    if (protocol==ntalkProtocol) debug("Exiting findProtocol with protocol = NTALK");
+    else if (protocol==talkProtocol) debug("Exiting findProtocol with protocol = NTALK");
     else p_error("FATAL : no protocol found.");
 }
 
-void TalkConnection::set_addr(const struct sockaddr * addr)
+void TalkConnection::set_addr(const struct talk_addr * addr)
 {
     old_msg.addr = *addr;
-    old_msg.addr.sa_family = htons(AF_INET);
+    old_msg.addr.ta_family = htons(AF_INET);
     new_msg.addr = *addr;
-    new_msg.addr.sa_family = htons(AF_INET);
+    new_msg.addr.ta_family = htons(AF_INET);
 }
 
-void TalkConnection::set_ctl_addr(const struct sockaddr * ctl_addr)
+void TalkConnection::set_ctl_addr(const struct talk_addr * ctl_addr)
 {
     old_msg.ctl_addr = *ctl_addr;
-    old_msg.ctl_addr.sa_family = htons(AF_INET);
+    old_msg.ctl_addr.ta_family = htons(AF_INET);
     new_msg.ctl_addr = *ctl_addr;
-    new_msg.ctl_addr.sa_family = htons(AF_INET);
+    new_msg.ctl_addr.ta_family = htons(AF_INET);
 }
 
 void TalkConnection::close_sockets()
@@ -414,10 +418,10 @@ void TalkConnection::ctl_transact(int type, int id_num)
                     continue;
                 p_error("Error waiting for daemon response");
             }
-            if (nready == 0) message("select returned 0 ! ");
+            if (nready == 0) debug("select returned 0 ! ");
         } while (nready == 0);
         /*
-         * Keep reading while there are queued messages 
+         * Keep reading while there are queued messages
          * (this is not necessary, it just saves extra
          * request/acknowledgements being sent)
          */
@@ -441,10 +445,10 @@ void TalkConnection::ctl_transact(int type, int id_num)
     } while (!ok);
     if (protocol == talkProtocol) {
         old_resp.id_num = ntohl(old_resp.id_num);
-        old_resp.addr.sa_family = ntohs(old_resp.addr.sa_family);
+        old_resp.addr.ta_family = ntohs(old_resp.addr.ta_family);
     } else {
         new_resp.id_num = ntohl(new_resp.id_num);
-        new_resp.addr.sa_family = ntohs(new_resp.addr.sa_family);
+        new_resp.addr.ta_family = ntohs(new_resp.addr.ta_family);
     }
 }
 
@@ -454,8 +458,8 @@ int TalkConnection::look_for_invite(int mandatory)
     /* Check for invitation on caller's machine */
     ctl_transact(LOOK_UP, 0);
 
-    char answer;
-    int id_num;
+    u_int8_t answer;
+    u_int32_t id_num;
     getResponseItems(&answer, &id_num, &lookup_addr);
 
     if (!mandatory) return 0;
@@ -466,14 +470,14 @@ int TalkConnection::look_for_invite(int mandatory)
 	case SUCCESS:
             new_msg.id_num = htonl(id_num);
             old_msg.id_num = htonl(id_num);
-            message("TalkConnection::look_for_invite : got SUCCESS");
-            if (lookup_addr.sa_family != AF_INET)
+            debug("TalkConnection::look_for_invite : got SUCCESS");
+            if (lookup_addr.ta_family != AF_INET)
                 p_error("Response uses invalid network address");
             return (1);
 
 	default:
             /* there wasn't an invitation waiting for us */
-            message("TalkConnection::look_for_invite : didn't get SUCCESS");
+            debug("TalkConnection::look_for_invite : didn't get SUCCESS");
             return (0);
     }
 }
@@ -502,20 +506,20 @@ int TalkConnection::accept()
 /** Connect to another talk client. */
 int TalkConnection::connect()
 {
-    message("Waiting to connect");
+    debug("Waiting to connect");
     do {
         errno = 0;
-        if (::connect(sockt, &lookup_addr, sizeof (struct sockaddr)) != -1)
+        if (::connect(sockt, (const sockaddr *)(&lookup_addr), sizeof (struct talk_addr)) != -1)
             return 1;
     } while (errno == EINTR);
     if (errno == ECONNREFUSED) {
         /*
          * The caller gave up, but his invitation somehow
-         * was not cleared. Clear it and initiate an 
+         * was not cleared. Clear it and initiate an
          * invitation. (We know there are no newer invitations,
          * the talkd works LIFO.)
          */
-        message("ECONNREFUSED");
+        debug("ECONNREFUSED");
         ctl_transact(DELETE, 0);
         close_sockets();
         return 0;
@@ -527,7 +531,7 @@ int TalkConnection::connect()
 
 /** Trade edit characters with the other talk. By agreement
  * the first three characters each talk transmits after
- * connection are the three edit characters. 
+ * connection are the three edit characters.
  * A normal talk client uses tcgetattr() to get the chars,
  * but the daemon isn't connected to a terminal, so we can't call it.
  * We just send dummy chars, to disable control chars. */
@@ -564,7 +568,7 @@ void TalkConnection::write_banner(char * banner)
     write(sockt,"\n",1);
 }
 
-void TalkConnection::getResponseItems(char * answer, int * id_num, struct sockaddr * addr) {
+void TalkConnection::getResponseItems(u_int8_t * answer, u_int32_t * id_num, struct talk_addr * addr) {
     if (protocol == talkProtocol) {
         if (answer) *answer = old_resp.answer;
         if (id_num) *id_num = old_resp.id_num;
@@ -580,6 +584,6 @@ void TalkConnection::getResponseItems(char * answer, int * id_num, struct sockad
  * and then exits. */
 void TalkConnection::p_error(const char *str)
 {
-    syslog(LOG_ERR,str);
+    syslog(LOG_ERR, "%s", str);
     _exit(0);
 }
