@@ -35,9 +35,33 @@ struct MyNIC
    struct sockaddr_in netmask;
 };
 
+struct LisaConfigInfo
+{
+   LisaConfigInfo();
+   void clear();
+   QString pingAddresses;
+   QString broadcastNetwork;
+   QString allowedAddresses;
+   int secondWait;
+   bool secondScan;
+   int firstWait;
+   int maxPingsAtOnce;
+   int updatePeriod;
+   bool useNmblookup;
+   bool unnamedHosts;
+};
+
+
 typedef QPtrList<MyNIC> NICList;
 
+//some plain C-like helper functions
+
+///Return a list with all NIC, which are up and broadcast-able
 NICList* findNICs();
+///Enter some settings into lci which might be appropriate for nic
+void suggestSettingsForNic(MyNIC* nic, LisaConfigInfo& lci);
+
+void suggestSettingsForAddress(const QString& addrMask, LisaConfigInfo& lci);
 
 #endif
 
