@@ -1,7 +1,7 @@
 /*
  * kcmreslisa.cpp
  *
- * Copyright (c) 2000 Alexander Neundorf <neundorf@kde.org>
+ * Copyright (c) 2000-2002 Alexander Neundorf <neundorf@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,85 +20,105 @@
 #include "kcmreslisa.h"
 #include "findnic.h"
 
+#include <qspinbox.h>
+#include <qcheckbox.h>
+#include <qvbox.h>
+#include <qpushbutton.h>
+#include <qgrid.h>
 #include <qlabel.h>
 #include <qtooltip.h>
 #include <qlayout.h>
+#include <qvbuttongroup.h>
 
+#include <kdialogbase.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <krestrictedline.h>
+#include <keditlistbox.h>
 
 ResLisaSettings::ResLisaSettings(const QString& config, QWidget *parent)
-:QVBox(parent)
+: KCModule(parent)
 ,m_config(config,false,true)
+,m_advancedSettingsButton(0)
+,m_suggestSettings(0)
+,m_useNmblookup(0)
+,m_pingNames(0)
+,m_allowedAddresses(0)
+,m_firstWait(0)
+,m_secondScan(0)
+,m_secondWait(0)
+,m_updatePeriod(0)
+,m_deliverUnnamedHosts(0)
+,m_maxPingsAtOnce(0)
+,m_reslisaAdvancedDlg(0)
 {
-   QHBox *hbox1 = new QHBox(this);
-   new QWidget(hbox1);
-   m_autoSetup=new QPushButton(i18n("A&utosetup..."),hbox1);
-   m_autoSetup->setFixedWidth( m_autoSetup->sizeHint().width() );   
+   QVBoxLayout *layout = new QVBoxLayout(this, KDialog::marginHint(), KDialog::spacingHint());
+   layout->setAutoAdd(true);
+   
+   QVButtonGroup *gb=new QVButtonGroup(i18n("Tell the ResLISa daemon how to search for hosts"),this);
+   gb->setInsideSpacing(10);
+
+   m_useNmblookup=new QCheckBox(i18n("Send &NetBIOS broadcasts using &nmblookup"),gb);
+   QToolTip::add(m_useNmblookup,i18n("Only hosts running SMB servers will answer"));
+
+   m_pingNames=new KEditListBox(i18n("A&dditionally Check These Hosts"),gb,"a",false, KEditListBox::Add|KEditListBox::Remove);
+   QToolTip::add(m_pingNames,i18n("The hosts listed here will be pinged."));
 
    QHBox *hbox=new QHBox(this);
    hbox->setSpacing(10);
-   QLabel *label=new QLabel(i18n("&Trusted addresses"),hbox);
-   QString comment = i18n("usually your network address/subnet mask (e.g. 192.168.0.0/255.255.255.0;)");
+   QLabel *label=new QLabel(i18n("&Trusted addresses:"),hbox);
+   QString comment = i18n("Usually your network address/subnet mask (e.g. 192.168.0.0/255.255.255.0;)");
    QToolTip::add(label, comment);
-
    m_allowedAddresses=new KRestrictedLine(hbox,"a","0123456789./;");
    QToolTip::add(m_allowedAddresses,comment);
    label->setBuddy(m_allowedAddresses);
 
-   m_pingNames=new KEditListBox(i18n("A&dditionally check the following hosts"),this,"a",false, KEditListBox::Add|KEditListBox::Remove);
-   QToolTip::add(m_pingNames,i18n("The names of the hosts you want to check"));
-   setStretchFactor(m_pingNames,1);
+   hbox = new QHBox(this);
+   m_suggestSettings=new QPushButton(i18n("&Suggest Settings"),hbox);
 
-   QGroupBox *gb=new QGroupBox(i18n("Advanced settings"),this);
-   gb->setColumnLayout(0, Qt::Vertical );
-   gb->layout()->setSpacing( 6 );
-   gb->layout()->setMargin( 6 );
-   QGridLayout *grid2 = new QGridLayout( gb->layout(), 4, 4);
+   new QWidget(hbox);
 
-   m_useNmblookup=new QCheckBox(i18n("Send NetBIOS broadcasts using &nmblookup for searching"),gb);
-   QToolTip::add(m_useNmblookup,i18n("only hosts running smb servers will answer"));
-   grid2->addMultiCellWidget(m_useNmblookup, 0, 0, 0, 3);
+   m_advancedSettingsButton=new QPushButton(i18n("Ad&vanced Settings..."),hbox);
 
-   m_deliverUnnamedHosts=new QCheckBox(i18n("Re&port unnamed hosts"),gb);
-   QToolTip::add(m_deliverUnnamedHosts,i18n("report hosts without DNS names"));
-   grid2->addMultiCellWidget(m_deliverUnnamedHosts, 1, 1, 0, 1);
 
-   m_secondScan=new QCheckBox(i18n("Al&ways scan twice"),gb);
-   QToolTip::add(m_secondScan,i18n("check the hosts twice"));
-   grid2->addMultiCellWidget(m_secondScan, 1, 1, 2, 3);
+   m_reslisaAdvancedDlg=new KDialogBase(0,0,true,i18n("Advanced Settings for ResLISa"),KDialogBase::Close, KDialogBase::Close);
+   connect(m_advancedSettingsButton,SIGNAL(clicked()),m_reslisaAdvancedDlg,SLOT(show()));
 
-   label=new QLabel(i18n("Update interval:"),gb);
-   QToolTip::add(label,i18n("search hosts after this number of seconds"));
-   grid2->addWidget(label, 2, 0);
-   m_updatePeriod=new QSpinBox(30,1800,10,gb);
+   QVBox *vbox=m_reslisaAdvancedDlg->makeVBoxMainWidget();
+
+   m_deliverUnnamedHosts=new QCheckBox(i18n("Show &hosts without DNS names"),vbox);
+
+   QGrid *advGrid = new QGrid(2, Qt::Horizontal, vbox);
+   advGrid->setSpacing(10);
+
+   label=new QLabel(i18n("Host list update interval:"),advGrid);
+   QToolTip::add(label,i18n("Search hosts after this number of seconds"));
+   m_updatePeriod=new QSpinBox(30,1800,10,advGrid);
    m_updatePeriod->setSuffix(i18n(" sec"));
-   QToolTip::add(m_updatePeriod,i18n("search hosts after this number of seconds"));
-   grid2->addWidget(m_updatePeriod, 2, 1);
+   QToolTip::add(m_updatePeriod,i18n("Search hosts after this number of seconds"));
 
+   m_secondScan=new QCheckBox(i18n("Always check twice for hosts when searching"),advGrid);
+   new QWidget(advGrid);
 
-   label=new QLabel(i18n("Wait for replies after first scan"),gb);
-   QToolTip::add(label,i18n("how long to wait for replies to the ICMP echo requests from hosts"));
-   grid2->addWidget(label, 2, 2);
-   m_firstWait=new QSpinBox(10,1000,50,gb);
+   label=new QLabel(i18n("Wait for replies from hosts after first scan:"),advGrid);
+   QToolTip::add(label,i18n("How long to wait for replies to the ICMP echo requests from hosts"));
+   m_firstWait=new QSpinBox(10,1000,50,advGrid);
    m_firstWait->setSuffix(i18n(" ms"));
-   QToolTip::add(m_firstWait,i18n("how long to wait for replies to the ICMP echo requests from hosts"));
-   grid2->addWidget(m_firstWait, 2, 3);
+   QToolTip::add(m_firstWait,i18n("How long to wait for replies to the ICMP echo requests from hosts"));
 
-   label=new QLabel(i18n("Send pings at once"),gb);
-   QToolTip::add(label,i18n("send this number of ping packets at once"));
-   grid2->addWidget(label, 3, 0);
-   m_maxPingsAtOnce=new QSpinBox(8,1024,5,gb);
-   QToolTip::add(m_maxPingsAtOnce,i18n("send this number of ping packets at once"));
-   grid2->addWidget(m_maxPingsAtOnce, 3, 1);
-
-   label=new QLabel(i18n("Wait for replies after second scan "),gb);
-   QToolTip::add(label,i18n("how long to wait for replies to the ICMP echo requests from hosts"));
-   grid2->addWidget(label, 3, 2);
-   m_secondWait=new QSpinBox(0,1000,50,gb);
+   label=new QLabel(i18n("Wait for replies from hosts after second scan:"),advGrid);
+   QToolTip::add(label,i18n("How long to wait for replies to the ICMP echo requests from hosts"));
+   m_secondWait=new QSpinBox(0,1000,50,advGrid);
    m_secondWait->setSuffix(i18n(" ms"));
    QToolTip::add(m_secondWait,i18n("how long to wait for replies to the ICMP echo requests from hosts"));
-   grid2->addWidget(m_secondWait, 3, 3);
+
+   label=new QLabel(i18n("Max. number of ping packets to send at once:"),advGrid);
+   m_maxPingsAtOnce=new QSpinBox(8,1024,5,advGrid);
+
+   QWidget *dummy=new QWidget(advGrid);
+   dummy->setMinimumHeight(10);
+
+
 
    connect(m_secondScan,SIGNAL(toggled(bool)),m_secondWait,SLOT(setEnabled(bool)));
 
@@ -114,10 +134,7 @@ ResLisaSettings::ResLisaSettings(const QString& config, QWidget *parent)
    connect(m_updatePeriod,SIGNAL(valueChanged(int)),this,SIGNAL(changed()));
    connect(m_pingNames,SIGNAL(changed()),this,SIGNAL(changed()));
    connect(m_useNmblookup,SIGNAL(toggled(bool)),this,SIGNAL(changed()));
-   connect(m_autoSetup,SIGNAL(clicked()),this,SLOT(autoSetup()));
-
-   setMargin(10);
-   setSpacing(15);
+   connect(m_suggestSettings,SIGNAL(clicked()),this,SLOT(suggestSettings()));
 }
 
 void ResLisaSettings::load()
@@ -179,54 +196,46 @@ void ResLisaSettings::save()
 };
 
 //#include <iostream>
-void ResLisaSettings::autoSetup()
+void ResLisaSettings::suggestSettings()
 {
-   NICList* nl=findNICs();
+   NICList* nics=findNICs();
 //   std::cerr<<"autoSetup() "<<nl->count()<<std::endl;
-   if (nl->count()==0)
+   if (nics->count()==0)
    {
       //ok, easy one :-)
-      KMessageBox::sorry(0,i18n("Sorry, it seems you don't have any network interfaces installed on your system."));
+      KMessageBox::sorry(0,i18n("It appears you do not have any network interfaces installed on your system."));
+      delete nics;
+      return;
    }
-   else if (nl->count()==1)
+
+   MyNIC *nic=nics->first();
+   QString address=inet_ntoa(nic->addr.sin_addr);
+   QString netmask=inet_ntoa(nic->netmask.sin_addr);
+   m_allowedAddresses->setText(address+"/"+netmask+";");
+   m_secondWait->setValue(0);
+   m_secondScan->setChecked(FALSE);
+   m_secondWait->setEnabled(FALSE);
+   m_firstWait->setValue(300);
+   m_maxPingsAtOnce->setValue(256);
+   m_updatePeriod->setValue(300);
+   m_useNmblookup->setChecked(true);
+   if (nics->count()>1)
    {
-      //still easy
-      //if the host part is less than 20 bits simply take it
-      MyNIC *nic=nl->first();
-      QString address=inet_ntoa(nic->addr.sin_addr);
-      QString netmask=inet_ntoa(nic->netmask.sin_addr);
-      //int tmp=ntohl(nic->addr.sin_addr.s_addr);
-      //if the host part is less than 20 bits simply take it
-      //this might be a problem on 64 bit machines
-      //if (tmp<0xfffff000)
-      //{
-      m_allowedAddresses->setText(address+"/"+netmask+";");
-      m_secondWait->setValue(0);
-      m_secondScan->setChecked(FALSE);
-      m_secondWait->setEnabled(FALSE);
-      m_firstWait->setValue(300);
-      m_maxPingsAtOnce->setValue(256);
-      m_updatePeriod->setValue(300);
-      m_useNmblookup->setChecked(true);
-      //};
-      KMessageBox::information(0,QString("<html>%1</html>").arg(i18n("The ResLISa daemon is now configured "
-         "correctly, hopefully.<br>Make sure that the reslisa binary is installed <i>suid root</i>.")));
-   }
-   else
-   {
-      QString msg(i18n("You have more than one network interface installed, I can't \
-                       setup automatically.<br><br>I found the following:<br><br>"));
+      QString msg(i18n("You have more than one network interface installed.<br>"
+                       "Please make sure the suggested settings are correct.<br>"
+                       "<br>The following interfaces were found:<br><br>"));
       //not that easy to handle
-      for (MyNIC* tmp=nl->first(); tmp!=0; tmp=nl->next())
+      for (MyNIC* tmp=nics->first(); tmp!=0; tmp=nics->next())
       {
          msg+="<b>"+tmp->name+": </b>"+inet_ntoa(tmp->addr.sin_addr)+"/"+inet_ntoa(tmp->netmask.sin_addr)+";<br>";
       }
-      msg+=i18n("Please set the right options manually");
-      KMessageBox::sorry(0,QString("<html>%1</html>").arg(msg));
+      KMessageBox::information(0,QString("<html>%1</html>").arg(msg));
    }
+   KMessageBox::information(0,QString("<html>%1</html>").arg(i18n("The ResLISa daemon is now configured "
+                                                                  "correctly, hopefully.<br>Make sure that the reslisa binary is installed <i>suid root</i>.")));
 
    emit changed();
-   delete nl;
+   delete nics;
 }
 
 #include "kcmreslisa.moc"
