@@ -1,7 +1,7 @@
 /*
  * kcmreslisa.h
  *
- * Copyright (c) 2000 Alexander Neundorf <neundorf@kde.org>
+ * Copyright (c) 2000-2002 Alexander Neundorf <neundorf@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,41 +21,44 @@
 #ifndef KCMRESLISA_H
 #define KCMRESLISA_H
 
-#include <krestrictedline.h>
+#include <kcmodule.h>
 #include <kconfig.h>
 
-#include <keditlistbox.h>
 
-#include <qspinbox.h>
-#include <qcheckbox.h>
-#include <qvbox.h>
-#include <qpushbutton.h>
+class QSpinBox;
+class QCheckBox;
+class QPushButton;
+class KEditListBox;
+class KDialogBase;
+class KRestrictedLine;
 
-class ResLisaSettings:public QVBox
+class ResLisaSettings:public KCModule
 {
    Q_OBJECT
    public:
       ResLisaSettings(const QString& config, QWidget *parent=0);
-      virtual ~ResLisaSettings() {};
+      virtual ~ResLisaSettings() {}
       void load();
       void save();
    signals:
       void changed();
    protected slots:
-      void autoSetup();
+      void suggestSettings();
    protected:
       KConfig m_config;
-      QPushButton *m_autoSetup;
-      QCheckBox *m_useNmblookup;
+      QPushButton* m_advancedSettingsButton;
+      QPushButton* m_suggestSettings;
+      QCheckBox* m_useNmblookup;
       KEditListBox *m_pingNames;
+
       KRestrictedLine *m_allowedAddresses;
-      //KRestrictedLine *m_broadcastNetwork;
       QSpinBox *m_firstWait;
       QCheckBox *m_secondScan;
       QSpinBox *m_secondWait;
       QSpinBox *m_updatePeriod;
       QCheckBox *m_deliverUnnamedHosts;
       QSpinBox *m_maxPingsAtOnce;
+      KDialogBase* m_reslisaAdvancedDlg;
 };
 
 #endif
