@@ -94,6 +94,7 @@ LANProtocol::LANProtocol(int isLanIoslave, const QCString &pool, const QCString
    m_protocolInfo[KIOLAN_HTTP].enabled=config->readNumEntry("Support_HTTP",PORTSETTINGS_CHECK);
    m_protocolInfo[KIOLAN_NFS].enabled=config->readNumEntry("Support_NFS",PORTSETTINGS_CHECK);
    m_protocolInfo[KIOLAN_SMB].enabled=config->readNumEntry("Support_SMB",PORTSETTINGS_CHECK);
+   m_protocolInfo[KIOLAN_FISH].enabled=config->readNumEntry("Support_FISH",PORTSETTINGS_CHECK);
 
    m_shortHostnames=config->readBoolEntry("ShowShortHostnames",false);
 
@@ -104,11 +105,13 @@ LANProtocol::LANProtocol(int isLanIoslave, const QCString &pool, const QCString
    strcpy(m_protocolInfo[KIOLAN_FTP].name,"FTP");
    strcpy(m_protocolInfo[KIOLAN_SMB].name,"SMB");
    strcpy(m_protocolInfo[KIOLAN_HTTP].name,"HTTP");
+   strcpy(m_protocolInfo[KIOLAN_FISH].name,"FISH");
 
    m_protocolInfo[KIOLAN_NFS].port=2049;
    m_protocolInfo[KIOLAN_FTP].port=21;
    m_protocolInfo[KIOLAN_SMB].port=139;
    m_protocolInfo[KIOLAN_HTTP].port=80;
+   m_protocolInfo[KIOLAN_FISH].port=22;
 
    m_hostInfoCache.setAutoDelete(true);
 
@@ -132,7 +135,13 @@ int LANProtocol::readDataFromServer()
 int LANProtocol::lanReadDataFromServer()
 {
    kdDebug(7101)<<"LANProtocol::lanReadDataFromServer() host: "<<m_currentHost<<" port: "<<m_port<<endl;
-   connectToHost(m_currentHost.latin1(), m_port);
+   if (!connectToHost(m_currentHost.latin1(), m_port, false))
+   {
+      error(ERR_SLAVE_DEFINED, i18n("<qt>The lisa daemon does not appear to be running.<p>"
+                                    "In order to use the LAN Browser the lisa daemon must be "
+                                    "installed and activated by the system administrator."));
+      return 0;
+   }
    kdDebug(7101)<<"LANProtocol::lanReadDataFromServer() connected"<<endl;
 
    int receivedBytes(0);
@@ -176,13 +185,14 @@ int LANProtocol::lanReadDataFromServer()
 
    char *currentBuf=receiveBuffer;
    int bytesLeft=receivedBytes;
-   int tmpIP;
    //this should be large enough for a name
-   char tmpName[1024];
+   char tmpName[4*1024];
    //this should be large enough for the hostname
-   char tmpHostname[512];
+   char tmpHostname[4*1024];
    while (bytesLeft>0)
    {
+      int tmpIP=2;
+      tmpName[0]='\0';
       if ((memchr(currentBuf,0,bytesLeft)==0) || (memchr(currentBuf,int('\n'),bytesLeft)==0))
       {
          delete [] receiveBuffer;
@@ -190,17 +200,24 @@ int LANProtocol::lanReadDataFromServer()
          return 0;
       };
       kdDebug(7101)<<"LANProtocol::lanReadDataFromServer: processing "<<currentBuf;
-      sscanf(currentBuf,"%u %s\n",&tmpIP,tmpName);
       //since we check for 0 and \n with memchr() we can be sure
       //at this point that tmpBuf is correctly terminated
       int length=strlen(currentBuf)+1;
+      if (length<(4*1024))
+         sscanf(currentBuf,"%u %s\n",&tmpIP,tmpName);
+      else
+      {
+        kdDebug(7101)<<"LANProtocol::lanReadDataFromServer:  buffer overflow attempt detected, aborting"<<endl;
+        break;
+      }
+
       bytesLeft-=length;
       currentBuf+=length;
-      if ((bytesLeft==0) && (strstr(tmpName,"succeeded")!=0) && ((tmpIP==0) ||(tmpIP==1)))
+      if ((bytesLeft==0) && ((tmpIP==0) ||(tmpIP==1)) && (strstr(tmpName,"succeeded")!=0))
       {
          kdDebug(7101)<<"LANProtocol::lanReadDataFromServer: succeeded"<<endl;
       }
-      else
+      else if (tmpIP!=2)
       {
          kdDebug(7101)<<"LANProtocol::lanReadDataFromServer: listing host: "<<tmpName<<" with ip: "<<tmpIP<<endl;
          UDSAtom atom;
@@ -257,7 +274,7 @@ int LANProtocol::rlanReadDataFromServer()
       //should never happen
       socketname+="???";
 
-   strcpy(addr.sun_path,socketname);
+   strlcpy(addr.sun_path,socketname,sizeof(addr.sun_path));
    int result=::connect(sockFD,(sockaddr*)&addr, sizeof(addr));
 
    kdDebug(7101)<<"readDataFromServer(): result: "<<result<<" name: "<<addr.sun_path<<" socket: "<<sockFD<<endl;
@@ -287,7 +304,7 @@ int LANProtocol::rlanReadDataFromServer()
 
       memset((char*)&addr,0,sizeof(addr));
       addr.sun_family=AF_LOCAL;
-      strcpy(addr.sun_path,socketname);
+      strlcpy(addr.sun_path,socketname,sizeof(addr.sun_path));
 
       kdDebug(7101)<<"connecting..."<<endl;
       result=::connect(sockFD,(sockaddr*)&addr, sizeof(addr));
@@ -344,13 +361,14 @@ int LANProtocol::rlanReadDataFromServer()
 
    char *currentBuf=receiveBuffer;
    int bytesLeft=receivedBytes;
-   int tmpIP;
    //this should be large enough for a name
-   char tmpName[1024];
+   char tmpName[4*1024];
    //this should be large enough for the hostname
-   char tmpHostname[512];
+   char tmpHostname[4*1024];
    while (bytesLeft>0)
    {
+      int tmpIP=2;
+      tmpName[0]='\0';
       if ((memchr(currentBuf,0,bytesLeft)==0) || (memchr(currentBuf,int('\n'),bytesLeft)==0))
       {
          delete [] receiveBuffer;
@@ -358,17 +376,24 @@ int LANProtocol::rlanReadDataFromServer()
          return 0;
       };
       kdDebug(7101)<<"RLANProtocol::readDataFromServer: processing "<<currentBuf;
-      sscanf(currentBuf,"%u %s\n",&tmpIP,tmpName);
       //since we check for 0 and \n with memchr() we can be sure
       //at this point that tmpBuf is correctly terminated
       int length=strlen(currentBuf)+1;
+      if (length<(4*1024))
+         sscanf(currentBuf,"%u %s\n",&tmpIP,tmpName);
+      else
+      {
+        kdDebug(7101)<<"RLANProtocol::readDataFromServer:  buffer overflow attempt detected, aborting"<<endl;
+        break;
+      }
+
       bytesLeft-=length;
       currentBuf+=length;
-      if ((bytesLeft==0) && (strstr(tmpName,"succeeded")!=0) && ((tmpIP==0) ||(tmpIP==1)))
+      if ((bytesLeft==0) && ((tmpIP==0) ||(tmpIP==1)) && (strstr(tmpName,"succeeded")!=0))
       {
          kdDebug(7101)<<"RLANProtocol::readDataFromServer: succeeded"<<endl;
       }
-      else
+      else if (tmpIP!=2)
       {
          kdDebug(7101)<<"RLANProtocol::readDataFromServer: listing host: "<<tmpName<<" with ip: "<<tmpIP<<endl;
          UDSAtom atom;
@@ -436,7 +461,7 @@ int LANProtocol::checkHost(const QString& host)
          delete hostInfo;
          return 0;
       }
-      memcpy(&ip, hp->h_addr, hp->h_length);
+      memcpy(&ip, hp->h_addr, sizeof(ip));
 
       for (int i=0; i<KIOLAN_MAX; i++)
       {
@@ -611,7 +636,7 @@ void LANProtocol::setHost( const QString& host, int port, const QString& s1, con
    else
    {
       if (!host.isEmpty())
-         error(ERR_MALFORMED_URL,i18n("No hosts allowed in rlan:/ url"));
+         error(ERR_MALFORMED_URL,i18n("No hosts allowed in rlan:/ URL"));
    };
 };
 
