@@ -22,6 +22,8 @@
 #include <arpa/inet.h>
 #include <sys/socket.h>
 
+using namespace std;
+
 AddressValidator::AddressValidator(const MyString& addressSpecs)
 //this is 127.0.0.0
 :localhostNet(htonl(0x7f000000))
@@ -66,12 +68,42 @@ void AddressValidator::setValidAddresses(MyString addressSpecs)
       nextPart=addressSpecs.left(pos);
       addressSpecs=addressSpecs.mid(pos+1);
       getDebug()<<"AddressValidator::setValidAddresses: nextPart: "<<nextPart<<std::endl;
-      if (nextPart.contains('.')==3)
+      if ((nextPart.contains('.')==3) && (nextPart.contains('-')==0))
       {
          addSpec(EXACTADDR_SPEC,inet_addr(nextPart.data()));
          getDebug()<<"AddressValidator::setValidAddresses: exact addr: "
 	 <<std::ios::hex<<inet_addr(nextPart.data())<<std::ios::dec<<std::endl;
       }
+      else if ((nextPart.contains('-')==1) && (nextPart.contains('.')==6))
+      {
+         int p2=nextPart.find("-");
+         MyString from=nextPart.left(p2);
+         MyString to=nextPart.mid(p2+1);
+         addSpec(RANGE_SPEC,ntohl(inet_addr(from.data())),ntohl(inet_addr(to.data())));
+      }
+      else if ((nextPart.contains('-')==4) && (nextPart.contains('.')==3))
+      {
+         unsigned int i1=0;
+         unsigned int i2=0;
+         int p2=0;
+         MyString rest=nextPart+'.';
+         MyString digit;
+
+         for (int i=0; i<4; i++)
+         {
+            p2=rest.find("-");
+            digit=rest.left(p2);
+            i1=i1<<8;
+            i1+=atoi(digit.data());
+            rest=rest.mid(p2+1);
+            p2=rest.find(".");
+            digit=rest.left(p2);
+            i2=i2<<8;
+            i2+=atoi(digit.data());
+            rest=rest.mid(p2+1);
+         };
+         addSpec(MULTIRANGE_SPEC,i1,i2);
+      }
       else
       {
          pos=nextPart.find('/');
@@ -152,6 +184,35 @@ int AddressValidator::isValid(int addressNBO)
             getDebug()<<"AddressValidator::isValid: net/mask"<<std::endl;
             return 1;
          };
+      }
+      else if (specs[i].typeOfSpec==RANGE_SPEC)
+      {
+         if ((ntohl(addressNBO)>=specs[i].address) && (ntohl(addressNBO)<=specs[i].mask))
+         {
+            getDebug()<<"AddressValidator::isValid: range"<<std::endl;
+            return 1;
+         };
+      }
+      else if (specs[i].typeOfSpec==MULTIRANGE_SPEC)
+      {
+         unsigned int addr=ntohl(addressNBO);
+         getDebug()<<" ntohl="<<hex<<addr<<" addr: "<<specs[i].address<<" ma: "<<specs[i].mask<<dec<<std::endl;
+         unsigned int mask=0x000000ff;
+         int failure=0;
+         for (int j=0; j<4; j++)
+         {
+            getDebug()<<hex<<"mask="<<mask<<" addr="<<(addr&mask)<<" addr="<<(specs[i].address & mask)<<" ma="<<(specs[i].mask&mask)<<std::endl;
+            if (((addr & mask) < (specs[i].address & mask))
+               || ((addr & mask) > (specs[i].mask & mask)))
+            {
+               failure=1;
+               break;
+            };
+            mask=mask<<8;
+         };
+         getDebug()<<"AddressValidator::isValid: multirange"<<std::endl;
+         if (!failure)
+            return 1;
       };
    };
    //if ((addressNBO==htonl(0x0a040801)) || (addressNBO==htonl(0xc0a80001))) return 0;
