@@ -16,6 +16,8 @@
 
 #include "ipaddress.h"
 
+#include <iostream>
+
 IPAddress::IPAddress()
 {
 };
@@ -87,9 +89,9 @@ struct in_addr IPAddress::string2Struct(MyString ip)
 
 void IPAddress::print()
 {
-   cout<<"address as string: "<<s<<endl;
-   cout<<"address in host byte order:    "<<ios::hex<<i<<ios::dec<<endl;
-   cout<<"address in network byte order: "<<ios::hex<<n.s_addr<<ios::dec<<endl;
+   std::cout<<"address as string: "<<s<<std::endl;
+   std::cout<<"address in host byte order:    "<<std::ios::hex<<i<<std::ios::dec<<std::endl;
+   std::cout<<"address in network byte order: "<<std::ios::hex<<n.s_addr<<std::ios::dec<<std::endl;
 };
 
 
