@@ -1,4 +1,4 @@
-/*    netmanager.cpp
+ /*    netmanager.cpp
  *
  *    Copyright (c) 2000, Alexander Neundorf
  *    neundorf@kde.org
@@ -14,6 +14,7 @@
  */
 
 #include "config.h"
+
 #include "netmanager.h"
 #include "lisadefines.h"
 
@@ -27,6 +28,8 @@
 #include <strings.h>
 #include <errno.h>
 #include <string.h>
+#include <signal.h>
+#include <pwd.h>
 
 #ifndef AF_LOCAL
 #define AF_LOCAL AF_UNIX
@@ -37,6 +40,16 @@
 #define mgetDebug() getDebug()<<procId
 //#define mgetDebug() std::cerr<<procId
 
+static void sig_child_handler(int)
+{
+   int saved_errno = errno;
+   
+   while (waitpid(-1, NULL, WNOHANG) > 0)
+      ; // repeat
+   
+   errno = saved_errno;
+}
+
 NetManager::NetManager(int& rawSocketFD, int portToUse, MyString configFile, int configStyle, int strictMode)
 :NetScanner(rawSocketFD,strictMode)
 //,validator()
@@ -66,15 +79,29 @@ NetManager::NetManager(int& rawSocketFD, int portToUse, MyString configFile, int
 {
    mgetDebug()<<"NetManager::NetManager"<<std::endl;
    m_startedAt=time(0);
-};
+   
+   struct sigaction act;
+   act.sa_handler=sig_child_handler;
+   sigemptyset(&(act.sa_mask));
+   sigaddset(&(act.sa_mask), SIGCHLD);
+   // Make sure we don't block this signal. gdb tends to do that :-(
+   sigprocmask(SIG_UNBLOCK, &(act.sa_mask), 0);
+
+   act.sa_flags = SA_NOCLDSTOP;
+#ifdef SA_RESTART
+   act.sa_flags |= SA_RESTART;
+#endif
+
+   sigaction( SIGCHLD, &act, NULL );
+}
 
 NetManager::~NetManager()
 {
-   mgetDebug()<<"netknife destructor ..."<<std::endl;
+   mgetDebug()<<"NetManager destructor ..."<<std::endl;
    if (m_receiveBuffer!=0) delete [] m_receiveBuffer;
    ::close(m_listenFD);
    ::close(m_bcFD);
-};
+}
 
 void NetManager::readConfig()
 {
@@ -84,7 +111,7 @@ void NetManager::readConfig()
       std::cout<<"configfile not found"<<std::endl;
       std::cout<<"use the command line option --help and \ntake a look at the README for more information"<<std::endl;
       exit(1);
-   };
+   }
 
    Config config(m_usedConfigFileName);
    NetManager::configure(config);
@@ -92,7 +119,7 @@ void NetManager::readConfig()
    validator.configure(config);
    //after reading the new configuration we should really update
    m_lastUpdate=0;
-};
+}
 
 void NetManager::configure(Config& config)
 {
@@ -117,7 +144,7 @@ void NetManager::configure(Config& config)
    if (m_refreshTime<30) m_refreshTime=30;
    if (m_refreshTime>1800) m_refreshTime=1800;
    m_initialRefreshTime=m_refreshTime;
-};
+}
 
 int NetManager::prepare()
 {
@@ -131,14 +158,28 @@ int NetManager::prepare()
       m_listenFD=::socket(AF_LOCAL, SOCK_STREAM, 0);
       //m_listenFD=::socket(AF_LOCAL, SOCK_STREAM, IPPROTO_TCP);
       MyString socketName("/tmp/resLisa-");
-      socketName+=getenv("LOGNAME");
+      struct passwd *user = getpwuid( getuid() );
+      if ( user )
+         socketName+=user->pw_name;
+      else
+         //should never happen
+         socketName+="???";
       ::unlink(socketName.data());
       sockaddr_un serverAddr;
-//      bzero((char*)&serverAddr, sizeof(serverAddr));
+      if (socketName.length() >= sizeof(serverAddr.sun_path))
+      {
+         std::cout<<"NetManager::prepare: your user name  \""<<user->pw_name<<"\" is too long, exiting."<<std::endl;
+         return 0;
+      }
       memset((void*)&serverAddr, 0, sizeof(serverAddr));
-      serverAddr.sun_family      = AF_LOCAL;
-      strcpy(serverAddr.sun_path,socketName.data());
-      ::bind(m_listenFD,(sockaddr*) &serverAddr,sizeof(serverAddr));
+      serverAddr.sun_family=AF_LOCAL;
+      strncpy(serverAddr.sun_path,socketName.data(),sizeof(serverAddr.sun_path));
+      result=::bind(m_listenFD,(sockaddr*) &serverAddr,sizeof(serverAddr));
+      if (result!=0)
+      {
+         std::cout<<"NetManager::prepare: bind (UNIX socket) failed, errno: "<<errno<<std::endl;
+         return 0;
+      }
    }
    else
    {
@@ -148,7 +189,7 @@ int NetManager::prepare()
       {
          std::cout<<"NetManager::prepare: socket(TCP) failed, errno: "<<errno<<std::endl;
          return 0;
-      };
+      }
 
       sockaddr_in serverAddress;
 //      bzero((char*)&serverAddress, sizeof(serverAddress));
@@ -163,24 +204,24 @@ int NetManager::prepare()
       {
          std::cout<<"NetManager::prepare: setsockopt(SO_REUSEADDR) failed"<<std::endl;
          return 0;
-      };
+      }
       result=::bind(m_listenFD, (struct sockaddr *) &serverAddress, sizeof(serverAddress));
       if (result!=0)
       {
          std::cout<<"NetManager::prepare: bind (TCP) failed, errno: "<<errno<<std::endl;
          return 0;
-      };
-   };
+      }
+   }
    result=::listen(m_listenFD, 32);
    if (result!=0)
    {
       std::cout<<"NetManager::prepare: listen failed"<<std::endl;
       return 0;
-   };
+   }
    mgetDebug()<<"NetManager::prepare: listening on port "<<m_basePort<<"..."<<std::endl;
 
    return 1;
-};
+}
 
 void NetManager::generateFDset(fd_set *tmpFDs)
 {
@@ -194,20 +235,20 @@ void NetManager::generateFDset(fd_set *tmpFDs)
       {
          mgetDebug()<<"NetManager::generateFDset: adding client FD="<<tmpClient->fd()<<std::endl;
          FD_SET(tmpClient->fd(),tmpFDs);
-      };
+      }
 
    if (m_pipeFD!=-1)
    {
       mgetDebug()<<"NetManager::generateFDset: adding pipeFD="<<m_pipeFD<<std::endl;
       FD_SET(m_pipeFD,tmpFDs);
-   };
+   }
 
    if ((m_bcFD!=-1) && (!m_strictMode))
    {
       mgetDebug()<<"NetManager::generateFDset: adding m_bcFD="<<m_bcFD<<std::endl;
       FD_SET(m_bcFD,tmpFDs);
-   };
-};
+   }
+}
 
 int NetManager::waitForSomethingToHappen(fd_set *tmpFDs)
 {
@@ -225,7 +266,7 @@ int NetManager::waitForSomethingToHappen(fd_set *tmpFDs)
    int result=select(getMaxFD(),tmpFDs,0,0,&tv);
    if (result>0) return 1;
    else return 0;
-};
+}
 
 int NetManager::getMaxFD()
 {
@@ -238,7 +279,7 @@ int NetManager::getMaxFD()
 
    mgetDebug()<<"NetManager::getMaxFD()="<<maxFD+1<<std::endl;
    return maxFD+1;
-};
+}
 
 int fileReadable(const MyString& filename)
 {
@@ -247,7 +288,7 @@ int fileReadable(const MyString& filename)
       return 0;
    fclose(file);
    return 1;
-};
+}
 
 MyString NetManager::getConfigFileName()
 {
@@ -278,7 +319,7 @@ MyString NetManager::getConfigFileName()
       if (fileReadable(tmpFilename))
          return tmpFilename;
       return "";
-   };
+   }
 /*   else if (m_configStyle==KDE1CONFIGSTYLE)
    {
       tmpFilename=getenv("HOME");
@@ -303,6 +344,8 @@ MyString NetManager::getConfigFileName()
       char buf[4*1024];
       memset(buf,0,4*1024);
       fgets(buf,4*1024-1,kdeConfig);
+      // Warning: The return value of plcose may be incorrect due to the
+      // SIGCHLD handler that is installed. Ignore it!
       pclose(kdeConfig);
       int length=strlen(buf);
       if (buf[length-1]=='\n') buf[length-1]='\0';
@@ -327,7 +370,7 @@ MyString NetManager::getConfigFileName()
       return "";
    }*/
    return "";
-};
+}
 
 int NetManager::run()
 {
@@ -371,14 +414,14 @@ int NetManager::run()
             {
                mgetDebug()<<"NetManager::run: kicking client"<<std::endl;
                ::close(connectionFD);
-            };
-         };
+            }
+         }
          checkClientsAndPipes(&tmpFDs);
-      };
+      }
       serveAndClean();
-   };   
+   }
    return 1;
-};
+}
 
 void NetManager::addClient(int socketFD)
 {
@@ -386,7 +429,7 @@ void NetManager::addClient(int socketFD)
    if (socketFD==-1) return;
    Client c(this,socketFD,0);
    clients.append(c);
-};
+}
 
 void NetManager::checkClientsAndPipes(fd_set *tmpFDs)
 {
@@ -399,8 +442,8 @@ void NetManager::checkClientsAndPipes(fd_set *tmpFDs)
       {
          mgetDebug()<<"NetManager::checkClientsAndPipes: client sent something"<<std::endl;
          tmpClient->read();
-      };
-   };
+      }
+   }
 
    //now check wether we received a broadcast
    //m_bcFD should always be -1 in strictMode
@@ -410,7 +453,7 @@ void NetManager::checkClientsAndPipes(fd_set *tmpFDs)
       //yes !
       if (FD_ISSET(m_bcFD,tmpFDs))
          answerBroadcast();
-   };
+   }
 
    //read the stuff from the forked pipe
    if (m_pipeFD!=-1)
@@ -423,24 +466,13 @@ void NetManager::checkClientsAndPipes(fd_set *tmpFDs)
          if (result!=1)
          {
             ::close(m_pipeFD);
-            
-            //can I really be 100% sure that the child will quit very soon ?
-            //::waitpid(m_childPid,0,0);
-
-            //I better wait for a short moment and then continue
-            //it's better to have possibly a zombie than a blocking server IMO
-            timeval tv;
-            tv.tv_sec=0;
-            tv.tv_usec=5*10*1000;//0.05 sec
-            ::select(0,0,0,0,&tv);
-            ::waitpid(m_childPid,0,WNOHANG);
             m_pipeFD=-1;
             mgetDebug()<<"NetManager::checkClientsAndPipes: everything read from pipe from proc "<<m_childPid<<std::endl;
             processScanResults();
-         };
-      };
-   };
-};
+         }
+      }
+   }
+}
 
 void NetManager::answerBroadcast()
 {
@@ -468,7 +500,7 @@ void NetManager::answerBroadcast()
    {
       mgetDebug()<<"NetManager::answerBroadcast: must be the same machine"<<std::endl;
       return;
-   };
+   }
 
    //mgetDebug()<<"received "<<ntohl(buf[0])<<" from "<<inet_ntoa(sAddr.sin_addr)<<hex<<" ";
    //mgetDebug()<<sAddr.sin_addr.s_addr<<" //"<<ntohl(sAddr.sin_addr.s_addr)<<dec<<std::endl;
@@ -477,7 +509,7 @@ void NetManager::answerBroadcast()
    {
       mgetDebug()<<"NetManager::answerBroadcast: invalid sender"<<std::endl;
       return;
-   };
+   }
 
    //create the answering socket
 
@@ -486,7 +518,7 @@ void NetManager::answerBroadcast()
    {
       mgetDebug()<<"NetManager::answerBroadcast: could not create answering socket"<<std::endl;
       return;
-   };
+   }
    
    sAddr.sin_family=AF_INET;
    sAddr.sin_port=htons(m_basePort+1);
@@ -501,7 +533,7 @@ void NetManager::answerBroadcast()
    mgetDebug()<<"sent "<<result<<" byte using sendto"<<std::endl;
    ::close(answerFD);
    //sent answer
-};
+}
 
 void NetManager::serveAndClean()
 {
@@ -510,7 +542,7 @@ void NetManager::serveAndClean()
    {
       mgetDebug()<<"NetManager::serveAndClean: trying to get info"<<std::endl;
       tmpClient->tryToGetInfo();
-   };
+   }
    
    //try to delete served the clients
    for (Client* tmpClient=clients.first();tmpClient!=0; tmpClient=clients.next())
@@ -524,9 +556,9 @@ void NetManager::serveAndClean()
          mgetDebug()<<"NetManager::serveAndClean: removing Client"<<std::endl;
          clients.remove(tmpClient);
          tmpClient=clients.first();
-      };
-   };
-};
+      }
+   }
+}
 
 void NetManager::scan()
 {
@@ -556,7 +588,7 @@ void NetManager::scan()
       m_pipeFD=fileDescr[0];
       m_childPid=pid;
       return;
-   };
+   }
    //child
    procId="** child ** ";
    mgetDebug()<<" NetManager::scan: a child was born"<<std::endl;
@@ -586,14 +618,14 @@ void NetManager::scan()
          mgetDebug()<<" NetManager::scan: getting list from serverServer"<<std::endl;
          getListFromServerServer(serverAddress);
          m_serverServer=0;
-      };
-   };
+      }
+   }
    mgetDebug()<<" NetScanner::scan: sending information to parent process"<<std::endl;
    writeDataToFD(fileDescr[1],m_serverServer);
    mgetDebug()<<" NetScanner::scan: closed FD: "<<::close(fileDescr[1])<<std::endl;
    mgetDebug()<<" NetScanner::scan: exiting now"<<std::endl;
    ::exit(0);
-};
+}
 
 int NetManager::writeDataToFD(int fd, int serverServer)
 {
@@ -616,11 +648,11 @@ int NetManager::writeDataToFD(int fd, int serverServer)
          {
             perror("hmmpf: ");
             return 0;
-         };
+         }
          length-=result;
          currentBuf+=result;
-      };
-   };
+      }
+   }
    //and a last line
    sprintf(buffer,"%d succeeded\n",serverServer);
    length=strlen(buffer)+1;
@@ -632,9 +664,9 @@ int NetManager::writeDataToFD(int fd, int serverServer)
       if (result==-1) return 0;
       length-=result;
       currentBuf+=result;
-   };
+   }
    return 1;
-};
+}
 
 int NetManager::readDataFromFD(int fd)
 {
@@ -647,15 +679,19 @@ int NetManager::readDataFromFD(int fd)
    {
       mgetDebug()<<"NetManager::readDataFromFD: FD was closed"<<std::endl;
       return 0;
-   };
+   }
    char *newBuf=new char[m_receivedBytes+result];
    if (m_receiveBuffer!=0) memcpy(newBuf,m_receiveBuffer,m_receivedBytes);
    memcpy(newBuf+m_receivedBytes,tmpBuf,result);
    m_receivedBytes+=result;
    if (m_receiveBuffer!=0) delete [] m_receiveBuffer;
    m_receiveBuffer=newBuf;
+   // too much data - abort at 2MB to avoid memory exhaustion
+   if (m_receivedBytes>2*1024*1024)
+     return 0;
+
    return 1;
-};
+}
 
 int NetManager::processScanResults()
 {
@@ -665,14 +701,15 @@ int NetManager::processScanResults()
 
    char *tmpBuf=m_receiveBuffer;
    int bytesLeft=m_receivedBytes;
-   int tmpIP;
    mgetDebug()<<"m_receivedBytes: "<<m_receivedBytes<<" bytesLeft: "<<bytesLeft<<std::endl;
    //this should be large enough for a name
    //and the stuff which is inserted into the buffer
-   //comes only from ourselves
+   //comes only from ourselves ... or attackers :-(
    char tmpName[1024*4];
    while (bytesLeft>0)
    {
+      int tmpIP=2;  // well, some impossible IP address, 0 and 1 are already used for the last line of output
+      tmpName[0]='\0';
       if ((memchr(tmpBuf,0,bytesLeft)==0) || (memchr(tmpBuf,int('\n'),bytesLeft)==0))
       {
          delete newNodes;
@@ -685,16 +722,18 @@ int NetManager::processScanResults()
          m_isInformed=1;
          m_isBeingScanned=0;
          return 0;
-      };
+      }
       //mgetDebug()<<"NetManager::processScanResults: processing -"<<tmpBuf;
-      sscanf(tmpBuf,"%u %s\n",&tmpIP,tmpName);
       //since we check for 0 and \n with memchr() we can be sure
       //at this point that tmpBuf is correctly terminated
       int length=strlen(tmpBuf)+1;
+      if (length<(4*1024))
+         sscanf(tmpBuf,"%u %s\n",&tmpIP,tmpName);
+
       bytesLeft-=length;
       tmpBuf+=length;
       mgetDebug()<<"length: "<<length<<" bytesLeft: "<<bytesLeft<<std::endl;
-      if ((bytesLeft==0) && (strstr(tmpName,"succeeded")!=0) && ((tmpIP==0) ||(tmpIP==1)))
+      if ((bytesLeft==0) && ((tmpIP==0) ||(tmpIP==1)) && (strstr(tmpName,"succeeded")!=0))
       {
          mgetDebug()<<"NetManager::processScanResults: succeeded :-)"<<std::endl;
          delete hostList;
@@ -712,12 +751,12 @@ int NetManager::processScanResults()
 
          return 1;
       }
-      else
+      else if (tmpIP!=2)
       {
          //mgetDebug()<<"NetManager::processScanResults: adding host: "<<tmpName<<" with ip: "<<tmpIP<<std::endl;
          newNodes->append(Node(tmpName,tmpIP));
-      };
-   };
+      }
+   }
    //something failed :-(
    delete newNodes;
    hostList->clear();
@@ -731,7 +770,7 @@ int NetManager::processScanResults()
 
    mgetDebug()<<"NetScanner::processScanResult: finished"<<std::endl;
    return 0;
-};
+}
 
 void NetManager::adjustRefreshTime(int serverServer)
 {
