@@ -1,8 +1,8 @@
-/*
+/*  -*- c++ -*
     kmime_headers.h
 
     KMime, the KDE internet mail/usenet news message library.
-    Copyright (c) 2001 the KMime authors.
+    Copyright (c) 2001-2002 the KMime authors.
     See file AUTHORS for details
 
     This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,7 @@
 // - incompatible, GStructured-based field classes
 // - compatible, GUnstructured-based field classes
 
-#include <time.h>
+#include "kmime_header_parsing.h"
 
 #include <qstring.h>
 #include <qstrlist.h>
@@ -33,21 +33,15 @@
 #include <qmap.h>
 #include <qptrlist.h>
 
+#include <time.h>
+
 namespace KMime {
 
 //forward declaration
 class Content;
 
-
 namespace Headers {
 
-// for when we can't make up our mind what to use...
-struct QStringOrQPair {
-  QStringOrQPair() : qstring(), qpair(0,0) {}
-  QString qstring;
-  QPair<char*,int> qpair;
-};
-
 
 enum contentCategory    { CCsingle,
                           CCcontainer,
@@ -68,7 +62,6 @@ enum contentDisposition { CDinline,
 //often used charset
 static const QCString Latin1("ISO-8859-1");
 
-
 #define mk_trivial_subclass_with_name( subclass, subclassName, baseclass ) \
 class subclass : public Generics::baseclass { \
 public: \
@@ -99,13 +92,12 @@ public: \
   \
   const char * type() const { return #subclassName; } \
 protected: \
-  bool parse( char* & scursor, const char * send, bool isCRLF=false ); \
+  bool parse( const char* & scursor, const char * const send, bool isCRLF=false ); \
 }
 
 #define mk_parsing_subclass( subclass, baseclass ) \
 mk_parsing_subclass_with_name( subclass, subclass, baseclass )
 
-
 //
 //
 // HEADER'S BASE CLASS. DEFINES THE COMMON INTERFACE
@@ -181,23 +173,6 @@ class Base {
     bool isXHeader()  { return (strncmp(type(), "X-", 2)==0); }
 
   protected:
-    /** Parse the encoded word in @p str pointed to by @p pos
-        (actually, @p pos-2, see below).
-
-	@param str the source string
-	@param pos in: the starting position (must already point to the
-                   character following the initial '=?';
-		   out: the new postion
-	@param ok  only out: if true, the encoded-word was correct up
-	           to and including the encoding specifier. The
-		   encoded-text is quite generously parsed and @p ok
-		   is still set to @p true when e.g. the encoded-word
-		   appears to be truncated or contains whitespace.
-	@return the decoded string the encoded word represented.
-    */
-    bool parseEncodedWord( char* & scursor, const char * send,
-			   QString & result, QCString & language );
-    
     QCString typeIntro()  { return (QCString(type())+": "); }
 
     const char *e_ncCS;
@@ -301,149 +276,24 @@ public:
 
   
 protected:
-  /** Types of tokens to be found in structured header fields:
-      <pre>
-          Type     |def'ed in| ... as
-      -------------+------------------------------------------
-      Token        | rfc2045 | 1*<CHAR w/o SPACE, CTLs and tspecials>
-      TSpecial     | rfc2045 | Special / "/" / "?" / "="
-      Atom         | rfc822  | 1*atext
-                   |         | atext := <CHAR w/o specials, SPACE and CTLs>
-		   | KMime   | 1*(token / "/" / "?" / "=")
-      Special      | rfc822  | "()<>@,;:.[]" / <"> / "\"
-      Comment      | rfc822  | "(" *(ctext / quoted-pair / comment) ")"
-                   |         | ctext := <CHAR w/o "()\", CR, but LWSP>
-      QuotedString | rfc822  | <"> *(qtext/quoted-pair) <">
-                   |         | qtext := <CHAR w/o <">, "\", CR, but LWSP>
-      DomainLiteral| rfc822  | "[" *(dtext / quoted-pair) "]"
-                   |         | dtext := <CHAR w/o "[]\", CR, but LWSP>
-      EncodedWord  | rfc2047 | "=?" charset "?" ( "q" / "b" ) "?" encoded-text "?="
-      Phrase       | rfc822  | 1*word
-                   | rfc2047 | 1*(word / encoded-word)
-      DotAtom      | rfc2822 | 1*atext *("." 1*atext)
-                   | KMime   | atom *("." atom) ["."]
-      Word         | rfc822  | atom / quoted-string
-                   | KMime   | atom / token / quoted-string
-      </pre>
-   */
-  enum TokenType { None = 0x00, Atom = 0x001, Special = 0x002, Comment = 0x004,
-		   QuotedString = 0x008, DomainLiteral = 0x010,
-		   EncodedWord = 0x020, Phrase = 0x040,
-		   Token = 0x080, TSpecial = 0x100, DotAtom = 0x200,
-		   EightBit = 0x400, Word = Token|Atom|QuotedString,
-		   All = Word|Special|Comment|DomainLiteral|EncodedWord|Phrase|TSpecial|DotAtom|EightBit};
-  /** Starting at index @p pos in @p source, return the next token of
-      a type specified by @p tt (can be or'ed).
-
-      Different tokens behave differently when they are disallowed. In
-      particular, Comment is considered part of CFWS and simply
-      ignored (but still parsed correctly!). If you choose to disallow
-      QuotedString or DomainLiteral, you will get them as a list of
-      Atom's and Specials, one per call. Note that this might lead to
-      parse errors when you change the type mask later on to allow
-      them again. Tokens can't be disallowed. If you forbid TSpecials,
-      you must allow Atoms and Specials. The lowest syntactic entities
-      are then Atom and Special (ie. like in rfc822). If you forbid
-      EncodedWords, you'll get them as an Atom, or, if you allowed
-      TSpecials, as a list of TSpecials and Tokens. If you forbit
-      Phrase, getToken doesn't condense consecutive Word's into a
-      Phrase, but returns them one by one.  If you disallow DotAtom,
-      you'll get them as a list of Atoms and Specials.
-
-      NOTE: Due to keeping the tokenizer lookahead (and complexity)
-      under control, you must take into account several issues with
-      some of the token types listed above. In particular, the
-      definition of a dot-atom deviates from the RFC one in that it is
-      allowed to have a trailing dot. The lookahead necessary to make
-      sure that this case does not occur would be unlimited (remember
-      that rfc822 allows arbitrary CFWS in between dot-atoms, though
-      rfc2822 forbids that). You should therefore exclude dot-atoms
-      from the list of requested types unless you really expect one,
-      and check for trailing dots if you get one.
-
-      Also, a sequence of encoded-words is condensed into a single
-      one. This is because there are special rules for when encoded
-      words are adjacent.
-
-      @param source source string
-      @param pos    in: starting position; out: new position
-      @param tt     in: types of tokens requested; out: type of token returned
-      @param CRLF   if true, the mail is in canonical CRLF form, so the parsers
-                    can handle lonely '\n' and '\r' according to rfc822.
-                    If false, the mail is in Unix-native format and the parser
-		    will eat every occurence of unescaped '\n' followed by
-		    SPACE or HTAB as meaning line folding.
-      @return the decoded token or @ref QString::null if there are no
-              more tokens
-      @see tokenType */
-  virtual QString getToken( char* & scursor, const char * send,
-			    TokenType & tt, bool isCRLF=false );
-
-  //
-  // The parsing squad:
-  //
-
-  /** You may or may not have already started parsing into the
-      atom. This function will go on where you left off. */
-  bool parseAtom( char* & scursor, const char * send, QString & result,
-		  bool allow8Bit=false );
-  bool parseAtom( char* & scursor, const char * send,
-		  QPair<char*,int> & result, bool allow8Bit=false );
-  /** You may or may not have already started parsing into the
-      token. This function will go on where you left off. */
-  bool parseToken( char* & scursor, const char * send, QString & result,
-		   bool allow8Bit=false );
-  bool parseToken( char* & scursor, const char * send,
-		   QPair<char*,int> & result, bool allow8Bit=false );
-  /** @p scursor must be positioned after the opening openChar. */
-  bool parseGenericQuotedString( char* & scursor, const char* send,
-				 QString & result, bool isCRLF,
-				 const char openChar='"',
-				 const char closeChar='"' );
-  /** @p scursor must be positioned right after the opening '(' */
-  bool parseComment( char* & scursor, const char * send, QString & result,
-		     bool isCRLF=false, bool reallySave=true );
-  /** You may or may not have already started parsing into the phrase,
-      but only if it starts with atext. If you setup this function to
-      parse a phrase starting with an encoded-word or quoted-string,
-      @p scursor has to point to the char introducing the encoded-word
-      or quoted-string, resp. */
-  bool parsePhrase( char* & scursor, const char * send, QString & result,
-		    bool isCRLF=false );
-  /** You may or may not have already started parsing into the initial
-      atom, but not up to it's end. */
-  bool parseDotAtom( char* & scursor, const char * send, QString & result,
-		     bool isCRLF=false );
-
-  /** Eats comment-folding-white-space, skips whitespace, folding and
-      comments (even nested ones) and stops at the next non-CFWS
-      character. After calling this function, you should check whether
-      @p scursor == @p send (end of header reached).
-
-      If a comment with unbalanced parantheses is encountered, @p
-      scursor is being positioned on the opening '(' of the outmost
-      comment.
-  */
-  void eatCFWS( char* & scursor, const char * send, bool isCRLF );
-
 #if 0
   // the assembly squad:
 
-  bool writeAtom( char* & dcursor, const char * dend, const QString & input );
-  bool writeAtom( char* & dcursor, const char * dend,
-		  const QPair<char*,int> & input );
-  bool writeToken( char* & dcursor, const char * dend, const QString & input );
-  bool writeToken( char* & dcursor, const char * dend,
-		   const QPair<char*int> & input );
+  bool writeAtom( char* & dcursor, const char * const dend, const QString & input );
+  bool writeAtom( char* & dcursor, const char * const dend,
+		  const QPair<const char*,int> & input );
+  bool writeToken( char* & dcursor, const char * const dend, const QString & input );
+  bool writeToken( char* & dcursor, const char * const dend,
+		   const QPair<const char*int> & input );
 
-  bool writeGenericQuotedString( char* & dcursor, const char * dend,
+  bool writeGenericQuotedString( char* & dcursor, const char * const dend,
 				 const QString & input, bool withCRLF=false );
-  bool writeComment( char* & dcursor, const char * dend,
+  bool writeComment( char* & dcursor, const char * const dend,
+		     const QString & input, bool withCRLF=false );
+  bool writePhrase( char* & dcursor, const char * const dend,
+		    const QString & input, bool withCRLF=false );
+  bool writeDotAtom( char* & dcursor, const char * const dend,
 		     const QString & input, bool withCRLF=false );
-  bool writePhrase( char* & dcursor, const char * dend, const QString & input,
-		    bool withCRLF=false );
-  bool writeDotAtom( char* & dcursor, const char * dend, const QString & input,
-		     bool withCRLF=false );
 #endif
 };
 
@@ -458,39 +308,7 @@ public:
     : GStructured( p )  { fromUnicodeString( s, cs ); }
   ~GAddress()  {}
 
-  struct AddrSpec {
-    QString localPart;
-    QString domain;
-  };
-
-  struct Mailbox {
-    QString displayName;
-    AddrSpec addrSpec;
-  };
-
-  struct Address {
-    QString displayName;
-    QValueList<Mailbox> mailboxList;
-  };
-
 protected:
-  bool parseDomain( char* & scursor, const char * send,
-		    QString & result, bool isCRLF=false );
-  bool parseObsRoute( char* & scursor, const char * send, QStringList & result,
-		      bool isCRLF=false, bool save=false );
-  bool parseAddrSpec( char* & scursor, const char * send,
-		      AddrSpec & result, bool isCRLF=false );
-  bool parseAngleAddr( char* & scursor, const char * send,
-		       AddrSpec & result, bool isCRLF=false );
-  bool parseMailbox( char* & scursor, const char * send,
-		     Mailbox & result, bool isCRLF=false );
-  bool parseGroup( char* & scursor, const char * send,
-		   Address & result, bool isCRLF=false );
-  bool parseAddress( char* & scursor, const char * send,
-		     Address & result, bool isCRLF=false );
-  bool parseAddressList( char* & scursor, const char * send,
-			 QValueList<Address> & result, bool isCRLF=false );
-
 };
 
 
@@ -507,10 +325,10 @@ public:
   ~MailboxList()  {}
 
 protected:
-  bool parse( char* & scursor, const char * send, bool isCRLF=false );
+  bool parse( const char* & scursor, const char * const send, bool isCRLF=false );
 
   /** The list of mailboxes */
-  QValueList<Mailbox> mMailboxList;
+  QValueList<Types::Mailbox> mMailboxList;
 };
 
 
@@ -531,10 +349,10 @@ public:
   ~AddressList()  {}
 
 protected:
-  bool parse( char* & scursor, const char * send, bool isCRLF=false );
+  bool parse( const char* & scursor, const char * const send, bool isCRLF=false );
 
   /** The list of addresses */
-  QValueList<Address> mAddressList;
+  QValueList<Types::Address> mAddressList;
 };
 
 /** Base class for headers which deal with a list of msg-id's */
@@ -549,10 +367,10 @@ public:
   ~GIdent()  {}
 
 protected:
-  bool parse( char* & scursor, const char * send, bool isCRLF=false );
+  bool parse( const char* & scursor, const char * const send, bool isCRLF=false );
 
   /** The list of msg-id's */
-  QValueList<AddrSpec> mMsgIdList;
+  QValueList<Types::AddrSpec> mMsgIdList;
 };
 
 /** Base class for headers which deal with a list of msg-id's */
@@ -570,7 +388,7 @@ public:
   ~GToken()  {}
 
 protected:
-  bool parse( char* & scursor, const char * send, bool isCRLF=false );
+  bool parse( const char* & scursor, const char * const send, bool isCRLF=false );
 
   QCString mToken;
 };
@@ -587,7 +405,7 @@ public:
   ~GPhraseList()  {}
 
 protected:
-  bool parse( char* & scursor, const char * send, bool isCRLF=false );
+  bool parse( const char* & scursor, const char * const send, bool isCRLF=false );
 
   QStringList mPhraseList;
 };
@@ -603,7 +421,7 @@ public:
   ~GDotAtom()  {}
 
 protected:
-  bool parse( char* & scursor, const char * send, bool isCRLF=false );
+  bool parse( const char* & scursor, const char * const send, bool isCRLF=false );
 
   QString mDotAtom;
 };
@@ -619,18 +437,9 @@ public:
   ~GParametrized()  {}
 
 protected:
-  bool parseParameter( char* & scursor, const char * send,
-		       QPair<QString,QStringOrQPair> & result,
-		       bool isCRLF=false );
-  bool parseParameterList( char* & scursor, const char * send,
-			   QMap<QString,QString> & result, bool isCRLF=false );
-
   QMap<QString,QString> mParameterHash;
 
 private:
-  bool parseRawParameterList( char* & scursor, const char * send,
-			      QMap<QString,QStringOrQPair> & result,
-			      bool isCRLF=false );
 };
 
 class GContentType : public GParametrized {
@@ -644,7 +453,7 @@ public:
   ~GContentType()  {}
 
 protected:
-  bool parse( char* & scursor, const char * send, bool isCRLF=false );
+  bool parse( const char* & scursor, const char * const send, bool isCRLF=false );
 
   QCString mMimeType;
   QCString mMimeSubType;
@@ -662,7 +471,7 @@ public:
   ~GCISTokenWithParameterList()  {}
 
 protected:
-  bool parse( char* & scursor, const char * send, bool isCRLF=false );
+  bool parse( const char* & scursor, const char * const send, bool isCRLF=false );
 
   QCString mToken;
 };
@@ -691,7 +500,7 @@ public:
   const char * type() const { return "Return-Path"; }
 
 protected:
-  bool parse( char* & scursor, const char * send, bool isCRLF=false );
+  bool parse( const char* & scursor, const char * const send, bool isCRLF=false );
 };
 
 #if defined(KMIME_NEW_STYLE_CLASSTREE)
