@@ -45,7 +45,7 @@ Base2::~Base2()
 
 
 int
-Base2::encrypt( Block& block, const QStrList& recipients )
+Base2::encrypt( Block& block, const KeyIDList& recipients )
 {
   return encsign( block, recipients, 0 );
 }
@@ -54,12 +54,12 @@ Base2::encrypt( Block& block, const QStrList& recipients )
 int
 Base2::clearsign( Block& block, const char *passphrase )
 {
-  return encsign( block, 0, passphrase );
+  return encsign( block, KeyIDList(), passphrase );
 }
 
 
 int
-Base2::encsign( Block& block, const QStrList& recipients,
+Base2::encsign( Block& block, const KeyIDList& recipients,
                 const char *passphrase )
 {
   QCString cmd;
@@ -87,9 +87,8 @@ Base2::encsign( Block& block, const QStrList& recipients,
       cmd += Module::getKpgp()->user();
     }
     
-    QStrListIterator it(recipients);
-    for(; it.current(); ++it)
-    {
+    for( KeyIDList::ConstIterator it = recipients.begin();
+         it != recipients.end(); ++it ) {
       cmd += " 0x";
       cmd += (*it);
     }
@@ -461,7 +460,7 @@ Base2::decrypt( Block& block, const char *passphrase )
 
 
 Key*
-Base2::readPublicKey( const QCString& keyID,
+Base2::readPublicKey( const KeyID& keyID,
                       const bool readTrust /* = false */,
                       Key* key /* = 0 */ )
 {
@@ -532,7 +531,7 @@ Base2::secretKeys()
 
 
 int
-Base2::signKey(const QCString& keyID, const char *passphrase)
+Base2::signKey(const KeyID& keyID, const char *passphrase)
 {
   QCString cmd;
   int exitStatus = 0;
@@ -551,7 +550,7 @@ Base2::signKey(const QCString& keyID, const char *passphrase)
 }
 
 
-QCString Base2::getAsciiPublicKey(const QCString& keyID)
+QCString Base2::getAsciiPublicKey(const KeyID& keyID)
 {
   int exitStatus = 0;
 
