@@ -46,7 +46,7 @@ Base5::~Base5()
 
 
 int
-Base5::encrypt( Block& block, const QStrList& recipients )
+Base5::encrypt( Block& block, const KeyIDList& recipients )
 {
   return encsign( block, recipients, 0 );
 }
@@ -55,12 +55,12 @@ Base5::encrypt( Block& block, const QStrList& recipients )
 int
 Base5::clearsign( Block& block, const char *passphrase )
 {
-  return encsign( block, 0, passphrase );
+  return encsign( block, KeyIDList(), passphrase );
 }
 
 
 int
-Base5::encsign( Block& block, const QStrList& recipients,
+Base5::encsign( Block& block, const KeyIDList& recipients,
                 const char *passphrase )
 {
   QCString cmd;
@@ -97,9 +97,8 @@ Base5::encsign( Block& block, const QStrList& recipients,
       cmd += Module::getKpgp()->user();
     }
 
-    QStrListIterator it(recipients);
-    for(; it.current(); ++it)
-    {
+    for( KeyIDList::ConstIterator it = recipients.begin();
+         it != recipients.end(); ++it ) {
       cmd += " -r 0x";
       cmd += (*it);
     }
@@ -318,7 +317,7 @@ Base5::decrypt( Block& block, const char *passphrase )
 
 
 Key*
-Base5::readPublicKey( const QCString& keyId, const bool readTrust, Key* key )
+Base5::readPublicKey( const KeyID& keyId, const bool readTrust, Key* key )
 {
   int exitStatus = 0;
 
@@ -399,7 +398,7 @@ Base5::secretKeys()
 }
 
 
-QCString Base5::getAsciiPublicKey(const QCString& keyID)
+QCString Base5::getAsciiPublicKey(const KeyID& keyID)
 {
   int exitStatus = 0;
 
@@ -419,7 +418,7 @@ QCString Base5::getAsciiPublicKey(const QCString& keyID)
 
 
 int
-Base5::signKey(const QCString& keyID, const char *passphrase)
+Base5::signKey(const KeyID& keyID, const char *passphrase)
 {
   QCString cmd;
   int exitStatus = 0;
