@@ -184,7 +184,7 @@ Base6::decrypt( Block& block, const char *passphrase )
 
 
 Key*
-Base6::readPublicKey( const QCString& keyID,
+Base6::readPublicKey( const KeyID& keyID,
                       const bool readTrust /* = false */,
                       Key* key /* = 0 */ )
 {
@@ -315,24 +315,14 @@ Base6::pubKeys()
 	line = line.stripWhiteSpace();
       } else {
 	// line with new key
-	int index4 = error.find(
-#if QT_VERSION < 290
-	  QRegExp("/[0-9][0-9]/[0-9][0-9] "),
-#else
-	  QRegExp("/\\d{2}/\\d{2} "),
-#endif
-	  index);
+	int index4 = error.find(QRegExp("/\\d{2}/\\d{2} "), index);
 	line = error.mid(index4+7,index2-index4-7);
       }
       //kdDebug(5100) << "Base: found key for " << (const char *)line << endl;
 
       // don't add PGP's comments to the key list
       if (strncmp(line.data(),"*** KEY EXPIRED ***",19) &&
-#if QT_VERSION < 290
-          line.find(QRegExp("^expires [0-9][0-9][0-9][0-9]/[0-9][0-9]/[0-9][0-9]")) < 0 &&
-#else
           line.find(QRegExp("^expires \\d{4}/\\d{2}/\\d{2}")) < 0 &&
-#endif
           strncmp(line.data(),"*** DEFAULT SIGNING KEY ***",27)) {
         publicKeys.append(line);
       }
