@@ -43,7 +43,7 @@ BaseG::~BaseG()
 
 
 int
-BaseG::encrypt( Block& block, const QStrList& recipients )
+BaseG::encrypt( Block& block, const KeyIDList& recipients )
 {
   return encsign( block, recipients, 0 );
 }
@@ -52,12 +52,12 @@ BaseG::encrypt( Block& block, const QStrList& recipients )
 int
 BaseG::clearsign( Block& block, const char *passphrase )
 {
-  return encsign( block, 0, passphrase );
+  return encsign( block, KeyIDList(), passphrase );
 }
 
 
 int
-BaseG::encsign( Block& block, const QStrList& recipients,
+BaseG::encsign( Block& block, const KeyIDList& recipients,
                 const char *passphrase )
 {
   QCString cmd;
@@ -88,9 +88,8 @@ BaseG::encsign( Block& block, const QStrList& recipients,
       cmd += pgpUser;
     }
 
-    QStrListIterator it(recipients);
-    for(; it.current(); ++it)
-    {
+    for( KeyIDList::ConstIterator it = recipients.begin();
+         it != recipients.end(); ++it ) {
       cmd += " -r 0x";
       cmd += (*it);
     }
@@ -356,7 +355,7 @@ BaseG::decrypt( Block& block, const char *passphrase )
 
 
 Key*
-BaseG::readPublicKey( const QCString& keyID,
+BaseG::readPublicKey( const KeyID& keyID,
                       const bool readTrust /* = false */,
                       Key* key /* = 0 */ )
 {
@@ -430,7 +429,7 @@ BaseG::secretKeys()
 
 
 int
-BaseG::signKey(const QCString& keyID, const char *passphrase)
+BaseG::signKey(const KeyID& keyID, const char *passphrase)
 {
   QCString cmd;
   int exitStatus = 0;
@@ -451,7 +450,7 @@ BaseG::signKey(const QCString& keyID, const char *passphrase)
 
 
 QCString
-BaseG::getAsciiPublicKey(const QCString& keyID)
+BaseG::getAsciiPublicKey(const KeyID& keyID)
 {
   int exitStatus = 0;
 
@@ -492,11 +491,11 @@ BaseG::parsePublicKeyData( const QCString& output, Key* key /* = 0 */ )
   else
     key->clear();
 
+  QCString keyID;
+
   while( true )
   {
     int index2;
-    QCString keyID;
-
     // search the end of the current line
     if( ( index2 = output.find( '\n', index ) ) == -1 )
       break;
@@ -536,7 +535,8 @@ BaseG::parsePublicKeyData( const QCString& output, Key* key /* = 0 */ )
               subkey->setExpired( true );
               key->setExpired( true );
               break;
-            case 'q': // undefined (no value assigned)
+            case '-': // undefined (no path leads to the key)
+            case 'q': // undefined (no trusted path leads to the key)
             case 'n': // don't trust this key at all
             case 'm': // the key is marginally trusted
             case 'f': // the key is fully trusted
@@ -631,7 +631,8 @@ BaseG::parsePublicKeyData( const QCString& output, Key* key /* = 0 */ )
             case 'r': // the user id has been revoked
               userID->setRevoked( true );
               break;
-            case 'q': // undefined (no value assigned)
+            case '-': // undefined (no path leads to the key)
+            case 'q': // undefined (no trusted path leads to the key)
               userID->setValidity( KPGP_VALIDITY_UNDEFINED );
               break;
             case 'n': // don't trust this key at all
@@ -721,9 +722,10 @@ BaseG::parseKeyList(const QCString& output, bool secretKeys)
       index++;
   }
 
+  QCString keyID;
+
   while(true) {
     int index2;
-    QCString keyID;
 
     // search the end of the current line
     if ((index2 = output.find('\n',index)) == -1)
@@ -771,7 +773,8 @@ BaseG::parseKeyList(const QCString& output, bool secretKeys)
               subkey->setExpired( true );
               key->setExpired( true );
               break;
-            case 'q': // undefined (no value assigned)
+            case '-': // undefined (no path leads to the key)
+            case 'q': // undefined (no trusted path leads to the key)
             case 'n': // don't trust this key at all
             case 'm': // the key is marginally trusted
             case 'f': // the key is fully trusted
@@ -869,7 +872,8 @@ BaseG::parseKeyList(const QCString& output, bool secretKeys)
             case 'r': // the user id has been revoked
               userID->setRevoked( true );
               break;
-            case 'q': // undefined (no value assigned)
+            case '-': // undefined (no path leads to the key)
+            case 'q': // undefined (no trusted path leads to the key)
               userID->setValidity( KPGP_VALIDITY_UNDEFINED );
               break;
             case 'n': // don't trust this key at all
