@@ -21,6 +21,30 @@
 
 namespace Kpgp {
 
+/* member functions of Kpgp::KeyIDList --------------------------------- */
+
+/** Converts from a KeyIDList to a QStringList.
+*/
+QStringList KeyIDList::toStringList() const
+{
+  QStringList res;
+  for( KeyIDList::ConstIterator it = begin(); it != end(); ++it ) {
+    res << (*it).data();
+  }
+  return res;
+}
+
+/** Converts from a QStringList to a KeyIDList.
+*/
+KeyIDList KeyIDList::fromStringList( const QStringList& l )
+{
+  KeyIDList res;
+  for( QStringList::ConstIterator it = l.begin(); it != l.end(); ++it ) {
+    res << (*it).local8Bit();
+  }
+  return res;
+}
+
 /* member functions of Kpgp::UserID ------------------------------------ */
 
 UserID::UserID(const QString& str, const Validity validity,
@@ -35,7 +59,7 @@ UserID::UserID(const QString& str, const Validity validity,
 
 /* member functions of Kpgp::Subkey ------------------------------------ */
 
-Subkey::Subkey(const QCString& keyID, const bool secret)
+Subkey::Subkey(const KeyID& keyID, const bool secret)
 {
   mSecret = secret;
   mKeyID = keyID;
@@ -57,7 +81,7 @@ Subkey::Subkey(const QCString& keyID, const bool secret)
 
 /* member functions of Kpgp::Key --------------------------------------- */
 
-Key::Key(const QCString& keyid, const QString& uid, const bool secret) :
+Key::Key(const KeyID& keyid, const QString& uid, const bool secret) :
   mSubkeys(), mUserIDs()
 {
   mSecret = secret;
@@ -79,7 +103,7 @@ Key::Key(const QCString& keyid, const QString& uid, const bool secret) :
 
 Key::~Key()
 {
-  //kdDebug(5100) << "Kpgp::Key: Deleting key " << primaryUserID() << endl;
+  kdDebug(5100) << "Kpgp::Key: Deleting key " << primaryUserID() << endl;
   mUserIDs.setAutoDelete(true);
   mUserIDs.clear();
   mSubkeys.setAutoDelete(true);
@@ -137,6 +161,24 @@ Key::keyTrust( const QString& uid ) const
   return trust;
 }
 
+void
+Key::cloneKeyTrust( const Key* key )
+{
+  if( !key )
+    return;
+
+  for( UserIDListIterator it(mUserIDs); it.current(); ++it )
+  {
+    (*it)->setValidity( key->keyTrust( (*it)->text() ) );
+  }
+}
+
+bool
+Key::isValid() const
+{
+  return ( !mRevoked && !mExpired && !mDisabled && !mInvalid );
+}
+
 
 bool
 Key::isValidEncryptionKey() const
@@ -174,7 +216,7 @@ bool Key::matchesUserID(const QString& str, bool cs)
   return false;
 }
 
-void Key::addSubkey(const QCString& keyID, const bool secret)
+void Key::addSubkey(const KeyID& keyID, const bool secret)
 {
   if (!keyID.isEmpty()) {
     Subkey *key = new Subkey(keyID, secret);
@@ -182,7 +224,7 @@ void Key::addSubkey(const QCString& keyID, const bool secret)
   }
 }
 
-Subkey *Key::getSubkey(const QCString& keyID)
+Subkey *Key::getSubkey(const KeyID& keyID)
 {
   if (keyID.isEmpty() || mSubkeys.isEmpty())
     return 0;
@@ -204,12 +246,15 @@ Subkey *Key::getSubkey(const QCString& keyID)
   return 0;
 }
 
-void Key::setFingerprint(const QCString& keyID, const QCString &fpr)
+void Key::setFingerprint(const KeyID& keyID, const QCString &fpr)
 {
   Subkey *key;
   if ((key = getSubkey(keyID)) != 0) {
     key->setFingerprint(fpr);
   }
+  else
+    kdDebug(5006) << "Error: Can't set fingerprint. A subkey with key ID 0x"
+                  << keyID << " doesn't exist." << endl;
 }
 
 }; // namespace Kpgp
