@@ -23,8 +23,8 @@
 
 #include <qcstring.h>
 #include <qstring.h>
-#include <qstrlist.h>
 #include <qstringlist.h>
+#include <qvaluelist.h>
 
 namespace Kpgp {
 
@@ -53,6 +53,22 @@ typedef enum
   AskWheneverPossible = 4
 } EncryptPref;
 
+
+typedef QCString KeyID;
+
+class KeyIDList : public QValueList<KeyID>
+{
+ public:
+  KeyIDList() { }
+  KeyIDList( const KeyIDList& l ) : QValueList<KeyID>(l) { }
+  KeyIDList( const QValueList<KeyID>& l ) : QValueList<KeyID>(l) { }
+  KeyIDList( const KeyID& i ) { append(i); }
+
+  QStringList toStringList() const;
+
+  static KeyIDList fromStringList( const QStringList& );
+};
+
 /** This class is used to store information about a user id of a PGP key.
  */
 class UserID
@@ -147,7 +163,7 @@ class Subkey
 {
  public:
   /** Constructs a new subkey with the given key ID. */
-  Subkey(const QCString& keyID, const bool secret = false);
+  Subkey(const KeyID& keyID, const bool secret = false);
   ~Subkey() {};
 
   /** Returns true if the subkey is a secret subkey. */
@@ -182,10 +198,10 @@ class Subkey
 
   /** Returns the long 64 bit key ID of the subkey if it's available. 
       Otherwise the short 32 bit key ID is returned. */
-  QCString longKeyID() const;
+  KeyID longKeyID() const;
 
   /** Returns the (short) 32 bit key ID of the subkey. */
-  QCString keyID() const;
+  KeyID keyID() const;
   
   /** Returns the fingerprint of the subkey. */
   QCString fingerprint() const;
@@ -230,7 +246,7 @@ class Subkey
   void setKeyLength(const unsigned int keyLen);
 
   /** Sets the key ID of the subkey to <em>keyID</em>. */
-  void setKeyID(const QCString& keyID);
+  void setKeyID(const KeyID& keyID);
 
   /** Sets the fingerprint of the subkey to <em>fingerprint</em>. */
   void setFingerprint(const QCString& fingerprint);
@@ -256,7 +272,7 @@ class Subkey
 
   unsigned int mKeyAlgo;
   unsigned int mKeyLen;
-  QCString mKeyID;
+  KeyID mKeyID;
   QCString mFingerprint;
   time_t mTimestamp; /* -1 for invalid, 0 for not available */
   time_t mExpiration; /* -1 for never, 0 for not available */
@@ -312,12 +328,12 @@ inline unsigned int Subkey::keyLength() const
   return mKeyLen;
 }
 
-inline QCString Subkey::longKeyID() const
+inline KeyID Subkey::longKeyID() const
 {
   return mKeyID;
 }
 
-inline QCString Subkey::keyID() const
+inline KeyID Subkey::keyID() const
 {
   return mKeyID.right(8);
 }
@@ -387,7 +403,7 @@ inline void Subkey::setKeyLength(const unsigned int keyLen)
   mKeyLen = keyLen;
 }
 
-inline void Subkey::setKeyID(const QCString& keyID)
+inline void Subkey::setKeyID(const KeyID& keyID)
 {
   mKeyID = keyID;
 }
@@ -418,7 +434,7 @@ class Key
  public:
   /** Constructs a new PGP key with <em>keyid</em> as key ID of the
       primary key and <em>uid</em> as primary user ID. */
-  Key( const QCString& keyid = QCString(),
+  Key( const KeyID& keyid = KeyID(),
        const QString& uid = QString::null,
        const bool secret = false);
   ~Key();
@@ -491,7 +507,11 @@ class Key
 
   /** Returns the key ID of the primary key or a null string if there
       are no subkeys. */
-  QCString primaryKeyID() const;
+  KeyID primaryKeyID() const;
+
+  /** Returns the fingerprint of the primary key or a null string if there
+      are no subkeys. */
+  QCString primaryFingerprint() const;
 
   /** Returns true if there are no user IDs or no subkeys.*/
   bool isNull() const;
@@ -509,6 +529,17 @@ class Key
    */
   Validity keyTrust( const QString& uid ) const;
 
+  /** Set the validity values for the user ids to the validity values of
+      the given key. This is useful after rereading a key without expensive
+      trust checking.
+   */
+  void cloneKeyTrust( const Key* key );
+
+  /** Returns true if the key is valid, i.e. not revoked, expired, disabled
+      or invalid.
+   */
+  bool isValid() const;
+
   /** Returns true if the key is a valid encryption key. The trust is not
       checked.
    */
@@ -541,16 +572,16 @@ class Key
 
   /** Adds a subkey with the given values to the key if <em>keyID</em> isn't
       an empty string. */
-  void addSubkey(const QCString& keyID, const bool secret = false);
+  void addSubkey(const KeyID& keyID, const bool secret = false);
 
   /** Adds the given subkey to the key. */
   void addSubkey(const Subkey *subkey);
 
   /** Returns a pointer to the subkey with the given key ID. */
-  Subkey *getSubkey(const QCString& keyID);
+  Subkey *getSubkey(const KeyID& keyID);
 
   /** Sets the fingerprint of the given subkey to <em>fpr</em>. */
-  void setFingerprint(const QCString& keyID, const QCString& fpr);
+  void setFingerprint(const KeyID& keyID, const QCString& fpr);
 
  protected:
   bool mSecret : 1;
@@ -669,12 +700,22 @@ inline QString Key::primaryUserID() const
     return QString::null;
 }
 
-inline QCString Key::primaryKeyID() const 
+inline KeyID Key::primaryKeyID() const 
 {
   Subkey *key = mSubkeys.getFirst();
 
   if (key)
     return key->keyID();
+  else
+    return KeyID();
+}
+
+inline QCString Key::primaryFingerprint() const
+{
+  Subkey *key = mSubkeys.getFirst();
+
+  if (key)
+    return key->fingerprint();
   else
     return QCString();
 }
