@@ -36,10 +36,6 @@
 #include <qapplication.h>
 #include <qtimer.h>
 
-#if QT_VERSION < 290
-# define Q_ASSERT ASSERT
-#endif
-
 // works for both ListBox and ComboBox
 template <class T> static int setCurrentItem(T *box, const QString& s)
 {
@@ -65,18 +61,18 @@ SingleConditionWidget::SingleConditionWidget(KScoringManager *m,QWidget *p, cons
   QBoxLayout *topL = new QVBoxLayout(this,5);
   QBoxLayout *firstRow = new QHBoxLayout(topL);
   neg = new QCheckBox(i18n("Not"),this);
-  QToolTip::add(neg,i18n("negate this condition"));
+  QToolTip::add(neg,i18n("Negate this condition"));
   firstRow->addWidget(neg);
   headers = new KComboBox(this);
   headers->insertStringList(manager->getDefaultHeaders());
-  QToolTip::add(headers,i18n("select the header against this condition matches"));
+  QToolTip::add(headers,i18n("Select the header against this condition match"));
   firstRow->addWidget(headers,1);
   matches = new KComboBox(this);
   matches->insertStringList(KScoringExpression::conditionNames());
-  QToolTip::add(matches,i18n("select the type of match"));
+  QToolTip::add(matches,i18n("Select the type of match"));
   firstRow->addWidget(matches,1);
   expr = new KLineEdit(this);
-  QToolTip::add(expr,i18n("the condition for the match"));
+  QToolTip::add(expr,i18n("The condition for the match"));
   // reserve space for at least 20 characters
   expr->setMinimumWidth(fontMetrics().maxWidth()*20);
   topL->addWidget(expr);
@@ -188,6 +184,10 @@ SingleActionWidget::SingleActionWidget(KScoringManager *m,QWidget *p, const char
   topL->addWidget(types);
   stack = new QWidgetStack(this);
   topL->addWidget(stack);
+  
+  dummyLabel = new QLabel(i18n("Select an action:"), stack);
+  stack->addWidget(dummyLabel, dummyIndex);
+
   // init widget stack and the types combo box
   int index = 0;
   QStringList l = ActionBase::userNames();
@@ -212,9 +212,7 @@ SingleActionWidget::SingleActionWidget(KScoringManager *m,QWidget *p, const char
     }
   }
   types->insertItem(QString::null);
-  dummyLabel = new QLabel(i18n("Please select an action."), stack);
   dummyIndex = index;
-  stack->addWidget(dummyLabel, dummyIndex);
 
   connect(types,SIGNAL(activated(int)),stack,SLOT(raiseWidget(int)));
 
@@ -774,7 +772,7 @@ KScoringEditor::KScoringEditor(KScoringManager* m,
   connect(ruleLister, SIGNAL(leavingRule()),
           ruleEditor, SLOT(updateRule()));
   connect(ruleEditor, SIGNAL(shrink()), SLOT(slotShrink()));
-  connect(this,SIGNAL(finished()),SLOT(slotDelayedDestruct()));
+  connect(this,SIGNAL(finished()),SLOT(slotFinished()));
   ruleLister->slotRuleSelected(0);
   resize(550, sizeHint().height());
 }
@@ -842,13 +840,18 @@ void KScoringEditor::slotCancel()
   KDialogBase::slotCancel();
 }
 
+void KScoringEditor::slotFinished()
+{
+  delayedDestruct();
+}
+
 //============================================================================
 //
 // class KScoringEditorWidgetDialog (a dialog for the KScoringEditorWidget)
 //
 //============================================================================
 KScoringEditorWidgetDialog::KScoringEditorWidgetDialog(KScoringManager *m, const QString& r, QWidget *p, const char *n)
-  : KDialogBase(p,n,true,i18n("Edit rule"),
+  : KDialogBase(p,n,true,i18n("Edit Rule"),
                 KDialogBase::Ok|KDialogBase::Apply|KDialogBase::Close,
                 KDialogBase::Ok,true),
     manager(m), ruleName(r)
