@@ -1,4 +1,4 @@
-/*
+/* -*- c++ -*-
     kwidgetlister.h
 
     Copyright (c) 2001 Marc Mutz <mutz@kde.org>
@@ -15,15 +15,7 @@
 #define _KWIDGETLISTER_H_
 
 #include <qwidget.h>
-#include <qglobal.h>
-#if QT_VERSION >= 290
-#  include <qptrlist.h>
-#else
-// remove when Qt3 become mandatory
-#  include <qlist.h>
-#  define QPtrList QList
-#  define QPtrListIterator QListIterator
-#endif
+#include <qptrlist.h>
 
 class QPushButton;
 class QVBoxLayout;
