@@ -3,12 +3,12 @@
 // Contents:   Definitions for DwTokenizer, DwRfc822Tokenizer
 // Maintainer: Doug Sauder <dwsauder@fwb.gulf.net>
 // WWW:        http://www.fwb.gulf.net/~dwsauder/mimepp.html
-// $Revision: 1.7 $
-// $Date: 2001/12/25 14:34:36 $
+// $Revision: 1.10 $
+// $Date: 2002/04/22 10:01:28 $
 //
 // Copyright (c) 1996, 1997 Douglas W. Sauder
 // All rights reserved.
-// 
+//
 // IN NO EVENT SHALL DOUGLAS W. SAUDER BE LIABLE TO ANY PARTY FOR DIRECT,
 // INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 // THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF DOUGLAS W. SAUDER
@@ -224,7 +224,7 @@ void DwTokenizer::ParseDomainLiteral()
 void DwTokenizer::PrintToken(std::ostream* aOut)
 {
     if (!aOut) return;
-    char* type = 0;
+    const char* type = 0;
     switch (mTkType) {
     case eTkError:
         type = "error          ";
